/*
 * arch/arm/mach-cxd4115/clock.c
 *
 * clock manager
 *
 * Copyright 2007,2008,2009,2010 Sony Corporation
 *
 * This code is based on arch/arm/mach-integrator/clock.c.
 */
/*
 *  linux/arch/arm/mach-integrator/clock.c
 *
 *  Copyright (C) 2004 ARM Limited.
 *  Written by Deep Blue Solutions Limited.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/list.h>
#include <linux/errno.h>
#include <linux/err.h>
#include <linux/string.h>
#include <linux/clk.h>
#include <linux/mutex.h>

#include <asm/clkdev.h>
#include <mach/hardware.h>
#include <asm/io.h>
#include <mach/timex.h>
#include <mach/regs-clk.h>

#include "clock.h"

/*
 *  Enable/Disable UART BAUD clock.
 *    (note) UART PCLK should be always enabled.
 */
static int cxd4115_enable_uart(struct clk *clk, int enable)
{
	writel(enable, VA_CLKBLK_UART_CLK(clk->chan));
	return 0;
}

static struct clk uart_clk[] = {
	/*-------------------- UART --------------------*/
	{
		.name		= "UARTCLK0",
		.rate		= CXD4115_UART_CLK,
		.chan		= 0,
		.enable		= cxd4115_enable_uart,
	},
	{
		.name		= "UARTCLK1",
		.rate		= CXD4115_UART_CLK,
		.chan		= 1,
		.enable		= cxd4115_enable_uart,
	},
	{
		.name		= "UARTCLK2",
		.rate		= CXD4115_UART_CLK,
		.chan		= 2,
		.enable		= cxd4115_enable_uart,
	},
#ifdef CONFIG_ARCH_CXD4120
	{
		.name		= "UARTCLK3",
		.rate		= CXD4115_UART_CLK,
		.chan		= 3,
		.enable		= cxd4115_enable_uart,
	},
#endif /* CONFIG_ARCH_CXD4120 */
	/*----------------------------------------------*/
};

#ifdef CONFIG_QEMU
static int qemu_clk_enable(struct clk *clk, int enable)
{
	return 0;
}

static unsigned long qemu_clk_getrate(struct clk *clk)
{
	return 1;
}

static struct clk qemu_clk = {
	.name		= "QEMU",
	.chan		= 2,
	.get_rate       = qemu_clk_getrate,
	.enable		= qemu_clk_enable,
};
#endif

static struct clk_lookup lookups[] = {
	{	/* UART0 */
		.dev_id		= "apb:050",
		.clk		= &uart_clk[0],
	},
	{	/* UART1 */
		.dev_id		= "apb:051",
		.clk		= &uart_clk[1],
	},
	{	/* UART2 */
		.dev_id		= "apb:052",
		.clk		= &uart_clk[2],
	},
#ifdef CONFIG_ARCH_CXD4120
	{	/* UART3 */
		.dev_id		= "apb:053",
		.clk		= &uart_clk[3],
	},
#endif /* CONFIG_ARCH_CXD4120 */
#ifdef CONFIG_QEMU
	{	/* clcd */
		.dev_id		= "apb:clcd",
		.clk		= &qemu_clk,
	},
#endif
};

int clk_enable(struct clk *clk)
{
	(clk->enable)(clk, 1);
	return 0;
}
EXPORT_SYMBOL(clk_enable);

void clk_disable(struct clk *clk)
{
	(clk->enable)(clk, 0);
}
EXPORT_SYMBOL(clk_disable);

unsigned long clk_get_rate(struct clk *clk)
{
	if (clk->rate != 0)
		return clk->rate;
	if (clk->get_rate != NULL)
		return (clk->get_rate)(clk);
	return clk->rate;
}
EXPORT_SYMBOL(clk_get_rate);

long clk_round_rate(struct clk *clk, unsigned long rate)
{
	return rate;
}
EXPORT_SYMBOL(clk_round_rate);

int clk_set_rate(struct clk *clk, unsigned long rate)
{
	return 0;
}
EXPORT_SYMBOL(clk_set_rate);

static int __init clk_init(void)
{
	int i;

	for (i = 0; i < ARRAY_SIZE(lookups); i++)
		clkdev_add(&lookups[i]);
	return 0;
}
arch_initcall(clk_init);
