/*
 * arch/arm/mach-cxd4115/core.c
 *
 * machine_desc and initialize function
 *
 * Copyright 2007,2008,2009,2010 Sony Corporation
 *
 * This code is based on arch/arm/mach-realview/realview_eb.c.
 */
/*
 *  linux/arch/arm/mach-realview/realview_eb.c
 *
 *  Copyright (C) 2004 ARM Limited
 *  Copyright (C) 2000 Deep Blue Solutions Ltd
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include <linux/device.h>
#include <linux/init.h>
#include <linux/sched.h>
#include <linux/interrupt.h>
#include <linux/irq.h>
#include <linux/sysdev.h>
#include <linux/mman.h>

#ifdef CONFIG_PM
#include <mach/pm.h>
#endif /* CONFIG_PM */

#include <asm/pgtable.h>
#include <asm/page.h>
#include <asm/mach/map.h>
#include <mach/hardware.h>

#include <asm/io.h>
#include <asm/irq.h>

#include <mach/gic.h>
#include <mach/regs-timer.h>
#include <mach/regs-dmac.h>
#include <mach/regs-emc.h>
#include <mach/regs-misc.h>
#include <mach/regs-gpio.h>
#ifdef CONFIG_CXD4115_SUBSYSTEM_MAPPING
# include <mach/pci.h>
#endif /* CONFIG_CXD4115_SUBSYSTEM_MAPPING */
#include <mach/cxd4115_board.h>

#ifdef __MACH_HAS_VALID_PHYS_ADDR_RANGE
int valid_phys_addr_range(unsigned long addr, size_t size)
{
	unsigned long high = addr + size;

	/* DDR0 */
	if (CXD4115_DDR0_BASE <= addr
	    && high <= CXD4115_DDR0_BASE+CXD4120_DDR0_SIZE)
		return 1;
	/* DDR1 */
	if (CXD4115_DDR1_BASE <= addr
	    && high <= CXD4115_DDR1_BASE+CXD4120_DDR1_SIZE)
		return 1;
	/* SDRAM */
	if (EMC_CS1_BASE <= addr
	    && high <= EMC_CS1_BASE+CXD4120_SDR_SIZE)
		return 1;

	return 0;
}
#endif /* __MACH_HAS_VALID_PHYS_ADDR_RANGE */

/*---------------------------- I/O mapping --------------------------*/
/*
  Logical    Physical
  F8000000   78000000 Peripherals
  FE000000   FFF00000 Peripherals
*/

static struct map_desc cxd4115_io_desc[] __initdata = {
	{
		.virtual	= IO_ADDRESS(CXD4115_AHB_BASE),
		.pfn		= __phys_to_pfn(CXD4115_AHB_BASE),
		.length		= CXD4115_AHB_SIZE,
		.type		= MT_DEVICE },
	{
		.virtual	= IO_ADDRESS(CXD4115_APB_BASE),
		.pfn		= __phys_to_pfn(CXD4115_APB_BASE),
		.length		= CXD4115_APB_SIZE,
		.type		= MT_DEVICE },
	{
		.virtual	= HIO_ADDRESS(CXD4115_HIO_BASE),
		.pfn		= __phys_to_pfn(CXD4115_HIO_BASE),
		.length		= CXD4115_HIO_SIZE,
		.type		= MT_DEVICE },
#ifdef CONFIG_SUBSYSTEM
	{	/* for BOOTRAM */
		.virtual	= 0xfff00000,
		.pfn		= __phys_to_pfn(CXD4115_BOOTRAM_BASE),
		.length		= SZ_16K,
		.type		= MT_DEVICE },
#endif /* CONFIG_SUBSYSTEM */
#ifdef CONFIG_CXD4115_SUBSYSTEM_MAPPING
# ifdef CONFIG_ARCH_CXD4120
#  if CONFIG_PAGE_OFFSET > 0x40000000
#   error "CONFIG_PAGE_OFFSET > 0x40000000"
#  else
	{	/* DDR0 uncached */
		.virtual	= 0x90000000,
		.pfn		= __phys_to_pfn(CXD4115_DDR0_BASE),
		.length		= SZ_64M,
		.type		= MT_DEVICE },
	{	/* DDR1 uncached */
		.virtual	= 0xb0000000,
		.pfn		= __phys_to_pfn(CXD4115_DDR1_BASE),
		.length		= SZ_32M,
		.type		= MT_DEVICE },
	{	/* PCI window2 */
		.virtual	= 0x50000000,
		.pfn		= __phys_to_pfn(AHBPCI_WINDOW2_BASE),
		.length		= AHBPCI_WINDOW2_SIZE,
		.type		= MT_DEVICE },
	{	/* SDRAM cached */
		.virtual	= 0x60000000,
		.pfn		= __phys_to_pfn(EMC_CS1_BASE),
		.length		= SZ_64M,
		.type		= MT_MEMORY },
	{	/* SDRAM uncached */
		.virtual	= 0x70000000,
		.pfn		= __phys_to_pfn(EMC_CS1_BASE),
		.length		= SZ_64M,
		.type		= MT_DEVICE },
	{	/* AHB, APB */
		.virtual	= 0x78000000,
		.pfn		= __phys_to_pfn(CXD4115_AHB_BASE),
		.length		= SZ_128M,
		.type		= MT_DEVICE },
	{	/* PCI window3 */
		.virtual	= 0x80000000,
		.pfn		= __phys_to_pfn(AHBPCI_WINDOW3_BASE),
		.length		= AHBPCI_WINDOW3_SIZE,
		.type		= MT_DEVICE },
	{	/* BOOTRAM, Bridge, MPCore, ROM */
		.virtual	= 0xaff00000,
		.pfn		= __phys_to_pfn(CXD4115_HIO_BASE),
		.length		= SZ_1M,
		.type		= MT_DEVICE },
#  endif
# endif /* CONFIG_ARCH_CXD4120 */
#endif /* CONFIG_CXD4115_SUBSYSTEM_MAPPING */
};

void __init cxd4115_map_io(void)
{
	iotable_init(cxd4115_io_desc, ARRAY_SIZE(cxd4115_io_desc));
}

/*------------------------------ SYSDEV -----------------------------*/
#ifdef CONFIG_PM
static int cxd4115_suspend(struct sys_device *dev, pm_message_t state)
{
	gic_suspend();
	return 0;
}

static int cxd4115_resume(struct sys_device *dev)
{
	/* Enable DMAC */
	writel(0x1, DMACCONFIGRATION);

	cxd4115_board_init();

	gic_resume();
	return 0;
}
#endif /* CONFIG_PM */

static struct sysdev_class cxd4115_sysclass = {
	.name		= "cxd4115",
#ifdef CONFIG_PM
	.suspend	= cxd4115_suspend,
	.resume		= cxd4115_resume,
#endif
};

static struct sys_device cxd4115_device = {
	.id		= 0,
	.cls		= &cxd4115_sysclass,
};

#ifdef CONFIG_PM
static void cxd4115_reset(int force)
{
	local_irq_disable();
	/* assert XRESET_REQ */
	writel(1, VA_MISCCTRL(MISCCTRL_XRESETREQ)+MISCCTRL_CLR);

	while (1) ;
}
void (*pm_machine_reset)(int) = cxd4115_reset;
EXPORT_SYMBOL(pm_machine_reset);

void cxd4115_poweroff(void)
{
	unsigned int port, bit;
	extern unsigned long cxd41xx_xpower_off;

	local_irq_disable();

	if (0xffffffffUL != cxd41xx_xpower_off) {
		port = cxd41xx_xpower_off & 0xff;
		bit  = (cxd41xx_xpower_off >> 8) & 0xff;

		/* DATA=L */
		writel(1 << bit, VA_GPIO(port)+GPIO_DATA_CLEAR);
		/* DIR=Output */
		gpio_set(port, GPIO_DIR, bit);
	}

	while (1) ;
}
#endif /* CONFIG_PM */

int __init cxd4115_core_init(void)
{
	int ret;
#ifdef CONFIG_OVERCOMMIT_ALWAYS
	extern int sysctl_overcommit_memory;
#endif

#ifdef CONFIG_PM
	pm_power_off = cxd4115_poweroff;
	cxd41xx_pm_setup();
#endif
#ifdef CONFIG_OVERCOMMIT_ALWAYS
	sysctl_overcommit_memory = OVERCOMMIT_ALWAYS;
#endif

	/*
	 *  Enable DMAC0
	 *    (note) DMAC0 HCLK is enabled (default).
	 */
	writel(0x1, DMACCONFIGRATION);

	ret = sysdev_class_register(&cxd4115_sysclass);
	if (ret) {
		printk(KERN_ERR "cxd4115_core_init: sysdev_class_register failed.\n");
		return ret;
	}
	ret = sysdev_register(&cxd4115_device);
	if (ret) {
		printk(KERN_ERR "cxd4115_core_init: sysdev_register failed.\n");
		return ret;
	}
	return ret;
}

/* List of edge trigger IRQs */
static u8 edge_irqs[] __initdata = {
#ifdef CONFIG_MPCORE_GIC_INIT
	IRQ_GPIO_RISE(0), IRQ_GPIO_RISE(1), IRQ_GPIO_RISE(2), IRQ_GPIO_RISE(3),
	IRQ_GPIO_RISE(4), IRQ_GPIO_RISE(5), IRQ_GPIO_RISE(6), IRQ_GPIO_RISE(7),
	IRQ_GPIO_RISE(8), IRQ_GPIO_RISE(9), IRQ_GPIO_RISE(10),IRQ_GPIO_RISE(11),
	IRQ_GPIO_RISE(12),IRQ_GPIO_RISE(13),IRQ_GPIO_RISE(14),IRQ_GPIO_RISE(15),
	IRQ_GPIO_RISE(16),IRQ_GPIO_RISE(17),IRQ_GPIO_RISE(18),IRQ_GPIO_RISE(19),
	IRQ_GPIO_RISE(20),IRQ_GPIO_RISE(21),IRQ_GPIO_RISE(22),
	IRQ_GPIO_FALL(0), IRQ_GPIO_FALL(1), IRQ_GPIO_FALL(2), IRQ_GPIO_FALL(3),
	IRQ_GPIO_FALL(4), IRQ_GPIO_FALL(5), IRQ_GPIO_FALL(6), IRQ_GPIO_FALL(7),
	IRQ_GPIO_FALL(8), IRQ_GPIO_FALL(9), IRQ_GPIO_FALL(10),IRQ_GPIO_FALL(11),
	IRQ_GPIO_FALL(12),IRQ_GPIO_FALL(13),IRQ_GPIO_FALL(14),IRQ_GPIO_FALL(15),
	IRQ_GPIO_FALL(16),IRQ_GPIO_FALL(17),IRQ_GPIO_FALL(18),IRQ_GPIO_FALL(19),
	IRQ_GPIO_FALL(20),IRQ_GPIO_FALL(21),IRQ_GPIO_FALL(22),
	88, 89, 90, 91, 92, 93, 94, 95, 96, 97, 98, 99,
	100, 101, 102, 103, 104, 105, 106, 107, 108, 109, 110, 111,
	IRQ_VBUSVALID_RISE,IRQ_VBUSVALID_FALL,IRQ_AVALID_RISE,IRQ_AVALID_FALL,
	IRQ_SESSEND_RISE,IRQ_SESSEND_FALL,
#ifdef CONFIG_ARCH_CXD4120
	IRQ_GPIO_RISE(23), IRQ_GPIO_FALL(23), 148, 149, 203, 204,
#endif
#endif /* CONFIG_MPCORE_GIC_INIT */
};

void __init cxd4115_init_irq(void)
{
	gic_dist_init(0, VA_GIC_DIST, 29, edge_irqs, sizeof edge_irqs);
	gic_cpu_init(0, VA_GIC_CPU);
}
