/*
 * arch/arm/mach-cxd4115/cxd4115.c
 *
 * CXD4115 devices
 *
 * Copyright 2007,2008,2009,2010 Sony Corporation
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 *
 */
#include <linux/device.h>
#include <linux/init.h>
#include <linux/platform_device.h>
#include <linux/amba/bus.h>
#include <mach/hardware.h>
#include <asm/irq.h>
#include <asm/mach/arch.h>
#include <asm/mach-types.h>
#include <mach/uart.h>
#include <mach/regs-pci.h>
#include <mach/regs-emc.h>
#include <mach/regs-gpio.h>
#include <mach/regs-octrl.h>
#include <asm/io.h>
#include <mach/cxd4115_board.h>

#ifdef CONFIG_SMSC911X
#include <linux/smsc911x.h>
#endif

static struct uart_platform_data uart0_data = {
	.chan		= 0,
	.fifosize	= 32,
	.clock		= "UARTCLK0",
};

static struct uart_platform_data uart1_data = {
	.chan		= 1,
	.fifosize	= 32,
	.clock		= "UARTCLK1",
};

static struct uart_platform_data uart2_data = {
	.chan		= 2,
	.fifosize	= 32,
	.clock		= "UARTCLK2",
};

#ifdef CONFIG_ARCH_CXD4120
static struct uart_platform_data uart3_data = {
	.chan		= 3,
	.fifosize	= 32,
	.clock		= "UARTCLK3",
};
#endif /* CONFIG_ARCH_CXD4120 */

static struct amba_device uart0_device = {
	.dev = {
		.bus_id = "apb:050",
		.platform_data	= &uart0_data,
	},
	.res = {
		 .start = CXD4115_UART(0),
		 .end   = CXD4115_UART(0)+SZ_4K-1,
		 .flags = IORESOURCE_MEM,
	 },
	.irq = { IRQ_UART(0), },
};

static struct amba_device uart1_device = {
	.dev = {
		.bus_id = "apb:051",
		.platform_data	= &uart1_data,
	},
	.res = {
		 .start = CXD4115_UART(1),
		 .end   = CXD4115_UART(1)+SZ_4K-1,
		 .flags = IORESOURCE_MEM,
	 },
	.irq = { IRQ_UART(1), },
};

static struct amba_device uart2_device = {
	.dev = {
		.bus_id = "apb:052",
		.platform_data	= &uart2_data,
	},
	.res = {
		 .start = CXD4115_UART(2),
		 .end   = CXD4115_UART(2)+SZ_4K-1,
		 .flags = IORESOURCE_MEM,
	 },
	.irq = { IRQ_UART(2), },
};

#ifdef CONFIG_ARCH_CXD4120
static struct amba_device uart3_device = {
	.dev = {
		.bus_id = "apb:053",
		.platform_data	= &uart3_data,
	},
	.res = {
		.start = CXD4115_UART(3),
		.end   = CXD4115_UART(3)+SZ_4K-1,
		.flags = IORESOURCE_MEM,
	},
	.irq = { IRQ_UART(3), },
};
#endif /* CONFIG_ARCH_CXD4120 */

static struct amba_device *uart_devs[] __initdata = {
	&uart0_device,
	&uart1_device,
	&uart2_device,
#ifdef CONFIG_ARCH_CXD4120
	&uart3_device,
#endif /* CONFIG_ARCH_CXD4120 */
};

#ifdef CONFIG_CXD4115_PCIBRIDGE_ENABLE
static struct amba_device pcibridge_device = {
	.dev = {
		.bus_id = "ahb:6",
	},
	.res = {
		 .start = CXD4115_PCI_BASE,
		 .end   = CXD4115_PCI_BASE+CXD4115_PCI_SIZE-1,
		 .flags = IORESOURCE_MEM,
	 },
	.irq = { IRQ_PCI_BRIDGE, },
	.periphid = CXD4115_PCI_BASE,
};
#endif /* CONFIG_CXD4115_PCIBRIDGE_ENABLE */

static struct amba_device *amba_devs[] __initdata = {
#ifdef CONFIG_CXD4115_PCIBRIDGE_ENABLE
	&pcibridge_device,
#endif
};

#if defined(CONFIG_SMC9118) || defined(CONFIG_SMSC911X)

#define SMC911X_INT	CONFIG_CXD41XX_SMC911X_INT
#define SMC9118_EMC_CS	CONFIG_CXD4115_SMC9118_EMC_CS
#ifdef CONFIG_CXD4115_SMC9118_ON_EMC_CS1
# define SMC9118_MEM_START	EMC_CS1_BASE
#else
# define SMC9118_MEM_START	EMC_CS2_BASE
#endif
static struct resource smc9118_resources[] = {
	[0] = {
		.start	= SMC9118_MEM_START,
		.end	= SMC9118_MEM_START + SZ_4K-1,
		.flags	= IORESOURCE_MEM,
	},
	[1] = {
		.start  = IRQ_GPIO_FALL(SMC911X_INT),
		.end	= IRQ_GPIO_FALL(SMC911X_INT),
		.flags	= IORESOURCE_IRQ,
	}
};

#if defined(CONFIG_SMSC911X)
static struct smsc911x_platform_config smsc911x_config = {
	.irq_polarity	= 0,  /* active low */
	.irq_type	= 1,
	.flags		= SMSC911X_USE_32BIT,
};
#endif

static struct platform_device smc9118_device = {
#if defined(CONFIG_SMC9118)
	.name		= "smc911x",
#else /* CONFIG_SMSC911X */
	.name		= "smsc911x",
#endif
	.id		= 0,
	.num_resources	= ARRAY_SIZE(smc9118_resources),
	.resource	= smc9118_resources,
#if defined(CONFIG_SMSC911X)
	.dev = {
		.platform_data = &smsc911x_config,
	},
#endif
};
#endif /* CONFIG_SMC9118  || CONFIG_SMSC911X */

#ifdef CONFIG_ARCH_CXD4120
# define EMCSYSCLK 129000000
#else /* CONFIG_ARCH_CXD4115 */
# define EMCSYSCLK (CONFIG_CXD4115_CPUCLK/2)
#endif

#if EMCSYSCLK == 129000000
 /* period=7.75ns */
 /* AM2 PRM1 (TWEA=0x02, TWEN=0x06, TWDH=0x02, TWWAIT=0x04) */
 /*                15.5,      46.5,      15.5,        31.0  */
# define EMC_PARAM1 0x02060204
 /* AM2 PRM2 (TOEA=0x02, TRDLAT=0x08, TRAH=0x00, TRWAIT=0x04) */
 /*                15.5,      62.0,           0,        31.0  */
# define EMC_PARAM2 0x02080004
#elif EMCSYSCLK == 78000000
 /* period=12.8ns */
 /* AM2 PRM1 (TWEA=0x01, TWEN=0x04, TWDH=0x01, TWWAIT=0x02) */
 /*                12.8,      51.2,      12.8,        25.6  */
# define EMC_PARAM1 0x01040102
 /* AM2 PRM2 (TOEA=0x01, TRDLAT=0x05, TRAH=0x00, TRWAIT=0x02) */
 /*                12.8,        64.0,         0,        25.6  */
# define EMC_PARAM2 0x01050002
#elif EMCSYSCLK == 66000000
 /* period=15ns */
 /* AM2 PRM1 (TWEA=0x01, TWEN=0x03, TWDH=0x01, TWWAIT=0x02) */
 /*                15.0,      45.0,        15,        30.0  */
# define EMC_PARAM1 0x01030102
 /* AM2 PRM2 (TOEA=0x01, TRDLAT=0x04, TRAH=0x00, TRWAIT=0x02) */
 /*                15.0,        60.0,         0,        30.0  */
# define EMC_PARAM2 0x01040002
#else
# error "Unknown EMCSYSCLK"
#endif

void cxd4115_board_init(void)
{
#if defined(CONFIG_SMC9118) || defined(CONFIG_SMSC911X)
	u32 dat;
	extern unsigned long cxd41xx_ether_irq;

	/* bind EMC CSx to AM2 */
	dat = readl(VA_EMC + EMC_CS_PHY_BIND);
	dat &= ~(EMC_CS_MASK << EMC_CS_SHIFT(SMC9118_EMC_CS));
	dat |= EMC_PHY_AM2 << EMC_CS_SHIFT(SMC9118_EMC_CS);
	writel(dat, VA_EMC + EMC_CS_PHY_BIND);
	/* config AM2 as PageMode:OFF,DW=16bit */
	writel(EMC_PHY_AM_DW16, VA_EMC + EMC_PHY_AM2_CFG);
	writel(EMC_PARAM1, VA_EMC + EMC_PHY_AM2_PRM1);
	writel(EMC_PARAM2, VA_EMC + EMC_PHY_AM2_PRM2);
	/* INT pin setup */
	/* Pullup
         *   (note) It is OK to set OCTRL regs for both
         *          CXD4115 and CXD9938. No side effects.
         */
	/* for CXD4115 */
	writel(1<<cxd41xx_ether_irq, VA_OCTRL_SET(23)); /* pullup */
	writel(1<<cxd41xx_ether_irq, VA_OCTRL_CLR(22)); /* Pu/Pd enable */
	/* for CXD9938 */
	writel(1<<cxd41xx_ether_irq, VA_OCTRL_SET(535)); /* P1=1 */
	writel(1<<cxd41xx_ether_irq, VA_OCTRL_CLR(534)); /* P2=0 */
	dat = readl(VA_GPIO(0) + GPIO_INEN);
	dat |= 1 << cxd41xx_ether_irq;
	writel(dat, VA_GPIO(0) + GPIO_INEN); /* input enable */
#endif /* CONFIG_SMC9118 || CONFIG_SMSC911X */
}

static void __init cxd4115_init(void)
{
	int i, ret;
	struct amba_device **dev;
	extern unsigned long cxd41xx_uart_bitmap;

	cxd4115_board_init();
	ret = cxd4115_core_init();
	for (i = 0, dev = uart_devs; i < ARRAY_SIZE(uart_devs); i++, dev++) {
		if (cxd41xx_uart_bitmap & (1 << i)) {
			ret = amba_device_register(*dev, &iomem_resource);
			if (ret) {
				printk(KERN_ERR "cxd4115_init: amba_device_register(uart_devs[%d]) failed.\n", i);
			}
		}
	}
	for (i = 0, dev = amba_devs; i < ARRAY_SIZE(amba_devs); i++, dev++) {
		ret = amba_device_register(*dev, &iomem_resource);
		if (ret) {
			printk(KERN_ERR "cxd4115_init: amba_device_register(amba_devs[%d]) failed.\n", i);
		}
	}

#if defined(CONFIG_SMC9118) || defined(CONFIG_SMSC911X)
	{ /* Override IRQ pin */
		extern unsigned long cxd41xx_ether_irq;
		smc9118_resources[1].start = IRQ_GPIO_FALL(cxd41xx_ether_irq);
		smc9118_resources[1].end   = IRQ_GPIO_FALL(cxd41xx_ether_irq);
	}
	ret = platform_device_register(&smc9118_device);
	if (ret) {
		printk(KERN_ERR "cxd4115_init: platform_device_register failed.\n");
	}
#endif /* CONFIG_SMC9118 || CONFIG_SMSC911X */
}

MACHINE_START(CXD4115, "ARM-CXD4115")
	/* Maintainer: Sony Corporation */
	.phys_io	= CXD4115_APB_BASE,
	.io_pg_offst	= (IO_ADDRESS(CXD4115_APB_BASE) >> 18) & 0xfffc,
	.map_io		= cxd4115_map_io,
	.init_irq	= cxd4115_init_irq,
	.timer		= &cxd4115_timer,
	.init_machine	= cxd4115_init,
#ifdef CONFIG_QEMU
	.boot_params	= PHYS_OFFSET + 0x900100,
#endif
MACHINE_END
