/*-----------------------------------------------------------------------------
 * cxd4115_gpio.h
 *
 * a header file for gpio
 *
 * Copyright 2008,2009,2010 Sony Corporation
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 * MA  02110-1301, USA.
 *---------------------------------------------------------------------------*/

#ifndef	__CXD4115_GPIO__H

#define	__CXD4115_GPIO__H

// GPIO Port Number
enum gpio_portid {
	GPIO_PORT_0 = 0,
	GPIO_PORT_1,
	GPIO_PORT_2,
	GPIO_PORT_3,
	GPIO_PORT_4,
	GPIO_PORT_5,
	GPIO_PORT_6,
	GPIO_PORT_7,
#ifdef CONFIG_ARCH_CXD4120
	GPIO_PORT_8,
	GPIO_PORT_9,
	GPIO_PORT_10,
	GPIO_PORT_11,
	GPIO_PORT_12,
	GPIO_PORT_13,
	GPIO_PORT_14,
	GPIO_PORT_15,
	GPIO_PORT_16,
	GPIO_PORT_17,
	GPIO_PORT_18,
	GPIO_PORT_19,
	GPIO_PORT_20,
	GPIO_PORT_21,
	GPIO_PORT_22,
	GPIO_PORT_23,
	GPIO_PORT_24,
#endif /* CONFIG_ARCH_CXD4120 */
	GPIO_PORT_MAX
};
typedef enum gpio_portid gpio_portid;

// function
#define GPIOMODE_GPIO       (0x00000001) // General PORT
#define GPIOMODE_PERIPHERAL (0x00000002) // Peripheral
// Direction
#define GPIOMODE_INPUT      (0x00000004) // Input
#define GPIOMODE_OUTPUT_H   (0x00000008) // Output with High level
#define GPIOMODE_OUTPUT_L   (0x00000010) // Output with Low level

// bit width
#define GPIO_MAXBIT_WIDTH (32) //set maximum bit width

// data type(for bit number)
typedef unsigned char gpio_bitnum;

// data type(for bitmap)
#if GPIO_MAXBIT_WIDTH <= 8
typedef unsigned char gpio_bitmap;
#elif GPIO_MAXBIT_WIDTH <= 16
typedef unsigned short gpio_bitmap;
#else
typedef unsigned long gpio_bitmap;
#endif

// data type(for bitmask)
typedef gpio_bitmap  gpio_bitmask;

// data type(for GPIO mode)
typedef unsigned long gpio_mode;

// data type(for bit value)
typedef unsigned char gpio_bit;

// ioctl command
enum gpio_ioctl_cmd {
	GPIO_CMD_OPEN = 0,
	GPIO_CMD_CLOSE,
	GPIO_CMD_READ,
	GPIO_CMD_WRITE,
	GPIO_CMD_SET_MODE,
	GPIO_CMD_GET_MODE,
};
typedef enum gpio_ioctl_cmd gpio_ioctl_cmd;

// for GPIO_CMD_OPEN and GPIO_CMD_CLOSE
struct gpio_ioctl_port{
	unsigned char	portctl; // port contrl flag
	gpio_portid	portid;  // Port number
	gpio_bitnum	bitnum;  // bit number
	gpio_bitmask	bitmask; // bitmask
};
typedef struct gpio_ioctl_port gpio_ioctl_port;

// for GPIO_CMD_READ and GPIO_CMD_WRITE
struct gpio_ioctl_value{
	gpio_ioctl_port port;
	gpio_bitmap value;            // when read, value is set from GPIO
                                      // when write, value is set to GPIO
};
typedef struct gpio_ioctl_value gpio_ioctl_value;

// for GPIO_CMD_SET_MODE
struct gpio_ioctl_mode{
	gpio_ioctl_port port;
	gpio_mode mode;               // mode
};
typedef struct gpio_ioctl_mode gpio_ioctl_mode;



int gpio_register_bit( gpio_portid port, gpio_bitnum bit );
int gpio_unregister_bit( gpio_portid port, gpio_bitnum bit );
int gpio_register_port( gpio_portid port, gpio_bitmask bitmask );
int gpio_unregister_port( gpio_portid port, gpio_bitmask bitmask );
int gpio_set_mode_bit( gpio_portid port,
				gpio_bitnum bit, gpio_mode mode );
int gpio_set_mode_port( gpio_portid port,
                              gpio_bitmask bitmask, gpio_mode mode );
int gpio_get_mode_bit( gpio_portid port,
				gpio_bitnum bit, gpio_mode* mode );
int gpio_get_data_bit( gpio_portid port,
				gpio_bitnum bit, gpio_bit* bit_value );
int gpio_set_data_bit( gpio_portid port,
				gpio_bitnum bit, gpio_bit bit_value );
int gpio_get_data_port( gpio_portid port,
				gpio_bitmask bitmask,
				gpio_bitmap* bitmap_value );
int gpio_set_data_port( gpio_portid port,
				gpio_bitmask bitmask,
				gpio_bitmap bitmap_value );

extern int gpio_init(void);
#endif
