/*
 * include/asm-arm/arch-cxd4115/dma.h
 *
 * DMA definitions
 *
 * Copyright 2007,2008,2009,2010 Sony Corporation
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 *
 */
#ifndef __ASM_ARCH_DMA_H
#define __ASM_ARCH_DMA_H

#include <asm/io.h>
#include <mach/hardware.h>
#include <mach/regs-dmac.h>

struct dma_lli {
	unsigned long src;
	unsigned long dst;
	unsigned long next;
	unsigned long ctrl;
};

int dma_stop(unsigned int ch);

static inline int dma_start(unsigned int ch, struct dma_lli *p, unsigned int cfg)
{
	int ret;

	ret = dma_stop(ch);
	if (unlikely(ret))
		return ret;

	writel(p->src, DMACCSRCADDR_(ch));
	writel(p->dst, DMACCDESTADDR_(ch));
	writel(p->next, DMACCLLI_(ch));
	writel(p->ctrl, DMACCCONTROL_(ch));
	writel(cfg, DMACCCONFIGURATION_(ch));

	return ret;
}

static inline void dma_setup(struct dma_lli *p,
			    unsigned long src, unsigned long dst,
			    unsigned long next, unsigned long ctrl)
{
	p->src	= src;
	p->dst	= dst;
	p->next	= next;
	p->ctrl	= ctrl;
}

static inline void dma_clear(unsigned int ch)
{
	writel(1 << ch, DMACINTTCCLR); /* INT clear */
}

static inline int dma_inttc_status(unsigned int ch)
{
	return !!(readl(DMACINTTCSTATUS) & (1 << ch));
}

#endif /* _ASM_ARCH_DMA_H */
