/*
 * include/asm-arm/arch-cxd4115/memory.h
 *
 * memory address definitions
 *
 * Copyright 2007,2008,2009,2010 Sony Corporation
 *
 * This code is based on include/asm-arm/arch-integrator/memory.h.
 */
/*
 *  linux/include/asm-arm/arch-integrator/mmu.h
 *
 *  Copyright (C) 1999 ARM Limited
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#ifndef __ARCH_CXD4115_MEMORY_H
#define __ARCH_CXD4115_MEMORY_H
#include <asm/page.h>
#include <mach/hardware.h>

#ifdef CONFIG_SUBSYSTEM
#define PAGE_OFFSET		UL(0xc0000000)
# ifdef CONFIG_ARCH_CXD4120
#  define TASK_SIZE		UL(CONFIG_PAGE_OFFSET)
#  define TASK_UNMAPPED_BASE	UL(CONFIG_PAGE_OFFSET / 8)
# else /* CONFIG_ARCH_CXD4120 */
#  define TASK_SIZE		(UL(CONFIG_PAGE_OFFSET) - UL(0x01000000))
#  define TASK_UNMAPPED_BASE	UL(CONFIG_PAGE_OFFSET / 2)
# endif /* CONFIG_ARCH_CXD4120 */
#endif /* CONFIG_SUBSYSTEM */

#ifdef __ASSEMBLY__
#define PHYS_OFFSET	(CONFIG_CXD4115_PHYS_OFFSET)
#define BUS_OFFSET	(CONFIG_CXD4115_PHYS_OFFSET)
#else
#define PHYS_OFFSET	((unsigned long)CONFIG_CXD4115_PHYS_OFFSET)
#define BUS_OFFSET	((unsigned long)CONFIG_CXD4115_PHYS_OFFSET)
#endif

/*
 * Virtual view <-> DMA view memory address translations
 * virt_to_bus: Used to translate the virtual address to an
 *              address suitable to be passed to set_dma_addr
 * bus_to_virt: Used to convert an address for DMA operations
 *              to an address that the kernel can use.
 */
#define __virt_to_bus__is_a_macro
#define __bus_to_virt__is_a_macro
#define __virt_to_bus(x)	__virt_to_phys((unsigned long)(x))
#define __bus_to_virt(x)	__phys_to_virt((unsigned long)(x))

#define MAX_DMA_ADDRESS		pfn_to_kaddr(max_pfn + PHYS_PFN_OFFSET)

#if defined(CONFIG_SPARSEMEM)
#define SECTION_SIZE_BITS	CONFIG_SNSC_SPARSE_SECTION_SHIFT
#define MAX_PHYSADDR_BITS	32
#define MAX_PHYSMEM_BITS	32

#ifdef CONFIG_ARCH_CXD4120
/*
   Ascending order of VA is SDR, DDR1, DDR0.
   So, memend is the end of DDR0.
 */
#define MAX_HOTPLUG_MEMEND	(0x14000000UL)
/*
  a trick to change the order of memory.
                         physical            modified physical
  DDR0(MAX128MB): 0x10000000-0x17ffffff -> 0x78000000-0x7fffffff
  DDR1(MAX128MB): 0x30000000-0x37ffffff -> 0x70000000-0x77ffffff
  SDR (MAX128MB): 0x60000000-0x67ffffff -> 0x60000000-0x67ffffff
 */
/* X is SDR. Where x is physical address. */
#define __PRESDR(x)		((x) >= 0x60000000UL)
/* X is SDR. Where x is modified physical address. */
#define __AFTSDR(x)		((x) <  0x70000000UL)
#define __SEGSIZE		(0x08000000UL)
#define __SEGMASK		(__SEGSIZE - 1)
#define __SEGADR(x)		((x) & ~__SEGMASK)
#define __SEGOFF(x)		((x) & __SEGMASK)
#define __SEGSHIFT		(2)
#define __PREMODIFIER           (0xf0000000UL)
#define __AFTMODIFIER           (0x40000000UL)
#define __MOD(x)  (__SEGADR((((x)^__PREMODIFIER)>>__SEGSHIFT)+__AFTMODIFIER) | __SEGOFF(x))
#define __RMOD(x) (__AFTSDR(x)?(x):(((__SEGADR(x)<<__SEGSHIFT)^__PREMODIFIER) | __SEGOFF(x)))

#define __virt_to_phys(x)	__RMOD((x) - PAGE_OFFSET + PHYS_OFFSET)
#define __phys_to_virt(x)	(__MOD(x) - PHYS_OFFSET + PAGE_OFFSET)

#define __MACH_HAS_VALID_PHYS_ADDR_RANGE
#define CXD4120_DDR0_SIZE	SZ_64M
#define CXD4120_DDR1_SIZE	SZ_32M
#define CXD4120_SDR_SIZE	SZ_64M

#else /* CONFIG_ARCH_CXD4120 */
# define MAX_HOTPLUG_MEM	(256 * SZ_1M)
#endif /* CONFIG_ARCH_CXD4120 */
#endif /* CONFIG_SPARSEMEM */

#if defined(CONFIG_NUMA) && defined(CONFIG_ARCH_CXD4120)
#define PFN_TO_NID(pfn)         pfn_to_nid(pfn)
#else
#define PFN_TO_NID(addr)	(0)
#endif /* CONFIG_NUMA */

#endif /* __ARCH_CXD4115_MEMORY_H */
