/*
 * noncache.h
 *
 * Physical address to NonCacheable/Cacheable address converter
 *
 * Copyright 2010 Sony Corporation
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 *
 */

#ifndef __MACH_CXD4115_INCLUDE_MACH_NONCACHE_H
#define __MACH_CXD4115_INCLUDE_MACH_NONCACHE_H
/*
 * NAME
 * 	PHYS_TO_NONCACHE, PHYS_TO_CACHE MACRO - Convert physical address to
 *	{noncacheable,cacheable} kernel address
 *
 * SYNOPSIS
 *	#include <mach/noncache.h>
 *	unsigned long PHYS_TO_NONCACHE(unsigned long);
 *	unsigned long PHYS_TO_CACHE(unsigned long);
 * INPUT
 *	Physical address of DDR0, DDR1, SDR.
 * RETURN VALUE
 *	PHYS_TO_NONCACHE: Noncacheable kernel address
 *	PHYS_TO_CACHE:    Cacheable kernel address
 *
 *	If input is out of range, PA2NC_ERR shall be returned.
 */
/*
 * NAME
 * 	VA_TO_PHYS - Convert virtual address to physical address
 *
 * SYNOPSIS
 *	#include <mach/noncache.h>
 *	unsigned long VA_TO_PHYS(unsigned long);
 * INPUT
 *	Kernel direct map address of DDR0, DDR1, SDR.
 * RETURN VALUE
 *	Physical address
 *
 *	If input is out of range, PA2NC_ERR shall be returned.
 */
#include <mach/noncache-common.h>

#if defined(CONFIG_ARCH_CXD4120) || !defined(AUTOCONF_INCLUDED)
/*
 *  CXD4120
 *
 *   Physical     NonCacheable  Cacheable
 *  0x10000000 --> 0x90000000	0xd8000000  ;DDR0
 *  0x30000000 --> 0xb0000000	0xd0000000  ;DDR1
 *  0x60000000 --> 0x70000000	0xc0000000  ;SDRAM
 */
/* Physical address definitions */
#define PA2NC_DDR0_SIZE		(0x04000000UL)
#define PA2NC_DDR1_SIZE		(0x02000000UL)
#define PA2NC_SDR_SIZE		(0x04000000UL)
#define PA2NC_P_DDR0_START	(0x10000000UL)
#define PA2NC_P_DDR0_END	PA2NC_END(DDR0, P)
#define PA2NC_P_DDR1_START	(0x30000000UL)
#define PA2NC_P_DDR1_END	PA2NC_END(DDR1, P)
#define PA2NC_P_SDR_START	(0x60000000UL)
#define PA2NC_P_SDR_END		PA2NC_END(SDR,  P)
/* NonCacheable */
#define PA2NC_N_DDR0_START	(0x90000000UL)
#define PA2NC_N_DDR0_END	PA2NC_END(DDR0, N)
#define PA2NC_N_DDR1_START	(0xb0000000UL)
#define PA2NC_N_DDR1_END	PA2NC_END(DDR1, N)
#define PA2NC_N_SDR_START	(0x70000000UL)
#define PA2NC_N_SDR_END		PA2NC_END(SDR,  N)
/* Cacheable */
#define PA2NC_C_DDR0_START	(0xd8000000UL)
#define PA2NC_C_DDR0_END	PA2NC_END(DDR0, C)
#define PA2NC_C_DDR1_START	(0xd0000000UL)
#define PA2NC_C_DDR1_END	PA2NC_END(DDR1, C)
#define PA2NC_C_SDR_START	(0xc0000000UL)
#define PA2NC_C_SDR_END		PA2NC_END(SDR,  C)

/* Physical <--> NonCacheable stuff */
#define PA2NC_SDR_NC_BIT	(0x10000000UL)
#define PA2NC_DDR_NC_BIT	(0x80000000UL)
#define PA2NC_SDR_P2N(x)	((x) | PA2NC_SDR_NC_BIT)
#define PA2NC_DDR_P2N(x)	((x) | PA2NC_DDR_NC_BIT)
#define PA2NC_SDR_N2P(x)	((x) & ~PA2NC_SDR_NC_BIT)
#define PA2NC_DDR_N2P(x)	((x) & ~PA2NC_DDR_NC_BIT)
/* Physical <--> Cacheable stuff (from __phys_to_virt) */
#define PA2NC_SEGSIZE		(0x08000000UL)
#define PA2NC_SEGMASK		(PA2NC_SEGSIZE - 1)
#define PA2NC_SEGADR(x)		((x) & ~PA2NC_SEGMASK)
#define PA2NC_SEGOFF(x)		((x) & PA2NC_SEGMASK)
#define PA2NC_SEGSHIFT		(2)
#define PA2NC_PREMOD		(0xf0000000UL)
#define PA2NC_AFTMOD		(0x40000000UL)
#define PA2NC_MOD(x)  (PA2NC_SEGADR((((x)^PA2NC_PREMOD)>>PA2NC_SEGSHIFT)+PA2NC_AFTMOD) | PA2NC_SEGOFF(x))
#define PA2NC_RMOD(x) (((PA2NC_SEGADR(x)<<PA2NC_SEGSHIFT)^PA2NC_PREMOD) | PA2NC_SEGOFF(x))
#define PA2NC_P2C(x)		(PA2NC_MOD(x) + 0x60000000UL)
#define PA2NC_SDR_C2P(x)	((x) - 0x60000000UL)
#define PA2NC_DDR_C2P(x)	PA2NC_RMOD((x) - 0x60000000UL)

/* range check stuff */
#define PA2NC_IN_P_DDR(x)  (PA2NC_IN((x), P_DDR0) || PA2NC_IN((x), P_DDR1))
#define PA2NC_IN_C_DDR(x)  (PA2NC_IN((x), C_DDR0) || PA2NC_IN((x), C_DDR1))
#define PA2NC_IN_N_DDR(x)  (PA2NC_IN((x), N_DDR0) || PA2NC_IN((x), N_DDR1))
#define PA2NC_IN_P_SDR(x)  PA2NC_IN((x), P_SDR)
#define PA2NC_IN_C_SDR(x)  PA2NC_IN((x), C_SDR)
#define PA2NC_IN_N_SDR(x)  PA2NC_IN((x), N_SDR)


#define PHYS_TO_NONCACHE(x) \
	(PA2NC_IN_P_DDR(x) ? PA2NC_DDR_P2N(x) : \
	(PA2NC_IN_P_SDR(x) ? PA2NC_SDR_P2N(x) : PA2NC_ERR))
#define PHYS_TO_CACHE(x) \
	((PA2NC_IN_P_DDR(x) || PA2NC_IN_P_SDR(x)) ? PA2NC_P2C(x) : PA2NC_ERR)
#define VA_TO_PHYS(x) \
	(PA2NC_IN_C_DDR(x) ? PA2NC_DDR_C2P(x) : \
	(PA2NC_IN_N_DDR(x) ? PA2NC_DDR_N2P(x) : \
	(PA2NC_IN_C_SDR(x) ? PA2NC_SDR_C2P(x) : \
	(PA2NC_IN_N_SDR(x) ? PA2NC_SDR_N2P(x) : PA2NC_ERR))))

#endif /* CONFIG_ARCH_CXD4120 */
#endif /* __MACH_CXD4115_INCLUDE_MACH_NONCACHE_H */
