/*
 * include/asm-arm/arch-cxd4115/platform.h
 *
 * CXD4115 address map
 *
 * Copyright 2007,2008,2009,2010 Sony Corporation
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 *
 */
#ifndef __ARCH_CXD4115_PLATFORM_H
#define __ARCH_CXD4115_PLATFORM_H

#define CXD4115_DDR0_BASE	0x10000000
#define CXD4115_DDR1_BASE	0x30000000

#define EMC_CS0_BASE		0x00000000
#define EMC_CS1_BASE		0x60000000
#define EMC_CS2_BASE		0x70000000

/*---------------- AHB ----------------*/
#define CXD4115_AHB_BASE	0x78000000
#define CXD4115_AHB_SIZE	(SZ_1M)

#define CXD4115_EMC_BASE	(CXD4115_AHB_BASE + 0x00000)
#define CXD4115_EMC_SIZE	(SZ_1K)
#define VA_EMC			IO_ADDRESSP(CXD4115_EMC_BASE)
#define CXD4115_DMAC0_BASE	(CXD4115_AHB_BASE + 0x08000)
#define CXD4115_DMAC0_SIZE	(SZ_4K)
#define CXD4115_CF_BASE		(CXD4115_AHB_BASE + 0x50000)
#define VA_CF			IO_ADDRESSP(CXD4115_CF_BASE)
#define CXD4115_PCI_BASE	(CXD4115_AHB_BASE + 0x60000)
#define CXD4115_PCI_SIZE	(SZ_1K)

/*---------------- APB ----------------*/
#define CXD4115_APB_BASE	0x7a000000
#define CXD4115_APB_SIZE	(SZ_16M)

#define CXD4115_APB0_BASE	(CXD4115_APB_BASE + 0x000000)
#define CXD4115_TIMER_BASE(x)	(CXD4115_APB0_BASE + 0x00000 + 0x20*(x))
#define VA_TIMER(x)		IO_ADDRESSP(CXD4115_TIMER_BASE(x))
#define CXD4115_SIO_BASE(x)	(CXD4115_APB0_BASE + 0x08000 + 0x200*(x))
#define __CXD4115_UART(x)	(CXD4115_APB0_BASE + 0x50000 + 0x1000*(x))
#define CXD4115_UART_CLK	12000000 /* Hz */
#define CXD4115_HSTCTRL_BASE	(CXD4115_APB0_BASE + 0x58000)
#define VA_HSTCTRL		IO_ADDRESSP(CXD4115_HSTCTRL_BASE)

#define CXD4115_APB1_BASE	(CXD4115_APB_BASE + 0x400000)
#define CXD4115_GPIO(x)		(CXD4115_APB1_BASE + 0x00000 + 0x100*(x))
#define PA_GPIO(x)		CXD4115_GPIO(x)
#define VA_GPIO(x)		IO_ADDRESSP(CXD4115_GPIO(x))
#define CXD4115_OCTRL(x)	(CXD4115_APB1_BASE + 0x10000 + 0x10*(x))
#define VA_OCTRL(x)		IO_ADDRESSP(CXD4115_OCTRL(x))
#define CXD4115_CLKRST(x)	(CXD4115_APB1_BASE + 0x20000 + 0x10*(x))
#define VA_CLKRST(x)		IO_ADDRESSP(CXD4115_CLKRST(x))
#define CXD4115_MISCCTRL(x)	(CXD4115_APB1_BASE + 0x60000 + 0x10*(x))
#define VA_MISCCTRL(x)		IO_ADDRESSP(CXD4115_MISCCTRL(x))

#define CXD4115_APB2_BASE	(CXD4115_APB_BASE + 0x800000)
#define CXD4115_PRC_BASE	(CXD4115_APB2_BASE + 0x03200)

#define CXD4115_APB3_BASE	(CXD4115_APB_BASE + 0xc00000)

#ifdef CONFIG_ARCH_CXD4120
# define __CXD4120_UART3	(CXD4115_APB1_BASE + 0x108000)
#ifdef __ASSEMBLY__
/* In assembly, FLASE=0, TRUE=0xffffffff */
# define CXD4115_UART(x)	((((x) == 3) & __CXD4120_UART3) | (((x) != 3) & __CXD4115_UART(x)))
#else /* __ASSEMBLY__ */
# define CXD4115_UART(x)	((x) == 3 ? __CXD4120_UART3 : __CXD4115_UART(x))
#endif /* __ASSEMBLY__ */
# define CXD4120_HSTCTRL_C_BASE	(CXD4115_APB1_BASE + 0x110000)
# define VA_HSTCTRL_C		IO_ADDRESSP(CXD4120_HSTCTRL_C_BASE)
# define CXD4115_SDC_BASE	(CXD4115_APB3_BASE + 0x75000)
# define CXD4115_SDC(x)		(CXD4115_SDC_BASE + 0x1000*(x))
#else
# define CXD4115_UART(x)	__CXD4115_UART(x)
# define CXD4115_SDC_BASE	(CXD4115_APB2_BASE + 0x15000)
#endif /* CONFIG_ARCH_CXD4120 */

#ifdef CONFIG_ARCH_CXD4120
#define CXD4115_BAM_BASE	(CXD4115_APB3_BASE + 0x71000)
#else
#define CXD4115_BAM_BASE	(CXD4115_APB2_BASE + 0x11000)
#endif /* CONFIG_ARCH_CXD4120 */

/*---------------- BOOT ----------------*/
#define CXD4115_HIO_BASE	0xfff00000
#define CXD4115_HIO_SIZE	(SZ_1M)
#define CXD4115_BOOTRAM_BASE	(CXD4115_HIO_BASE+0x00000)
#define CXD4115_BOOTRAM_SIZE	(SZ_32K)
#define VA_BOOTRAM		HIO_ADDRESS(CXD4115_BOOTRAM_BASE)

#define CXD4115_BRIDGE(x)	(CXD4115_HIO_BASE+0xc0000+0x2000*(x))
#define VA_BRIDGE(x)		HIO_ADDRESS(CXD4115_BRIDGE(x))

#define CXD4115_MPCORE_BASE	(CXD4115_HIO_BASE+0xd0000)
#define CXD4115_SCU_BASE	(CXD4115_MPCORE_BASE+0x0000)
#define CXD4115_GIC_CPU_BASE	(CXD4115_MPCORE_BASE+0x0100) /* CPU I/F */
#define CXD4115_LOCALTIMER_BASE	(CXD4115_MPCORE_BASE+0x0600)
#define CXD4115_LOCALTIMER(x)	(CXD4115_MPCORE_BASE+0x0700+0x100*(x))
#define CXD4115_GIC_DIST_BASE	(CXD4115_MPCORE_BASE+0x1000) /* INT DIST */
#define CXD4115_LOCALTIMER_BASE_SIZE (0x0100)
#define SCU_BASE		CXD4115_SCU_BASE
#define VA_SCU			HIO_ADDRESSP(CXD4115_SCU_BASE)
#define VA_GIC_CPU		HIO_ADDRESSP(CXD4115_GIC_CPU_BASE)
#define VA_LOCALTIMER		HIO_ADDRESSP(CXD4115_LOCALTIMER_BASE)
#define VA_LOCALTIMER_X(x)	HIO_ADDRESSP(CXD4115_LOCALTIMER(x))
#define VA_GIC_DIST		HIO_ADDRESSP(CXD4115_GIC_DIST_BASE)
#define GIC_CPU_BASE		CXD4115_GIC_CPU_BASE

#ifndef __ASSEMBLY__
extern int cxd4115_core_init(void);
extern struct sys_timer cxd4115_timer;
extern void cxd4115_map_io(void);
extern void cxd4115_init_irq(void);
#endif /* !__ASSEMBLY__ */

#endif /* __ARCH_CXD4115_PLATFORM_H */
