/*
 * arch/arm/mach-cxd4115/pm_macro.S
 *
 * sleep macro
 *
 * Copyright 2010 Sony Corporation
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 *
 */
#include <mach/hardware.h>
#include <mach/bootram.h>
#include <mach/regs-clk.h>
#include <mach/regs-misc.h>
#include <mach/regs-dmc.h>
#include <mach/regs-emc.h>
#include <mach/regs-prc.h>
#include <mach/regs-bam.h>
#include <mach/regs-bridge.h>

	.macro	pm_workarea_va,rx
	ldr	\rx, =VA_BOOTRAM+CXD4115_SUSPEND_WORKAREA
	.endm
	.macro	pm_workarea_phys,rx
	ldr	\rx, =CXD4115_BOOTRAM_BASE+CXD4115_SUSPEND_WORKAREA
	.endm
	.macro  pm_stack_phys,rx
	ldr	\rx, =CXD4115_BOOTRAM_BASE+CXD4115_SUSPEND_WORKAREA
	.endm

	.macro	pm_outer_cache
	.endm

	.macro	pm_hook,rd
#ifndef CONFIG_ARCH_CXD4120
	@ force RESET ?
	ldr	\rd, modeflag
	cmp	\rd, #PM_ENTER_RESET
	bne	1f
	mov	\rd, #1
	str	\rd, reboot
1:
#endif /* !CONFIG_ARCH_CXD4120 */
	.endm

	.macro	pm_hook2,rd
	.endm

	.macro	pm_reset,rd,rx
#ifndef CONFIG_ARCH_CXD4120
	ldr	\rx, =CXD4115_MISCCTRL(MISCCTRL_XRESETREQ)
	mov	\rd, #1
	str	\rd, [\rx, #MISCCTRL_CLR]		@ assert
#endif /* !CONFIG_ARCH_CXD4120 */
	.endm

	.macro	pm_memory,rx
#ifdef CONFIG_CXD4115_DDR_ACCESS_DETECT
	ldr	\rx, =CXD4115_BAM_BASE
# ifdef CONFIG_ARCH_CXD4120
	mov	r0, #0x0310			@ MON0=DMC0, MON1=DMC1
	str	r0, [\rx, #BAM_DMC_SELECT]
	@ MON0:	Monitor from ALL IP
	mov	r0, #BAM_AXI_MON_IP1_ALL	@ IP31-IP0
	str	r0, [\rx, #BAM_AXI_MON1_IP1]
	mov	r0, #BAM_AXI_MON_IP2_ALL	@ IP63-IP32
	str	r0, [\rx, #BAM_AXI_MON1_IP2]
	@ MON1:	Monitor from ALL IP (MON1)
	mov	r0, #BAM_AXI_MON_IP1_ALL	@ IP31-IP0
	str	r0, [\rx, #BAM_AXI_MON2_IP1]
	mov	r0, #BAM_AXI_MON_IP2_ALL	@ IP63-IP32
	str	r0, [\rx, #BAM_AXI_MON2_IP2]
	@ set mode
	mov	r0, #BAM_MODE_COUNT|BAM_CTRL_CPU
	str	r0, [\rx, #BAM_LOAD_SET]
	@ start
	mov	r0, #BAM_START
	str	r0, [\rx, #BAM_MON_START]
	@ wait 20ms
	ldr	r0, =20000
	bl	__udelay			@ destruct r0, r2
	@ stop
	mov	r0, #0
	str	r0, [\rx, #BAM_MON_START]
	@ show result
	ldr	r0, [\rx, #BAM_MON1_BURST_COUNT]
	bl	__printhex8			@ destruct r0-r3
	mov	r0, #'\n'
	bl	__printch
	ldr	r0, [\rx, #BAM_MON2_BURST_COUNT]
	bl	__printhex8			@ destruct r0-r3
	mov	r0, #'\n'
	bl	__printch
# else /* CONFIG_ARCH_CXD4120 */
	@ Monitor DMC0 from ALL IP
	mov	r0, #BAM_AXI_MON_IP1_ALL	@ DMC0, IP59-IP32
	str	r0, [\rx, #BAM_AXI_MON1_IP1]
	mov	r0, #BAM_AXI_MON_IP2_ALL	@ IP31-IP0
	str	r0, [\rx, #BAM_AXI_MON1_IP2]
	@ start
	mov	r0, #BAM_MODE_COUNT|BAM_CTRL_CPU|BAM_START
	str	r0, [\rx, #BAM_LOAD_SET]
	@ wait 20ms
	ldr	r0, =20000
	bl	__udelay			@ destruct r0, r2
	@ stop
	mov	r0, #BAM_MODE_COUNT|BAM_CTRL_CPU
	str	r0, [\rx, #BAM_LOAD_SET]
	@ show result
	ldr	r0, [\rx, #BAM_MON1_BURST_COUNT]
	bl	__printhex8			@ destruct r0-r3
	mov	r0, #'\n'
	bl	__printch
# endif /* CONFIG_ARCH_CXD4120 */
#endif /* CONFIG_CXD4115_DDR_ACCESS_DETECT */
	/*
	 * DDR0 power down
	 */
	ldrb	r0, cxd41xx_ddr_ctrl		@ PC relative
	cmp	r0, #DMC_DDRCTRL_NO
	beq	ddr_ctrl_end			@ do not control
	@ set DDRs to SREF or DPD
	ldr	\rx, =CXD4115_SDC_BASE
	mov	r1, #0				@ SREF command word
	mov	r2, #0				@ DPD  command word
	cmp	r0, #DMC_DDRCTRL_SREF
	orreq	r1, r1, #DMC_APBCMDREQ0		@ set DDR0 to SREF
	orrne	r2, r2, #DMC_APBCMDREQ0		@ set DDR0 to DPD
	ldr	r0, [\rx, #DMC_MEMORY]
	ands	r0, r0, #DMC_MODE2RANK		@ 2RANKs populated ?
	beq	1f
	ldrb	r0, cxd41xx_ddr_ctrl+1		@ PC relative
	cmp	r0, #DMC_DDRCTRL_SREF
	orreq	r1, r1, #DMC_APBCMDREQ1		@ set DDR1 to SREF
	orrne	r2, r2, #DMC_APBCMDREQ1		@ set DDR1 to DPD
1:
	bl	__dmc_issue
ddr_ctrl_end:

#ifdef CONFIG_ARCH_CXD4120
	/*
	 * DDR1 power down
	 */
	ldrb	r0, cxd41xx_ddr_ctrl2		@ PC relative
	cmp	r0, #DMC_DDRCTRL_NO
	beq	ddr1_ctrl_end			@ do not control
	@ set DDRs to SREF or DPD
	ldr	\rx, =CXD4115_SDC(1)
	mov	r1, #0				@ SREF command word
	mov	r2, #0				@ DPD  command word
	cmp	r0, #DMC_DDRCTRL_SREF
	orreq	r1, r1, #DMC_APBCMDREQ0		@ set DDR2 to SREF
	orrne	r2, r2, #DMC_APBCMDREQ0		@ set DDR2 to DPD
	ldr	r0, [\rx, #DMC_MEMORY]
	ands	r0, r0, #DMC_MODE2RANK		@ 2RANKs populated ?
	beq	1f
	ldrb	r0, cxd41xx_ddr_ctrl2+1		@ PC relative
	cmp	r0, #DMC_DDRCTRL_SREF
	orreq	r1, r1, #DMC_APBCMDREQ1		@ set DDR3 to SREF
	orrne	r2, r2, #DMC_APBCMDREQ1		@ set DDR3 to DPD
1:
	bl	__dmc_issue
ddr1_ctrl_end:
#endif /* CONFIG_ARCH_CXD4120 */

	/*
	 * DMC reset
	 */
	/* Check TYPEID */
	ldr	\rx, =CXD4115_MISCCTRL(MISCCTRL_TYPEID)
	ldr	r0, [\rx]
	and	r0, r0, #TYPEID_IDMASK
	cmp	r0, #TYPEID_CXD4120		@ do not reset
	beq	1f
	cmp	r0, #TYPEID_CXD9938		@ do not reset
	beq	1f

	@ wait 10us (Undocumented. not mandatory.)
	mov	r0, #10
	bl	__udelay

	@ reset DDR PHY
	ldr	\rx, =CXD4115_PRC_BASE
	ldr	r0, [\rx, #PRC_PHY_MODE1_MST]
	ldr	r1, =PRC_PHY_RESET_MASK
	and	r0, r0, r1
	str	r0, [\rx, #PRC_PHY_MODE1_MST]	@ reset
1:

/*--------------------- SDR power down ----------------------*/
#ifdef CONFIG_CXD4115_USE_SDR
	@ set SDRAM to SREF or DPD
	ldr	\rx, =CXD4115_EMC_BASE
# ifdef CONFIG_CXD4115_EMC_DETECT
	mov	r0, #0
	str	r0, [\rx, #EMC_BM_CTRL]
	ldr	r0, =0x80280000			@ period 16.7ms
	str	r0, [\rx, #EMC_CNT_MODE]
	mov	r0, #EMC_BP_ON
	str	r0, [\rx, #EMC_BP_ON_MODE]	@ count start
# endif /* CONFIG_CXD4115_EMC_DETECT */
	@ Flush FIFOs
	mov	r0, #0x07
	str	r0, [\rx, #EMC_TRANS_CTRL]
1:
	ldr	r0, [\rx, #EMC_TRANS_CTRL]
	cmp	r0, #0
	bne	1b

        mov	r0, #0x8000
# ifdef CONFIG_CXD4115_SDR_DPD
	orr	r0, r0, #EMC_SDR_SEQCMD_DPD	@ DPD
# else
	orr	r0, r0, #EMC_SDR_SEQCMD_SREF	@ SREF
# endif
        str	r0, [\rx, #EMC_PHY_SDR_CMD]	@ issue
	@ delay > 1ms for SDR SREF
	mov	r0, #0x400
	bl	__udelay
#endif /* CONFIG_CXD4115_USE_SDR */
#ifdef CONFIG_CXD4115_EMC_DETECT
	ldr	r0, =20000			@ wait 20ms
	bl	__udelay
	@ show result
	add	r8, \rx, #EMC_C0_SINGLE_WR
	mov	r7, #0x17
1:
	ldr	r0, [r8], #4
	bl	__printhex8
	ands	r0, r7, #0x7
	moveq	r0, #'\n'
	movne	r0, #' '
	bl	__printch
	subs	r7, r7, #1
	bpl	1b
#endif /* CONFIG_CXD4115_EMC_DETECT */
	.endm

	.macro	pm_functions
/*
 * issue DMC_COMMAND
 *    input:	r4=DMC base address, r1=SREF param, r2=DPD param
 *    output:	none.
 *    destruct: r0, r3
 */
__dmc_issue:
	mov	r3, lr				@ save lr
	bl	__dmc_ready			@ wait for ready
	cmp	r1, #0
	beq	1f
	orr	r1, r1, #DMC_APBCMD_SIN		@ SREF Enter command
	str	r1, [r4, #DMC_COMMAND]		@ issue
	bl	__dmc_ready			@ wait for done
1:
	cmp	r2, #0
	beq	1f
	mov	r0, #0
	str	r0, [r4, #DMC_REFRESH]		@ stop auto-refresh
	orr	r2, r2, #DMC_APBCMD_DPD		@ DPD command
	str	r2, [r4, #DMC_COMMAND]		@ issue
	bl	__dmc_ready			@ wait for done
1:
	mov	pc, r3

/*
 * Wait for DMC_COMMAND register ready
 *    input:	r4=DMC base address
 *    output:	none.
 *    destruct:	r0
 */
__dmc_ready:
	ldr	r0, [r4, #DMC_COMMAND]
	ands	r0, r0, #DMC_APBCMDREQMASK
	bne	__dmc_ready
	mov	pc, lr
	.endm
