/*
 * include/asm-arm/arch-cxd4115/regs-clk.h
 *
 * CXD4115 HSTCTRL registers
 *
 * Copyright 2007,2008,2009,2010 Sony Corporation
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 *
 */
#ifndef __REGS_CLK_H__
#define __REGS_CLK_H__

/* CLKRST */
#define CLKRST_DATA		0x00
#define CLKRST_SET		0x04
#define CLKRST_CLR		0x08

#define VA_CLKRST_DATA(x)	(VA_CLKRST(x) + CLKRST_DATA)
#define VA_CLKRST_SET(x)	(VA_CLKRST(x) + CLKRST_SET)
#define VA_CLKRST_CLR(x)	(VA_CLKRST(x) + CLKRST_CLR)

#ifndef CONFIG_ARCH_CXD4120
#define CLKRST_XPS3		16
# define CLKRST_XPS3_PCI	0x00002000
#endif /* !CONFIG_ARCH_CXD4120 */

/* HSTCTRL */
#define CLKBLK_CLOCK_ON         1
#define CLKBLK_CLOCK_OFF        0
#define CLKBLK_DMAC0_HCLK	0x01c
#define CLKBLK_ONA_HCLK		0x034
#define CLKBLK_LDEC_HCLK	0x03c
#define CLKBLK_PCI_HCLK		0x074
#define CLKBLK_PCI_OCLK		0x07c
#define CLKBLK_PCI_STAT		0x080

#define CLKBLK_UART_CLKBASE	0x148
#define __CLKBLK_UART(x)	(CXD4115_HSTCTRL_BASE + CLKBLK_UART_CLKBASE + 0x10*(x))
#define __VA_CLKBLK_UART(x)	(VA_HSTCTRL + CLKBLK_UART_CLKBASE + 0x10*(x))
#define VA_CLKBLK_UART_PCLK(x)	(VA_CLKBLK_UART(x) + CLKBLK_UART_PCLK(x))
#define VA_CLKBLK_UART_CLK(x)	(VA_CLKBLK_UART(x) + CLKBLK_UART_CLK(x))
#define VA_CLKBLK_UART_STAT(x)	(VA_CLKBLK_UART(x) + CLKBLK_UART_STAT(x))

#define CLKBLK_TIMER_PCLK	0x1d0
#define CLKBLK_TIMER_CLK	0x1d8
#define CLKBLK_TIMER_STAT	0x1dc

/* HSTCTRL */
#define CLKBLK_RESET_ASSERT     0
#define CLKBLK_RESET_NEGATE     1
#define CLKBLK_DMAC0_RST	0x308
#define CLKBLK_ONA_RST		0x318
#define CLKBLK_LDEC_RST		0x31c
#define CLKBLK_PCI_RST		0x328
#define __VA_CLKBLK_UART_RST(x)	(VA_HSTCTRL + 0x35c + 4*(x))
#define CLKBLK_TIMER_RST	0x384

#ifdef CONFIG_ARCH_CXD4120
# define CLKBLK_UART3_CLKBASE	0x14
# define __CLKBLK_UART3		(CXD4120_HSTCTRL_C_BASE + CLKBLK_UART3_CLKBASE)
# define __VA_CLKBLK_UART3	(VA_HSTCTRL_C + CLKBLK_UART3_CLKBASE)
#ifdef __ASSEMBLY__
/* In assembly, FLASE=0, TRUE=0xffffffff */
# define CXD4115_CLKBLK_UART(x)	((((x) == 3) & __CLKBLK_UART3) | (((x) != 3) & __CLKBLK_UART(x)))
#else /* __ASSEMBLY__ */
# define CXD4115_CLKBLK_UART(x)	((x) == 3 ? __CLKBLK_UART3 : __CLKBLK_UART(x))
#endif /* __ASSEMBLY__ */
# define VA_CLKBLK_UART(x)	((x) == 3 ? __VA_CLKBLK_UART3 : __VA_CLKBLK_UART(x))
#  define CLKBLK_UART_PCLK(x)	0x0
#  define CLKBLK_UART_CLK(x)	((x) == 3 ? 0x4 : 0x8)
#  define CLKBLK_UART_STAT(x)	((x) == 3 ? 0x8 : 0xc)
# define __VA_CLKBLK_UART3_RST	(VA_HSTCTRL_C + 0x40)
# define VA_CLKBLK_UART_RST(x)	((x) == 3 ? __VA_CLKBLK_UART3_RST : __VA_CLKBLK_UART_RST(x))
#else
# define CXD4115_CLKBLK_UART(x) __CLKBLK_UART(x)
# define VA_CLKBLK_UART(x) __VA_CLKBLK_UART(x)
#  define CLKBLK_UART_PCLK(x)	0x0
#  define CLKBLK_UART_CLK(x)	0x8
#  define CLKBLK_UART_STAT(x)	0xc
# define VA_CLKBLK_UART_RST(x)	__VA_CLKBLK_UART_RST(x)
#endif /* CONFIG_ARCH_CXD4120 */

#endif /* __REGS_CLK_H__ */
