/*
 * include/asm-arm/arch-cxd4115/regs-pci.h
 *
 * CXD4115 PCI address map
 *
 * Copyright 2007,2008,2009,2010 Sony Corporation
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 *
 */
#ifndef __REGS_PCI_H__
#define __REGS_PCI_H__

#define PCI_CTRL(x) 		(IO_ADDRESSP(CXD4115_PCI_BASE)+(x))
#define PCI_CFGREG(x)		(IO_ADDRESSP(CXD4115_PCIWINDOW_IOBASE)+(x))

/* PCI Communication Registers */
#define PCI_PCIAHB_WIN1_CTR	0x00
#define PCI_PCIAHB_WIN2_CTR	0x04
#define PCI_PCIAHB_DCT_CTR	0x08  /* Discard Timer Control */
#define PCI_AHBPCI_WIN1_CTR	0x10
#define PCI_AHBPCI_WIN2_CTR	0x14
#define PCI_AHBPCI_WIN3_CTR	0x18
#define PCI_AHBPCI_DCT_CTR	0x1c
#define PCI_INT_ENABLE		0x20
#define PCI_INT_STATUS		0x24
# define PCI_INT_INTA			0x10000
# define PCI_INT_INTB			0x20000
# define PCI_INT_INTC			0x40000
# define PCI_INT_INTD			0x80000
# define PCI_INT_AHBPCIWIN		0x04000
# define PCI_INT_PCIAHBWIN2		0x02000
# define PCI_INT_PCIAHBWIN1		0x01000
# define PCI_INT_DMAAHBPCI		0x00200
# define PCI_INT_DMAPCIAHB		0x00100
# define PCI_INT_CLKRUN			0x00080
# define PCI_INT_PM			0x00040
# define PCI_INT_SERR			0x00020
# define PCI_INT_SIGSERR		0x00010
# define PCI_INT_PERR			0x00008
# define PCI_INT_MABORT			0x00004
# define PCI_INT_TABORT			0x00002
# define PCI_INT_SIGTABORT		0x00001
# define PCI_INT_BRIDGEMASK		0x0ffff

#define PCI_PCIERR_ADR		0x28
#define PCI_AHBERR_ADR		0x2c
#define PCI_AHB_BUS_CTR		0x30
#define PCI_ARBITER_CTR		0x40
#define PCI_EXT_CTR		0x44
#define PCI_UNIT_REV		0x48
#define PCI_UNIT_DEBUG_CTR	0x4c
#define PCI_PME_CTR		0x50
#define PCI_DMA_AHBPCI_PCIADR	0x60
#define PCI_DMA_AHBPCI_AHBADR	0x64
#define PCI_DMA_AHBPCI_TSIZE	0x68
#define PCI_DMA_AHBPCI_CTR	0x6c
#define PCI_DMA_AHBPCI_DCT	0x70
#define PCI_DMA_PCIAHB_PCIADR	0x80
#define PCI_DMA_PCIAHB_AHBADR	0x84
#define PCI_DMA_PCIAHB_TSIZE	0x88
#define PCI_DMA_PCIAHB_CTR	0x8c
#define PCI_DMA_PCIAHB_DCT	0x90

/* PCI Bus Commands */
#define PCI_IO_RW   0x1
#define PCI_MEM_RW  0x3
#define PCI_CFG_RW  0x5

#define PCI_CFG_TYPE0 0x00
#define PCI_CFG_TYPE1 0x10

#define CXD4115_CFG_SPACE ((1<<PCI_IDSEL(PCI_CXD4115_SLOT))|(PCI_CFG_RW<<1))
#define CXD4115_IO_SPACE (CXD4115_PCIWINDOW_IOBASE|(PCI_IO_RW<<1)) /* I/O space, non-burst, non-prefetch */

#endif /* __REGS_PCI_H__ */
