/*-----------------------------------------------------------------------------
 * sonymisc.h
 *
 * a header file for sonymisc
 *
 * Copyright 2005,2006,2007,2008,2009,2010 Sony Corporation
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 * MA  02110-1301, USA.
 *---------------------------------------------------------------------------*/
#ifndef __SONYMISC_H__
#define __SONYMISC_H__

#include <linux/blkdev.h>

/*=============================================================================
 *
 * sonymisc-dev's public method.
 *
 *===========================================================================*/
/*=============================================================================
 * SONYMISC API
 *===========================================================================*/
/*-----------------------------------------------------------------------------
 * open( struct inode *inode, struct file *file )
 *-----------------------------------------------------------------------------
 * Summary	: Open method
 *     IN param	: inode	:
 *              : file  :
 *     OUT param: -
 *     Return	: Err (0:OK / -1:NG )
 *---------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------
 * release( struct inode *inode, struct file *file )
 *-----------------------------------------------------------------------------
 * Summary	: Close method
 *     IN param	: inode	:
 *              : file  :
 *     OUT param: -
 *     Return	: Err (0:OK / -1:NG )
 *---------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------
 * sonymisc_register()
 *-----------------------------------------------------------------------------
 * Summary		: register to sonymisc
 *     IN param	: *dev	: pointer of struct sonymisc_device
 *     OUT param: -	: -
 *     Return	: err	: 0:succeed / -EINVAL:param err
 *---------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------
 * sonymisc_deregister()
 *-----------------------------------------------------------------------------
 * Summary		: deregister to sonymisc
 *     IN param	: *dev	: pointer of struct sonymisc_device
 *     OUT param: -	: -
 *     Return	: err	: 0:succeed / -EINVAL:param err
 *---------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------
 * sonymisc struct
 *---------------------------------------------------------------------------*/
struct sonymisc_device
{
    char			class;
    int 			minor;
    const char			*name;
    struct file_operations	*fops; /* if use character device use */
    struct gendisk 		*gd;	/* if use block device use */
};

#define DEV_CLASS_CHR	0
#define DEV_CLASS_BLK	1

#ifndef __SONYMISC_C__
extern int sonymisc_register( struct sonymisc_device *dev );
extern int sonymisc_deregister( struct sonymisc_device *dev );
#endif /* __SONYMISC_C__ */

#endif /* __SONYMISC_DEV_H__ */
