/*
 * arch/arm/mach-cxd4115/target_init.c
 *
 * Serialized and ordered initcall for target specific static kernel modules.
 *
 * Copyright 2007,2008,2009,2010 Sony Corporation
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 *
 */
/*
 * Each static driver should care that his/her init()
 * is called from module_init() macro or this target_init().
 *
 */

#include <linux/kernel.h>
#include <linux/module.h>


#ifdef CONFIG_ARCH_CXD4115BASED

#ifdef CONFIG_CXD4115_COMPAT
extern int sonymisc_init(void);
extern int gpio_init(void);
#else
# define sonymisc_init() do {} while (0)
# define gpio_init() do {} while (0)
#endif /* CONFIG_CXD4115_COMPAT */

extern int MGC_ModuleInit(void);
extern int usb_mcd_module_init(void);
extern int usb_event_module_init(void);
extern int usb_otgcore_module_init(void);
extern int usb_gcore_module_init(void);

static int __init target_init(void)
{
	printk(KERN_INFO "%s: CXD4115\n", __func__);

	/*
	 * put the static moudle's init()s here.
	 */
	sonymisc_init();
	gpio_init();

#if defined(CONFIG_USB_MCD_CD)
# if defined(CONFIG_USB_MCD_REL)
	usb_mcd_module_init();
# else
	MGC_ModuleInit();
# endif
#endif

#if defined(CONFIG_USB_EVENT)
	usb_event_module_init();
#endif

#if defined(CONFIG_USB_OTG_CORE)
	usb_otgcore_module_init();
#endif

#if defined(CONFIG_USB_GADGET_CORE)
	usb_gcore_module_init();
#endif

	return 0;
}

/* module_init() is too early. */
late_initcall(target_init);

#endif  /* end of ARCH_CXD4115BASED */
