/*
 * arch/arm/mach-cxd4115/udif/platform.c
 *
 *
 * Copyright 2010 Sony Corporation
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 *
 */
#include <linux/module.h>
#include <linux/interrupt.h>
#include <linux/udif/device.h>
#include <linux/udif/string.h>
#include <linux/udif/macros.h>
#include <mach/platform.h>
#include <mach/regs-dmac.h>
#include <mach/udif/platform.h>

#define SZ_32K 0x00008000

#define __UDIF_IOMEM_INIT(n,pa,sz)	UDIF_IOMEM_INIT(n,pa,sz,UDIF_IO_FLAGS_NONCACHED)

#define UDIF_INTRPT_INIT(n,i,f) \
{ \
	.name	= n, \
	.irq	= i, \
	.flags	= f, \
}

#define UDIF_DMAC_INIT(n,c,r,f) \
{ \
	.ch	= c, \
	.rqid	= r, \
	.intr	= UDIF_INTRPT_INIT(n "_dmac", IRQ_DMAC0(c), f), \
}

#define __UDIF_DMAC_INIT(n,v) \
{ \
	.num	= n, \
	.dmac	= v, \
}

static inline void udif_intr_init(UDIF_INTRPT *intr, const UDIF_U8 *name,
				  UDIF_IRQ irq, UDIF_U32 flags)
{
	__strncpy(intr->name, name, sizeof(intr->name));
	intr->irq = irq;
	intr->flags = flags;
}

/*
 * Each Device
 */

/* GPIO stuff */
#define GPIO_NAME 		"gpio"
#define GPIO_IO_SIZE		0x100
#define GPIO_IO_BASE(x)		(CXD4115_APB1_BASE + GPIO_IO_SIZE * (x))

static UDIF_CHANNELS gpio[UDIF_NR_GPIO] = {
	[0] = { .iomem = __UDIF_IOMEM_INIT(GPIO_NAME "0", GPIO_IO_BASE(0), GPIO_IO_SIZE), },
	[1] = { .iomem = __UDIF_IOMEM_INIT(GPIO_NAME "1", GPIO_IO_BASE(1), GPIO_IO_SIZE), },
	[2] = { .iomem = __UDIF_IOMEM_INIT(GPIO_NAME "2", GPIO_IO_BASE(2), GPIO_IO_SIZE), },
	[3] = { .iomem = __UDIF_IOMEM_INIT(GPIO_NAME "3", GPIO_IO_BASE(3), GPIO_IO_SIZE), },
	[4] = { .iomem = __UDIF_IOMEM_INIT(GPIO_NAME "4", GPIO_IO_BASE(4), GPIO_IO_SIZE), },
	[5] = { .iomem = __UDIF_IOMEM_INIT(GPIO_NAME "5", GPIO_IO_BASE(5), GPIO_IO_SIZE), },
	[6] = { .iomem = __UDIF_IOMEM_INIT(GPIO_NAME "6", GPIO_IO_BASE(6), GPIO_IO_SIZE), },
	[7] = { .iomem = __UDIF_IOMEM_INIT(GPIO_NAME "7", GPIO_IO_BASE(7), GPIO_IO_SIZE), },
	[8] = { .iomem = __UDIF_IOMEM_INIT(GPIO_NAME "8", GPIO_IO_BASE(8), GPIO_IO_SIZE), },
	[9] = { .iomem = __UDIF_IOMEM_INIT(GPIO_NAME "9", GPIO_IO_BASE(9), GPIO_IO_SIZE), },
	[10] = { .iomem = __UDIF_IOMEM_INIT(GPIO_NAME "10", GPIO_IO_BASE(10), GPIO_IO_SIZE), },
	[11] = { .iomem = __UDIF_IOMEM_INIT(GPIO_NAME "11", GPIO_IO_BASE(11), GPIO_IO_SIZE), },
	[12] = { .iomem = __UDIF_IOMEM_INIT(GPIO_NAME "12", GPIO_IO_BASE(12), GPIO_IO_SIZE), },
	[13] = { .iomem = __UDIF_IOMEM_INIT(GPIO_NAME "13", GPIO_IO_BASE(13), GPIO_IO_SIZE), },
	[14] = { .iomem = __UDIF_IOMEM_INIT(GPIO_NAME "14", GPIO_IO_BASE(14), GPIO_IO_SIZE), },
	[15] = { .iomem = __UDIF_IOMEM_INIT(GPIO_NAME "15", GPIO_IO_BASE(15), GPIO_IO_SIZE), },
	[16] = { .iomem = __UDIF_IOMEM_INIT(GPIO_NAME "16", GPIO_IO_BASE(16), GPIO_IO_SIZE), },
	[17] = { .iomem = __UDIF_IOMEM_INIT(GPIO_NAME "17", GPIO_IO_BASE(17), GPIO_IO_SIZE), },
	[18] = { .iomem = __UDIF_IOMEM_INIT(GPIO_NAME "18", GPIO_IO_BASE(18), GPIO_IO_SIZE), },
	[19] = { .iomem = __UDIF_IOMEM_INIT(GPIO_NAME "19", GPIO_IO_BASE(19), GPIO_IO_SIZE), },
	[20] = { .iomem = __UDIF_IOMEM_INIT(GPIO_NAME "20", GPIO_IO_BASE(20), GPIO_IO_SIZE), },
	[21] = { .iomem = __UDIF_IOMEM_INIT(GPIO_NAME "21", GPIO_IO_BASE(21), GPIO_IO_SIZE), },
	[22] = { .iomem = __UDIF_IOMEM_INIT(GPIO_NAME "22", GPIO_IO_BASE(22), GPIO_IO_SIZE), },
	[23] = { .iomem = __UDIF_IOMEM_INIT(GPIO_NAME "23", GPIO_IO_BASE(23), GPIO_IO_SIZE), },
	[24] = { .iomem = __UDIF_IOMEM_INIT(GPIO_NAME "24", GPIO_IO_BASE(24), GPIO_IO_SIZE), },
};

/* GPE stuff */
#include <linux/platform_device.h>
#include <linux/uio_driver.h>
#include <linux/mm.h>

#define GPE_NAME 	"gpe"
#define GPE_REV 	"1.0"
#define GPE_IO_BASE	(CXD4115_APB3_BASE + 0x20000)
#define GPE_IO_SIZE	SZ_1M
#define GPE_IRQ		IRQ_GPE
#define GPE_IRQ_FLAGS	IRQF_DISABLED

static UDIF_CHANNELS gpe[UDIF_NR_GPE] = {
	[0] = {
		.iomem = __UDIF_IOMEM_INIT(GPE_NAME, GPE_IO_BASE, GPE_IO_SIZE),
		.intr = UDIF_INTRPT_INIT(GPE_NAME, GPE_IRQ, 0),
	},
};

static irqreturn_t gpe_uio_handler(int irq, struct uio_info *info)
{
	return IRQ_HANDLED;
}

#if 0
static int gpe_uio_mmap(struct uio_info *info, struct vm_area_struct *vma)
{
	vma->vm_page_prot = pgprot_noncached(vma->vm_page_prot);
	vma->vm_flags |= VM_IO | VM_RESERVED;

	return remap_pfn_range(vma,
			       vma->vm_start,
			       info->mem[0].addr >> PAGE_SHIFT,
			       vma->vm_end - vma->vm_start,
			       vma->vm_page_prot);
}
#endif

struct resource gpe_res = {
	.start	= GPE_IO_BASE,
	.end	= GPE_IO_BASE + GPE_IO_SIZE - 1,
	.flags	= IORESOURCE_MEM,
};

static struct uio_info gpe_uio_info = {
	.name		= GPE_NAME,
	.version	= GPE_REV,
	.irq		= GPE_IRQ,
	.irq_flags	= GPE_IRQ_FLAGS,
	.handler	= gpe_uio_handler,
#if 0
	.mmap		= gpe_uio_mmap,
#endif
};

static struct platform_device gpe_device = {
	.name			= "uio_pdrv",
	.num_resources		= 1,
	.resource		= &gpe_res,
	.dev.platform_data	= &gpe_uio_info,
};

static int gpe_init(UDIF_CHANNELS *chs)
{
	UDIF_ERR ret = platform_device_register(&gpe_device);

	if (unlikely(ret)) {
		UDIF_PERR("gpe: failed platform_device_register(), ret = %d\n", ret);
	}

	return ret;
}

static void gpe_exit(UDIF_CHANNELS *chs)
{
	platform_device_unregister(&gpe_device);
}

/* SIO */
#define SIO_NAME	"sio"
#define SIO_IO_BASE(ch)	CXD4115_SIO_BASE(ch)
#define SIO_IO_SIZE	0x200
#define SIO_IRQ(ch)	(IRQ_SIO0 + (ch))

static UDIF_CHANNELS sio[UDIF_NR_SIO] = {
	[0] = {
		.iomem = __UDIF_IOMEM_INIT(SIO_NAME "0", SIO_IO_BASE(0), SIO_IO_SIZE),
		.intr = UDIF_INTRPT_INIT(SIO_NAME "0", SIO_IRQ(0), 0),
	},
	[1] = {
		.iomem = __UDIF_IOMEM_INIT(SIO_NAME "1", SIO_IO_BASE(1), SIO_IO_SIZE),
		.intr = UDIF_INTRPT_INIT(SIO_NAME "1", SIO_IRQ(1), 0),
	},
	[2] = {
		.iomem = __UDIF_IOMEM_INIT(SIO_NAME "2", SIO_IO_BASE(2), SIO_IO_SIZE),
		.intr = UDIF_INTRPT_INIT(SIO_NAME "2", SIO_IRQ(2), 0),
	},
	[3] = {
		.iomem = __UDIF_IOMEM_INIT(SIO_NAME "3", SIO_IO_BASE(3), SIO_IO_SIZE),
		.intr = UDIF_INTRPT_INIT(SIO_NAME "3", SIO_IRQ(3), 0),
	},
	[4] = {
		.iomem = __UDIF_IOMEM_INIT(SIO_NAME "4", SIO_IO_BASE(4), SIO_IO_SIZE),
		.intr = UDIF_INTRPT_INIT(SIO_NAME "4", SIO_IRQ(4), 0),
	},
};

/* SIRCS */
#define SIRCS_NAME	"sircs"
#define SIRCS_IO_BASE	(CXD4115_APB0_BASE + 0x28000)
#define SIRCS_IO_SIZE	0x100
#define SIRCS_IRQ	186 /* GIRRXINTR */

/* overrun */
#define SIRCS_OVERRUN_NAME	"sircs_overun"
#define SIRCS_OVERRUN_IRQ	184 /* GIRORINTR */

static UDIF_CHANNELS sircs[UDIF_NR_SIRCS] = {
	[UDIF_SIRCS_RX] = {
		.iomem = __UDIF_IOMEM_INIT(SIRCS_NAME, SIRCS_IO_BASE, SIRCS_IO_SIZE),
		.intr = UDIF_INTRPT_INIT(SIRCS_NAME, SIRCS_IRQ, 0),
	},
	[UDIF_SIRCS_OVERRUN] = {
		.intr = UDIF_INTRPT_INIT(SIRCS_OVERRUN_NAME, SIRCS_OVERRUN_IRQ, 0),
	},
};

/* DMAC stuff */
#define DMAC_NAME 	"dmac"
#define DMAC_IO_BASE	(CXD4115_AHB_BASE + 0x00008000)
#define DMAC_IO_SIZE	SZ_32K

static UDIF_CHANNELS dmac[UDIF_NR_DMAC] = {
	[0] = {
		.iomem = __UDIF_IOMEM_INIT(DMAC_NAME, DMAC_IO_BASE, DMAC_IO_SIZE),
	},
};

static UDIF_DMAC dmac_dmac[] = {
	UDIF_DMAC_INIT(DMAC_NAME "-0", 0, 0, 0),
	UDIF_DMAC_INIT(DMAC_NAME "-1", 1, 1, 0),
	UDIF_DMAC_INIT(DMAC_NAME "-2", 2, 2, 0),
	UDIF_DMAC_INIT(DMAC_NAME "-3", 3, 3, 0),
	UDIF_DMAC_INIT(DMAC_NAME "-4", 4, 4, 0),
	UDIF_DMAC_INIT(DMAC_NAME "-5", 5, 5, 0),
	UDIF_DMAC_INIT(DMAC_NAME "-6", 6, 6, 0),
	UDIF_DMAC_INIT(DMAC_NAME "-7", 7, 7, 0),
};

/* ONA */
#define ONA_NAME	"ona"
#define ONA_IO_BASE	(CXD4115_AHB_BASE + 0x00098000)
#define ONA_IO_SIZE	SZ_32K
#define ONA_DMAC_RQID	DMA0_REQID_ONA
#define ONA_DMAC_CH	6
#define ONA_IRQ		IRQ_ONA
#if 0
# define ONA_IRQ_FLAGS	IRQF_NODELAY
#else
# define ONA_IRQ_FLAGS	0
#endif

static UDIF_CHANNELS ona[UDIF_NR_ONA] = {
	[0] = {
		.iomem = __UDIF_IOMEM_INIT(ONA_NAME, ONA_IO_BASE, ONA_IO_SIZE),
		.intr = UDIF_INTRPT_INIT(ONA_NAME, ONA_IRQ, ONA_IRQ_FLAGS),
	},
};

static UDIF_DMAC ona_dmac[] = {
	UDIF_DMAC_INIT(ONA_NAME, ONA_DMAC_CH, ONA_DMAC_RQID, ONA_IRQ_FLAGS),
};

/* NAND */
#define NAND_NAME	"nand"
#define NAND_IO_BASE	EMC_CS0_BASE
#define NAND_IO_SIZE	SZ_128K
#define NAND_IRQ	IRQ_EMC_XINT
#if 0
# define NAND_IRQ_FLAGS	IRQF_NODELAY
#else
# define NAND_IRQ_FLAGS	0
#endif

#define NAND_DMAC_CH	ONA_DMAC_CH
#define NAND_DMAC_IRQ	DMAC_CH_TO_IRQ(DMAC_CH_NAND)

static UDIF_CHANNELS nand[UDIF_NR_NAND] = {
	[0] = {
		.iomem = __UDIF_IOMEM_INIT(NAND_NAME, NAND_IO_BASE, NAND_IO_SIZE),
		.intr = UDIF_INTRPT_INIT(NAND_NAME, NAND_IRQ, NAND_IRQ_FLAGS),
	},
};

static UDIF_ERR nand_init(UDIF_CHANNELS *chs)
{
	if (unlikely(!udif_ioremap(&chs->iomem))) {
		UDIF_PERR("nand ioremap failed\n");
		return UDIF_ERR_NOMEM;
	}

	return UDIF_ERR_OK;
}

static void nand_exit(UDIF_CHANNELS *chs)
{
	udif_iounmap(&chs->iomem);
}

static UDIF_DMAC nand_dmac[] = {
	UDIF_DMAC_INIT(NAND_NAME, NAND_DMAC_CH, UDIF_DMAC_NORQID, NAND_IRQ_FLAGS),
};

/* LDEC */
#define LDEC_NAME	"ldec"
#define LDEC_IO_BASE	(CXD4115_AHB_BASE + 0x00090000)
#define LDEC_IO_SIZE	SZ_32K
#define LDEC_IRQ	IRQ_LDEC

#define LDEC_DMAC_CH_TODEV	2
#define LDEC_DMAC_CH_FROMDEV	3
#define LDEC_DMAC_RQID_TODEV	DMA0_REQID_LDECWR
#define LDEC_DMAC_RQID_FROMDEV	DMA0_REQID_LDECRD

static UDIF_CHANNELS ldec[UDIF_NR_LDEC] = {
	[0] = {
		.iomem = __UDIF_IOMEM_INIT(LDEC_NAME, LDEC_IO_BASE, LDEC_IO_SIZE),
		.intr = UDIF_INTRPT_INIT(LDEC_NAME, LDEC_IRQ, 0),
	},
};

static UDIF_DMAC ldec_dmac[] = {
	UDIF_DMAC_INIT(LDEC_NAME "-1", LDEC_DMAC_CH_TODEV, LDEC_DMAC_RQID_TODEV, 0),
	UDIF_DMAC_INIT(LDEC_NAME "-2", LDEC_DMAC_CH_FROMDEV, LDEC_DMAC_RQID_FROMDEV, 0),
};

/* MMC stuff */
#define MMC_NAME 	"mmc"

#define MMC_IO_SIZE	0x8000
#define MMC_IO_BASE(x)	(CXD4115_AHB_BASE + 0x28000 + MMC_IO_SIZE * (x))

#define MMC_IRQ(x)	(IRQ_SDIF0_INT + (x) * 2)
#define MMC_HCLK(x)	(CXD4115_HSTCTRL_BASE + 0xbc + 0x10 * (x))
#define MMC_HCLK_SHIFT	0

#define MMC_DCLK	CXD4115_CLKRST(0x14)
#define MMC_DCLK_SET	(MMC_DCLK + 0x4)
#define MMC_DCLK_CLR	(MMC_DCLK + 0x8)
#define MMC_DCLK_SHIFT(x) (11 + (x))

#define MMC_DSEL	CXD4115_CLKRST(0x15)
#define MMC_DSEL_SET	(MMC_DSEL + 0x4)
#define MMC_DSEL_CLR	(MMC_DSEL + 0x8)
#define MMC_DSEL_SHIFT(x) (12 + (x) * 4)

static UDIF_CHANNELS mmc[UDIF_NR_MMC] = {
	[0] = {
		.iomem = __UDIF_IOMEM_INIT(MMC_NAME "0", MMC_IO_BASE(0), MMC_IO_SIZE),
		.intr = UDIF_INTRPT_INIT(MMC_NAME "0", MMC_IRQ(0), 0),
		.clock = UDIF_CLOCK_INIT(MMC_HCLK(0),  MMC_HCLK(0),  MMC_HCLK_SHIFT,
					 MMC_DCLK_SET, MMC_DCLK_CLR, MMC_DCLK_SHIFT(0),
					 MMC_DSEL_SET, MMC_DSEL_CLR, MMC_DSEL_SHIFT(0)),
	},
	[1] = {
		.iomem = __UDIF_IOMEM_INIT(MMC_NAME "2", MMC_IO_BASE(1), MMC_IO_SIZE),
		.intr = UDIF_INTRPT_INIT(MMC_NAME "2", MMC_IRQ(1), 0),
		.clock = UDIF_CLOCK_INIT(MMC_HCLK(1),  MMC_HCLK(1), MMC_HCLK_SHIFT,
					 MMC_DCLK_SET, MMC_DCLK_CLR, MMC_DCLK_SHIFT(1),
					 MMC_DSEL_SET, MMC_DSEL_CLR, MMC_DSEL_SHIFT(1)),
	},
	[2] = {
		.iomem = __UDIF_IOMEM_INIT(MMC_NAME "2", MMC_IO_BASE(2), MMC_IO_SIZE),
		.intr = UDIF_INTRPT_INIT(MMC_NAME "2", MMC_IRQ(2), 0),
		.clock = UDIF_CLOCK_INIT(MMC_HCLK(2),  MMC_HCLK(2),   MMC_HCLK_SHIFT,
					 MMC_DCLK_SET, MMC_DCLK_CLR, MMC_DCLK_SHIFT(2),
					 MMC_DSEL_SET, MMC_DSEL_CLR, MMC_DSEL_SHIFT(2)),
	},
};

/* MS stuff */
#define MS_NAME		"ms"
#define MS_IO_BASE(x)	(CXD4115_AHB_BASE + 0x80000 + MS_IO_SIZE * (x))
#define MS_IRQ(x)	(IRQ_MS0 + (x))

#define MS_HCLK(x)	(CXD4115_HSTCTRL_BASE + 0x9c + 0x10 * (x))
#define MS_DCLK_SHIFT(x) (9 + (x))
#define MS_DSEL_SHIFT(x) (4 + (x) * 4)

#define MS_IO_SIZE	0x8000
#define MS_HCLK_SHIFT	0

#define MS_DCLK		CXD4115_CLKRST(0x14)
#define MS_DCLK_SET	(MS_DCLK + 4)
#define MS_DCLK_CLR	(MS_DCLK + 8)

#define MS_DSEL		CXD4115_CLKRST(0x15)
#define MS_DSEL_SET	(MS_DSEL + 4)
#define MS_DSEL_CLR	(MS_DSEL + 8)

static UDIF_CHANNELS ms[UDIF_NR_MS] = {
	[0] = {
#ifdef CONFIG_ARCH_CXD4115
		.iomem = __UDIF_IOMEM_INIT(MS_NAME "0", MS_IO_BASE(0), MS_IO_SIZE),
		.intr = UDIF_INTRPT_INIT(MS_NAME "0", MS_IRQ(0), 0),
		.clock = UDIF_CLOCK_INIT(MS_HCLK(0),  MS_HCLK(0),  MS_HCLK_SHIFT,
					 MS_DCLK_SET, MS_DCLK_CLR, MS_DCLK_SHIFT(0),
					 MS_DSEL_SET, MS_DSEL_CLR, MS_DSEL_SHIFT(0)),
#else /* CONFIG_ARCH_CXD4120 */
		.iomem = __UDIF_IOMEM_INIT(MS_NAME "1", MS_IO_BASE(1), MS_IO_SIZE),
		.intr = UDIF_INTRPT_INIT(MS_NAME "1", MS_IRQ(1), 0),
		.clock = UDIF_CLOCK_INIT(MS_HCLK(1),  MS_HCLK(1),  MS_HCLK_SHIFT,
					 MS_DCLK_SET, MS_DCLK_CLR, MS_DCLK_SHIFT(1),
					 MS_DSEL_SET, MS_DSEL_CLR, MS_DSEL_SHIFT(1)),
#endif
	},
};

/* i2c stuff */
#define I2C_NAME	"i2c"
#define I2C_IO_BASE	(CXD4115_APB0_BASE + 0x10000)
#define I2C_IO_SIZE	0x3c
#define I2C_IRQ		IRQ_I2C

#define I2C_HCLK	(CXD4115_HSTCTRL_BASE + 0x138)
#define I2C_HCLK_SHIFT	0

static UDIF_CHANNELS i2c[UDIF_NR_I2C] = {
	[0] = {
		.iomem = __UDIF_IOMEM_INIT(I2C_NAME, I2C_IO_BASE, I2C_IO_SIZE),
		.intr = UDIF_INTRPT_INIT(I2C_NAME, I2C_IRQ, 0),
		.clock = UDIF_CLOCK_INIT(I2C_HCLK, I2C_HCLK, I2C_HCLK_SHIFT,
					 0, 0, 0,
					 0, 0, 0),
	},
};

/* cec stuff */
#define CEC_NAME	"cec"
#define CEC_IO_BASE	(CXD4115_APB0_BASE + 0x48000)
#define CEC_IO_SIZE	0x190

static UDIF_CHANNELS cec[UDIF_NR_CEC] = {
	[0] = {
		.iomem = __UDIF_IOMEM_INIT(CEC_NAME, CEC_IO_BASE, CEC_IO_SIZE),
	},
};

/*
 * All Devices
 */
UDIF_DEVICE udif_devices[UDIF_ID_NUM] = {
	[UDIF_ID_GPIO] = {
		.name	= GPIO_NAME,
		.nr_ch 	= NARRAY(gpio),
		.chs	= gpio,
	},
	[UDIF_ID_GPE] = {
		.name	= GPE_NAME,
		.nr_ch 	= NARRAY(gpe),
		.chs	= gpe,
		.init	= gpe_init,
		.exit	= gpe_exit,
	},
	[UDIF_ID_SIO] = {
		.name	= SIO_NAME,
		.nr_ch = NARRAY(sio),
		.chs	= sio,
	},
	[UDIF_ID_SIRCS] = {
		.name	= SIRCS_NAME,
		.nr_ch = NARRAY(sircs),
		.chs	= sircs,
	},
	[UDIF_ID_DMAC] = {
		.name	= DMAC_NAME,
		.dmac	= __UDIF_DMAC_INIT(NARRAY(dmac_dmac), dmac_dmac),
		.nr_ch 	= NARRAY(dmac),
		.chs	= dmac,
	},
	[UDIF_ID_ONA] = {
		.name	= ONA_NAME,
		.dmac	= __UDIF_DMAC_INIT(NARRAY(ona_dmac), ona_dmac),
		.nr_ch = NARRAY(ona),
		.chs	= ona,
	},
	[UDIF_ID_NAND] = {
		.name	= NAND_NAME,
		.dmac	= __UDIF_DMAC_INIT(NARRAY(nand_dmac), nand_dmac),
		.nr_ch = NARRAY(nand),
		.chs	= nand,
		.init	= nand_init,
		.exit	= nand_exit,
	},
	[UDIF_ID_LDEC] = {
		.name	= LDEC_NAME,
		.dmac	= __UDIF_DMAC_INIT(NARRAY(ldec_dmac), ldec_dmac),
		.nr_ch = NARRAY(ldec),
		.chs	= ldec,
	},
	[UDIF_ID_MMC] = {
		.name	= MMC_NAME,
		.nr_ch 	= NARRAY(mmc),
		.chs	= mmc,
	},
	[UDIF_ID_MS] = {
		.name	= MS_NAME,
		.nr_ch 	= NARRAY(ms),
		.chs	= ms,
	},
	[UDIF_ID_I2C] = {
		.name	= I2C_NAME,
		.nr_ch 	= NARRAY(i2c),
		.chs	= i2c,
	},
	[UDIF_ID_CEC] = {
		.name	= CEC_NAME,
		.nr_ch 	= NARRAY(cec),
		.chs	= cec,
	},
};

EXPORT_SYMBOL(udif_devices);

static int udif_platform_register(void)
{
	int i, ret = 0;

	for (i = 0; i < NARRAY(udif_devices); i++)
		ret = udif_device_register(&udif_devices[i]);

	return ret;
}

arch_initcall(udif_platform_register);

#if 0
static void udif_platform_unregister(void)
{
	int i;

	for (i = NARRAY(udif_devices) - 1; i >= 0; i--)
		udif_device_unregister(&udif_devices[i]);
}

module_exit(udif_platform_unregister);
#endif
