/*
 * arch/arm/mach-cxd4132/cachectrl.c
 *
 * cache lockdown driver
 *
 * Copyright 2011 Sony Corporation
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 *
 */
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/smp.h>
#include <linux/seq_file.h>
#include <linux/io.h>
#include <asm/hardware/cache-pl310.h>

#ifdef CONFIG_CACHE_PL310

#define N_MASTER 3

void l2cachectrl_write(char *arg, size_t len)
{
	int n, i;
	uint mask[N_MASTER];

	n = sscanf(arg, "%x %x %x", &mask[0], &mask[1], &mask[2]);
	if (n <= 0) {
		printk(KERN_ERR "invalid format\n");
		return;
	}
	for (i = n; i < 3; i++) {
		mask[i] = mask[n - 1];
	}
	pl310_lockdown(mask, N_MASTER);
}

int l2cachectrl_show(struct seq_file *seq, void *v)
{
	int i;
	__u32 lock_d, lock_i;

	seq_printf(seq, "L2:");
	for (i = 0; i < N_MASTER; i++) {
		lock_d = readl(PL310_REG(PL310_LOCKDOWN_WAY_D(i)));
		lock_i = readl(PL310_REG(PL310_LOCKDOWN_WAY_I(i)));
		seq_printf(seq, " 0x%02x,0x%02x", lock_d, lock_i);
	}
	seq_printf(seq, "\n");
	return 0;
}
#endif /* CONFIG_CACHE_PL310 */
