/*
 * arch/arm/mach-cxd4132/pm_macro-sio-type1.S
 *
 * SIO driver
 *
 * Copyright 2010 Sony Corporation
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 *
 */

#define SIO_DEBUG

#define MAX_RETRY 10
#define SIO_BUFSIZE	128

#ifdef CHECKSUM_EVAL
# define NVRAMSIZE 4
#else
# define NVRAMSIZE 2
#endif

/* packet */
#define LEN 28
#define OFFS_CHIPID	0
# define CHIP_ID	 0xb0
#define OFFS_PERR	2
# define PERR_MASK	 0x40
#if NVRAMSIZE == 4
#define OFFS_DATA0	14
#define OFFS_DATA1	15
#define OFFS_DATA2	16
#define OFFS_DATA3	18
#else
#define OFFS_DATA0	16
#define OFFS_DATA1	18
#endif
#define OFFS_PARITY	22
# define PAR_START	 2
# define PAR_END	 21

	.macro	sio_txbuf,rx
	ldr	\rx, pm_sio_buf
	.endm
	.macro	sio_rxbuf,rx
	ldr	\rx, pm_sio_buf
	add	\rx, \rx, #SIO_BUFSIZE
	.endm
	.macro	sio_wrbuf,rx
	ldr	\rx, pm_sio_buf
	add	\rx, \rx, #SIO_BUFSIZE*2
	.endm
	.macro	copyb,rd,rs,offs,rx
	ldrb	\rx, [\rs, \offs]
	strb	\rx, [\rd, \offs]
	.endm

	.macro	pm_sio_type1
type1_parity:
	add	r1, r0, #PAR_START		@ ptr
	mov	r0, #0				@ parity
	mov	r2, #PAR_END-PAR_START+1	@ len
1:
	ldrb	r3, [r1], #1
	eor	r0, r0, r3
	subs	r2, #1
	bne	1b
	eor	r0, r0, #0x0f			@ parity
	mov	pc, lr

type1_packet_init:
	stmdb	sp!, {r10, lr}
	mov	r10, r0
	mov	r1, #0
	mov	r2, #LEN
	bl	__memset
	mov	r0, #CHIP_ID
	strb	r0, [r10, #OFFS_CHIPID]
	ldmia	sp!, {r10, pc}

type1_packet_finish:
	stmdb	sp!, {r10, lr}
	mov	r10, r0
	bl	type1_parity
	strb	r0, [r10, #OFFS_PARITY]
	ldmia	sp!, {r10, pc}

type1_set_data:
	strb	r1, [r0, #OFFS_DATA0]
	lsr	r1, r1, #8
	strb	r1, [r0, #OFFS_DATA1]
#if NVRAMSIZE == 4
	lsr	r1, r1, #8
	strb	r1, [r0, #OFFS_DATA2]
	lsr	r1, r1, #8
	strb	r1, [r0, #OFFS_DATA3]
#endif
	mov	pc, lr

type1_get_data:
	ldrb	r2, [r0, #OFFS_DATA0]
	ldrb	r1, [r0, #OFFS_DATA1]
	orr	r2, r2, r1, lsl #8
#if NVRAMSIZE == 4
	ldrb	r1, [r0, #OFFS_DATA2]
	orr	r2, r2, r1, lsl #16
	ldrb	r1, [r0, #OFFS_DATA3]
	orr	r2, r2, r1, lsl #24
#endif
	mov	r0, r2			@ data
	mov	pc, lr

type1_setup_read_packet:
	stmdb	sp!, {r10, lr}
	mov	r10, r0
	bl	type1_packet_init
	@ Should not set parity byte.
	@ If parity is OK, some bits will be modified.
	ldmia	sp!, {r10, pc}

type1_setup_write_packet:
	stmdb	sp!, {r10-r12, lr}
	mov	r10, r0
	mov	r11, r1
	mov	r12, r2
	bl	type1_packet_init
	copyb	r10,r11,#3, r0
	ldrb	r0, [r11, #6]
	and	r0, r0, #0xf0
	strb	r0, [r10, #6]
	mov	r0, #0x01	@ write enable
	strb	r0, [r10, #11]
	copyb	r10,r11,#12, r0
	copyb	r10,r11,#13, r0
#if NVRAMSIZE != 4
	copyb	r10,r11,#14, r0
#endif
	copyb	r10,r11,#17, r0
	mov	r0, r10
	mov	r1, r12
	bl	type1_set_data
	mov	r0, r10
	bl	type1_packet_finish
	ldmia	sp!, {r10-r12, pc}

type1_verify_packet:
	stmdb	sp!, {r10, lr}
	mov	r10, r0
	bl	type1_parity
#ifdef ERROR_INJECTION
	ldr	r1, do_rx_perr
	tst	r1, #1
	lsr	r1, r1, #1
	str	r1, do_rx_perr
	addne	r0, r0, #1		@ parity error injection
#endif /* ERROR_INJECTION */
	ldrb	r1, [r10, #OFFS_PARITY]
	cmp	r0, r1
	moveq	r0, #0
	movne	r0, #-1
	ldmia	sp!, {r10, pc}

type1_transfer_packet:
	stmdb	sp!, {r9-r12, lr}
	mov	r10, r0
	mov	r11, r1
	mov	r12, r2
	mov	r9, #MAX_RETRY
2:
	mov	r0, r10
	mov	r1, r11
	mov	r2, r12
	bl	sio_transfer
	cmp	r0, #0
	bne	1f
	mov	r0, r11
	bl	type1_verify_packet
	cmp	r0, #0
	beq	9f			@ SUCCESS
	mov	r0, #'p'
	bl	__printch
1:
	subs	r9, r9, #1
	bne	2b
	mov	r0, #'r'
	bl	__printch
	mov	r0, #-1
9:
	ldmia	sp!, {r9-r12, pc}

type1_nvram_read:
	@ return value
	@   r0: status, r1: value
	stmdb	sp!, {lr}
#ifdef ERROR_INJECTION
	ldr	r0, pm_sio_rx_timeout
	str	r0, do_timeout
#endif
	sio_txbuf r0
	bl	type1_setup_read_packet
	sio_txbuf r0
	sio_rxbuf r1
	mov	r2, #LEN
	bl	type1_transfer_packet
	cmp	r0, #0
	bne	1f			@ FAIL
#ifdef SIO_DEBUG
	sio_rxbuf r0
	mov	r1, #LEN
	bl	__memdump
#endif
	sio_rxbuf r0
	bl	type1_get_data
	mov	r1, r0
	mov	r0, #0
1:
	ldmia	sp!, {pc}

type1_nvram_write:
	stmdb	sp!, {r9-r10, lr}
	mov	r10, r0
#ifdef ERROR_INJECTION
	ldr	r0, pm_sio_tx_timeout
	str	r0, do_timeout
#endif
	sio_txbuf r0
	bl	type1_setup_read_packet
	sio_txbuf r0
	sio_rxbuf r1
	mov	r2, #LEN
	bl	type1_transfer_packet
	cmp	r0, #0
	bne	9f			@ FAIL
#ifdef SIO_DEBUG
	sio_rxbuf r0
	mov	r1, #LEN
	bl	__memdump
#endif
	sio_wrbuf r0
	sio_rxbuf r1
	mov	r2, r10
	bl	type1_setup_write_packet

#ifdef ERROR_INJECTION
	ldr	r0, pm_sio_tx_perr
	str	r0, do_tx_perr
	ldr	r0, pm_sio_data_error
	str	r0, do_data_error
#endif

	mov	r9, #MAX_RETRY
3:
	@ write
	sio_wrbuf r0
	sio_rxbuf r1
	mov	r2, #LEN
	bl	type1_transfer_packet
	cmp	r0, #0
	bne	9f			@ FAIL
	@ read for verify
	sio_txbuf r0
	sio_rxbuf r1
	mov	r2, #LEN
	bl	type1_transfer_packet
	cmp	r0, #0
	bne	9f			@ FAIL
	@ verify
	sio_rxbuf r1
	ldrb	r0, [r1, #OFFS_PERR]
	ands	r0, r0, #PERR_MASK
	beq	1f
	@ TX parity error
	mov	r0, #'P'
	bl	__printch
	b	2f			@ retry
1:
	sio_rxbuf r0
	bl	type1_get_data
#ifdef ERROR_INJECTION
	ldr	r1, do_data_error
	tst	r1, #1
	lsr	r1, r1, #1
	str	r1, do_data_error
	addne	r0, r0, #1		@ data error injection
#endif /* ERROR_INJECTION */
	cmp	r0, r10
	beq	1f			@ SUCCESS
	@ TX error
	mov	r0, #'V'
	bl	__printch
2:
	@ retry
	subs	r9, r9, #1
	bne	3b
	mov	r0, #'R'
	bl	__printch
	mov	r0, #-1
	b	8f
9:
	sio_rxbuf r0
	mov	r1, #LEN
	bl	__memdump
	mov	r0, #-1
	b	8f
1:
	@ SUCCESS
#ifdef SIO_DEBUG
	sio_rxbuf r0
	mov	r1, #LEN
	bl	__memdump
#endif
	mov	r0, #0
8:
	ldmia	sp!, {r9-r10, pc}

type1_nvram_open:
	stmdb	sp!, {lr}
	mov	r0, #1
	bl	sio_clk
	bl	sio_setup
#ifdef ERROR_INJECTION
	ldr	r0, pm_sio_rx_perr
	str	r0, do_rx_perr
#endif
	ldmia	sp!, {pc}

type1_nvram_close:
	stmdb	sp!, {lr}
	mov	r0, #0
	bl	sio_clk
	ldmia	sp!, {pc}

#ifdef ERROR_INJECTION
do_rx_perr:	.long	0
do_tx_perr:	.long	0
do_timeout:	.long	0
do_data_error:	.long	0
	.globl	pm_sio_rx_perr
pm_sio_rx_perr:	.long	0
	.globl	pm_sio_tx_perr
pm_sio_tx_perr:	.long	0
	.globl	pm_sio_rx_timeout
pm_sio_rx_timeout:	.long	0
	.globl	pm_sio_tx_timeout
pm_sio_tx_timeout:	.long	0
	.globl	pm_sio_data_error
pm_sio_data_error:	.long	0
#endif /* ERROR_INJECTION */
	.endm
