/*
 * arch/arm/mach-cxd4132/pm_macro-sio.S
 *
 * SIO driver
 *
 * Copyright 2010 Sony Corporation
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 *
 */
#include <mach/hardware.h>
#include <mach/time.h>
#include <mach/regs-permisc.h>
#include <mach/regs-gpio.h>
#include <mach/regs-sio.h>

#define TIMEOUT 10000 /* us */
#define OFFS_RXBUF	128

/* SIO timings */
#define XCS_SETUP_HOLD		4 /* us */
#define XCS_NEGATE_PERIOD	10 /* us */

	.macro	pm_sio
sio_clk:
	@ destruct r0-r1
	ldr	r1, pm_sio_clkrst	@ CXD4132_PER_CLKRST_SIO(pm_sio_ch)
	cmp	r0, #0
	mov	r0, #1
	strne	r0, [r1, #PER_CLKRST_SET]
	streq	r0, [r1, #PER_CLKRST_CLR]
	mov	pc, lr

sio_xcs:
	@ destruct r0-r1
	ldr	r1, pm_sio_gpio		@ PA_GPIO(pm_sio_port)
	cmp	r0, #0
	ldr	r0, pm_sio_xcs_mask	@ 1 << pm_sio_xcs
	strne	r0, [r1, #GPIO_DATA_CLEAR]	@ assert
	streq	r0, [r1, #GPIO_DATA_SET]	@ negate
	mov	pc, lr

	@ XCS setup/hold
sio_xcs_wait:
	@ destruct r0-r2
	mov	r0, #XCS_SETUP_HOLD
	b	__udelay

	@ XCS nagate period
sio_wait:
	@ destruct r0-r2
	mov	r0, #XCS_NEGATE_PERIOD
	b	__udelay		@ destruct r0-r2

sio_setup:
	@ destruct r0-r1
	ldr	r1, pm_sio_addr		@ CXD4132_SIO(sio_ch)
	mov	r0, #SIOEN_EN
	str	r0, [r1, #SIOEN]
	ldr	r0, pm_sio_baud_l	@ (pm_sio_baud & 0x00ff) << 24
	str	r0, [r1, #SIOBGC0]
	ldr	r0, pm_sio_baud_h	@ (pm_sio_baud & 0xff00) << 16
	str	r0, [r1, #SIOBGC1]
	ldr	r0, =SIOCS_ICLSIO|SIOCS_CLOE|SIOCS_SCKOE|SIOCS_SOEN
	str	r0, [r1, #SIOCS]
	ldr	r0, =SIOM_IMOD|SIOM_TRM_SENDREC|SIOM_ICK_16
	str	r0, [r1, #SIOM]
	mov	pc, lr

sio_transfer:
	@ r0: return value
	stmdb	sp!, {r8-r12, lr}
	mov	r10, r0			@ tx
	mov	r11, r1			@ rx
	mov	r12, r2			@ len

	mov	r9, #0			@ return code
	ldr	r8, pm_sio_addr

	mov	r0, #1
	bl	sio_xcs			@ assert XCS
	bl	sio_xcs_wait

	mov	r0, #0
	str	r0, [r8, #SIOSA]
	sub	r0, r12, #1		@ len - 1
	lsl	r0, r0, #SIO_SHIFT
	str	r0, [r8, #SION]
	add	r0, r8, #SIOBUF
	mov	r1, r10
	mov	r2, r12
	bl	__memcpy
#ifdef ERROR_INJECTION
	ldr	r0, do_tx_perr
	tst	r0, #1
	lsr	r0, r0, #1
	str	r0, do_tx_perr
	beq	1f
	ldrb	r0, [r8,#SIOBUF+22]
	add	r0, r0, #1
	strb	r0, [r8,#SIOBUF+22]	@ parity error injection
1:
#endif /* ERROR_INJECTION */

	@ start
	ldr	r0, [r8, #SIOCS]
	orr	r0, r0, #SIOCS_SIOST
	str	r0, [r8, #SIOCS]
	@ wait for done
	ldr	r1, =TIMEOUT
#ifdef ERROR_INJECTION
	ldr	r0, do_timeout
	tst	r0, #1
	lsr	r0, r0, #1
	str	r0, do_timeout
	movne	r1, #0			@ timeout error injection
#endif /* ERROR_INJECTION */
	mach_usecs_to_cycles r1
	mach_freerun r2			@ timer
	mach_read_cycles r0,r2		@ now
	add	r1, r1, r0		@ expire time
1:
	ldr	r0, [r8, #SIOCS]
	tst	r0, #SIOCS_SIOST
	beq	1f
	mach_read_cycles r0,r2		@ now
	cmp	r0, r1			@ now - expire
	bmi	1b
	@ timeout
	ldr	r0, [r8, #SIOCS]
	and	r0, r0, #~SIOCS_SIOST
	str	r0, [r8, #SIOCS]	@ stop
	mov	r0, #'T'
	bl	__printch
	mov	r9, #-1			@ error
1:
	mov	r0, r11
	add	r1, r8, #SIOBUF
	mov	r2, r12
	bl	__memcpy

	bl	sio_xcs_wait
	mov	r0, #0
	bl	sio_xcs			@ negate XCS
	bl	sio_wait

	mov	r0, r9			@ return value
	ldmia	sp!, {r8-r12, pc}

	.globl	pm_sio_addr
pm_sio_addr:	.long	0
	.globl	pm_sio_gpio
pm_sio_gpio:	.long	0
	.globl	pm_sio_clkrst
pm_sio_clkrst:	.long	0
	.globl	pm_sio_xcs_mask
pm_sio_xcs_mask:.long	0
	.globl	pm_sio_baud_l
pm_sio_baud_l:	.long	0
	.globl	pm_sio_baud_h
pm_sio_baud_h:	.long	0
	.globl	pm_sio_buf
pm_sio_buf:	.long	0
	.endm
