/* 2010-09-02: File changed by Sony Corporation */
/*
 *  linux/arch/arm/mm/proc-v6.S
 *
 *  Copyright (C) 2001 Deep Blue Solutions Ltd.
 *  Modified by Catalin Marinas for noMMU support
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 *  This is the "shell" of the ARMv6 processor support.
 */
#include <linux/linkage.h>
#include <asm/assembler.h>
#include <asm/asm-offsets.h>
#include <asm/hwcap.h>
#include <asm/pgtable-hwdef.h>
#include <asm/pgtable.h>
#if defined(CONFIG_EJ_TTB_OUTER_NC) && defined(CONFIG_CACHE_PL310)
#include <asm/hardware/cache-l2x0.h>
#endif /* CONFIG_EJ_TTB_OUTER_NC && CONFIG_CACHE_PL310 */

#include "proc-macros.S"

#define HARVARD_CACHE
#define D_CACHE_LINE_SIZE	32

#define TTB_C		(1 << 0)
#define TTB_S		(1 << 1)
#define TTB_IMP		(1 << 2)
#define TTB_RGN_NC	(0 << 3)
#define TTB_RGN_WBWA	(1 << 3)
#define TTB_RGN_WT	(2 << 3)
#define TTB_RGN_WB	(3 << 3)

#ifndef CONFIG_SMP
#define TTB_FLAGS	TTB_RGN_WBWA
#define PMD_FLAGS	PMD_SECT_WB
#else
#ifdef CONFIG_EJ_TTB_OUTER_NC
#define TTB_FLAGS	TTB_RGN_NC|TTB_S
#else
#define TTB_FLAGS	TTB_RGN_WBWA|TTB_S
#endif /* CONFIG_EJ_TTB_OUTER_NC */
#define PMD_FLAGS	PMD_SECT_WBWA|PMD_SECT_S
#endif

ENTRY(cpu_v6_proc_init)
	mov	pc, lr

ENTRY(cpu_v6_proc_fin)
	stmfd	sp!, {lr}
	cpsid	if				@ disable interrupts
	bl	v6_flush_kern_cache_all
	mrc	p15, 0, r0, c1, c0, 0		@ ctrl register
	bic	r0, r0, #0x1000			@ ...i............
	bic	r0, r0, #0x0006			@ .............ca.
	mcr	p15, 0, r0, c1, c0, 0		@ disable caches
	ldmfd	sp!, {pc}

/*
 *	cpu_v6_reset(loc)
 *
 *	Perform a soft reset of the system.  Put the CPU into the
 *	same state as it would be if it had been reset, and branch
 *	to what would be the reset vector.
 *
 *	- loc   - location to jump to for soft reset
 *
 *	It is assumed that:
 */
	.align	5
ENTRY(cpu_v6_reset)
	mov	pc, r0

/*
 *	cpu_v6_do_idle()
 *
 *	Idle the processor (eg, wait for interrupt).
 *
 *	IRQs are already disabled.
 */
ENTRY(cpu_v6_do_idle)
	mov	r1, #0
	mcr	p15, 0, r1, c7, c10, 4		@ DWB - WFI may enter a low-power mode
	mcr	p15, 0, r1, c7, c0, 4		@ wait for interrupt
	mov	pc, lr

ENTRY(cpu_v6_dcache_clean_area)
#ifndef TLB_CAN_READ_FROM_L1_CACHE
1:	mcr	p15, 0, r0, c7, c10, 1		@ clean D entry
	add	r0, r0, #D_CACHE_LINE_SIZE
	subs	r1, r1, #D_CACHE_LINE_SIZE
	bhi	1b
#endif
	mov	pc, lr

/*
 *	cpu_arm926_switch_mm(pgd_phys, tsk)
 *
 *	Set the translation table base pointer to be pgd_phys
 *
 *	- pgd_phys - physical address of new TTB
 *
 *	It is assumed that:
 *	- we are not using split page tables
 */
ENTRY(cpu_v6_switch_mm)
#ifdef CONFIG_MMU
	mov	r2, #0
	ldr	r1, [r1, #MM_CONTEXT_ID]	@ get mm->context.id
	orr	r0, r0, #TTB_FLAGS
	mcr	p15, 0, r2, c7, c5, 6		@ flush BTAC/BTB
	mcr	p15, 0, r2, c7, c10, 4		@ drain write buffer
	mcr	p15, 0, r0, c2, c0, 0		@ set TTB 0
	mcr	p15, 0, r1, c13, c0, 1		@ set context ID
#endif
	mov	pc, lr

/*
 *	cpu_v6_set_pte_ext(ptep, pte, ext)
 *
 *	Set a level 2 translation table entry.
 *
 *	- ptep  - pointer to level 2 translation table entry
 *		  (hardware version is stored at -1024 bytes)
 *	- pte   - PTE value to store
 *	- ext	- value for extended PTE bits
 *
 *	#ifdef CONFIG_CPU_AFE
 *	Simplified permission translation (AP0 is the access flag):
 *	YUWD  AP2 AP1 AP0	SVC	User
 *	0xxx   0   0   0	no acc	no acc
 *	100x   1   0   1	r/o	no acc
 *	10x0   1   0   1	r/o	no acc
 *	1011   0   0   1	r/w	no acc
 *	110x   1   1   1	r/o	r/o
 *	11x0   1   1   1	r/o	r/o
 *	1111   0   1   1	r/w	r/w
 *	#endif
 */
#ifndef CONFIG_CPU_AFE
	armv6_mt_table cpu_v6
#endif

ENTRY(cpu_v6_set_pte_ext)
#ifdef CONFIG_MMU
#ifndef CONFIG_CPU_AFE
	armv6_set_pte_ext cpu_v6
#else
	tst	r1, #L_PTE_PRESENT
	beq	1f
	tst	r1, #L_PTE_DIRTY
	orreq	r1, #L_PTE_NOWRITE
	orr	r1, r1, r2
1:
#ifdef CONFIG_SNSC_DISABLE_IRQ_IN_SET_PTE_EXT
	mrs	r2, cpsr
	cpsid	i
#endif
	str	r1, [r0]
	mcr	p15, 0, r0, c7, c10, 1		@ flush_pte

#ifdef CONFIG_EJ_TTB_OUTER_NC
	@ flush L2 Cache
# ifdef CONFIG_CACHE_PL310
	mov	r1, #0
	mcr	p15, 0, r1, c7, c10, 4		@ dsb
	mov	r1, #0xbfffffe0			@ virt_to_phys
	and	r0, r0, r1
	ldr	r1, =VA_PL310			@ L2CC
	str	r0, [r1, #L2X0_CLEAN_LINE_PA]	@ CLEAN by PA
	str	r0, [r1, #L2X0_CACHE_SYNC]	@ SYNC
# endif /* CONFIG_CACHE_PL310 */
#endif /* CONFIG_EJ_TTB_OUTER_NC */

#ifdef CONFIG_SNSC_DISABLE_IRQ_IN_SET_PTE_EXT
	msr	cpsr_c, r2
#endif
#endif
#endif
	mov	pc, lr




cpu_v6_name:
	.asciz	"ARMv6-compatible processor"
	.align

	.section ".text.init", #alloc, #execinstr

/*
 *	__v6_setup
 *
 *	Initialise TLB, Caches, and MMU state ready to switch the MMU
 *	on.  Return in r0 the new CP15 C1 control register setting.
 *
 *	We automatically detect if we have a Harvard cache, and use the
 *	Harvard cache control instructions insead of the unified cache
 *	control instructions.
 *
 *	This should be able to cover all ARMv6 cores.
 *
 *	It is assumed that:
 *	- cache type register is implemented
 */
ENTRY(__v6_setup)
#ifdef CONFIG_EXCLUSIVE_CACHE
	@ Exclusive cache
	adr	r0, v6_excl
	ldmia	r0, {r5, r6}
	sub	r0, r0, r5
	add	r6, r6, r0
	ldr	r0, [r6]			@ exclusive_cache flag
	cmp	r0, #0
	beq	1f
	mrc	p15, 0, r0, c1, c0, 1
	orr	r0, r0, #0x10			@ L2 EXCL
	mcr	p15, 0, r0, c1, c0, 1
1:
#endif /* CONFIG_EXCLUSIVE_CACHE */
#ifdef CONFIG_SMP
	mrc	p15, 0, r0, c1, c0, 1		@ Enable SMP/nAMP mode
	orr	r0, r0, #0x20
	mcr	p15, 0, r0, c1, c0, 1
#endif

	mov	r0, #0
#ifdef HARVARD_CACHE
	mcr	p15, 0, r0, c7, c14, 0		@ clean+invalidate D cache
	mcr	p15, 0, r0, c7, c5, 0		@ invalidate I+BTB cache
#else
	mcr	p15, 0, r0, c7, c15, 0		@ clean+invalidate cache
#endif
	mcr	p15, 0, r0, c7, c10, 4		@ drain write buffer
#ifdef CONFIG_MMU
	mcr	p15, 0, r0, c8, c7, 0		@ invalidate I + D TLBs
	mcr	p15, 0, r0, c2, c0, 2		@ TTB control register
	orr	r4, r4, #TTB_FLAGS
	mcr	p15, 0, r4, c2, c0, 1		@ load TTB1
#ifdef CONFIG_EJ_SPLIT_PAGETABLE
	mov	r0, #CONFIG_EJ_SPLIT_SHIFT
	mcr	p15, 0, r0, c2, c0, 2		@ TTB control register
#endif
#endif /* CONFIG_MMU */
#ifdef CONFIG_CPU_AFE
	ldr	r5, =0x000a81a8
	ldr	r6, =0x40e040e0
	mcr	p15, 0, r5, c10, c2, 0		@ write PRRR
	mcr	p15, 0, r6, c10, c2, 1		@ write NMRR
#endif /* CONFIG_CPU_AFE */
	adr	r5, v6_crval
	ldmia	r5, {r5, r6}
#ifdef CONFIG_CPU_ENDIAN_BE8
	orr	r6, r6, #1 << 25		@ big-endian page tables
#endif
	mrc	p15, 0, r0, c1, c0, 0		@ read control register
	bic	r0, r0, r5			@ clear bits them
	orr	r0, r0, r6			@ set them
#ifdef CONFIG_ARM_ERRATA_364296
	/* Workaround for the 364296 ARM1136 r0pX errata (possible cache data
	 * corruption with hit-under-miss enabled). The conditional code below
	 * (setting the undocumented bit 31 in the auxiliary control register
	 * and the FI bit in the control register) disables hit-under-miss
	 * without putting the processor into full low interrupt latency mode.
	 */
	ldr	r6, =0x4107b360			@ id for ARM1136 r0pX
	mrc	p15, 0, r5, c0, c0, 0		@ get processor id
	bic	r5, r5, #0xf			@ mask out part bits [3:0]
	teq	r5, r6				@ check for the faulty core
	mrceq	p15, 0, r5, c1, c0, 1		@ load aux control reg
	orreq	r5, r5, #(1 << 31)		@ set the undocumented bit 31
	mcreq	p15, 0, r5, c1, c0, 1		@ write aux control reg
	orreq	r0, r0, #(1 << 21)		@ low interrupt latency configuration
#endif
	mov	pc, lr				@ return to head.S:__ret

	/*
	 * #ifndef CONFIG_CPU_AFE
	 *         V X F   I D LR
	 * .... ...E PUI. .T.T 4RVI ZFRS BLDP WCAM
	 * rrrr rrrx xxx0 0101 xxxx xxxx x111 xxxx < forced
	 *         0 110       0011 1.00 .111 1101 < we want
	 * #else
	 *   FT   EV X F   I D LR
	 * ..AR ..EE PUI. .T.T 4RVI ZFRS BLDP WCAM
	 * rxxx rrxx xxx0 01x1 xxxx xxxx x111 xxxx < forced
	 *   11    0 110       0011 1.00 .111 1101 < we want
	 * #endif
	 */
	.type	v6_crval, #object
v6_crval:
#ifndef CONFIG_CPU_AFE
	crval	clear=0x01e0fb7f, mmuset=0x00c0387d, ucset=0x00c0187c
#else
	crval	clear=0x01e0fb7f, mmuset=0x30c0387d, ucset=0x00c0187c
#endif

#ifdef CONFIG_EXCLUSIVE_CACHE
v6_excl:
	.long	.
	.long	exclusive_cache
#endif /* CONFIG_EXCLUSIVE_CACHE */

	.type	v6_processor_functions, #object
ENTRY(v6_processor_functions)
	.word	v6_early_abort
	.word	v6_pabort
	.word	cpu_v6_proc_init
	.word	cpu_v6_proc_fin
	.word	cpu_v6_reset
	.word	cpu_v6_do_idle
	.word	cpu_v6_dcache_clean_area
	.word	cpu_v6_switch_mm
	.word	cpu_v6_set_pte_ext
	.size	v6_processor_functions, . - v6_processor_functions

	.type	cpu_arch_name, #object
cpu_arch_name:
	.asciz	"armv6"
	.size	cpu_arch_name, . - cpu_arch_name

	.type	cpu_elf_name, #object
cpu_elf_name:
	.asciz	"v6"
	.size	cpu_elf_name, . - cpu_elf_name
	.align

	.section ".proc.info.init", #alloc, #execinstr

	/*
	 * Match any ARMv6 processor core.
	 */
	.type	__v6_proc_info, #object
__v6_proc_info:
	.long	0x0007b000
	.long	0x0007f000
	.long   PMD_TYPE_SECT | \
		PMD_SECT_AP_WRITE | \
		PMD_SECT_AP_READ | \
		PMD_FLAGS
	.long   PMD_TYPE_SECT | \
		PMD_SECT_XN | \
		PMD_SECT_AP_WRITE | \
		PMD_SECT_AP_READ
	b	__v6_setup
	.long	cpu_arch_name
	.long	cpu_elf_name
	.long	HWCAP_SWP|HWCAP_HALF|HWCAP_THUMB|HWCAP_FAST_MULT|HWCAP_EDSP|HWCAP_JAVA
	.long	cpu_v6_name
	.long	v6_processor_functions
	.long	v6wbi_tlb_fns
	.long	v6_user_fns
	.long	v6_cache_fns
	.size	__v6_proc_info, . - __v6_proc_info
