/*
 * arch/arm/mach-cxd4115/pm.c
 *
 * pm callback for cxd4115
 *
 * Copyright 2007,2008,2009,2010 Sony Corporation
 *
 * This code is based on arch/arm/mach-sa1100/pm.c.
 */
/*
 * SA1100 Power Management Routines
 *
 * Copyright (c) 2001 Cliff Brake <cbrake@accelent.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License.
 *
 * History:
 *
 * 2001-02-06:	Cliff Brake         Initial code
 *
 * 2001-02-25:	Sukjae Cho <sjcho@east.isi.edu> &
 * 		Chester Kuo <chester@linux.org.tw>
 * 			Save more value for the resume function! Support
 * 			Bitsy/Assabet/Freebird board
 *
 * 2001-08-29:	Nicolas Pitre <nico@cam.org>
 * 			Cleaned up, pushed platform dependent stuff
 * 			in the platform specific files.
 *
 * 2002-05-27:	Nicolas Pitre	Killed sleep.h and the kmalloced save array.
 * 				Storage is local on the stack now.
 */
#include <linux/module.h>
#include <linux/moduleparam.h>
#include <mach/moduleparam.h>
#include <linux/init.h>
#include <linux/suspend.h>
#include <linux/time.h>
#include <linux/bootmem.h>

#include <mach/hardware.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/setup.h>
#include <asm/mach-types.h>
#include <asm/page.h>
#include <asm/mach/arch.h>
#include <mach/system.h>
#include <asm/mach/time.h>
#include <mach/time.h>
#include <mach/pm.h>
#include <asm/uaccess.h>
#include <asm/irq.h>
#include <asm/system.h>
#include <asm/mach/irq.h>
#include <linux/delay.h>
#include <mach/regs-timer.h>
#include <mach/hardware.h>
#include <mach/memory.h>
#include <linux/em_export.h>
#ifdef CONFIG_SNSC_BOOT_TIME
#include <linux/snsc_boot_time.h>
#endif

#include <asm/mach/warmboot.h>

static int dc = 0;
module_param_named(dc, dc, int, S_IRUSR|S_IWUSR);
static int logbuf_noclear = 0;
module_param_named(logbuf_noclear, logbuf_noclear, int, S_IRUSR|S_IWUSR);

#ifndef CONFIG_CXD41XX_SIMPLE_SUSPEND
/*
 * DDR power down control
 */
extern unsigned long cxd41xx_ddr_ctrl;   /* sleep.S */
module_param_named(ddr_ctrl, cxd41xx_ddr_ctrl, ddrctrl, S_IRUSR|S_IWUSR);
#ifdef CONFIG_ARCH_CXD4120
extern unsigned long cxd41xx_ddr_ctrl2;  /* sleep.S */
module_param_named(ddr_ctrl2, cxd41xx_ddr_ctrl2, ddrctrl, S_IRUSR|S_IWUSR);
#endif /* CONFIG_ARCH_CXD4120 */
extern unsigned long cxd41xx_ddr_pasr0;   /* sleep.S */
module_param_named(pasr0, cxd41xx_ddr_pasr0, ddrpasr, S_IRUSR|S_IWUSR);
extern unsigned long cxd41xx_ddr_pasr1;   /* sleep.S */
module_param_named(pasr1, cxd41xx_ddr_pasr1, ddrpasr, S_IRUSR|S_IWUSR);
extern unsigned long cxd41xx_sus_end_msg;   /* sleep.S */
module_param_named(sus_msg, cxd41xx_sus_end_msg, ulong, S_IRUSR|S_IWUSR);
/*
 * resume vector
 */
static unsigned long cxd41xx_resume_vector = 0xffffffffUL;
module_param_named(resume, cxd41xx_resume_vector, ulongH, S_IRUSR);
/*
 * suspend error flag
 */
static unsigned long cxd41xx_sus_error = 0xffffffffUL;
module_param_named(sus_error, cxd41xx_sus_error, ulongH, S_IRUSR);
#endif /* !CONFIG_CXD41XX_SIMPLE_SUSPEND */

extern int cxd4115_cpu_suspend(int);
extern void cxd4115_cpu_resume(void);

#ifdef CONFIG_OSAL_UDIF
extern int udif_driver_suspend(void);
extern int udif_driver_resume(void);
#else
# define udif_driver_suspend() (0)
# define udif_driver_resume() do {} while (0)
#endif /* CONFIG_OSAL_UDIF */

void cxd41xx_pm_setup(void)
{
#ifndef CONFIG_CXD41XX_SIMPLE_SUSPEND
	unsigned int port, bit;
	extern unsigned long cxd41xx_stable_run;
	extern unsigned long cxd41xx_xpower_off;
	extern unsigned long cxd41xx_xpower_off_addr;    /* sleep.S */
	extern unsigned long cxd41xx_xpower_off_bitmask; /* sleep.S */
	extern unsigned long cxd41xx_stable_run_addr;    /* sleep.S */
	extern unsigned long cxd41xx_stable_run_bitmask; /* sleep.S */
	extern void cxd4115_cpu_resume(void);

	/* resume vector */
	if (0xffffffffUL == cxd41xx_resume_vector) {
		printk("resume vector: not defined\n");
	} else {
		printk("resume vector: 0x%08lx\n", cxd41xx_resume_vector);
		printk("resume entry:  0x%08lx\n", virt_to_phys(cxd4115_cpu_resume));
		*(unsigned long *)phys_to_virt(cxd41xx_resume_vector) = virt_to_phys(cxd4115_cpu_resume);
	}
	/* sus_error flag */
	if (0xffffffffUL == cxd41xx_sus_error) {
		printk("sus_error address: not defined\n");
	} else {
		printk("sus_error address: 0x%08lx\n", cxd41xx_sus_error);
		*(unsigned long *)phys_to_virt(cxd41xx_sus_error) = 0UL;
	}

	/* XPOWER_OFF */
	if (0xffffffffUL == cxd41xx_xpower_off) {
		cxd41xx_xpower_off_addr = 0xffffffffUL; /* invalid mark */
	} else {
		port = cxd41xx_xpower_off & 0xff;
		bit  = (cxd41xx_xpower_off >> 8) & 0xff;
		cxd41xx_xpower_off_addr = PA_GPIO(port);
		cxd41xx_xpower_off_bitmask = 1 << bit;
	}

	/* STABLE_RUN */
	if (0xffffffffUL == cxd41xx_stable_run) {
		cxd41xx_stable_run_addr = 0xffffffffUL; /* invalid mark */
	} else {
		port = cxd41xx_stable_run & 0xff;
		bit  = (cxd41xx_stable_run >> 8) & 0xff;
		cxd41xx_stable_run_addr = PA_GPIO(port);
		cxd41xx_stable_run_bitmask = 1 << bit;
	}
#endif /* CONFIG_CXD41XX_SIMPLE_SUSPEND */
}

void __attribute__((weak)) cxd41xx_early_resume(void)
{
	/* none */
}

static int _cxd4115_pm_enter(int mode)
{
	int err=0;
#ifdef CONFIG_SNSC_DEBUG_PROFILE
	extern void profile_soft_init(void);
#endif


#if 1
	/* go zzz */
	if(cxd4115_cpu_suspend(mode)) {
		err = -1;
		printk("suspend error!\n");
	}
#endif

	cpu_init();

#ifdef CONFIG_WARM_BOOT_IMAGE
	if (!err && PM_SUSPEND_DISK == pm_get_state() && !pm_is_mem_alive()) {
		extern void clear_logbuf(void);
		if (!logbuf_noclear) {
			clear_logbuf();
		}
		printk("WARMBOOT\n");
	}
#endif /* CONFIG_WARM_BOOT_IMAGE */

	cxd41xx_early_resume();

#ifdef CONFIG_SNSC_BOOT_TIME
	boot_time_takeover();
#endif
#ifdef CONFIG_SNSC_DEBUG_PROFILE
	profile_soft_init();
#endif
	cxd4115_timer_early_init();

	return err;
}

unsigned long sleep_phys_sp(void *sp)
{
	return virt_to_phys(sp);
}

static u8 reboot_reason;
static u16 reboot_timer;
static u16 state_timer;

static void cxd4115_pm_halt(int reboot)
{
	cxd4115_cpu_suspend((reboot)?PM_ENTER_RESET:PM_ENTER_ERROR);

	/* never reached */
}

static void pm_suspend_error(int mode)
{
	wbi_drop();
#ifndef CONFIG_CXD41XX_SIMPLE_SUSPEND
	*(unsigned long *)phys_to_virt(cxd41xx_sus_error) = 1UL;
#endif /* CONFIG_CXD41XX_SIMPLE_SUSPEND */
	cxd4115_pm_halt(0);
	/* never reached */
}

/*
 * Called after processes are frozen, but before we shut down devices.
 */
static int cxd4115_pm_begin(suspend_state_t state)
{
	return warm_boot_pm_begin(state);
}

extern void drop_pagecache(void);
extern void drop_slab(void);
static int cxd4115_pm_prepare(void)
{
	if (unlikely(udif_driver_suspend()))
		pm_suspend_error(0);

	if (pm_get_state() == PM_SUSPEND_MEM) {
		if (dc & 1)
			drop_pagecache();
		if (dc & 2)
			drop_slab();
	}

	return warm_boot_pm_prepare();
}

static int cxd4115_pm_enter(suspend_state_t state)
{
	return _cxd4115_pm_enter(PM_ENTER_NORMAL);
}

/*
 * Called after devices are re-setup, but before processes are thawed.
 */
static void cxd4115_pm_finish(void)
{
	reboot_reason = 0;
	reboot_timer = 0;
	state_timer = 0;

	warm_boot_pm_finish();
	udif_driver_resume();

	wbi_resume_drop();
}

static int cxd4115_valid(suspend_state_t state)
{
	return state == PM_SUSPEND_MEM || state == PM_SUSPEND_DISK;
}

/*
 * Set to PM_DISK_FIRMWARE so we can quickly veto suspend-to-disk.
 */
static struct platform_suspend_ops cxd4115_pm_ops = {
	.valid		= cxd4115_valid,
	.begin		= cxd4115_pm_begin,
	.prepare	= cxd4115_pm_prepare,
	.enter		= cxd4115_pm_enter,
	.finish		= cxd4115_pm_finish,
};

extern struct kobject *power_kobj;

#define wb_attr(_name) \
static ssize_t _name##_show(struct kobject *s, struct kobj_attribute *attr, \
			    char *buf)                                   \
{                                                                        \
	return snprintf(buf, PAGE_SIZE, "%u\n", _name);                              \
}                                                                        \
static ssize_t _name##_store(struct kobject *s, struct kobj_attribute *attr, \
			     const char * buf, size_t n)                 \
{                                                                        \
	char *endp;                                                      \
	int value;                                                       \
	                                                                 \
	value = simple_strtoul(buf, &endp, 0);                           \
	if(*endp)                                                        \
		return -EINVAL;                                          \
                                                                         \
	_name = value;                                                   \
	return n;                                                        \
}                                                                        \
static struct kobj_attribute _name##_attr = {	                         \
	.attr	= {				                         \
		.name = __stringify(_name),	                         \
		.mode = 0644,			                         \
	},					                         \
	.show	= _name##_show,			                         \
	.store	= _name##_store,		                         \
}

wb_attr(reboot_reason);
wb_attr(reboot_timer);
wb_attr(state_timer);

static int __init cxd4115_pm_init(void)
{
	int error = 0;

	error = sysfs_create_file(power_kobj, &reboot_reason_attr.attr);
	if (error) {
		goto out;
	}
	error = sysfs_create_file(power_kobj, &reboot_timer_attr.attr);
	if (error) {
		goto out;
	}
	error = sysfs_create_file(power_kobj, &state_timer_attr.attr);
	if (error) {
		goto out;
	}

	suspend_set_ops(&cxd4115_pm_ops);
 out:
	return error;
}

late_initcall(cxd4115_pm_init);

unsigned int cxd41xx_mem_alive = 0; /* set @ sleep.S */

/* check if memory data is kept during the last halt */
unsigned int pm_is_mem_alive(void)
{
	return cxd41xx_mem_alive;
}
EXPORT_SYMBOL(pm_is_mem_alive);

#ifdef CONFIG_WARM_BOOT_IMAGE
int cxd4115_create_warmbootimage(void)
{
	cxd41xx_mem_alive = 0;
	return cxd_create_warmbootimage((unsigned long)cxd4115_cpu_resume, 0);
}
#else
int cxd4115_create_warmbootimage(void)
{
	return 0;
}
#endif

void em_reboot(int force)
{
	pm_machine_reset(force);
}
