/*
 * arch/arm/plat-cxd41xx/sleep-simple.S
 *
 * pm callback for cxd41xx
 *
 * Copyright 2009,2010 Sony Corporation
 *
 * This code is based on arch/arm/mach-sa1100/sleep.S
 */
/*
 * SA11x0 Assembler Sleep/WakeUp Management Routines
 *
 * Copyright (c) 2001 Cliff Brake <cbrake@accelent.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License.
 *
 * History:
 *
 * 2001-02-06: Cliff Brake         Initial code
 *
 * 2001-08-29:	Nicolas Pitre	Simplified.
 *
 * 2002-05-27:	Nicolas Pitre	Revisited, more cleanup and simplification.
 *				Storage is on the stack now.
 */

#include <linux/linkage.h>
#include <asm/assembler.h>

/*
 * cxd4115_cpu_suspend()
 *
 * Causes cxd4115 to enter sleep state
 *
 */
ENTRY(cxd4115_cpu_suspend)
	stmfd	sp!, {r4 - r12, lr}		@ save registers on stack

	@ get coprocessor registers
	mrc 	p15, 0, r2, c1, c0, 0		@ control reg
	mrc 	p15, 0, r3, c1, c0, 1		@ aux control reg

	@ store them plus current virtual stack ptr on stack
	mov	r4, sp
	stmfd	sp!, {r2 - r4}

	@ get coprocessor registers
	mrc 	p15, 0, r2, c3, c0, 0		@ domain ID
	mrc 	p15, 0, r3, c2, c0, 2		@ TTB control register
	mrc 	p15, 0, r4, c2, c0, 0		@ TTB0
	mrc 	p15, 0, r5, c2, c0, 1		@ TTB1
	mrc	p15, 0, r6, c13, c0, 0		@ PID
	mrc	p15, 0, r7, c13, c0, 1		@ context ID
	mrc	p15, 0, r8, c13, c0, 3		@ TLS
	mrc 	p15, 0, r9, c10, c2, 0		@ PRRR
	mrc 	p15, 0, r10, c10, c2, 1		@ NMRR

	@ store them on stack
	stmfd	sp!, {r2 - r10}

	@ preserve phys address of stack
	mov	r0, sp
	bl	sleep_phys_sp
	ldr	r1, =sleep_save_sp
	str	r0, [r1]

	@ disable interrupt
	mov	r0, #PSR_F_BIT | PSR_I_BIT | SVC_MODE
	msr	cpsr_c, r0

	@ clean & invalide D&I cache
	bl	v6_flush_kern_cache_all
	mrc	p15, 0, r0, c1, c0, 0		@ ctrl register
	bic	r0, r0, #0x1000			@ ...i............
	bic	r0, r0, #0x0006			@ .............ca.
	mcr	p15, 0, r0, c1, c0, 0		@ disable caches
#ifdef CONFIG_CACHE_PL310
	bl	cxd4132_pl310_shutdown		@ r0-r2 corrupted
#endif /* CONFIG_CACHE_PL310 */


#ifdef CONFIG_DEBUG_LL
	mov	r0, #'E'
	bl	printch
	mov	r0, #'\r'
	bl	printch
	mov	r0, #'\n'
	bl	printch
#endif /* CONFIG_DEBUG_LL */

	@ wait for interrupt
	mov	r0, #0
1:
	wfi
	b	1b
	.ltorg
