/*
 * arch/arm/plat-cxd41xx/sleep.S
 *
 * pm callback for cxd41xx
 *
 * Copyright 2007,2008,2009,2010 Sony Corporation
 *
 * This code is based on arch/arm/mach-sa1100/sleep.S
 */
/*
 * SA11x0 Assembler Sleep/WakeUp Management Routines
 *
 * Copyright (c) 2001 Cliff Brake <cbrake@accelent.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License.
 *
 * History:
 *
 * 2001-02-06: Cliff Brake         Initial code
 *
 * 2001-08-29:	Nicolas Pitre	Simplified.
 *
 * 2002-05-27:	Nicolas Pitre	Revisited, more cleanup and simplification.
 *				Storage is on the stack now.
 */

#include <linux/linkage.h>
#include <asm/assembler.h>
#include <mach/debug-macro.S>
#include <mach/pm.h>
#include <mach/pm_macro.S>
#include <mach/time.h>

#include <mach/hardware.h>
#include <mach/regs-gpio.h>

		.data
/*
 * cxd4115_cpu_suspend()
 *
 * Causes cxd4115 to enter sleep state
 *
 */

ENTRY(cxd4115_cpu_suspend)
	@ r0: mode flag
	stmfd	sp!, {r4 - r12, lr}		@ save registers on stack
	str	r0, modeflag			@ save mode flag

	@ get coprocessor registers
	mrc 	p15, 0, r2, c1, c0, 0		@ control reg
	mrc 	p15, 0, r3, c1, c0, 1		@ aux control reg

	@ store them plus current virtual stack ptr on stack
	mov	r4, sp
	stmfd	sp!, {r2 - r4}

	@ get coprocessor registers
	mrc 	p15, 0, r2, c3, c0, 0		@ domain ID
	mrc 	p15, 0, r3, c2, c0, 2		@ TTB control register
	mrc 	p15, 0, r4, c2, c0, 0		@ TTB0
	mrc 	p15, 0, r5, c2, c0, 1		@ TTB1
	mrc	p15, 0, r6, c13, c0, 0		@ PID
	mrc	p15, 0, r7, c13, c0, 1		@ context ID
	mrc	p15, 0, r8, c13, c0, 3		@ TLS
	mrc 	p15, 0, r9, c10, c2, 0		@ PRRR
	mrc 	p15, 0, r10, c10, c2, 1		@ NMRR

	@ store them on stack
	stmfd	sp!, {r2 - r10}

	@ preserve phys address of stack
	mov	r0, sp
	bl	sleep_phys_sp
	ldr	r1, =sleep_save_sp
	str	r0, [r1]

	@ disable interrupt
	mov	r0, #PSR_F_BIT | PSR_I_BIT | SVC_MODE
	msr	cpsr_c, r0

	@ clean & invalide D&I cache
	bl	v6_flush_kern_cache_all

	ldr	r0, modeflag
	cmp	r0, #PM_ENTER_NORMAL
	bne	goto_suspend

	@ for creating Warm-Boot-Image
	bl	cxd4115_create_warmbootimage
	cmp	r0, #0
	beq	goto_suspend
	/*
		failure handling for warm_boot_image
	*/
	mov	r0, sp
	ldmfd	r0!, {r2 - r10}
	ldmfd	r0!, {r2 - r3, sp}
	ldmfd	sp!, {r4 - r12, pc}

goto_suspend:
	mov	r0, #1
	ldr	r1, =cxd41xx_mem_alive
	str	r0, [r1]			@ save memory alive flag
#ifdef CONFIG_SNSC_DEBUG_PROFILE
	adr	r0, __printascii
	bl	profile_dump_record
#endif
	@ copy the standby operation code to the resume ram
	pm_workarea_va r0
	ldr	r1, =exec_in_resume_ram
	ldr	r2, =exec_in_resume_ram_end

code_copy:
	ldr	r3, [r1], #4
	str	r3, [r0], #4
	teq	r1, r2
	bne	code_copy

	@ clean & invalide D&I cache
	bl	v6_flush_kern_cache_all
	mrc	p15, 0, r0, c1, c0, 0		@ ctrl register
	bic	r0, r0, #0x1000			@ ...i............
	bic	r0, r0, #0x0006			@ .............ca.
	mcr	p15, 0, r0, c1, c0, 0		@ disable caches
	pm_outer_cache

	pm_stack_phys sp			@ Physical address
	pm_workarea_phys r1			@ Physical address
 	@ disable MMU, the mapping is flat so
 	mrc	p15, 0, r0, c1, c0, 0		@ ctrl register
 	bic	r0, r0, #0x0001			@ ...............m
 	mcr	p15, 0, r0, c1, c0, 0		@ disable MMU
	@ jump to the resume ram(Physical address)
	mov	pc, r1				@ fetch: MMU active
	nop					@ fetch: MMU active
	nop

	.ltorg
	.align	5
exec_in_resume_ram:
	@ Invert STABLE_RUN for DEBUG
	ldr	r0, cxd41xx_stable_run_addr
	cmp	r0, #0xffffffff		@ not defined
	beq	1f
	ldr	r1, cxd41xx_stable_run_bitmask
	ldr	r2, [r0, #GPIO_DATA_READ]
	ands	r2, r2, r1
	streq	r1, [r0, #GPIO_DATA_SET]
	strne	r1, [r0, #GPIO_DATA_CLEAR]
1:

	pm_hook r0
/*--------------------- Memory power down ----------------------*/
	pm_memory r4
/*--------------------------------------------------------------*/

	ldr r0, cxd41xx_sus_end_msg
	cmp r0, #0
	beq 1f

	mov	r0, #'E'
	bl	__printch
	mov	r0, #'\r'
	bl	__printch
	mov	r0, #'\n'
	bl	__printch
1:

/*--------------------- D-D Conv. control ----------------------*/
	ldr	r0, reboot
	cmp	r0, #1
	beq	dd_reboot

dd_off:
	pm_hook2 r4

	ldr	r0, cxd41xx_xpower_off_addr
	cmp	r0, #0xffffffff		@ not defined
	beq	1f
	ldr	r1, cxd41xx_xpower_off_bitmask
	str	r1, [r0, #GPIO_DATA_CLEAR]	@ assert
1:
	b	end_job
dd_reboot:
	pm_reset r0 r4
end_job:
	@ wait for interrupt
	mov	r0, #0
1:
	mcr	p15, 0, r0, c7, c0, 4
	b	1b
/*--------------------------------------------------------------*/

__udelay:
	@ destruct r0-r2
	mach_usecs_to_cycles r0
	mach_freerun r2
	mach_read_cycles r1, r2			@ now
	add	r1, r1, r0			@ expire time
1:
	mach_read_cycles r0, r2			@ now
	cmp	r0, r1				@ now - expire
	bmi	1b
	mov	pc, lr

__memcpy:
	@ destruct r0-r3
	cmp	r2, #0
	moveq	pc, lr
1:
	ldrb	r3, [r1], #1
	strb	r3, [r0], #1
	subs	r2, r2, #1
	bne	1b
	mov	pc, lr

__memset:
	@ destruct r0-r2
	cmp	r2, #0
	moveq	pc, lr
1:
	strb	r1, [r0], #1
	subs	r2, r2, #1
	bne	1b
	mov	pc, lr

__memdump:
	stmdb   sp!, {r9-r11, lr}
	cmp	r1, #0
	beq	9f
	mov	r10, r0
	mov	r11, r1
	mov	r0, #'\n'
	bl	__printch
	mov	r9, #0
2:
	ldrb	r0, [r10, r9]
	bl	__printhex2
	mov	r0, #' '
	bl	__printch
	add	r9, r9, #1
	ands	r0, r9, #0xf
	bne	1f
	mov	r0, #'\n'
	bl	__printch
1:
	subs	r11, r11, #1
	bne	2b
	mov	r0, #'\n'
	bl	__printch
9:
	ldmia   sp!, {r9-r11, pc}


	pm_functions

modeflag:	.long	0x00
reboot:		.long	0x00
		.globl	cxd41xx_ddr_ctrl
cxd41xx_ddr_ctrl:	.long	0
#ifdef CONFIG_ARCH_CXD4120
		.globl	cxd41xx_ddr_ctrl2
cxd41xx_ddr_ctrl2:	.long	0
#endif /* CONFIG_ARCH_CXD4120 */
		.globl	cxd41xx_ddr_pasr0
cxd41xx_ddr_pasr0:	.long	0
		.globl	cxd41xx_ddr_pasr1
cxd41xx_ddr_pasr1:	.long	0
		.globl	cxd41xx_xpower_off_addr
cxd41xx_xpower_off_addr:
		.long	0xffffffff
		.globl	cxd41xx_xpower_off_bitmask
cxd41xx_xpower_off_bitmask:
		.long	0xffffffff
		.globl	cxd41xx_stable_run_addr
cxd41xx_stable_run_addr:
		.long	0xffffffff
		.globl	cxd41xx_stable_run_bitmask
cxd41xx_stable_run_bitmask:
		.long	0xffffffff
		.globl	cxd41xx_sus_end_msg
cxd41xx_sus_end_msg:
		.long	0

		.align	4
	@ for debug
__printhex8:
	mov	r1, #8
	b	__printhex

__printhex4:
	mov	r1, #4
	b	__printhex

__printhex2:
	mov	r1, #2
__printhex:
	adr	r2, hexbuf
	add	r3, r2, r1
	mov	r1, #0
	strb	r1, [r3]
1:	and	r1, r0, #15
	mov	r0, r0, lsr #4
	cmp	r1, #10
	addlt	r1, r1, #'0'
	addge	r1, r1, #'a' - 10
	strb	r1, [r3, #-1]!
	teq	r3, r2
	bne	1b
	mov	r0, r2
	b	__printascii

__printascii:
	addruart r3
	b	2f
1:	waituart r2, r3
	senduart r1, r3
	busyuart r2, r3
	teq	r1, #'\n'
	moveq	r1, #'\r'
	beq	1b
2:	teq	r0, #0
	ldrneb	r1, [r0], #1
	teqne	r1, #0
	bne	1b
	mov	pc, lr

__printch:
	addruart r3
	mov	r1, r0
	mov	r0, #0
	b	1b

hexbuf:		.space 16
	.ltorg
exec_in_resume_ram_end:
