/*
 * drivers/base/decompress.c
 *
 * decompress driver
 *
 * Copyright 2008 Sony Corporation
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 */

#include <linux/module.h>
#include <linux/decompress_driver.h>

static LIST_HEAD(drivers);

u32 call_decompress_driver(u32 flags, void *dst, u32 dstlen, void *src, u32 srclen)
{
	struct decompress_driver *p;

	list_for_each_entry(p, &drivers, list) {
		if ((flags & p->flag) && p->decompress) {
			int fill = p->decompress(dst, dstlen, src, srclen, p->data);
			if (unlikely(fill < 0)) {
				printk("Failed: decompress, dst:0x%p, dstlen:%d, src:0x%p, srclen:%d\n",
					dst, dstlen, src, srclen);
				fill = 0;
			}
			return fill;
		}
	}

	return 0;
}

int decompress_driver_register(struct decompress_driver *p)
{
	list_add(&p->list, &drivers);

	return 0;
}

void decompress_driver_unregister(struct decompress_driver *p)
{
	list_del(&p->list);
}

EXPORT_SYMBOL(call_decompress_driver);
EXPORT_SYMBOL(decompress_driver_register);
EXPORT_SYMBOL(decompress_driver_unregister);
