/*--------------------------------------------------------------------------*
 * drivers/usb/scd/scd.h
 * 
 * SCD internal interface
 * 
 * Copyright 2011 Sony Corporation
 *
 * This file is part of the HS-OTG Controller Driver SCD.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 *---------------------------------------------------------------------------*/

#ifndef _SCD_H_
#define _SCD_H_

/*
 *	󥿡ե
 */

/* scd_usec */

typedef signed long scd_usec;

/* scd_timer_callback - controllerѰդ륿ॢȥХå */

struct scd_timer;
typedef int (*scd_timer_callback)(struct scd_timer *timer);

/* scd_control_type */

enum scd_control_type {
	SCD_CONTROL_TYPE_GADGET = 0,	/* Enumͤscd_device.cˤƸ */
	SCD_CONTROL_TYPE_HCD,		/* ǼƤ뤿ᡢ */
	SCD_CONTROL_TYPE_INFO,		/* Ʊscd_device.cintfs */
	SCD_CONTROL_TYPE_SYSDEP,	/* 󥤥ǥåͤѹ뤳 */

	SCD_CONTROL_TYPE_DEVICE, /* special type */
};


/* scd_message_type */

enum scd_message_type {
	SCD_MESSAGE_TYPE_UNKNOWN = 0,
	SCD_MESSAGE_TYPE_ERROR,
	SCD_MESSAGE_TYPE_WARN,
	SCD_MESSAGE_TYPE_INFO,
	SCD_MESSAGE_TYPE_SETTING,
	SCD_MESSAGE_TYPE_STRING,
};

/* scd_timer */

struct scd_timer {
	scd_usec expire;
	scd_timer_callback cb;
	void *id;
};

/* scd_device_ops */

struct scd_device;
struct scd_device_ops {
	struct scd_timer *(*add_timer)(
		struct scd_device *device, scd_usec timeout, /* relative time */
		scd_timer_callback cb, void *id);
	int (*cancel_timer)(struct scd_device *device, struct scd_timer *timer);
	scd_usec (*get_time)(struct scd_device *device);
	int (*connect)(struct scd_device *device, bool onoff, int port);
	int (*suspend_bus)(struct scd_device *device, bool onoff, int port);
	int (*power)(struct scd_device *device, bool onoff, int port);
	int (*vbuserror)(struct scd_device *device, int port);
	int (*overcurrent)(struct scd_device *device, int port);
	int (*reset_link)(struct scd_device *device, bool pullup);
	int (*timeout)(struct scd_device *device);
	int (*message)(struct scd_device *device,
		       enum scd_control_type recipient,
		       enum scd_message_type type, const char *fmt, ...);
	int (*vmessage)(struct scd_device *device,
			enum scd_control_type recipient,
			enum scd_message_type type, const char *fmt, va_list ap);
};

/* scd_device  */

#define SCD_DEVICE_ATTR_HS_ENABLE		0 /* HSEnable */
#define SCD_DEVICE_ATTR_VBUSERRORED		8 /* VbusError */
#define SCD_DEVICE_ATTR_SUSPENDED		9 /* Suspended */
#define SCD_DEVICE_ATTR_IRQ_REQUESTED		12 /* already call request_irq */

struct scd_device {
	struct device *dev;
	int mode;
	void *regs;
	spinlock_t lock;
	unsigned long attributes;
	struct scd_device_ops *ops;
};

/* scd_control */

struct scd_intf;
struct scd_control {
	struct scd_intf *intf;
};

/* scd_intf */

struct scd_intf {
	enum scd_control_type type;
	struct scd_control *(*probe)(struct scd_device *device);
	int (*remove)(struct scd_control *control);
	void (*shutdown)(struct scd_control *control);
	int (*suspend)(struct scd_control *control, pm_message_t state);
	int (*suspend_late)(struct scd_control *control, pm_message_t state);
	int (*resume)(struct scd_control *control);
	int (*resume_early)(struct scd_control *control);
	int (*start)(struct scd_control *control);
	int (*stop)(struct scd_control *control);

	int (*vmessage)(struct scd_control *control, enum scd_message_type type,
			const char *fmt, va_list ap);

	irqreturn_t (*irq)(struct scd_control *control);
	void (*detect_vbus)(struct scd_control *control, bool onoff);
	void (*overcurrent)(struct scd_control *control, int port);
};

/* scd_sysdep_intf */

struct scd_sysdep_intf {
	struct scd_intf base_intf;
	/* 2nd argument 'expire' of set_timeout means absolute time */
	int (*set_timeout)(struct scd_control *control, scd_usec expire);
	int (*cancel_timeout)(struct scd_control *control);
	scd_usec (*get_time)(struct scd_control *control);
	int (*power)(struct scd_control *control, bool onoff, int port);
};

/*
 *	󥿡եؿ
 */

extern int usb_scd_device_set_intf(struct scd_intf *intf);
extern int usb_scd_device_unset_intf(struct scd_intf *intf);
extern int __init usb_scd_sysdep_init(void);
extern void __exit usb_scd_sysdep_exit(void);
extern int __init usb_scd_gadget_init(void);
extern void __exit usb_scd_gadget_exit(void);
extern int __init usb_scd_hcd_init(void);
extern void __exit usb_scd_hcd_exit(void);
extern int __init usb_scd_info_init(void);
extern void __exit usb_scd_info_exit(void);

static inline struct scd_timer *scd_device_add_timer(
	struct scd_device *device, scd_usec timeout,
	scd_timer_callback cb, void *id)
{
	return device->ops->add_timer(device, timeout, cb, id);
}

static inline int scd_device_cancel_timer(
	struct scd_device * device, struct scd_timer *timer)
{
	return device->ops->cancel_timer(device, timer);
}

static inline scd_usec scd_device_get_time(struct scd_device *device)
{
	return device->ops->get_time(device);
}

static inline int scd_device_connect(
	struct scd_device *device, bool onoff, int port)
{
	return device->ops->connect(device, onoff, port);
}

static inline int scd_device_suspend_bus(
	struct scd_device *device, bool onoff, int port)
{
	return device->ops->suspend_bus(device, onoff, port);
}

static inline int scd_device_power(
	struct scd_device *device, bool onoff, int port)
{
	return device->ops->power(device, onoff, port);
}

static inline int scd_device_vbuserror(struct scd_device *device, int port)
{
	return device->ops->vbuserror(device, port);
}

static inline int scd_device_overcurrent(struct scd_device * device, int port)
{
	return device->ops->overcurrent(device, port);
}

static inline int scd_device_reset_link(struct scd_device * device, bool pullup)
{
	return device->ops->reset_link(device, pullup);
}

static inline scd_usec scd_add_usec(scd_usec t1, scd_usec t2)
{
	return t1 + t2;
}

static inline scd_usec scd_dec_usec(scd_usec t1, scd_usec t2)
{
	return t1 - t2;
}

static inline int scd_cmp_usec(scd_usec t1, scd_usec t2)
{
	return t1 - t2;
}

#define scd_error(DEV, FMT, ARGS...) \
	(DEV)->ops->message((DEV), SCD_CONTROL_TYPE_INFO, \
			    SCD_MESSAGE_TYPE_ERROR, (FMT), ## ARGS)

#define scd_warn(DEV, FMT, ARGS...) \
	(DEV)->ops->message((DEV), SCD_CONTROL_TYPE_INFO, \
			     SCD_MESSAGE_TYPE_WARN, (FMT), ## ARGS)

#define scd_info(DEV, FMT, ARGS...) \
	(DEV)->ops->message((DEV), SCD_CONTROL_TYPE_INFO, \
			    SCD_MESSAGE_TYPE_INFO, (FMT), ## ARGS)

#define scd_print(DEV, FMT, ARGS...) \
	(DEV)->ops->message((DEV), SCD_CONTROL_TYPE_INFO, \
			    SCD_MESSAGE_TYPE_STRING, (FMT), ## ARGS)

#endif  /* _SCD_H_ */
