/*--------------------------------------------------------------------------*
 * drivers/usb/scd/scd_debug.c
 * 
 * Debug functions
 * 
 * Copyright 2011 Sony Corporation
 *
 * This file is part of the HS-OTG Controller Driver SCD.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *---------------------------------------------------------------------------*/
#include <linux/kernel.h>
#include <linux/list.h>
#include <linux/timer.h>
#include <linux/spinlock.h>
#include <linux/errno.h>
#include <linux/interrupt.h>
#include <linux/init.h>
#include <linux/usb.h>
#include <linux/usb/ch9.h>
#include <linux/usb/gadget.h>
#include <linux/usb/scd/usb_otg_control.h>
#include <linux/wait.h>

#include "scd.h"
#include "scd_version.h"
#include "scd_register.h"
#include "scd_gadget.h"
#include "scd_debug.h"

int scd_debug_level = SCD_DEBUG;
EXPORT_SYMBOL(scd_debug_level);


/*
 * dump memory for general
 *
 * str_adr : start address
 * size    : size
 * mode    : 1->byte  2->half word  3->word
 */
int scd_gl_mem_dump(struct scd_device *sdev, u32 str_adr, u32 size, u8 mode)
{
        unsigned long top, col, end_adr, i;
        unsigned char cpod;
        unsigned char cnt_space[] = {1, 2, 4}; /* 2 Τ٤ɽ */

        const char *prt_scale[3] = {
                "\n 0x%08X +0 +1 +2 +3 +4 +5 +6 +7 +8 +9 +A +B +C +D +E +F   0123456789ABCDEF\n",
                "\n 0x%08X   +0   +2   +4   +6   +8   +A   +C   +E   \n",
                "\n 0x%08X       +0       +4       +8       +C       \n"
            };

        if(2 < mode) {
                return -EINVAL;
        }

        end_adr = str_adr + size - 1;

        for (top = str_adr; top <= end_adr; top += 16) {
                /* 256ХOffset()ɽ */
                if (!((top - str_adr) & 0xff)) {
                        printk(prt_scale[mode], top);
                }
                
                /* Offsetɽ() */
                printk("+0x%08X ", (int)(top - str_adr));
                
                /* ǽ饤λϺǽɥ쥹ޤǤɽ */
                for (col = 0; (col < 16) && (top + col <= end_adr);
                     col += (0x01 << mode)) {
                        switch(mode) {
                            case 0:
                                printk("%02X ",
                                          (int)*(unsigned char *)
                                          ((top + col)));
                                break;
                            case 1:
                                printk("%04X ",
                                          (int)*(unsigned short *)
                                          ((top + col)));
                                break;
                            default: /* 2 */
                                printk("%08X ",
                                          (int)*(unsigned long *)
                                          ((top + col)));
                                break;
                        }
                }
                
                /* 16 byte ܿǤʤκǽζɽ */
                if (col != 16) {
                        for (; col < 16; col += (0x01 << mode)) {
                                for (i = 0; i < cnt_space[mode]; i++) {
                                        printk("  ");
                                }
                                /* ɽζڤ */
                                scd_print(sdev, " ");
                        }
                }
                
                /* byte dump ξϡʸɽ */
                if(mode == 0) {
                        scd_print(sdev, "  ");
                        for (col = 0; (col < 16) && (top + col <= end_adr);
                             col++) {
                                cpod = *(unsigned char *)(top + col);
                                if ((cpod >= 0x20) && (cpod <= 0x7e)) {
                                        scd_print(sdev, "%c", cpod);
                                } else {
                                        scd_print(sdev, ".");
                                }
                        }
                }
                
                scd_print(sdev, "\n");
        } /* for () */

        return 0;
}
