/*-----------------------------------------------------------------------------
 * drivers/usb/scd/scd_gadget.h
 * 
 * Gadget common definitions
 * 
 * Copyright 2011 Sony Corporation
 *
 * This file is part of the HS-OTG Controller Driver SCD.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *---------------------------------------------------------------------------*/

#ifndef _USB_SCD_GADGET_H_
#define _USB_SCD_GADGET_H_

/*---------------------------------------------------------------------------*/
/* for DEBUG */
#define SCD_CONFIG_MAP	/* config map */
/* ɽ */
#define DISPLAY_REQ	/* DEBUG function to check a request */
#define WARN_RESET
#define CHK_SUPCNT	/* checking supcnt overwrite for debug */

/* option or new functions */
#define SCD_POLL_ON		/* polling mode on Flush&Stall operations */
#define RW_LOCK_ON  	
#define EP_SKIP_FLUSH2
//#define SCD_DEV_REG		/* sysfsѤˤͭˤ롣 */
/*---------------------------------------------------------------------------*/

/*==================================================
 *	 spin_lock
 ==================================================*/
#ifdef PR_SPIN_LOCK
#define SCD_G_SPIN_LOCK_IRQ(lock, flag)	\
	{   DBG(1,"spin_lock irq\n");\
		spin_lock_irqsave(lock, flag);		}
#else
#define SCD_G_SPIN_LOCK_IRQ(lock, flag)	\
	{   spin_lock_irqsave(lock, flag);		}
#endif

#ifdef PR_SPIN_LOCK
#define SCD_G_SPIN_UNLOCK_IRQ(lock, flag)	\
	{   DBG(1,"spin_unlock irq\n");\
		spin_unlock_irqrestore(lock, flag);		}
#else
#define SCD_G_SPIN_UNLOCK_IRQ(lock, flag)	\
	{   spin_unlock_irqrestore(lock, flag);		}
#endif

#ifdef PR_SPIN_LOCK
#define SCD_G_SPIN_LOCK(lock)	\
	{   DBG(1,"spin_lock\n");\
		spin_lock(lock);		}
#else
#define SCD_G_SPIN_LOCK(lock)	\
	{   spin_lock(lock);		}
#endif

#ifdef PR_SPIN_LOCK
#define SCD_G_SPIN_UNLOCK(lock)	\
	{   DBG(1,"spin_unlock\n");\
		spin_unlock(lock);		}
#else
#define SCD_G_SPIN_UNLOCK(lock)	\
	{   spin_unlock(lock);		}
#endif

/*==================================================
 *	 rwlock
 ==================================================*/
#ifdef PR_RW_LOCK
#define SCD_G_READ_LOCK(lock)	\
	{   DBG(1,"read_lock\n");\
		read_lock(lock);		}
#else
#define SCD_G_READ_LOCK(lock)	\
	{   read_lock(lock);		}
#endif

#ifdef PR_RW_LOCK
#define SCD_G_READ_UNLOCK(lock)	\
	{   DBG(1,"read_unlock irq\n");\
		read_unlock(lock);		}
#else
#define SCD_G_READ_UNLOCK(lock)	\
	{   read_unlock(lock);		}
#endif

#ifdef PR_RW_LOCK
#define SCD_G_READ_LOCK_IRQ(lock, flag)	\
	{   DBG(1,"read_lock irq \n");\
		read_lock_irqsave(lock, flag);		}
#else
#define SCD_G_READ_LOCK_IRQ(lock, flag)	\
	{   read_lock_irqsave(lock, flag);		}
#endif

#ifdef PR_RW_LOCK
#define SCD_G_READ_UNLOCK_IRQ(lock, flag)	\
	{   DBG(1,"read_unlock irq\n");\
		read_unlock_irqrestore(lock, flag);		}
#else
#define SCD_G_READ_UNLOCK_IRQ(lock, flag)	\
	{   read_unlock_irqrestore(lock, flag);		}
#endif

#ifdef PR_RW_LOCK
#define SCD_G_WRITE_LOCK(lock)	\
	{   DBG(1,"write_lock \n");\
		write_lock(lock);		}
#else
#define SCD_G_WRITE_LOCK(lock)	\
	{   write_lock(lock);		}
#endif

#ifdef PR_RW_LOCK
#define SCD_G_WRITE_UNLOCK(lock)	\
	{   DBG(1,"write_unlock\n");\
		write_unlock(lock);		}
#else
#define SCD_G_WRITE_UNLOCK(lock)	\
	{   write_unlock(lock);		}
#endif

#ifdef PR_RW_LOCK
#define SCD_G_WRITE_LOCK_IRQ(lock, flag)	\
	{   DBG(1,"write_lock irq\n");\
		write_lock_irqsave(lock, flag);		}
#else
#define SCD_G_WRITE_LOCK_IRQ(lock, flag)	\
	{   write_lock_irqsave(lock, flag);		}
#endif

#ifdef PR_RW_LOCK
#define SCD_G_WRITE_UNLOCK_IRQ(lock, flag)	\
	{   DBG(1,"write_unlock irq\n");\
		write_unlock_irqrestore(lock, flag);		}
#else
#define SCD_G_WRITE_UNLOCK_IRQ(lock, flag)	\
	{   write_unlock_irqrestore(lock, flag);		}

#endif
#define SCD_G_EP0				0	/* EP0 for Control */
#define SCD_G_EP1				1	/* EP1  */
#define SCD_G_EP2				2	/* EP2  */
#define SCD_G_EP3				3	/* EP3  */
#define SCD_G_EP4				4	/* EP4  */

#define	SCD_NUM_EPS				4	/* ep0 not included */
#define SCD_NUM_EP0				1	/* the number of ctrol transfer */
#define SCD_NUM_ALLEPS			(SCD_NUM_EPS+SCD_NUM_EP0)	/* include ep0 */

#define SCD_EP0_MAXPSZ			64  /* maxp size for EP0 */
#define SCD_MAX_EP0_PACKET		1	/* number of maxp for EP0 */
#define SCD_LAST_PACKET			1	/* last packet size except EP0 */
#define SCD_EP_NAMESZ			6	
#define SCD_GADGET_NAMESZ		20	/* a name of usb_gadget */
#define SCD_GRX_MAXFIFOSZ		1024  /* max fifo size for rx */
#define SCD_GRX_MAXBUFNUM		2   /* max buf number for rx */
#define SCD_SKIP_EP0			1	/* FIFOޥå ep0 */

#define SCD_MAXP_HS_SZ			64	/* maxp size for HS */
#define SCD_MAXP_FS_SZ			64  /* maxp size for FS */
#define SCD_FIFO0_NUMBUF		2	/* the number of buf for EP0*/
#define SCD_FIFO_NUMBUF			2	/* the number of buf except EP0 */
#define SCD_SETUP_NUM			3	/* the number of Setup  */ 
#define SCD_GREQ_SZ				8	/* the number of request  */ 
#define SCD_SUPCNT_VAL			3	/* the value of supcnt */
#define SCD_GREQ_1				0	/* index value to refer the 1st setup */
#define SCD_GREQ_1_PP			8	/* indicate the setup packet at the 1st*/
#define SCD_GREQ_SDONE			12	/* indicate the setup stage done  */
#define SCD_GREQ_2				16	/* index value to refer the 2nd setup */
#define SCD_GREQ_2_PP			24	/* indicate the setup packet at the 2nd*/
#define SCD_GREQ_3				28	/* index value to refer the 3nd setup */
#define SCD_GREQ_3_PP			36	/* indicate the setup packet at the 3rd*/

#define SCD_GREQ_IND1			0	/* index value to refer the 1st setup */
#define SCD_GREQ_IND2			8	/* index value to refer the 2nd setup */
#define SCD_GREQ_IND3			16	/* index value to refer the 3rd setup */

/* buffer for DMA */
#define SCD_DMA_EP0_SZ			24	/* 3xsetup */

/* For Interrupt */
#define SCD_GCLR_GINTSTS	~(SCD_B_GINTSTS_SESSREQINT|SCD_B_GINTSTS_CONIDSTSCHNG)

/* scd_gadget bTransMode */
#define SCD_GADGET_TRANS_DMA	0
#define SCD_GADGET_TRANS_CPU	1

/* FIFO FLUSH */
#define SCD_GRSTCTL_FFLUSH_NUM_ALL	0x10
#define SCD_FLUSH_HS_UDELAY			25
#define SCD_FLUSH_FS_UDELAY			25
#define SCD_FLUSH_HS_CNT			20
#define SCD_FLUSH_FS_CNT			20
#define SCD_FLUSH_HS_LOOP_OUT		(SCD_FLUSH_HS_UDELAY*SCD_FLUSH_HS_CNT)
#define SCD_FLUSH_FS_LOOP_OUT		(SCD_FLUSH_FS_UDELAY*SCD_FLUSH_FS_CNT)

#ifdef SCD_POLL_ON
/* STALL&FLUSH */
#define SCD_POLL_WAIT_GOUTNAKEFF	800	/* worst FS 800um */
#define SCD_POLL_WAIT_EPDIS			100
#define SCD_POLL_WAIT_INNAKEFF		800
#define SCD_POLL_UDELAY				1
#define SCD_POLL_CNT				20
#define SCD_POLL_SKIP				1
#endif

/* Resuest field */
#define SCD_G_REQTYPE_DIR		0x80

/* request decode return value */
#define SCD_GREQ_THRU			1
#define SCD_GREQ_SKIP			2

#define SCD_DEVST_SKIP			1

/* OUT transfer size check status */
#define SCD_G_TRNS_END		0
#define SCD_G_TRNS_CONT		1
#define SCD_G_TRNS_SHORT	2
#define SCD_G_TRNS_ZERO		3
#define SCD_G_TRNS_OVER		4
#define SCD_G_TRNS_ERR		5

/* BusState */
enum G_BUS_STATE
{
	SCD_BUS_IDLE=0,
	SCD_BUS_RESET,
	SCD_BUS_ENUMDONE,
	SCD_BUS_SUSPEND,
	SCD_BUS_RESUME
};

/* State Control(EP0) */
#define	SCD_EP0_STAGE_INIT			0
#define	SCD_EP0_STAGE_SETUP			1
#define	SCD_EP0_STAGE_DATA_IN		2
#define	SCD_EP0_STAGE_DATA_OUT		3
#define	SCD_EP0_STAGE_STATUS_IN		4
#define	SCD_EP0_STAGE_STATUS_OUT	5

#define SCD_GST_INIT				0x0
#define SCD_GST_PULLUPON			0x1
#define SCD_GST_DFLT				0x2
#define SCD_GST_ADDR				0x3
#define SCD_GST_CNFG				0x4
#define SCD_GST_PULLUPON_SUS		0xf1
#define SCD_GST_DFLT_SUS			0xf2
#define SCD_GST_ADDR_SUS			0xf3
#define SCD_GST_CNFG_SUS			0xf4
#define SCD_GST_PULLUPOFF_EN		0x10
#define SCD_GST_PULLUPON_EN			0x11
#define SCD_GST_RESET_EN			0x12
#define SCD_GST_SUS_EN				0xf0
#define SCD_GST_RESUME_EN			~((u8)0xf0)

/* EP state */
#define SCD_EP_IDLE				0

/* EP state  */
#define SCD_EP_GET_SETUP		0x01
#define SCD_EP_SETUP_DONE		0x11
#define SCD_EP_INTR_SETUP		0x21
#define SCD_EP_QUE_WAIT			0x02
#define SCD_EP_OUT_READY		0x03
#define SCD_EP_IN_READY			0x04
#define SCD_EP_DATA_CONT		0x05
#define SCD_EP_DATA_END			0x06
#define SCD_EP_DATA_COMP		0x16
#define SCD_EP_ZERO_END			0x07
#define SCD_EP_FLUSH			0x08
#define SCD_EP_CANCEL			0x09
#define SCD_EP_STALL			0x0a
#define SCD_EP_DATA_ERR			0x0e

/* EP SubState */
#define SCD_EPSUB_IDLE			0
#define SCD_EPSUB_WAIT_NAKEFF	1
#define SCD_EPSUB_NAK			2
#define SCD_EPSUB_WAIT_DIS		3
#define SCD_EPSUB_DIS			4
#define SCD_EPSUB_WAIT_STALL	5
#define SCD_EPSUB_STALL			6

/* wq parameter */
#define SCD_GQ_IDLE				0
#define SCD_GQ_WAIT_NAKEFF		1
#define SCD_GQ_NAKEFF			2
#define SCD_GQ_WAIT_EPDIS		3
#define SCD_GQ_EPDIS			4
#define SCD_GQ_CONNRESET		5
#define SCD_GQ_SHUTDOWN			6

/* usb_gadget parameter */
#define SCD_GADGET_SPEEDDUAL		1
#define SCD_GADGET_OTG_SUPPORT		1
#define SCD_GADGET_A_HNP_SUPPORT	0

/* For a register setting */
#define SCD_GINTMSK_INIT			0x7f810007

/* DAINT */
#define SCD_G_DAINT_EP0				1

/* ơơ*/
#define SCD_G_PKT1					1	/* a packet count */
#define SCD_G_SZ0					0	/* a packet size */

/* gcoreȤξ */
#define EP_NUMBER(_ep)		(((struct scd_gep*)(_ep))->bEpnum )

/* usb_request */
#define SCD_G_REQ_OK				0

/* device address */
#define SCD_DFLT_DEVADDR			0
#define SCD_ADDR_LIMIT				0x80
/* supcnt */
#define SCD_G_SUPCNT_MODE0			0	/* read modifywrite */
#define SCD_G_SUPCNT_MODE1			1	/* clean set */

/* set_halt ops */
#define SCD_G_SET_STALL				1

/* data_out at intr_int */
#define SCD_G_DOSETUP	1

/* flush */
#define SCD_G_FLUSH_NORMAL	0
#define SCD_G_FLUSH_STSKIP	1

/* struct device bus_id */
#define SCD_GADGET_DEV_BUSID "gadget"

/* limit size */
#define SCD_MAX_TRNSF_SZ	SCD_M_DXEPTSIZN_XFERSIZE	
	/* GHWCFG3 XferSizeWidth=0x8 => bit19 */
	/* bit19 => 0x7FFFF => 524287 (byte) */
#define SCD_MAX_PKT_CNT		SCD_M_DXEPTSIZN_PKTCNT		
	/* bit10(fixed) => 1023(times) */

/*-------------------------------------------------------
--	init mask value for the gadget interrupt(GINTSTS)

	WkUpInt				used
	SessReqInt			not handled
	DisconnInt			not handled
	ConIDStsChng		not handled
	LPM_Int				non-used
	PTxFEmp				not handled
	HChInt				not handled
	PrtInt				not handled
	
	ResetDet			non-used
	FetSusp				non-used
	incompISOOUT		non-used
	incompISOIN			non-used
	OEPInt				used
	IEPInt				used
	EPMis				non-used
	Reserved			-
	
	EOPF				non-used
	ISOOutDrop			non-used
	EnumDone			used
	USBRst				used
	USBSusp				used
	ErlySusp			non-used
	I2CINT				not handled
	ULPICKINT			not handled

	GOUTNakEff			used
	GINNakEff			non-used
	NPTxFEmp			not handled
	RxFLvl				used
	Sof					non-used
	OTGInt				not handled
	ModeMis				not handled
	CurMod				not handled
-------------------------------------------------------*/

extern struct scd_gadget *scd_gadget_info;
extern struct scd_gadget_fmapinfo *scd_gadget_fminfo;
extern struct reqdone_queue	scd_ReqDoneQueue;
/*-------------------------------------------------------
	name	: struct scd_gadget_fmapinfo
	description: a info for FIFO mapping
	date	: 2010/03/25
-------------------------------------------------------*/
struct scd_gadget_fmapinfo {
	u32 all_dfifo_dep;			/* ȡ륵 */
	u32 all_rx_dfifo_dep;		/* EP0 */
	u32 *max_tx_dfifo_dep;		/* EPk ưŪ롣 */
	u32 req_dep;				/* ׵Ĺ */
	u32 chk_dep;				/* ȡ륵׻ */
	u32 chk_dep2;				/* ׵ȡ륵׻ */
	u32 num_ctrl_ep;			/* ControlžEP */
	u32 num_oep;				/* OUT-EP ׻ */
	u32 num_iep_max;			/* IN-EPEP0ޤ */
	u32 num_iep;				/* IN-EP ׻ */
	u32 rx_maxp_dep;			/* ǤκMAXP */
	u32 rxc;					/* rxͷ׻ѷ */	
};

/*-------------------------------------------------------
	name	: struct scd_fifo_each
	description: ep parameter 
	date	: 2010/03/25
-------------------------------------------------------*/
struct scd_fifo_each {
	u8	bEpAddr;
	u16	wSize;
	u8	wNumbuf;
};

/*-------------------------------------------------------
	name	: struct scd_gadget_fparam
	description: ep parameter 
	date	: 2010/03/25
-------------------------------------------------------*/
struct scd_gadget_fparam {
	struct scd_fifo_each *ep_param;
	u8	cnt;
};
/*-------------------------------------------------------
	name	: struct scd_gadget_fdepth
	description: fifo depth info 
	date	: 2010/03/25
-------------------------------------------------------*/
struct scd_gadget_fdepth {
	u32 rx_fifosz;
	u32 tx_fifosz[SCD_NUM_ALLEPS]; /* ߸ */
};
/*-------------------------------------------------------
	name	: struct scd_ep0_buf
	description: a data area for ep0
	date	: 2010/03/09
-------------------------------------------------------*/
struct scd_ep0_buf {
	u8	req[SCD_GREQ_SZ];
	u8	aEp0data[SCD_MAX_EP0_PACKET*SCD_EP0_MAXPSZ];
	u8	dcnt;	/* transfer count */
};
/*-------------------------------------------------------
	name	: struct scd_dmaep0_buf
	description: a data area for ep0 in a dma
	date	: 2010/04/12
-------------------------------------------------------*/
struct scd_dmaep0_buf {
	u8	*buf;		/* as a control transfer is one */
	u8	*dma_buf;	/* dreq mapping */
	u8	dcnt;	/* transfer count */
	spinlock_t lock;
};
/*-------------------------------------------------------
	name	: scd_gadget_wq
	description: event-flag in a substate
	date	: 2010/04/20
-------------------------------------------------------*/
struct scd_gadget_wq{
	wait_queue_head_t ep_stq;	/* used for transitions of substate */
	int	wq_flag;
};

/*-------------------------------------------------------
	name	: struct scd_lastbuf
	description: maxp饤ѼХåե
	date	: 2010/05/25
-------------------------------------------------------*/
struct scd_lastbuf {
	u8 *buf;
	u8 *dma;
	u16 size;
};
/*-------------------------------------------------------
	name	: struct scd_gep
	description: a info for gadget ep
	date	: 2010/03/09
-------------------------------------------------------*/
struct scd_gep {
	struct usb_ep end_point;
	spinlock_t Lock;
#ifdef RW_LOCK_ON
	rwlock_t	rwLock;
#endif
	struct scd_gadget_wq wq;	/* workqueue */
	struct list_head req_list;
#ifdef MUTEX_ON
	struct semaphore *gep_mtx;		/* mutex */
#endif
	struct scd_lastbuf lbuf;	/* last buffer */
	u8  bTransferType;	/* a attribute of transfer type */
	u16 wMaxp;			/* Max packet size */
	u32 dwLen;			/* setting length */
	u8  bIsTx;			/* tx direction */
	u8  bOverrun;		/* rx overrun */
	u8  bEpnum;			/* EP number */
	u8  bEpena;			/* EP enable status */
	u8  bState;			/* EP State */
	u8	bSubState;		/* for NAK or STALL */
};
/*-------------------------------------------------------
	name	: struct scd_bilimit
	description: the limit size info for bulk/interrupt
	date	: 2010/10/22
-------------------------------------------------------*/
struct scd_bilimit {
	u8	bWidthSize;		/* XferSizeWidth from ccParam */
	u32 maxLen;			/* Max number of length */
	u32 maxPktn;		/* Max number of packets */
};
/*-------------------------------------------------------
	name	: struct scd_gadget
	description: a info for gadget block
	date	: 2010/06/04
-------------------------------------------------------*/
struct scd_gadget {
	struct scd_control	g_control;				/* link to scd_intf */
	struct scd_device	*g_device;				/* link to scd_device */
	struct usb_gadget	*pGadget;				/* the gadget info in linux */
	struct usb_gadget_driver *pGadgetDriver;	/* the gadget driver */
	struct scd_gadget_fparam	fparam;			/* an Info for FIFO mapping */
	struct scd_gadget_fdepth	fdepth;			/* a depth info for FIFO mapping */
	spinlock_t	gLock;
#ifdef RW_LOCK_ON
	rwlock_t	rwLock;
#endif
#ifdef MUTEX_ON
	struct semaphore *g_mtx;		/* mutex */
#endif
	u8 bDevState;		/* protocol state */
	enum G_BUS_STATE bBusState;	/* bus state for interrupt */
	u8 bEp0Stage;		/* EP0 stage state */
	u8 bSuspended;		/* current suspend state */
	u8 bSetAddr;		/* SetAddress processing flag */
	u8 bDevAddr;		/* the device address set */
	u8 bFifoCnfg;		/* a FIFO configuration state */
	u8 bTestMode;		/* a test mode processing flag */
	u8 bTestModeNo;
	u8 bTransMode;		/* transfer type mode(cpu or dma) */
	u8 bEps;			/* number of eps from ccParam */
	struct scd_bilimit	bilimit;	/* the limit info for general transfers */
	void *pEp0buf;	/* the buffer only for EP0 */
	struct scd_gep gep[SCD_NUM_ALLEPS];	/* include ep0 */
};

/*-------------------------------------------------------
	name	: scd_get_currentreq
	argment	: struct scd_gep *gep
	return	: usb_request *
	description: 
			  to indicate tke current list of usb_request
	date	: 2010/02/26
-------------------------------------------------------*/
static inline struct usb_request *scd_get_currentreq(struct scd_gep *gep)
{
	struct usb_request *req;

	req = (list_empty(&((gep)->req_list)) ? NULL :
		list_entry((gep)->req_list.next, struct usb_request, list));
	return req;
}

/*-------------------------------------------------------
	name	: dump_usb_request
	argment	: struct usb_request *req
	return	: inline char *
	description: 
			  to show infos of usb_request
	date	: 2010/02/26
-------------------------------------------------------*/
static inline char *dump_usb_request(struct usb_request *req)
{
	static char buff[256];
	snprintf(buff, sizeof(buff),
		 "req=%p, req->request.length=0x%0x, req->request.zero=0x%x, "
		 "req->request.actual=0x%x, req->request.status=%d", req,
		 req->length, req->zero, req->actual, req->status);
	return buff;
}
/*-------------------------------------------------------
	name	: scd_gadget_find_end
	argment	: struct usb_ep *pGadgetEnd
	return	: inline int
	description: nonuse now
	date	: 2010/02/26
-------------------------------------------------------*/
static inline int scd_gadget_find_end(struct usb_ep *pGadgetEnd)
{
	return 1;
}
#define SCD_REQ_NONE	0
#define SCD_REQ_TX_SYNC	1
#define SCD_REQ_TX_MAP	2
#define SCD_REQ_RX_SYNC	3
#define SCD_REQ_RX_MAP	4
#define SCD_REQ_MAPMSK	0x7

#define SCD_REQ_ADDBUF		0x10
#define SCD_REQ_ADDBUFMSK	0x10
#define SCD_REQ_4ALIGIN		0x40
#define SCD_REQ_4ALIGINMSK	0x40


/*-------------------------------------------------------
	name	: struct scd_grequest
	description: Ѿեꥯ
	date	: 2010/05/25
-------------------------------------------------------*/
struct scd_grequest {
	struct usb_request	request;
	u8	bEp;		/* the reference ep number on the list completed */
	u8	bAttribute;	/* for request mapping  */
	struct list_head	compl_list;	/* the lists of completed request */
};

/*-------------------------------------------------------
	name	: struct reqdone_queue
	description: taskletѤΥꥯȴλ塼
	date	: 2010/04/21
-------------------------------------------------------*/
struct reqdone_queue{
	spinlock_t			lock;
	struct list_head	req_list;
	u16					cnt;		/* for debug */
};

/*-------------------------------------------------------
	name	: struct grxstsr_val
	description: GRXSTSR쥸
	date	: 2010/04/21
-------------------------------------------------------*/
struct grxstsr_val{
	u8	fn;
	u8	pktsts;
	u8	dpid;
	u16	bcnt;
	u8	epnum;
};
#endif	/* _USB_SCD_GADGET_H_ */
