/*--------------------------------------------------------------------------*
 * drivers/usb/scd/scd_register.h
 * 
 * HS-OTG register definitions
 * 
 * Copyright 2011 Sony Corporation
 *
 * This file is part of the HS-OTG Controller Driver SCD.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 *---------------------------------------------------------------------------*/

#ifndef _SCD_REGISTER_H_
#define _SCD_REGISTER_H_

#include <linux/bitops.h>
#include <asm/io.h>
#include <linux/kernel.h>

/*
  scd쥸ɥ饤
  ץեå
    - SCD_R : 쥸
    - SCD_B : ƥ쥸Υӥå
    - SCD_S : ƥ쥸Υեɳϥӥåֹ
    - SCD_M : ƥ쥸Υեɥӥåȥޥ
    - SCD_E : ƥ쥸Υե
    - SCD_F : եɴϢޥؿ
  ̾
    쥸 : 
    <ץեå>_<쥸̾>
    ӥå : 
    <ץեå>_<쥸̾>_<ӥå̾>
    եɳϥӥåֹ/ӥåȥޥ : 
    <ץեå>_<쥸̾>_<ե̾>
    ե : 
    <ץեå>_<쥸̾>_<ե̾>_<̾>
    ޥؿ : (ܤǽ˹碌ѹ)
    <ץեå>_<쥸̾>_<ե̾>
  ̾
    쥸̾(RN)ӥå̾(BN)ե̾(FN)ˡǽʸ¤ǡ֥å
    ܤ̾ΤѤ롣ʲѹԤ
    - ʸ : DoPng -> DOPNG
    
 */

/*
  scd쥸ʪŪˤI/Oɥ쥹֤¸ߤ뤬
  ioremapˤmemory mapped I/OȤƥ󤵤뤿ᡢ
  readl/writelޥˤƥ롣
 */

/*
 *	scd쥸ޥ
 */

/* ǡɤ߽Ф
  DEV : struct scd_device¤Υ֥
  RFN : Register Full Name - SCD_R_<쥸̾RN>
  RN : Register Name - RFNץեåSCD_R_̾
 */
#define SCD_READ(DEV, RFN) readl((DEV)->regs + (RFN))
#define SCD__READ(DEV, RN) SCD_READ((DEV), (SCD_R_##RN))

/* ǡ񤭹
  DEV : struct scd_device¤Υ֥
  RFN : Register Full Name - SCD_R_<쥸̾RN>
  RN : Register Name - RFNץեåSCD_R_̾
 */
#ifndef SCD_WRITE_PRN
#define SCD_WRITE(DEV, RFN, VAL) writel((VAL), (DEV)->regs + (RFN))
#else
#define SCD_WRITE(DEV, RFN, VAL) \
	printk("SCD_WRITE(sdev, 0x%08lx, 0x%08lx)\n", \
	(unsigned long)(RFN), (unsigned long)(VAL)); \
	writel((VAL), (DEV)->regs + (RFN))
#endif
#define SCD__WRITE(DEV, RN, VAL) SCD_WRITE((DEV), (SCD_R_##RN), (VAL))

/* ʣӥåȥեͼ
    - SCD_READǼ32bit쥸ͤ˾եɤͤФ
  RFSN : Restiger Field Shift Name - SCD_S_<쥸̾RN>_<ե̾FN>
  RFMN : Restiger Field Mask Name - SCD_M_<쥸̾RN>_<ե̾FN>
  RN : Register Name - RFNץեåSCD_R_̾
  FN : Field Name - ե̾

  SCD_GET_FIELD(val, SCD_S_XXXX_YYYY, SCD_M_XXXX_YYYY) 
  SCD__GET_FIELD(val, XXXX, YYYY) 
 */
#define SCD_GET_FIELD(VAL, RFSN, RFMN) (((VAL) >> (RFSN)) & (RFMN))
#define SCD__GET_FIELD(VAL, RN, FN) \
	SCD_GET_FIELD((VAL), (SCD_S_##RN##_##FN), (SCD_M_##RN##_##FN))

/* ʣӥåȥեͥե
    - եͤեɥեȤSCD_WRITEͿͤ򤹤
  RFSN : Restiger Field Shift Name - SCD_S_<쥸̾RN>_<ե̾FN>
  RFMN : Restiger Field Mask Name - SCD_M_<쥸̾RN>_<ե̾FN>
  RN : Register Name - RFNץեåSCD_R_̾
  FN : Field Name - ե̾

  SCD_SFT_FIELD(val, SCD_S_XXXX_YYYY, SCD_M_XXXX_YYYY) 
  SCD__SFT_FIELD(val, XXXX, YYYY) 
 */
#define SCD_SFT_FIELD(VAL, RFSN, RFMN) (((VAL) & (RFMN)) << (RFSN))
#define SCD_SFT__FIELD(VAL, RN, FN) \
	SCD_SFT_FIELD((VAL), (SCD_S_##RN##_##FN), (SCD_M_##RN##_##FN))

/*
 *	scd Global쥸
 */

/* GOTGCTL : Control and Status Register */
#define SCD_R_GOTGCTL	0x000

#define SCD_B_GOTGCTL_BSESVLD		BIT(19)
#define SCD_B_GOTGCTL_ASESVLD		BIT(18)
#define SCD_B_GOTGCTL_DBNCTIME		BIT(17)
#define SCD_B_GOTGCTL_CONIDSTS		BIT(16)
#define SCD_B_GOTGCTL_DEVHNPEN		BIT(11)
#define SCD_B_GOTGCTL_HSTSETHNPEN	BIT(10)
#define SCD_B_GOTGCTL_HNPREQ		BIT(9)
#define SCD_B_GOTGCTL_HSTNEGSCS		BIT(8)
#define SCD_B_GOTGCTL_SESREQ		BIT(1)
#define SCD_B_GOTGCTL_SESREQSCS		BIT(0)

/* GOTGINT : Interrupt Register */
#define SCD_R_GOTGINT	0x004

#define SCD_B_GOTGINT_DBNCEDONE		BIT(19)
#define SCD_B_GOTGINT_ADEVTOUTCHG	BIT(18)
#define SCD_B_GOTGINT_HSTNEGDET		BIT(17)
#define SCD_B_GOTGINT_HSTNEGSUCSTSCHNG	BIT(9)
#define SCD_B_GOTGINT_SESREQSUCSTSCHNG	BIT(8)
#define SCD_B_GOTGINT_SESENDDET	BIT(2)

/* GAHBCFG : AHB Configuration Register */
#define SCD_R_GAHBCFG	0x008

#define SCD_B_GAHBCFG_PTXFEMPLVL	BIT(8)
#define SCD_B_GAHBCFG_NPTXFEMPLVL	BIT(7)
#define SCD_B_GAHBCFG_DMAEN		BIT(5)
#define SCD_B_GAHBCFG_HBSTLEN		BIT(1)
#define SCD_M_GAHBCFG_HBSTLEN		0xf /* 4 bits mask */
#define SCD_S_GAHBCFG_HBSTLEN		1 /* shift */
#define SCD_B_GAHBCFG_GLBLINTRMSK	BIT(0)

/* GUSBCFG : USB COnfiguration Register */
#define SCD_R_GUSBCFG	0x00c

#define SCD_B_GUSBCFG_CORRUPTTXPKT	BIT(31)
#define SCD_B_GUSBCFG_FORCEDEVMODE	BIT(30)
#define SCD_B_GUSBCFG_FORCEHSTMODE	BIT(29)
#define SCD_B_GUSBCFG_TXENDDELAY	BIT(28)
#define SCD_B_GUSBCFG_INDCPASSTHRU	BIT(24)
#define SCD_B_GUSBCFG_INDCCOMPL		BIT(23)
#define SCD_B_GUSBCFG_TERMSELDLPULSE	BIT(22)
#define SCD_B_GUSBCFG_PHYLPWRCLKSEL	BIT(15)
#define SCD_B_GUSBCFG_USBTRDTIM		BIT(10)
#define SCD_M_GUSBCFG_USBTRDTIM		0xf /* 4 bits mask */
#define SCD_S_GUSBCFG_USBTRDTIM		10 /* shift */
#define SCD_B_GUSBCFG_HNPCAP		BIT(9)
#define SCD_B_GUSBCFG_SRPCAP		BIT(8)
#define SCD_B_GUSBCFG_DDRSEL		BIT(7)
#define SCD_B_GUSBCFG_PHYSEL		BIT(6)
#define SCD_B_GUSBCFG_FSINTF		BIT(5)
#define SCD_B_GUSBCFG_ULPIUTMISEL	BIT(4)
#define SCD_B_GUSBCFG_PHYIF		BIT(3)
#define SCD_B_GUSBCFG_TOUTCAL		BIT(0)
#define SCD_M_GUSBCFG_TOUTCAL		0x7 /* 3 bits mask */
#define SCD_S_GUSBCFG_TOUTCAL		0 /* shift */

/* GRSTCTL : Reset Register */
#define SCD_R_GRSTCTL	0x010

#define SCD_B_GRSTCTL_AHBIDLE		BIT(31)
#define SCD_B_GRSTCTL_DMAREQ		BIT(30)
#define SCD_B_GRSTCTL_TXFNUM		BIT(6)
#define SCD_M_GRSTCTL_TXFNUM		0x1f /* 5 bits mask */
#define SCD_S_GRSTCTL_TXFNUM		6 /* shift */
#define SCD_B_GRSTCTL_TXFFLSH		BIT(5)
#define SCD_B_GRSTCTL_RXFFLSH		BIT(4)
#define SCD_B_GRSTCTL_INTKNQFLSH	BIT(3)
#define SCD_B_GRSTCTL_FRMCNTRRST	BIT(2)
#define SCD_B_GRSTCTL_HSFTRST		BIT(1)
#define SCD_B_GRSTCTL_CSFTRST		BIT(0)

/* GINTSTS : Interrupt Register */
#define SCD_R_GINTSTS	0x014

#define SCD_B_GINTSTS_WKUPINT		BIT(31)
#define SCD_B_GINTSTS_SESSREQINT	BIT(30)
#define SCD_B_GINTSTS_DISCONNINT	BIT(29)
#define SCD_B_GINTSTS_CONIDSTSCHNG	BIT(28)
#define SCD_B_GINTSTS_PTXFEMP		BIT(26)
#define SCD_B_GINTSTS_HCHINT		BIT(25)
#define SCD_B_GINTSTS_PRTINT		BIT(24)
#define SCD_B_GINTSTS_RESETDET		BIT(23)
#define SCD_B_GINTSTS_FETSUSP		BIT(22)
#define SCD_B_GINTSTS_INCOMPLP		BIT(21)
#define SCD_B_GINTSTS_INCOMPISOO	BIT(21)
#define SCD_B_GINTSTS_INCOMPISOI	BIT(20)
#define SCD_B_GINTSTS_OEPINT		BIT(19)
#define SCD_B_GINTSTS_IEPINT		BIT(18)
#define SCD_B_GINTSTS_EOPF			BIT(15)
#define SCD_B_GINTSTS_ISOOUTDROP	BIT(14)
#define SCD_B_GINTSTS_ENUMDONE		BIT(13)
#define SCD_B_GINTSTS_USBRST		BIT(12)
#define SCD_B_GINTSTS_USBSUSP		BIT(11)
#define SCD_B_GINTSTS_ERLYSUSP		BIT(10)
#define SCD_B_GINTSTS_GOUTNAKEFF	BIT(7)
#define SCD_B_GINTSTS_GINNAKEFF		BIT(6)
#define SCD_B_GINTSTS_NPTXFEMP		BIT(5)
#define SCD_B_GINTSTS_RXFLVL		BIT(4)
#define SCD_B_GINTSTS_SOF			BIT(3)
#define SCD_B_GINTSTS_OTGINT		BIT(2)
#define SCD_B_GINTSTS_MODEMIS		BIT(1)
#define SCD_B_GINTSTS_CURMOD		BIT(0)

/* GINTMSK : Interrupt Mask Register */
#define SCD_R_GINTMSK	0x018

#define SCD_B_GINTMSK_WKUPINTMSK	BIT(31)
#define SCD_B_GINTMSK_SESSREQINTMSK	BIT(30)
#define SCD_B_GINTMSK_DISCONNINTMSK	BIT(29)
#define SCD_B_GINTMSK_CONIDSTSCHNGMSK	BIT(28)
#define SCD_B_GINTMSK_PTXFEMPMSK	BIT(26)
#define SCD_B_GINTMSK_HCHINTMSK		BIT(25)
#define SCD_B_GINTMSK_PRTINTMSK		BIT(24)
#define SCD_B_GINTMSK_RESETDETMSK	BIT(23)
#define SCD_B_GINTMSK_FETSUSPMSK	BIT(22)
#define SCD_B_GINTMSK_INCOMPLPMSK	BIT(21)
#define SCD_B_GINTMSK_OEPINTMSK		BIT(19)
#define SCD_B_GINTMSK_IEPINTMSK		BIT(18)
#define SCD_B_GINTMSK_EOPFMSK		BIT(15)
#define SCD_B_GINTMSK_ENUMDONEMSK	BIT(13)
#define SCD_B_GINTMSK_USBRSTMSK		BIT(12)
#define SCD_B_GINTMSK_USBSUSPMSK	BIT(11)
#define SCD_B_GINTMSK_ERLYSUSPMSK	BIT(10)
#define SCD_B_GINTMSK_GOUTNAKEFFMSK	BIT(7)
#define SCD_B_GINTMSK_GINNAKEFFMSK	BIT(6)
#define SCD_B_GINTMSK_NPTXFEMPMSK	BIT(5)
#define SCD_B_GINTMSK_RXFLVLMSK		BIT(4)
#define SCD_B_GINTMSK_SOFMSK		BIT(3)
#define SCD_B_GINTMSK_OTGINTMSK		BIT(2)
#define SCD_B_GINTMSK_MODEMISMSK	BIT(1)

/* GRXSTS (GRXSTSR/GRXSTSP) : Receive Status Debug Read/Status Read
                              and Pop Registers
  쥸̾ǽʸRReadPPopΰ̣ǡGRXSTSRGRXSTSPƱӥå
  եɤ̤2ĤΥ쥸ӥåȡե2ĥ쥸
  ͭ뤿ᡢ̾κǸ'R'ڤ'P'ɽƤʤȤա */
#define SCD_R_GRXSTSR	0x01c
#define SCD_R_GRXSTSP	0x020
#define SCD_R_GRXSTS	SCD_R_GRXSTSP

#define SCD_B_GRXSTS_FN			BIT(21)
#define SCD_M_GRXSTS_FN			0xf /* 4 bits mask */
#define SCD_S_GRXSTS_FN			21 /* shift */
#define SCD_B_GRXSTS_PKTSTS		BIT(17)
#define SCD_M_GRXSTS_PKTSTS		0xf /* 4 bits mask */
#define SCD_S_GRXSTS_PKTSTS		17 /* shift */
#define SCD_E_GRXSTS_PKTSTS_GOUTNAK	0x1 /* Global OUT NAK */
#define SCD_E_GRXSTS_PKTSTS_OUTDPRX	0x2 /* OUT data packet received */
#define SCD_E_GRXSTS_PKTSTS_INDPRX	0x2 /* IN data packet received */
#define SCD_E_GRXSTS_PKTSTS_OUTTXCMPL	0x3 /* OUT transfer complete */
#define SCD_E_GRXSTS_PKTSTS_INTXCMPL	0x3 /* IN transfer complete */
#define SCD_E_GRXSTS_PKTSTS_SETUPTXCMPL	0x4 /* SETUP transaction completed */
#define SCD_E_GRXSTS_PKTSTS_DTOGERR	0x5 /* Data toggle error */
#define SCD_E_GRXSTS_PKTSTS_SETUPDPRX	0x6 /* SETUP data packet received */
#define SCD_E_GRXSTS_PKTSTS_CHHALTED	0x7 /* Channel halted */
#define SCD_B_GRXSTS_DPID		BIT(15)
#define SCD_M_GRXSTS_DPID		0x3 /* 2 bits mask */
#define SCD_S_GRXSTS_DPID		15 /* shift */
#define SCD_E_GRXSTS_DPID_DATA0		0x0 /* DATA0 */
#define SCD_E_GRXSTS_DPID_DATA1		0x2 /* DATA1 */
#define SCD_E_GRXSTS_DPID_DATA2		0x1 /* DATA2 */
#define SCD_E_GRXSTS_DPID_MDATA		0x3 /* MDATA */
#define SCD_B_GRXSTS_BCNT		BIT(4)
#define SCD_M_GRXSTS_BCNT		0x7ff /* 11 bits mask */
#define SCD_S_GRXSTS_BCNT		4 /* shift */
#define SCD_B_GRXSTS_CHNUM		BIT(0)
#define SCD_M_GRXSTS_CHNUM		0xf /* 4 bits mask */
#define SCD_S_GRXSTS_CHNUM		0 /* shift */
#define SCD_B_GRXSTS_EPNUM		BIT(0)
#define SCD_M_GRXSTS_EPNUM		0xf /* 4 bits mask */
#define SCD_S_GRXSTS_EPNUM		0 /* shift */

/* GRXFSIZ : Receive FIFO Size Register */
#define SCD_R_GRXFSIZ	0x024

#define SCD_B_GRXFSIZ_RXFDEP		BIT(0)
#define SCD_M_GRXFSIZ_RXFDEP		0xffff /* 16 bits mask */
#define SCD_S_GRXFSIZ_RXFDEP		0 /* shift */

/* GNPTXFSIZ : Non-Periodic Transmit FIFO Size Register */
#define SCD_R_GNPTXFSIZ	0x028

#define SCD_B_GNPTXFSIZ_NPTXFDEP	BIT(16)
#define SCD_M_GNPTXFSIZ_NPTXFDEP	0xffff /* 16 bits mask (max) */
#define SCD_S_GNPTXFSIZ_NPTXFDEP	16
#define SCD_B_GNPTXFSIZ_INEPTXF0DEP	BIT(16)
#define SCD_M_GNPTXFSIZ_INEPTXF0DEP	0xffff /* 16 bits mask (max) */
#define SCD_S_GNPTXFSIZ_INEPTXF0DEP	16
#define SCD_B_GNPTXFSIZ_NPTXFSTADDR	BIT(0)
#define SCD_M_GNPTXFSIZ_NPTXFSTADDR	0xffff /* 16 bits mask (max) */
#define SCD_S_GNPTXFSIZ_NPTXFSTADDR	0
#define SCD_B_GNPTXFSIZ_INEPTXF0STADDR	BIT(0)
#define SCD_M_GNPTXFSIZ_INEPTXF0STADDR	0xffff /* 16 bits mask (max) */
#define SCD_S_GNPTXFSIZ_INEPTXF0STADDR	0

/* GSNPSID : Synopsys ID Register */
#define SCD_R_GSNPSID	0x040

#define SCD_B_GSNPSID_SYNOPSYSID	BIT(0)
#define SCD_M_GSNPSID_SYNOPSYSID	0xffffffff /* 32 bits mask */
#define SCD_S_GSNPSID_SYNOPSYSID	32

/* GHWCFG1 : User HW Config1 Register
   exget EP3 direction
         ep3dir = SCD_GET_FIELD(SCD_READ(device, SCD_R_GHWCFG1),
                                SCD_S_GHWCFG1_EPDIR(3),
                                SCD_M_GHWCFG1_EPDIR(3)) */
#define SCD_R_GHWCFG1	0x044

#define SCD_B_GHWCFG1_EPDIR		BIT(0)
#define SCD_M_GHWCFG1_EPDIR(EPN)	0x3 /* 2 bits mask for each EPDIR */
#define SCD_S_GHWCFG1_EPDIR(EPN)	((EPN) * 2)
#define SCD_E_GHWCFG1_EPDIR_BIDIR	0x0
#define SCD_E_GHWCFG1_EPDIR_IN		0x1
#define SCD_E_GHWCFG1_EPDIR_OUT		0x2

/* GHWCFG2 : User HW Config2 Register
  ex) get PTXQDEPTH
        ptxqdep = SCD_F_GHWCFG2PTXQDEPTH(
                    SCD_GET_FIELD(SCD_READ(device, SCD_R_GHWCFG2),
                                  SCD_S_GHWCFG2_PTXQDEPTH,
                                  SCD_M_GHWCFG2_PTXQDEPTH)); */
#define SCD_R_GHWCFG2	0x048

#define SCD_B_GHWCFG2_TKNQDEPTH		BIT(26)
#define SCD_M_GHWCFG2_TKNQDEPTH		0x1f /* 5 bits mask (max) */
#define SCD_S_GHWCFG2_TKNQDEPTH		26
#define SCD_B_GHWCFG2_PTXQDEPTH		BIT(24)
#define SCD_M_GHWCFG2_PTXQDEPTH		0x3 /* 2 bits mask (max) */
#define SCD_S_GHWCFG2_PTXQDEPTH		24
#define SCD_F_GHWCFG2_PTXQDEPTH(VAL)	(1UL << ((VAL) + 1))
#define SCD_B_GHWCFG2_NPTXQDEPTH	BIT(22)
#define SCD_M_GHWCFG2_NPTXQDEPTH	0x3 /* 2 bits mask (max) */
#define SCD_S_GHWCFG2_NPTXQDEPTH	22
#define SCD_F_GHWCFG2_NPTXQDEPTH(VAL)	(1UL << ((VAL) + 1))
#define SCD_B_GHWCFG2_MULTIPROCINTRPT	BIT(20)
#define SCD_B_GHWCFG2_DYNFIFOSIZING	BIT(19)
#define SCD_B_GHWCFG2_PERIOSUPPORT	BIT(18)
#define SCD_B_GHWCFG2_NUMHSTCHNL	BIT(14)
#define SCD_M_GHWCFG2_NUMHSTCHNL	0xf /* 4 bits mask (max) */
#define SCD_S_GHWCFG2_NUMHSTCHNL	14
#define SCD_B_GHWCFG2_NUMDEVEPS		BIT(10)
#define SCD_M_GHWCFG2_NUMDEVEPS		0xf /* 4 bits mask (max) */
#define SCD_S_GHWCFG2_NUMDEVEPS		10
#define SCD_B_GHWCFG2_FSPHYTYPE		BIT(8)
#define SCD_M_GHWCFG2_FSPHYTYPE		0x3 /* 2 bits mask (max) */
#define SCD_S_GHWCFG2_FSPHYTYPE		8
#define SCD_E_GHWCFG2_FSPHYTYPE_NOTSUPPORT	0x0
#define SCD_E_GHWCFG2_FSPHYTYPE_DEDICATED_FS	0x1
#define SCD_E_GHWCFG2_FSPHYTYPE_SHARED_UTMI	0x2
#define SCD_E_GHWCFG2_FSPHYTYPE_SHARED_ULPI	0x3
#define SCD_B_GHWCFG2_HSPHYTYPE		BIT(6)
#define SCD_M_GHWCFG2_HSPHYTYPE		0x3 /* 2 bits mask (max) */
#define SCD_S_GHWCFG2_HSPHYTYPE		6
#define SCD_E_GHWCFG2_HSPHYTYPE_NOTSUPPORT	0x0
#define SCD_E_GHWCFG2_HSPHYTYPE_UTMI		0x1
#define SCD_E_GHWCFG2_HSPHYTYPE_UTLP		0x2
#define SCD_E_GHWCFG2_HSPHYTYPE_UTMI_ULPI	0x3
#define SCD_B_GHWCFG2_SINGPNT		BIT(5)
#define SCD_B_GHWCFG2_OTGARCH		BIT(3)
#define SCD_M_GHWCFG2_OTGARCH		0x3 /* 2 bits mask (max) */
#define SCD_S_GHWCFG2_OTGARCH		3
#define SCD_E_GHWCFG2_OTGARCH_SLAVEONLY		0x0
#define SCD_E_GHWCFG2_OTGARCH_EXTERNALDMA	0x1
#define SCD_E_GHWCFG2_OTGARCH_INTERNALDMA	0x2
#define SCD_B_GHWCFG2_OTGMODE		BIT(0)
#define SCD_M_GHWCFG2_OTGMODE		0x7 /* 3 bits mask (max) */
#define SCD_S_GHWCFG2_OTGMODE		0
#define SCD_E_GHWCFG2_OTGMODE_HNPSRPOTG	0x0
#define SCD_E_GHWCFG2_OTGMODE_SRPOTG	0x1
#define SCD_E_GHWCFG2_OTGMODE_OTG	0x2
#define SCD_E_GHWCFG2_OTGMODE_SPRDEVICE	0x3
#define SCD_E_GHWCFG2_OTGMODE_DEVICE	0x4
#define SCD_E_GHWCFG2_OTGMODE_SRPHOST	0x5
#define SCD_E_GHWCFG2_OTGMODE_HOST	0x6
 
/* GHWCFG3 : User HW Config3 Register
  ex) get PKTSIZEWIDTH
        pktsizewidth = SCD_F_GHWCFG3_PKTSIZEWIDTH(
                         SCD_GET_FIELD(SCD_READ(device, SCD_R_GHWCFG3),
                                       SCD_S_GHWCFG3_PKTSIZEWIDTH,
                                       SCD_M_GHWCFG3_PKTSIZEWIDTH)); */
#define SCD_R_GHWCFG3	0x04c

#define SCD_B_GHWCFG3_DFIFODEPTH	BIT(16)
#define SCD_M_GHWCFG3_DFIFODEPTH	0xffff /* 16 bits mask (max) */
#define SCD_S_GHWCFG3_DFIFODEPTH	16
#define SCD_B_GHWCFG3_OTGENABLELPM	BIT(15)
#define SCD_B_GHWCFG3_OTGENABLEHSIC	BIT(13)
#define SCD_B_GHWCFG3_OTGENABLEICUSB	BIT(12)
#define SCD_B_GHWCFG3_RSTTYPE		BIT(11)
#define SCD_B_GHWCFG3_OPTFEATURE	BIT(10)
#define SCD_B_GHWCFG3_VNDCTLSUPT	BIT(9)
#define SCD_B_GHWCFG3_I2CINTSEL		BIT(8)
#define SCD_B_GHWCFG3_OTGEN		BIT(7)
#define SCD_B_GHWCFG3_PKTSIZEWIDTH	BIT(4)
#define SCD_M_GHWCFG3_PKTSIZEWIDTH	0x7 /* 3 bits mask (max) */
#define SCD_S_GHWCFG3_PKTSIZEWIDTH	4
#define SCD_F_GHWCFG3_PKTSIZEWIDTH(VAL)	((VAL) + 4)
#define SCD_B_GHWCFG3_XFERSIZEWIDTH	BIT(4)
#define SCD_M_GHWCFG3_XFERSIZEWIDTH	0xf /* 4 bits mask (max) */
#define SCD_S_GHWCFG3_XFERSIZEWIDTH	4
#define SCD_F_GHWCFG3_XFERSIZEWIDTH(VAL)	((VAL) + 11)

/* GHWCFG3 : User HW Config4 Register */
#define SCD_R_GHWCFG4	0x050

#define SCD_B_GHWCFG4_SGDMA		BIT(31)
#define SCD_B_GHWCFG4_SGDMACONFIG	BIT(30)
#define SCD_B_GHWCFG4_INEPS		BIT(26)
#define SCD_M_GHWCFG4_INEPS		0xf /* 4 bits mask (max) */
#define SCD_S_GHWCFG4_INEPS		26
#define SCD_B_GHWCFG4_DEDFIFOMODE	BIT(25)
#define SCD_B_GHWCFG4_SESSENDFLTR	BIT(24)
#define SCD_B_GHWCFG4_BVALIDFLTR	BIT(23)
#define SCD_B_GHWCFG4_AVALIDFLTR	BIT(22)
#define SCD_B_GHWCFG4_VBUSVALIDFLTR	BIT(21)
#define SCD_B_GHWCFG4_IDDGFLTR		BIT(20)
#define SCD_B_GHWCFG4_NUMCTLEPS		BIT(16)
#define SCD_M_GHWCFG4_NUMCTLEPS		0xf /* 4 bits mask (max) */
#define SCD_S_GHWCFG4_NUMCTLEPS		16
#define SCD_B_GHWCFG4_PHYDATAWIDTH	BIT(14)
#define SCD_M_GHWCFG4_PHYDATAWIDTH	0x3 /* 2 bits mask (max) */
#define SCD_S_GHWCFG4_PHYDATAWIDTH	14
#define SCD_E_GHWCFG4_PHYDATAWIDTH_8BITFIX	0x0
#define SCD_E_GHWCFG4_PHYDATAWIDTH_16BITFIX	0x1
#define SCD_E_GHWCFG4_PHYDATAWIDTH_SELECTABLE	0x2
#define SCD_B_GHWCFG4_AHBFREQ		BIT(5)
#define SCD_B_GHWCFG4_ENABLEPWROPT	BIT(4)
#define SCD_B_GHWCFG4_NUMDEVPERIOEPS	BIT(0)
#define SCD_M_GHWCFG4_NUMDEVPERIOEPS	0xf /* 4 bits mask (max) */
#define SCD_S_GHWCFG4_NUMDEVPERIOEPS	0

/* HPTXFSIZ : Host Periodic Transmit FIFO Size Register */
#define SCD_R_HPTXFSIZ	0x100

#define SCD_B_HPTXFSIZ_PTXFSIZE		BIT(16)
#define SCD_M_HPTXFSIZ_PTXFSIZE		0xffff /* 16 bits mask (max) */
#define SCD_S_HPTXFSIZ_PTXFSIZE		16
#define SCD_B_HPTXFSIZ_PTXFSTADDR	BIT(0)
#define SCD_M_HPTXFSIZ_PTXFSTADDR	0xffff /* 16 bits mask (max) */
#define SCD_S_HPTXFSIZ_PTXFSTADDR	0

/* DIEPTXFN : Device IN Endpoint Transmit FIFO Size Register
   exget EP3 tx fifo size
         ep3ineptxfstaddr = SCD_GET_FIELD(
                              SCD_READ(device, SCD_R_DIEPTXF(3)),
                              SCD_S_DIEPTXFN_INEPNTXFSTADDR,
                              SCD_M_DIEPTXFN_INEPNTXFSTADDR);
       SCD_READͿ쥸̾'N'դʤȤա */
#define SCD_R_DIEPTXF(EPN)	(0x104 + ((EPN) - 1) * 0x04)

#define SCD_B_DIEPTXFN_IEPNTXFDEP	BIT(16)
#define SCD_M_DIEPTXFN_IEPNTXFDEP	0xffff /* 16 bits mask (max) */
#define SCD_S_DIEPTXFN_IEPNTXFDEP	16
#define SCD_B_DIEPTXFN_INEPNTXFSTADDR	BIT(0)
#define SCD_M_DIEPTXFN_INEPNTXFSTADDR	0xffff /* 16 bits mask (max) */
#define SCD_S_DIEPTXFN_INEPNTXFSTADDR	0

/*
 *	scd Host Mode쥸
 */

/* HCFG : Host Configuration Register */
#define SCD_R_HCFG	0x400

#define SCD_B_HCFG_RESVALID		BIT(8)
#define SCD_M_HCFG_RESVALID		0xff /* 8 bits mask */
#define SCD_S_HCFG_RESVALID		8
#define SCD_B_HCFG_ENA32KHZS		BIT(7)
#define SCD_B_HCFG_FSLSSUPP		BIT(2)
#define SCD_B_HCFG_FSLSPCLKSEL		BIT(0)
#define SCD_M_HCFG_FSLSPCLKSEL		0x3 /* 2 bits mask */
#define SCD_S_HCFG_FSLSPCLKSEL		0
#define SCD_E_HCFG_FSLSPCLKSEL_30_60MHZ	0x0
#define SCD_E_HCFG_FSLSPCLKSEL_48MHZ	0x1

/* HFIR : Host Frame Interval Register */
#define SCD_R_HFIR	0x404

#define SCD_B_HFIR_FRINT		BIT(0)
#define SCD_M_HFIR_FRINT		0xffff /* 16 bits mask */
#define SCD_S_HFIR_FRINT		0

/* HFNUM : Host Frame Number/Frame Time Remaining Register */
#define SCD_R_HFNUM	0x408

#define SCD_B_HFNUM_FRREM		BIT(16)
#define SCD_M_HFNUM_FRREM		0xffff /* 16 bits mask */
#define SCD_S_HFNUM_FRREM		16
#define SCD_B_HFNUM_FRNUM		BIT(0)
#define SCD_M_HFNUM_FRNUM		0xffff /* 16 bits mask */
#define SCD_S_HFNUM_FRNUM		0

/* HPTXSTS : Host Periodic Transmit FIFO/Queue Status Register */
#define SCD_R_HPTXSTS	0x410

#define SCD_B_HPTXSTS_PTXQTOP		BIT(24)
#define SCD_M_HPTXSTS_PTXQTOP		0xff /* 8 bits mask */
#define SCD_S_HPTXSTS_PTXQTOP		24
#define SCD_B_HPTXSTS_PTXQTOP_ODDEVEN	BIT(31)
#define SCD_S_HPTXSTS_PTXQTOP_ODDEVEN	31
#define SCD_M_HPTXSTS_PTXQTOP_CHEPNUM	0xf /* 4 bits mask */
#define SCD_S_HPTXSTS_PTXQTOP_CHEPNUM	27
#define SCD_M_HPTXSTS_PTXQTOP_TYPE	0x3 /* 2 bits mask */
#define SCD_S_HPTXSTS_PTXQTOP_TYPE	25
#define SCD_E_HPTXSTS_PTXQTOP_TYPE_INOUT	0x0
#define SCD_E_HPTXSTS_PTXQTOP_TYPE_ZEROLEN	0x1
#define SCD_E_HPTXSTS_PTXQTOP_TYPE_CSPLIT	0x2
#define SCD_E_HPTXSTS_PTXQTOP_TYPE_DISABLE	0x3
#define SCD_B_HPTXSTS_PTXQTOP_TERMINATE	BIT(24)
#define SCD_B_HPTXSTS_PTXQSPCAVAIL	BIT(16)
#define SCD_M_HPTXSTS_PTXQSPCAVAIL	0xff /* 8 bits mask (max) */
#define SCD_S_HPTXSTS_PTXQSPCAVAIL	16
#define SCD_B_HPTXSTS_PTXFSPCAVAIL	BIT(0)
#define SCD_M_HPTXSTS_PTXFSPCAVAIL	0xffff /* 16 bits mask (max) */
#define SCD_S_HPTXSTS_PTXFSPCAVAIL	0

/* PCHN : number of physical channel */
#define CHN_OFFSET	1
#define PCHN(CHN)	(CHN+CHN_OFFSET)

/* HAINT : Host All Channels Interrupt Register */
#define SCD_R_HAINT	0x414

#define SCD_B_HAINT_HAINT		BIT(0)
#define SCD_M_HAINT_HAINT		0xffff /* 16 bits mask (max) */
#define SCD_S_HAINT_HAINT		0
#define SCD_F_HAINT_CH(CHN)		BIT(PCHN(CHN))

/* HAINTMSK : Host All Channels Interrupt Mask Register */
#define SCD_R_HAINTMSK	0x418

#define SCD_B_HAINTMSK_HAINTMSK		BIT(0)
#define SCD_M_HAINTMSK_HAINTMSK		0xffff /* 16 bits mask */
#define SCD_S_HAINTMSK_HAINTMSK		0
#define SCD_F_HAINTMSK_CH(CHN)		BIT(PCHN(CHN))

/* HPRT : Host Port Control and Status Register */
#define SCD_R_HPRT	0x440

#define SCD_B_HPRT_PRTSPD		BIT(17)
#define SCD_M_HPRT_PRTSPD		0x3 /* 2 bits mask */
#define SCD_S_HPRT_PRTSPD		17
#define SCD_E_HPRT_PRTSPD_HS		0x0
#define SCD_E_HPRT_PRTSPD_FS		0x1
#define SCD_E_HPRT_PRTSPD_LS		0x2
#define SCD_B_HPRT_PRTTSTCTL		BIT(13)
#define SCD_M_HPRT_PRTTSTCTL		0xf /* 4 bits mask */
#define SCD_S_HPRT_PRTTSTCTL		13
#define SCD_E_HPRT_PRTTSTCTL_DISABLE	0x0
#define SCD_E_HPRT_PRTTSTCTL_J		0x1
#define SCD_E_HPRT_PRTTSTCTL_K		0x2
#define SCD_E_HPRT_PRTTSTCTL_SE0NAK	0x3
#define SCD_E_HPRT_PRTTSTCTL_PACKET	0x4
#define SCD_E_HPRT_PRTTSTCTL_FORCE	0x5
#define SCD_B_HPRT_PRTPWR		BIT(12)
#define SCD_B_HPRT_PRTLNSTS		BIT(10)
#define SCD_M_HPRT_PRTLNSTS		0x3 /* 2 bits mask */
#define SCD_S_HPRT_PRTLNSTS		10
#define SCD_B_HPRT_PRTLNSTS_DM		BIT(11)
#define SCD_B_HPRT_PRTLNSTS_DP		BIT(10)
#define SCD_B_HPRT_PRTRST		BIT(8)
#define SCD_B_HPRT_PRTSUSP		BIT(7)
#define SCD_B_HPRT_PRTRES		BIT(6)
#define SCD_B_HPRT_PRTOVRCURRCHNG	BIT(5)
#define SCD_B_HPRT_PRTOVRCURRACT	BIT(4)
#define SCD_B_HPRT_PRTENCHNG		BIT(3)
#define SCD_B_HPRT_PRTENA		BIT(2)
#define SCD_B_HPRT_PRTCONNDET		BIT(1)
#define SCD_B_HPRT_PRTCONNSTS		BIT(0)

/* HCCHARN : Host Channel-n Characteristics Register
   exget CH3 Enable
         ch3en = SCD_READ(device, SCD_R_HCCHAR(3)) & SCD_B_HCCHARN_CHENA;
       SCD_READͿ쥸̾'N'դʤȤա */
#define SCD_R_HCCHAR(CHN)	(0x500 + (PCHN(CHN)) * 0x20)

#define SCD_B_HCCHARN_CHENA		BIT(31)
#define SCD_B_HCCHARN_CHDIS		BIT(30)
#define SCD_B_HCCHARN_ODDFRM		BIT(29)
#define SCD_B_HCCHARN_DEVADDR		BIT(22)
#define SCD_M_HCCHARN_DEVADDR		0x7f /* 7 bits mask */
#define SCD_S_HCCHARN_DEVADDR		22
#define SCD_B_HCCHARN_MCEC		BIT(20)
#define SCD_M_HCCHARN_MCEC		0x3 /* 2 bits mask */
#define SCD_S_HCCHARN_MCEC		20
#define SCD_B_HCCHARN_EPTYPE		BIT(18)
#define SCD_M_HCCHARN_EPTYPE		0x3 /* 2 bits mask */
#define SCD_S_HCCHARN_EPTYPE		18
#define SCD_E_HCCHARN_EPTYPE_CONTROL	0x0
#define SCD_E_HCCHARN_EPTYPE_ISO	0x1
#define SCD_E_HCCHARN_EPTYPE_BULK	0x2
#define SCD_E_HCCHARN_EPTYPE_INTERRUPT	0x3
#define SCD_B_HCCHARN_LSPDDEV		BIT(17)
#define SCD_B_HCCHARN_EPDIR		BIT(15)
#define SCD_B_HCCHARN_EPNUM		BIT(11)
#define SCD_M_HCCHARN_EPNUM		0xf /* 4 bits mask */
#define SCD_S_HCCHARN_EPNUM		11
#define SCD_B_HCCHARN_MPS		BIT(0)
#define SCD_M_HCCHARN_MPS		0x7ff /* 11 bits mask */
#define SCD_EHCCHARN_MPS		0

/* HCSPLTN : Host Channel-n Split Control Register
   exget CH3 Split Enable
         ch3spliten = SCD_READ(device, SCD_R_HCSPLT(3)) & SCD_B_HCSPLTN_SPLTENA;
       SCD_READͿ쥸̾'N'դʤȤա */
#define SCD_R_HCSPLT(CHN)	(0x504 + (PCHN(CHN)) * 0x20)

#define SCD_B_HCSPLTN_SPLTENA		BIT(31)
#define SCD_B_HCSPLTN_COMPSPLT		BIT(16)
#define SCD_B_HCSPLTN_XACTPOS		BIT(14)
#define SCD_M_HCSPLTN_XACTPOS		0x3 /* 2 bits mask */
#define SCD_S_HCSPLTN_XACTPOS		14
#define SCD_E_HCSPLTN_XACTPOS_BEGIN	0x2
#define SCD_E_HCSPLTN_XACTPOS_MID	0x0
#define SCD_E_HCSPLTN_XACTPOS_END	0x1
#define SCD_B_HCSPLTN_HUBADDR		BIT(7)
#define SCD_M_HCSPLTN_HUBADDR		0x7f /* 7 bits mask */
#define SCD_S_HCSPLTN_HUBADDR		7
#define SCD_B_HCSPLTN_PRTADDR		BIT(0)
#define SCD_M_HCSPLTN_PRTADDR		0x7f /* 7 bits mask */
#define SCD_S_HCSPLTN_PRTADDR		0

/* HCINTN : Host Channel-n Interrupt Register
   exget CH3 Data Toggle Error
         ch3datatglerr = SCD_READ(device, SCD_R_HCINT(3)) &
                           SCD_B_HCINTN_DATATGLERR;
       SCD_READͿ쥸̾'N'դʤȤա */
#define SCD_R_HCINT(CHN)	(0x508 + (PCHN(CHN)) * 0x20)

#define SCD_B_HCINTN_DATATGLERR		BIT(10)
#define SCD_B_HCINTN_FRMOVRUN		BIT(9)
#define SCD_B_HCINTN_BBLERR		BIT(8)
#define SCD_B_HCINTN_XACTERR		BIT(7)
#define SCD_B_HCINTN_NYET		BIT(6)
#define SCD_B_HCINTN_ACK		BIT(5)
#define SCD_B_HCINTN_NAK		BIT(4)
#define SCD_B_HCINTN_STALL		BIT(3)
#define SCD_B_HCINTN_AHBERR		BIT(2)
#define SCD_B_HCINTN_CHHLTD		BIT(1)
#define SCD_B_HCINTN_XFERCOMPL		BIT(0)

/* HCINTMSKN : Host Channel-n Interrupt Mask Register
   exset CH3 Transfer Completed Mask
         ep3hcintmsk = SCD_READ(device, SCD_R_HCINTMSK(3));
         SCD_WRITE(device, SCD_R_HCINTMSK(3),
                   ep3hcintmsk & ~SCD_B_HCINTMSKN_XFERCOMPLMSK);
       SCD_READͿ쥸̾'N'դʤȤա */
#define SCD_R_HCINTMSK(CHN)	(0x50c + (PCHN(CHN)) * 0x20)

#define SCD_B_HCINTMSKN_DATATGLERR	BIT(10)
#define SCD_B_HCINTMSKN_FRMOVRUN	BIT(9)
#define SCD_B_HCINTMSKN_BBLERR		BIT(8)
#define SCD_B_HCINTMSKN_XACTERR		BIT(7)
#define SCD_B_HCINTMSKN_NYET		BIT(6)
#define SCD_B_HCINTMSKN_ACK		BIT(5)
#define SCD_B_HCINTMSKN_NAK		BIT(4)
#define SCD_B_HCINTMSKN_STALL		BIT(3)
#define SCD_B_HCINTMSKN_AHBERR		BIT(2)
#define SCD_B_HCINTMSKN_CHHLTD		BIT(1)
#define SCD_B_HCINTMSKN_XFERCOMPL	BIT(0)

/* HCTSIZN : Host Channel-n Transfer Size Register
       SCD_READͿ쥸̾'N'դʤȤա */
#define SCD_R_HCTSIZ(CHN)	(0x510 + (PCHN(CHN)) * 0x20)

#define SCD_B_HCTSIZN_DOPNG		BIT(31)
#define SCD_B_HCTSIZN_PID		BIT(29)
#define SCD_M_HCTSIZN_PID		0x3 /* 2 bits mask */
#define SCD_S_HCTSIZN_PID		29
#define SCD_B_HCTSIZN_PID_DATA0		(0x0 << SCD_S_HCTSIZN_PID)
#define SCD_B_HCTSIZN_PID_DATA2		(0x1 << SCD_S_HCTSIZN_PID)
#define SCD_B_HCTSIZN_PID_DATA1		(0x2 << SCD_S_HCTSIZN_PID)
#define SCD_B_HCTSIZN_PID_MDATA		(0x3 << SCD_S_HCTSIZN_PID)
#define SCD_B_HCTSIZN_PID_SETUP		(0x3 << SCD_S_HCTSIZN_PID)
#define SCD_B_HCTSIZN_PKTCNT		BIT(19)
#define SCD_M_HCTSIZN_PKTCNT		0x3ff /* 10 bits mask */
#define SCD_S_HCTSIZN_PKTCNT		19
#define SCD_B_HCTSIZN_XFERSIZE		BIT(0)
#define SCD_M_HCTSIZN_XFERSIZE		0x7ffff /* 19 bits mask */
#define SCD_S_HCTSIZN_XFERSIZE		0

/* HCDMAN : Host Channel-n DMA Address Register
       SCD_READͿ쥸̾'N'դʤȤա */
#define SCD_R_HCDMA(CHN)	(0x514 + (PCHN(CHN)) * 0x20)

#define SCD_B_HCDMAN_DMAADDR		BIT(0)
#define SCD_M_HCDMAN_DMAADDR		0xffffffff /* 32 bits mask */
#define SCD_S_HCDMAN_DMAADDR		0

/*
 *	scd Device Mode쥸
 */

/* DCFG : Device Configuration Register */
#define SCD_R_DCFG	0x800

#define SCD_B_DCFG_RESVALID		BIT(26)
#define SCD_M_DCFG_RESVALID		0x3f /* 6 bits mask */
#define SCD_S_DCFG_RESVALID		26
#define SCD_B_DCFG_PERSCHINTVL		BIT(24)
#define SCD_M_DCFG_PERSCHINTVL		0x3 /* 2 bits mask */
#define SCD_S_DCFG_PERSCHINTVL		24
#define SCD_E_DCFG_PERSCHINTVL_25P	0x0
#define SCD_E_DCFG_PERSCHINTVL_50P	0x1
#define SCD_E_DCFG_PERSCHINTVL_75P	0x2
#define SCD_B_DCFG_DESCDMA		BIT(23)
#define SCD_B_DCFG_EPMISCNT		BIT(18)
#define SCD_M_DCFG_EPMISCNT		0x1f /* 5 bits mask */
#define SCD_S_DCFG_EPMISCNT		18
#define SCD_B_DCFG_PERFRINT		BIT(11)
#define SCD_M_DCFG_PERFRINT		0x3 /* 2 bits mask */
#define SCD_S_DCFG_PERFRINT		11
#define SCD_E_DCFG_PERFRINT_80P		0x0
#define SCD_E_DCFG_PERFRINT_85P		0x1
#define SCD_E_DCFG_PERFRINT_90P		0x2
#define SCD_E_DCFG_PERFRINT_95P		0x3
#define SCD_B_DCFG_DEVADDR		BIT(4)
#define SCD_M_DCFG_DEVADDR		0x7f /* 7 bits mask */
#define SCD_S_DCFG_DEVADDR		4
#define SCD_B_DCFG_ENA32KHZS		BIT(3)
#define SCD_B_DCFG_NZSTSOUTHSHK		BIT(2)
#define SCD_B_DCFG_DEVSPD		BIT(0)
#define SCD_M_DCFG_DEVSPD		0x3 /* 2 bits mask */
#define SCD_S_DCFG_DEVSPD		0
#define SCD_E_DCFG_DEVSPD_HS		0x0
#define SCD_E_DCFG_DEVSPD_FS		0x1
#define SCD_E_DCFG_DEVSPD_FS_PCLK48MHZ	0x3

/* DCTL : Device Control Register */
#define SCD_R_DCTL	0x804

#define SCD_B_DCTL_NAKONBBLE		BIT(16)
#define SCD_B_DCTL_IGNRFRMNUM		BIT(15)
#define SCD_B_DCTL_GMC			BIT(13)
#define SCD_M_DCTL_GMC			0x3 /* 2 bits mask */
#define SCD_S_DCTL_GMC			13
#define SCD_E_DCTL_GMC_INVALID		0x0
#define SCD_E_DCTL_GMC_1PACKET		0x1
#define SCD_E_DCTL_GMC_2PACKET		0x2
#define SCD_E_DCTL_GMC_3PACKET		0x3
#define SCD_B_DCTL_PWRONPRGDONE		BIT(11)
#define SCD_B_DCTL_CGOUTNAK		BIT(10)
#define SCD_B_DCTL_SGOUTNAK		BIT(9)
#define SCD_B_DCTL_CGNPINNAK		BIT(8)
#define SCD_B_DCTL_SGNPINNAK		BIT(7)
#define SCD_B_DCTL_TSTCTL		BIT(4)
#define SCD_M_DCTL_TSTCTL		0x7 /* 3 bits mask */
#define SCD_S_DCTL_TSTCTL		4
#define SCD_E_DCTL_TSTCTL_DISABLE	0x0
#define SCD_E_DCTL_TSTCTL_J		0x1
#define SCD_E_DCTL_TSTCTL_K		0x2
#define SCD_E_DCTL_TSTCTL_SE0NAK	0x3
#define SCD_E_DCTL_TSTCTL_PACKET	0x4
#define SCD_E_DCTL_TSTCTL_FORCE		0x5
#define SCD_B_DCTL_GOUTNAKSTS		BIT(3)
#define SCD_B_DCTL_GNPINNAKSTS		BIT(2)
#define SCD_B_DCTL_SFTDISCON		BIT(1)
#define SCD_B_DCTL_RMTWKUPSIG		BIT(0)

/* DSTS : Device Status Register */
#define SCD_R_DSTS	0x808

#define SCD_B_DSTS_SOFFN		BIT(8)
#define SCD_M_DSTS_SOFFN		0x3fff /* 14 bits mask */
#define SCD_S_DSTS_SOFFN		8
#define SCD_B_DSTS_ERRTICERR		BIT(3)
#define SCD_B_DSTS_ENUMSPD		BIT(1)
#define SCD_M_DSTS_ENUMSPD		0x3 /* 2 bits mask */
#define SCD_S_DSTS_ENUMSPD		1
#define SCD_E_DSTS_ENUMSPD_HS		0x0
#define SCD_E_DSTS_ENUMSPD_FS		0x1
#define SCD_E_DSTS_ENUMSPD_LS_PCLK48MHZ	0x2
#define SCD_E_DSTS_ENUMSPD_FS_PCLK48MHZ	0x3
#define SCD_B_DSTS_SUSPSTS		BIT(0)

/* DIEPMSK : Device IN Endpoint Common Interrupt Mask Register */
#define SCD_R_DIEPMSK	0x810

#define SCD_B_DIEPMSK_NAKMSK		BIT(13)
#define SCD_B_DIEPMSK_BNAININTRMSK	BIT(9)
#define SCD_B_DIEPMSK_TXFIFOUNDRNMSK	BIT(8)
#define SCD_B_DIEPMSK_INEPNAKEFFMSK	BIT(6)
#define SCD_B_DIEPMSK_INTKNEPMISMSK	BIT(5)
#define SCD_B_DIEPMSK_INTKNTXFEMPMSK	BIT(4)
#define SCD_B_DIEPMSK_TIMEOUTMSK	BIT(3)
#define SCD_B_DIEPMSK_AHBERRMSK		BIT(2)
#define SCD_B_DIEPMSK_EPDISBLDMSK	BIT(1)
#define SCD_B_DIEPMSK_XFERCOMPLMSK	BIT(0)

/* DOEPMSK : Device OUT Endpoint Common Interrupt Mask Register */
#define SCD_R_DOEPMSK	0x814

#define SCD_B_DOEPMSK_NYETMSK		BIT(14)
#define SCD_B_DOEPMSK_NAKMSK		BIT(13)
#define SCD_B_DOEPMSK_BBLEERRMSK	BIT(12)
#define SCD_B_DOEPMSK_BNAOUTINTRMSK	BIT(9)
#define SCD_B_DOEPMSK_OUTPKTERRMSK	BIT(8)
#define SCD_B_DOEPMSK_BACK2BACKSETUP	BIT(6)
#define SCD_B_DOEPMSK_OUTTKNEPDISMSK	BIT(4)
#define SCD_B_DOEPMSK_SETUPMSK		BIT(3)
#define SCD_B_DOEPMSK_AHBERRMSK		BIT(2)
#define SCD_B_DOEPMSK_EPDISBLDMSK	BIT(1)
#define SCD_B_DOEPMSK_XFERCOMPLMSK	BIT(0)

/* DAINT : Device All Endpoints Interrupt Register
  ex) get EP3 OUT Interrupt
        ep3outepint = SCD_F_DAINT_OUTEPINT(SCD_READ(device, SCD_R_DAINT), 3); */
#define SCD_R_DAINT	0x818

#define SCD_B_DAINT_OUTEPINT		BIT(16)
#define SCD_M_DAINT_OUTEPINT		0xffff /* 16 bits mask */
#define SCD_S_DAINT_OUTEPINT		16
#define SCD_F_DAINT_OUTEPINT(EPN)	BIT((EPN) + 16)
#define SCD_B_DAINT_INEPINT		BIT(0)
#define SCD_M_DAINT_INEPINT		0xffff /* 16 bits mask */
#define SCD_S_DAINT_INEPINT		0
#define SCD_F_DAINT_INEPINT(EPN)	BIT(EPN)

/* DAINTMSK : Device All Endpoints Interrupt Mask Register
  ex) unmask EP3 IN Interrupt Mask
        ep3outepmsk = SCD_READ(device, SCD_R_DAINTMSK);
        SCD_WRITE(device, SCD_R_DAINTMSK,
                  ep3outepmsk | SCD_F_DAINTMSK_INEPMSK(3)); */
#define SCD_R_DAINTMSK	0x81c

#define SCD_B_DAINTMSK_OUTEPMSK		BIT(16)
#define SCD_M_DAINTMSK_OUTEPMSK		0xffff /* 16 bits mask */
#define SCD_S_DAINTMSK_OUTEPMSK		16
#define SCD_F_DAINTMSK_OUTEPMSK(EPN)	BIT((EPN) + 16)
#define SCD_B_DAINTMSK_INEPMSK		BIT(0)
#define SCD_M_DAINTMSK_INEPMSK		0xffff /* 16 bits mask */
#define SCD_S_DAINTMSK_INEPMSK		0
#define SCD_F_DAINTMSK_INEPMSK(EPN)	BIT(EPN)

/* DVBUSDIS : Device VBUS Discharge Time Register */
#define SCD_R_DVBUSDIS	0x828

#define SCD_B_DVBUSDIS_DVBUSDIS		BIT(0)
#define SCD_M_DVBUSDIS_DVBUSDIS		0xffff /* 16 bits mask */
#define SCD_S_DVBUSDIS_DVBUSDIS		0

/* DVBUSPULSE : Device VBUS Pulsing Time Register */
#define SCD_R_DVBUSPULSE	0x82c

#define SCD_B_DVBUSPULSE_DVBUSPULSE	BIT(0)
#define SCD_M_DVBUSPULSE_DVBUSPULSE	0xfff /* 12 bits mask */
#define SCD_S_DVBUSPULSE_DVBUSPULSE	0

/* DTHRCTL : Device Threshold Control Register */
#define SCD_R_DTHRCTL	0x830

#define SCD_B_DTHRCTL_ARBPRKEN		BIT(27)
#define SCD_B_DTHRCTL_RXTHRLEN		BIT(17)
#define SCD_M_DTHRCTL_RXTHRLEN		0x1ff /* 9 bits mask */
#define SCD_S_DTHRCTL_RXTHRLEN		17
#define SCD_B_DTHRCTL_RXTHREN		BIT(16)
#define SCD_B_DTHRCTL_AHBTHRRATIO	BIT(11)
#define SCD_M_DTHRCTL_AHBTHRRATIO	0x3 /* 2 bits mask */
#define SCD_S_DTHRCTL_AHBTHRRATIO	11
#define SCD_E_DTHRCTL_AHBTHRRATIO_DIV1	0x0
#define SCD_E_DTHRCTL_AHBTHRRATIO_DIV2	0x1
#define SCD_E_DTHRCTL_AHBTHRRATIO_DIV4	0x2
#define SCD_E_DTHRCTL_AHBTHRRATIO_DIV8	0x3
#define SCD_B_DTHRCTL_TXTHRLEN		BIT(2)
#define SCD_M_DTHRCTL_TXTHRLEN		0x1ff /* 9 bits mask */
#define SCD_S_DTHRCTL_TXTHRLEN		2
#define SCD_B_DTHRCTL_ISOTHREN		BIT(1)
#define SCD_B_DTHRCTL_NONISOTHREN	BIT(0)

/* DIEPEMPMSK : Device IN Endpoint FIFO Empty Interrupt Mask Register */
#define SCD_R_DIEPEMPMSK	0x834

#define SCD_B_DIEPEMPMSK_INEPTXFEMPMSK	BIT(0)
#define SCD_M_DIEPEMPMSK_INEPTXFEMPMSK	0xffff /* 16 bits mask */
#define SCD_S_DIEPEMPMSK_INEPTXFEMPMSK	0

/* DIEPCTL0 : Device Control IN Endpoint 0 Control Register */
#define SCD_R_DIEPCTL0	0x900

#define SCD_B_DIEPCTL0_EPENA		BIT(31)
#define SCD_B_DIEPCTL0_EPDIS		BIT(30)
#define SCD_B_DIEPCTL0_SNAK		BIT(27)
#define SCD_B_DIEPCTL0_CNAK		BIT(26)
#define SCD_B_DIEPCTL0_TXFNUM		BIT(22)
#define SCD_M_DIEPCTL0_TXFNUM		0xf /* 4 bits mask */
#define SCD_S_DIEPCTL0_TXFNUM		22
#define SCD_B_DIEPCTL0_STALL		BIT(21)
#define SCD_B_DIEPCTL0_EPTYPE		BIT(18)
#define SCD_M_DIEPCTL0_EPTYPE		0x3 /* 2 bits mask */
#define SCD_S_DIEPCTL0_EPTYPE		18
#define SCD_E_DIEPCTL0_EPTYPE_CONTROL	0x0
#define SCD_B_DIEPCTL0_NAKSTS		BIT(17)
#define SCD_B_DIEPCTL0_USBACTEP		BIT(15)
#define SCD_B_DIEPCTL0_NEXTEP		BIT(11)
#define SCD_M_DIEPCTL0_NEXTEP		0xf /* 4 bits mask */
#define SCD_S_DIEPCTL0_NEXTEP		11
#define SCD_B_DIEPCTL0_MPS		BIT(0)
#define SCD_M_DIEPCTL0_MPS		0x3 /* 2 bits mask */
#define SCD_S_DIEPCTL0_MPS		0
#define SCD_E_DIEPCTL0_MPS_64B		0x0
#define SCD_E_DIEPCTL0_MPS_32B		0x1
#define SCD_E_DIEPCTL0_MPS_16B		0x2
#define SCD_E_DIEPCTL0_MPS_8B		0x3

/* DOEPCTL0 : Device Control OUT Endponit 0 Control Register */
#define SCD_R_DOEPCTL0	0xb00

#define SCD_B_DOEPCTL0_EPENA		BIT(31)
#define SCD_B_DOEPCTL0_EPDIS		BIT(30)
#define SCD_B_DOEPCTL0_SNAK		BIT(27)
#define SCD_B_DOEPCTL0_CNAK		BIT(26)
#define SCD_B_DOEPCTL0_STALL		BIT(21)
#define SCD_B_DOEPCTL0_SNP		BIT(20)
#define SCD_B_DOEPCTL0_EPTYPE		BIT(18)
#define SCD_M_DOEPCTL0_EPTYPE		0x3 /* 2 bits mask */
#define SCD_S_DOEPCTL0_EPTYPE		18
#define SCD_E_DOEPCTL0_EPTYPE_CONTROL	0x0
#define SCD_B_DOEPCTL0_NAKSTS		BIT(17)
#define SCD_B_DOEPCTL0_USBACTEP		BIT(15)
#define SCD_B_DOEPCTL0_MPS		BIT(0)
#define SCD_M_DOEPCTL0_MPS		0x3 /* 2 bits mask */
#define SCD_S_DOEPCTL0_MPS		0
#define SCD_E_DOEPCTL0_MPS_64B		0x0
#define SCD_E_DOEPCTL0_MPS_32B		0x1
#define SCD_E_DOEPCTL0_MPS_16B		0x2
#define SCD_E_DOEPCTL0_MPS_8B		0x3

/* DxEPCTLn(= DIEPCTLn/DOEPCTLn) : Device Endpoint-n Control Register
   exget EP3 Enable
         ep3inen = SCD_READ(device, SCD_R_DIEPCTL(3)) & SCD_B_DXEPCTLN_EPENA;
       SCD_READͿ쥸̾'N'դʤȤա
       ӥåΥ쥸̾IN/OUTʬXǤ뤳Ȥա */
#define SCD_R_DIEPCTL(EPN)	(0x900 + (EPN) * 0x20)
#define SCD_R_DOEPCTL(EPN)	(0xb00 + (EPN) * 0x20)

#define SCD_B_DXEPCTLN_EPENA		BIT(31)
#define SCD_B_DXEPCTLN_EPDIS		BIT(30)
#define SCD_B_DXEPCTLN_SETD1PID		BIT(29)
#define SCD_B_DXEPCTLN_SETD0PID		BIT(28)
#define SCD_B_DXEPCTLN_SNAK		BIT(27)
#define SCD_B_DXEPCTLN_CNAK		BIT(26)
#define SCD_B_DXEPCTLN_TXFNUM		BIT(22)
#define SCD_M_DXEPCTLN_TXFNUM		0xf /* 4 bits mask */
#define SCD_S_DXEPCTLN_TXFNUM		22
#define SCD_B_DXEPCTLN_STALL		BIT(21)
#define SCD_B_DXEPCTLN_SNP		BIT(20)
#define SCD_B_DXEPCTLN_EPTYPE		BIT(18)
#define SCD_M_DXEPCTLN_EPTYPE		0x3 /* 2 bits mask */
#define SCD_S_DXEPCTLN_EPTYPE		18
#define SCD_E_DXEPCTLN_EPTYPE_CONTROL	0x0
#define SCD_E_DXEPCTLN_EPTYPE_ISO	0x1
#define SCD_E_DXEPCTLN_EPTYPE_BULK	0x2
#define SCD_E_DXEPCTLN_EPTYPE_INTERRUPT	0x3
#define SCD_B_DXEPCTLN_NAKSTS		BIT(17)
#define SCD_B_DXEPCTLN_DPID		BIT(16)
#define SCD_B_DXEPCTLN_USBACTEP		BIT(15)
#define SCD_B_DXEPCTLN_NEXTEP		BIT(11)
#define SCD_M_DXEPCTLN_NEXTEP		0xf /* 4 bits mask */
#define SCD_S_DXEPCTLN_NEXTEP		11
#define SCD_B_DXEPCTLN_MPS		BIT(0)
#define SCD_M_DXEPCTLN_MPS		0x7ff /* 11 bits mask */
#define SCD_S_DXEPCTLN_MPS		0

/* DxEPINTn(= DIEPINTn/DOEPINTn) : Device Endpoint-n Interrupt Register
   exget EP3 NYET interrupt
         ep3innyetint = SCD_READ(device, SCD_R_DIEPINT(3)) &
                          SCD_B_DXEPINTN_NYETINTRPT;
       SCD_READͿ쥸̾'N'դʤȤա
       ӥåΥ쥸̾IN/OUTʬXǤ뤳Ȥա */
#define SCD_R_DIEPINT(EPN)	(0x908 + (EPN) * 0x20)
#define SCD_R_DOEPINT(EPN)	(0xb08 + (EPN) * 0x20)

#define SCD_B_DXEPINTN_NYETINTRPT	BIT(14)
#define SCD_B_DXEPINTN_NAKINTRPT	BIT(13)
#define SCD_B_DXEPINTN_BBLEERRINTRPT	BIT(12)
#define SCD_B_DXEPINTN_PKTDRPSTS	BIT(11)
#define SCD_B_DXEPINTN_BNAINTR		BIT(9)
#define SCD_B_DXEPINTN_TXFIFOUNDRN	BIT(8)
#define SCD_B_DXEPINTN_OUTPKTERR	BIT(8)
#define SCD_B_DXEPINTN_TXFEMP		BIT(7)
#define SCD_B_DXEPINTN_INEPNAKEFF	BIT(6)
#define SCD_B_DXEPINTN_BACK2BACKSETUP	BIT(6)
#define SCD_B_DXEPINTN_INTKNEPMIS	BIT(5)
#define SCD_B_DXEPINTN_STSPHSERCVD	BIT(5)
#define SCD_B_DXEPINTN_INTKNTXFEMP	BIT(4)
#define SCD_B_DXEPINTN_OUTTKNEPDIS	BIT(4)
#define SCD_B_DXEPINTN_TIMEOUT		BIT(3)
#define SCD_B_DXEPINTN_SETUP		BIT(3)
#define SCD_B_DXEPINTN_AHBERR		BIT(2)
#define SCD_B_DXEPINTN_EPDISBLD		BIT(1)
#define SCD_B_DXEPINTN_XFERCOMPL	BIT(0)

/* DIEPTSIZ0 : Device IN Endpoint 0 Transfer Size Register */
#define SCD_R_DIEPTSIZ0		0x910

#define SCD_B_DIEPTSIZ0_PKTCNT		BIT(19)
#define SCD_M_DIEPTSIZ0_PKTCNT		0x3 /* 2 bits mask */
#define SCD_S_DIEPTSIZ0_PKTCNT		19
#define SCD_B_DIEPTSIZ0_XFERSIZE	BIT(0)
#define SCD_M_DIEPTSIZ0_XFERSIZE	0x7f /* 7bits mask */
#define SCD_S_DIEPTSIZ0_XFERSIZE	0

/* DOEPTSIZ0 : Device OUT Endpoint 0 Transfer Size Register */
#define SCD_R_DOEPTSIZ0		0xb10

#define SCD_B_DOEPTSIZ0_SUPCNT		BIT(29)
#define SCD_M_DOEPTSIZ0_SUPCNT		0x3 /* 2 bits mask */
#define SCD_S_DOEPTSIZ0_SUPCNT		29
#define SCD_B_DOEPTSIZ0_PKTCNT		BIT(19)
#define SCD_M_DOEPTSIZ0_PKTCNT		0x3 /* 2 bits mask */
#define SCD_S_DOEPTSIZ0_PKTCNT		19
#define SCD_B_DOEPTSIZ0_XFERSIZE	BIT(0)
#define SCD_M_DOEPTSIZ0_XFERSIZE	0x7f /* 7 bits mask */
#define SCD_S_DOEPTSIZ0_XFERSIZE	0

/* DxEPTSIZn(= DIEPTSIZn/DOEPTSIZn) : Device Endpoint-n Transfer Size Register
   exget EP3 IN Packet Count
         ep3inpktcnt = SCD_GET_FIELD(SCD_READ(device, SCD_R_DIEPTSIZ(3)),
                                     SCD_S_DXEPTSIZN_PKTCNT, 
                                     SCD_M_DXEPTSIZN_PKTCNT);
       SCD_READͿ쥸̾'N'դʤȤա
       ӥåΥ쥸̾IN/OUTʬXǤ뤳Ȥա */
#define SCD_R_DIEPTSIZ(EPN)	(0x910 + (EPN) * 0x20)
#define SCD_R_DOEPTSIZ(EPN)	(0xb10 + (EPN) * 0x20)

#define SCD_B_DXEPTSIZN_MC		BIT(29)
#define SCD_M_DXEPTSIZN_MC		0x3 /* 2 bits mask */
#define SCD_S_DXEPTSIZN_MC		29
#define SCD_B_DXEPTSIZN_RXDPID		BIT(29)
#define SCD_M_DXEPTSIZN_RXDPID		0x3 /* 2 bits mask */
#define SCD_S_DXEPTSIZN_RXDPID		29
#define SCD_E_DXEPTSIZN_RXDPID_DATA0	0x0
#define SCD_E_DXEPTSIZN_RXDPID_DATA1	0x1
#define SCD_E_DXEPTSIZN_RXDPID_DATA2	0x2
#define SCD_E_DXEPTSIZN_RXDPID_MDATA	0x3
#define SCD_B_DXEPTSIZN_SUPCNT		BIT(29)
#define SCD_M_DXEPTSIZN_SUPCNT		0x3 /* 2 bits mask */
#define SCD_S_DXEPTSIZN_SUPCNT		29
#define SCD_B_DXEPTSIZN_PKTCNT		BIT(19)
#define SCD_M_DXEPTSIZN_PKTCNT		0x3ff /* 10 bits mask */
#define SCD_S_DXEPTSIZN_PKTCNT		19
#define SCD_B_DXEPTSIZN_XFERSIZE	BIT(0)
#define SCD_M_DXEPTSIZN_XFERSIZE	0x7ffff	/* 19 bits mask */
#define SCD_S_DXEPTSIZN_XFERSIZE	0

/* DxEPDMAn(= DIEPDMAn/DOEPDMAn) : Device Endpoint-n DMAAddress Register
       SCD_READͿ쥸̾'N'դʤȤա
       ӥåΥ쥸̾IN/OUTʬXǤ뤳Ȥա */
#define SCD_R_DIEPDMA(EPN)	(0x914 + (EPN) * 0x20)
#define SCD_R_DOEPDMA(EPN)	(0xb14 + (EPN) * 0x20)

#define SCD_B_DXEPDMAN_DMAADDR		BIT(0)
#define SCD_M_DXEPDMAN_DMAADDR		0xffffffff /* 32 bits mask */
#define SCD_S_DXEPDMAN_DMAADDR		0

/* DTXFSTSn : Device IN Endpoint Transmit FIFO Status Register */
#define SCD_R_DTXFSTS(EPN)	(0x918 + (EPN) * 0x20)

#define SCD_B_DTXFSTSN_INEPTXFSPCAVAIL	BIT(0)
#define SCD_M_DTXFSTSN_INEPTXFSPCAVAIL	0xffff /* 16 bits mask (max) */
#define SCD_S_DTXFSTSN_INEPTXFSPCAVAIL	0

/*
 *	scd Power & Clock Gating 쥸
 */

/* PCGCCTL : Power and Clock Gating Control Register */
#define SCD_R_PCGCCTL		0xe00

#define SCD_B_PCGCCTL_RESETAFTERSUSP	BIT(8)
#define SCD_B_PCGCCTL_L1SUSPENDED	BIT(7)
#define SCD_B_PCGCCTL_PHYSLEEP		BIT(6)
#define SCD_B_PCGCCTL_ENBLL1GATING	BIT(5)
#define SCD_B_PCGCCTL_RSTPDWNMODULE	BIT(3)
#define SCD_B_PCGCCTL_PWRCLMP		BIT(2)
#define SCD_B_PCGCCTL_GATEHCLK		BIT(1)
#define SCD_B_PCGCCTL_STOPPCLK		BIT(0)

#define SCD_R_DFIFO	0x1000

#endif  /* _SCD_REGISTER_H_ */
