/*--------------------------------------------------------------------------*
 * drivers/usb/scd/scd_sysdep_integrator.c
 * 
 * System dependent definitions
 * 
 * Copyright 2011 Sony Corporation
 *
 * This file is part of the HS-OTG Controller Driver SCD.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 *---------------------------------------------------------------------------*/

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/init.h>
#include <linux/list.h>
#include <linux/interrupt.h>
#include <linux/time.h>
#include <linux/timer.h>
#include <linux/platform_device.h>

#include "scd.h"
#include "scd_debug.h"

static int cancel_timeout(struct scd_control *control);

/* IntegratorLinkĤʤ */
static struct scd_sysdep_control {
	struct scd_device *sdev;
	struct scd_control control;
	struct timer_list timer;
} sysdepc;

static inline
struct scd_sysdep_control *control_to_sysdep(struct scd_control* control)
{
	return container_of(control, struct scd_sysdep_control, control);
}

/* timer handler */
static void timer_function(unsigned long ptr)
{
	sysdepc.sdev->ops->timeout(sysdepc.sdev);
}

/*
 *	scd intf ops
 */

static struct scd_control *probe(struct scd_device *sdev)
{
	/* staticǥ֥Ƥ뤿ᡢؤɤ뤳̵ */
	sysdepc.sdev = sdev;

	init_timer(&sysdepc.timer);
	sysdepc.timer.function = timer_function;
	sysdepc.timer.data = 0;

	return &sysdepc.control;
}

static int remove(struct scd_control *control)
{
	/* staticǥ֥Ƥ뤿ᡢؤɤ뤳̵ */
	sysdepc.sdev = NULL;

	return 0;
}

static void shutdown(struct scd_control *control)
{
	cancel_timeout(control);
}

static int start(struct scd_control *control)
{
	struct scd_sysdep_control *sysdepc;

	sysdepc = control_to_sysdep(control);

	// ¾
	/*
	  ƥ९å(USB HCLK)γ
	 */
	// ¾λ

	return 0;
}

static int stop(struct scd_control *control)
{
	struct scd_sysdep_control *sysdepc;

	sysdepc = control_to_sysdep(control);

	// ¾
	/*
	  ƥ९å(USB HCLK)νλ
	 */
	// ¾λ
	del_timer(&sysdepc->timer);

	return 0;
}

static scd_usec get_time(struct scd_control *control)
{
	struct timeval tv;

	do_gettimeofday(&tv);

	return (scd_usec)(tv.tv_sec * 1000000 + tv.tv_usec);
}

static int set_timeout(struct scd_control *control, scd_usec expire)
{
	struct scd_sysdep_control *sysdepc;
	scd_usec timeout;
	unsigned long jexpire;

	sysdepc = control_to_sysdep(control);
	timeout = scd_dec_usec(expire, get_time(control));
	if (timeout < 0) {
		timeout = 0;
	}
	jexpire = jiffies + usecs_to_jiffies(timeout) + 1;
	mod_timer(&sysdepc->timer, jexpire);

	return 0;
}

static int cancel_timeout(struct scd_control *control)
{
	struct scd_sysdep_control *sysdepc;

	sysdepc = control_to_sysdep(control);

	del_timer(&sysdepc->timer);

	return 0;
}

int power(struct scd_control *control, bool onoff, int port)
{
	return 0;
}

static struct scd_sysdep_intf sysdep_intf = {
	.base_intf = {
		.type = SCD_CONTROL_TYPE_SYSDEP,
		.probe = probe,
		.remove = remove,
		.shutdown = shutdown,
		.start = start,
		.stop = stop,
	},
	.set_timeout = set_timeout,
	.cancel_timeout = cancel_timeout,
	.get_time = get_time,
	.power = power,
};

/*
 * device
 */
/* ɥ쥹 */
#define ADR_START	0xC0000000
#define ADR_END  	0xC0001000
#define IRQ_NUM		9

static struct resource scd_resources[] = {
	[0] = {
		.start = ADR_START,
		.end = ADR_END,
		.flags = IORESOURCE_MEM,
	},
	[1] = {
		.name = "USB0",
		.start = IRQ_NUM,
		.end = 0,
		.flags = IORESOURCE_IRQ,
	},
};

static u64 scd_dma_mask = 0xffffffffUL;

/* dma dummy release */
static void nop_release(struct device *dev)
{
    DBG(1, "dev_name=%s\n", dev_name(dev));
}

struct platform_device scd_platform_device = {
	/* name fieldscd_info.cˤƥե̾ǻѤǤʸȤ뤳 */
	.name = "usb_scd",
	.id = 0,
	.dev = {
		.dma_mask = &scd_dma_mask,
		.coherent_dma_mask = 0xffffffffUL,
		.release = nop_release,
		},
	.num_resources = ARRAY_SIZE(scd_resources),
	.resource = scd_resources,
};

/* ؿ - otgľܸƤФ */
int __init usb_scd_sysdep_init(void)
{
	usb_scd_device_set_intf(&sysdep_intf.base_intf);

    	/* Platform deviceȤϿ */
	platform_device_register(&scd_platform_device);

	return 0;
}

/* λؿ */
void __exit usb_scd_sysdep_exit(void)
{
	platform_device_unregister(&scd_platform_device);

	usb_scd_device_unset_intf(&sysdep_intf.base_intf);
}
