/*--------------------------------------------------------------------------*
 * include/linux/usb/scd/usb_otg_control.h
 * 
 * USB controller driver OTG API
 * 
 * Copyright 2005,2006,2008,2011 Sony Corporation
 *
 * This file is part of the HS-OTG Controller Driver SCD.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 *---------------------------------------------------------------------------*/

#ifndef _USB_OTG_CONTROL_H_
#define _USB_OTG_CONTROL_H_

/*
 *	¤
 */

/*
 * 󷿡usb_otg_vbus_stat
 * VbusŰ֤򼨤UNKOWN=-1, OFF=0, LOW=1, SESS=2, VALID=3
 */

enum usb_otg_vbus_stat {
	USB_OTG_VBUS_STAT_UNKNOWN = -1,
	USB_OTG_VBUS_STAT_OFF = 0,
	USB_OTG_VBUS_STAT_LOW = 1,
	USB_OTG_VBUS_STAT_SESS = 2,
	USB_OTG_VBUS_STAT_VALID = 3,
};

/*
 * 󷿡usb_otg_control_speed
 * ̿®٤򼨤
 */

enum usb_otg_control_speed {
	USB_OTG_SPEED_LS,
	USB_OTG_SPEED_FS,
	USB_OTG_SPEED_HS,
};

/*
 * 󷿡usb_otg_control_test_mode
 * ̿®٤򼨤
 */

enum usb_otg_control_test_mode {
	USB_OTG_TEST_MODE_NORMAL,
	/* Gadgetɸƥ */
	USB_OTG_TEST_MODE_G_PACKET,
	USB_OTG_TEST_MODE_G_K,
	USB_OTG_TEST_MODE_G_J,
	USB_OTG_TEST_MODE_G_SE0_NAK,
	/* Gadgetϡɰ¸ƥ */
	USB_OTG_TEST_MODE_G_FORCE_PACKET,	/* ѥåХ⡼ */
	/* Hostɸƥ */
	USB_OTG_TEST_MODE_H_PACKET,
	USB_OTG_TEST_MODE_H_K,
	USB_OTG_TEST_MODE_H_J,
	USB_OTG_TEST_MODE_H_SE0_NAK,
	/* Hostϡɰ¸ƥ */
	USB_OTG_TEST_MODE_H_SETUP,
	USB_OTG_TEST_MODE_H_IN,
	USB_OTG_TEST_MODE_H_AS,	/* ®̵ */
	USB_OTG_TEST_MODE_H_LS,	/* LowSpeed */
	USB_OTG_TEST_MODE_H_FS,	/* FullSpeed */
	USB_OTG_TEST_MODE_H_HS,	/* HighSpeed */
};

/*
 * 󷿡usb_otg_line_state_value
 * linestate򼨤
 */
enum usb_otg_line_state_value {
        USB_OTG_LINE_STATE_LOW = 0,
        USB_OTG_LINE_STATE_HIGH = 1,
};

/*
 * 󷿡usb_otg_event_type
 * OTG٥ȥפ򼨤
 */

enum usb_otg_event_type {
	USB_OTG_EVENT_TYPE_CID = 1,
	USB_OTG_EVENT_TYPE_VBUS,
	USB_OTG_EVENT_TYPE_VBUSERROR,
	USB_OTG_EVENT_TYPE_CON,
	USB_OTG_EVENT_TYPE_SRP,
	USB_OTG_EVENT_TYPE_RCHOST,
	USB_OTG_EVENT_TYPE_RCGADGET,
};

/*
 * ¤Ρusb_otg_event_cid
 * CIDͤѲ򼨤
 */

struct usb_otg_event_cid {
	enum usb_otg_event_type type;
	unsigned int value;
	unsigned int port;
};

/*
 * ¤Ρusb_otg_event_vbus
 * VbusŰ֤򼨤
 */

struct usb_otg_event_vbus {
	enum usb_otg_event_type type;
	enum usb_otg_vbus_stat vbus_stat;
	unsigned int port;
};

/*
 * ¤Ρusb_otg_event_vbuserror
 * Vbusΰ۾򼨤
 */

struct usb_otg_event_vbuserror {
	enum usb_otg_event_type type;
};

/*
 * ¤Ρusb_otg_event_con
 * Hostư³GadgetD+ Pullup˴Ť³Ǥ򼨤
 */

struct usb_otg_event_con {
	enum usb_otg_event_type type;
	unsigned int value;
};

/*
 * ¤Ρusb_otg_event_srp
 * a-device³GadgetSRPѥ륷󥰤򼨤
 */

struct usb_otg_event_srp {
	enum usb_otg_event_type type;
};

/*
 * ¤Ρusb_otg_event_rc
 * HNPRoleChangeδλӤݤ򼨤
 */

struct usb_otg_event_rc {
	enum usb_otg_event_type type;
	unsigned int status;
};

#define USB_OTG_SUCCESS			0	/* RoleChange */
#define USB_OTG_FAIL_T_B_ASE0_BRST	1	/* b-deviceRcHostD+饤
						   Pullup OFF3.125[ms]³
						   a-devicePullup ON򤷤ʤä
						 */
#define USB_OTG_FAIL_T_A_AIDL_BDIS	2	/* a-deviceRcGadgetХ򥵥
						   ɤˤƤ200[ms]³
						   b-deviceD+饤Pullup OFF
						   ʤä */

/*
 * Ρusb_otg_event
 * OTG٥ȤɽControllerˤUSBOTG˴ؤ륤٥ȤꤷAPIؿ
 * usb_otg_core_notifyˤCoreΤ롣
 */

union usb_otg_event {
	enum usb_otg_event_type type;
	struct usb_otg_event_cid cid;
	struct usb_otg_event_vbus vbus;
	struct usb_otg_event_vbuserror vbuserror;
	struct usb_otg_event_con con;
	struct usb_otg_event_srp srp;
	struct usb_otg_event_rc rchost;
	struct usb_otg_event_rc rcgadget;
};

/*
 * ¤ : usb_otg_control_port_info
 * USBݡȤξɽʣUSBݡȤ¾Ū˻Ѥݤ
 * usb_otg_control_get_port_infoˤݡȿȸ򤵤Ƥݡֹ
 * 롣
 */

struct usb_otg_control_port_info {
	unsigned int nr;
	unsigned int current_port;
};

/*
 * ¤Ρusb_otg_line_state
 * linestateξּIOCTLؿˤƼ롣
 */
struct usb_otg_line_state {
        uint8_t dp;             /* D+ line state */
	uint8_t dm;             /* D- line state */
	uint16_t reserved;      /* Reserved */
};

/*
 * ¤Ρusb_otg_control_ops
 * OTGȥؿɽؿб륤饤ޥ
 * usb_otg_control.hƤꡢľܤؿѤɬפ̵
 * ξܺϡؿιܤˤб륤饤ؿ򻲾Ȥ
 * ȡ
 */

struct usb_otg_control;		/* prototype */

struct usb_otg_control_ops {
	int (*get_port_info) (struct usb_otg_control *,
			      struct usb_otg_control_port_info * info);
	int (*select_port) (struct usb_otg_control *, unsigned int pn);
	int (*start_control) (struct usb_otg_control *);
	int (*stop_control) (struct usb_otg_control *);
	int (*start_gadget) (struct usb_otg_control *);
	int (*stop_gadget) (struct usb_otg_control *);
	int (*enable_rchost) (struct usb_otg_control *);
	int (*disable_rchost) (struct usb_otg_control *);
	int (*stop_rchost) (struct usb_otg_control *);
	int (*start_host) (struct usb_otg_control *);
	int (*stop_host) (struct usb_otg_control *);
	int (*start_rcgadget) (struct usb_otg_control *, unsigned int port);
	int (*stop_rcgadget) (struct usb_otg_control *, unsigned int port);
	int (*get_mode) (struct usb_otg_control *);
	int (*request_session) (struct usb_otg_control *);
	int (*set_speed) (struct usb_otg_control *,
			  enum usb_otg_control_speed speed);
	enum usb_otg_control_speed (*get_speed) (struct usb_otg_control *);
	int (*set_test_mode) (struct usb_otg_control *,
			      enum usb_otg_control_test_mode test_mode);
	enum usb_otg_control_test_mode (*get_test_mode) (struct usb_otg_control
							 *);
        int (*ioctl) (struct usb_otg_control *, unsigned, unsigned long);
};

/*
 * ¤Ρusb_otg_control
 * OTGȥɽǤȤơOTGȥؿ
 * ӥ桼ؿˤƻѤOTGȥμ̾ġ
 */

struct usb_otg_control {
	struct usb_otg_control_ops *ops;
	struct usb_otg_core *otg_core;
	const char *name;
	void *data;
};

/*
 * ¤Ρusb_otg_core
 * OTG coreɽǤȤơOTGȥХɤȥХɤ
 * USB٥ȤΤؿġ
 */

struct usb_otg_core {
	int (*bind) (struct usb_otg_control * otg_control);
	int (*unbind) (struct usb_otg_control * otg_control);
	int (*notify) (union usb_otg_event * otg_event);
};

/*
 *	ؿ
 */

/*
 * ؿusb_otg_control_register_core
 * OTG Core򥳥ȥϿ롣
 */

extern int usb_otg_control_register_core(struct usb_otg_core *otg_core);

/*
 * ؿusb_otg_control_unregister_core
 * OTG Core򥳥ȥϿ롣
 */

extern int usb_otg_control_unregister_core(struct usb_otg_core *otg_core);

/*
 * ؿusb_otg_core_notify
 * OTGȥ餫Coreإ٥ȤΤ롣
 */

static inline int
usb_otg_core_notify(struct usb_otg_core *otg_core,
		    union usb_otg_event *otg_event)
{
	return otg_core->notify(otg_event);
}

/*
 * ؿusb_otg_core_bind
 * OTGȥCoreإХɤ롣
 */

static inline int
usb_otg_core_bind(struct usb_otg_core *otg_core,
		  struct usb_otg_control *otg_control)
{
	return otg_core->bind(otg_control);
}

/*
 * ؿusb_otg_core_unbind
 * OTGȥCoreإХɤ롣
 */

static inline int
usb_otg_core_unbind(struct usb_otg_core *otg_core,
                    struct usb_otg_control *otg_control)
{
	return otg_core->unbind(otg_control);
}

/*
 * ؿusb_otg_control_get_port_info
 * USBݡȾ롣
 */

static inline int
usb_otg_control_get_port_info(struct usb_otg_control *otg_control,
			      struct usb_otg_control_port_info *info)
{
	if (!otg_control) {
		return -EINVAL;
	}
	return otg_control->ops->get_port_info(otg_control, info);
}

/*
 * ؿusb_otg_control_select_port
 * USBݡȤ򤹤롣
 */

static inline int
usb_otg_control_select_port(struct usb_otg_control *otg_control,
			    unsigned int pn)
{
	if (!otg_control) {
		return -EINVAL;
	}
	return otg_control->ops->select_port(otg_control, (u32)pn);
}

/*
 * ؿusb_otg_control_start_control
 * Controller򳫻Ϥ롣
 */

static inline int
usb_otg_control_start_control(struct usb_otg_control *otg_control)
{
	if (!otg_control) {
		return -EINVAL;
	}
	return otg_control->ops->start_control(otg_control);
}

/*
 * ؿusb_otg_control_stop_control
 * Controllerߤ롣
 */

static inline int
usb_otg_control_stop_control(struct usb_otg_control *otg_control)
{
	if (!otg_control) {
		return -EINVAL;
	}
	return otg_control->ops->stop_control(otg_control);
}

/*
 * ؿusb_otg_control_start_gadget
 * ControllerGADGET⡼ɤˤ롣
 */

static inline int
usb_otg_control_start_gadget(struct usb_otg_control *otg_control)
{
	if (!otg_control) {
		return -EINVAL;
	}
	return otg_control->ops->start_gadget(otg_control);
}

/*
 * ؿusb_otg_control_stop_gadget
 * ControllerIDLE⡼ɤˤ롣
 */

static inline int
usb_otg_control_stop_gadget(struct usb_otg_control *otg_control)
{
	if (!otg_control) {
		return -EINVAL;
	}
	return otg_control->ops->stop_gadget(otg_control);
}

/*
 * ؿusb_otg_control_enable_rchost
 * ControllerRCHOST⡼ɤˤʤ뤳ȤĤ롣
 */

static inline int
usb_otg_control_enable_rchost(struct usb_otg_control *otg_control)
{
	if (!otg_control) {
		return -EINVAL;
	}
	return otg_control->ops->enable_rchost(otg_control);
}

/*
 * ؿusb_otg_control_disable_rchost
 * ControllerRCHOST⡼ɤˤʤ뤳ȤԵĤȤ롣
 */

static inline int
usb_otg_control_disable_rchost(struct usb_otg_control *otg_control)
{
	if (!otg_control) {
		return -EINVAL;
	}
	return otg_control->ops->disable_rchost(otg_control);
}

/*
 * ؿusb_otg_control_stop_rchost
 * ControllerIDLE⡼ɤˤ롣
 */

static inline int
usb_otg_control_stop_rchost(struct usb_otg_control *otg_control)
{
	if (!otg_control) {
		return -EINVAL;
	}
	return otg_control->ops->stop_rchost(otg_control);
}

/*
 * ؿusb_otg_control_start_host
 * ControllerHOST⡼ɤˤ롣
 */

static inline int
usb_otg_control_start_host(struct usb_otg_control *otg_control)
{
	if (!otg_control) {
		return -EINVAL;
	}
	return otg_control->ops->start_host(otg_control);
}

/*
 * ؿusb_otg_control_stop_host
 * ControllerIDLE⡼ɤˤ롣
 */

static inline int usb_otg_control_stop_host(struct usb_otg_control *otg_control)
{
	if (!otg_control) {
		return -EINVAL;
	}
	return otg_control->ops->stop_host(otg_control);
}

/*
 * ؿusb_otg_control_start_rcgadget
 * ControllerRCGADGET⡼ɤˤ롣
 */

static inline int
usb_otg_control_start_rcgadget(struct usb_otg_control *otg_control,
			       unsigned int port)
{
	if (!otg_control) {
		return -EINVAL;
	}
	return otg_control->ops->start_rcgadget(otg_control, port);
}

/*
 * ؿusb_otg_control_stop_rcgadget
 * ControllerIDLE⡼ɤˤ롣
 */

static inline int
usb_otg_control_stop_rcgadget(struct usb_otg_control *otg_control,
			      unsigned int port)
{
	if (!otg_control) {
		return -EINVAL;
	}
	return otg_control->ops->stop_rcgadget(otg_control, port);
}

/*
 * ؿusb_otg_control_get_mode
 * OTG⡼ɤ롣
 */

static inline int usb_otg_control_get_mode(struct usb_otg_control *otg_control)
{
	if (!otg_control) {
		return -EINVAL;
	}
	return otg_control->ops->get_mode(otg_control);
}

#define USB_OTG_CONTROL_STOP		1	/* STOP⡼ */
#define USB_OTG_CONTROL_IDLE		2	/* IDLE⡼ */
#define USB_OTG_CONTROL_GADGET		3	/* GADGET⡼ */
#define USB_OTG_CONTROL_RCHOST		4	/* RCHOST⡼ */
#define USB_OTG_CONTROL_HOST		5	/* HOST⡼ */
#define USB_OTG_CONTROL_RCGADGET	6	/* RCGADGET⡼ */

/*
 * ؿusb_otg_control_request_session
 * ControllerSRPѥ륷󥰤׵᤹롣
 */

static inline int
usb_otg_control_request_session(struct usb_otg_control *otg_control)
{
	if (!otg_control) {
		return -EINVAL;
	}
	return otg_control->ops->request_session(otg_control);
}

/*
 * ؿusb_otg_control_set_speed
 * ̿ԡɤꤹ롣
 */

static inline int
usb_otg_control_set_speed(struct usb_otg_control *otg_control,
			  unsigned int speed)
{
	if (!otg_control) {
		return -EINVAL;
	}
	return otg_control->ops->set_speed(otg_control, speed);
}

#define USB_OTG_HS_DISABLE	1	/* High SpeedĤʤ */
#define USB_OTG_HS_ENABLE	2	/* High SpeedĤ롣 */

/*
 * ؿusb_otg_control_get_speed
 * ̿ԡɤ롣
 */

static inline enum usb_otg_control_speed
usb_otg_control_get_speed(struct usb_otg_control *otg_control)
{
	if (!otg_control) {
		return -EINVAL;
	}
	return otg_control->ops->get_speed(otg_control);
}

/*
 * ؿusb_otg_control_set_test_mode
 * ƥȥ⡼ɤꤹ롣
 */

static inline int
usb_otg_control_set_test_mode(struct usb_otg_control *otg_control,
			      enum usb_otg_control_test_mode test_mode)
{
	if (!otg_control) {
		return -EINVAL;
	}
	return otg_control->ops->set_test_mode(otg_control, test_mode);
}

/*
 * ؿusb_otg_control_get_test_mode
 * ƥȥ⡼ɤ롣
 */

static inline enum usb_otg_control_test_mode
usb_otg_control_get_test_mode(struct usb_otg_control *otg_control)
{
	if (!otg_control) {
		return -EINVAL;
	}
	return otg_control->ops->get_test_mode(otg_control);
}

/*
 * ؿusb_otg_control_ioctl
 * IOCTLԤ
 */
static inline int
usb_otg_control_ioctl(struct usb_otg_control* otg_control, unsigned code,
                      unsigned long param)
{
	if (!otg_control) {
		return -EINVAL;
	}
	return otg_control->ops->ioctl(otg_control, code, param);
}

#define USB_IOCTL_GET_LINE_STATE	1 /* linestateͼǽ */

#endif				/* _USB_OTG_CONTROL_H_ */
