/* 2010-09-02: File changed by Sony Corporation */
/*
 *  linux/init/main.c
 *
 *  Copyright (C) 1991, 1992  Linus Torvalds
 *
 *  GK 2/5/95  -  Changed to support mounting root fs via NFS
 *  Added initrd & change_root: Werner Almesberger & Hans Lermen, Feb '96
 *  Moan early if gcc is old, avoiding bogus kernels - Paul Gortmaker, May '96
 *  Simplified starting of init:  Michael A. Griffith <grif@acm.org> 
 */

#include <linux/types.h>
#include <linux/module.h>
#include <linux/proc_fs.h>
#include <linux/kernel.h>
#include <linux/syscalls.h>
#include <linux/stackprotector.h>
#include <linux/string.h>
#include <linux/ctype.h>
#include <linux/delay.h>
#include <linux/utsname.h>
#include <linux/ioport.h>
#include <linux/init.h>
#include <linux/smp_lock.h>
#include <linux/initrd.h>
#include <linux/bootmem.h>
#include <linux/tty.h>
#include <linux/gfp.h>
#include <linux/percpu.h>
#include <linux/kmod.h>
#include <linux/vmalloc.h>
#include <linux/kernel_stat.h>
#include <linux/start_kernel.h>
#include <linux/security.h>
#include <linux/smp.h>
#include <linux/workqueue.h>
#include <linux/profile.h>
#include <linux/rcupdate.h>
#include <linux/posix-timers.h>
#include <linux/moduleparam.h>
#include <linux/kallsyms.h>
#include <linux/writeback.h>
#include <linux/cpu.h>
#include <linux/cpuset.h>
#include <linux/cgroup.h>
#include <linux/efi.h>
#include <linux/tick.h>
#include <linux/interrupt.h>
#include <linux/taskstats_kern.h>
#include <linux/delayacct.h>
#include <linux/unistd.h>
#include <linux/rmap.h>
#include <linux/irq.h>
#include <linux/mempolicy.h>
#include <linux/key.h>
#include <linux/buffer_head.h>
#include <linux/page_cgroup.h>
#include <linux/debug_locks.h>
#include <linux/debugobjects.h>
#include <linux/lockdep.h>
#include <linux/kmemleak.h>
#include <linux/pid_namespace.h>
#include <linux/device.h>
#include <linux/kthread.h>
#include <linux/sched.h>
#include <linux/signal.h>
#include <linux/idr.h>
#include <linux/kmemcheck.h>
#include <linux/ftrace.h>
#include <linux/async.h>
#ifdef CONFIG_SNSC_FREE_MEM_MEASUREMENT
#include <linux/swap.h>
#endif
#include <trace/boot.h>

#include <asm/io.h>
#include <asm/bugs.h>
#include <asm/setup.h>
#include <asm/sections.h>
#include <asm/cacheflush.h>
#include <trace/kmemtrace.h>

#ifdef CONFIG_X86_LOCAL_APIC
#include <asm/smp.h>
#endif

#ifdef CONFIG_DEBUG_LL
extern void printascii(const char *string);
#endif

static int kernel_init(void *);

extern void init_IRQ(void);
extern void fork_init(unsigned long);
extern void mca_init(void);
extern void sbus_init(void);
extern void prio_tree_init(void);
extern void radix_tree_init(void);
extern void free_initmem(void);
#ifdef	CONFIG_ACPI
extern void acpi_early_init(void);
#else
static inline void acpi_early_init(void) { }
#endif
#ifndef CONFIG_DEBUG_RODATA
static inline void mark_rodata_ro(void) { }
#endif

#ifdef CONFIG_TC
extern void tc_init(void);
#endif
#ifdef USE_IMMEDIATE
extern void imv_init_complete(void);
#else
static inline void imv_init_complete(void) { }
#endif

enum system_states system_state __read_mostly;
EXPORT_SYMBOL(system_state);

#include <linux/snsc_boot_time.h>

#ifdef CONFIG_SNSC_PAGES_ACCOUNTING
int mem_init_state = 0;
#endif

#ifdef CONFIG_SNSC_USE_NODE_ORDER
extern void nodeorder_init(void);
extern void nodeorder_make_zonelists(void);
#endif

/*
 * Boot command-line arguments
 */
#define MAX_INIT_ARGS CONFIG_INIT_ENV_ARG_LIMIT
#define MAX_INIT_ENVS CONFIG_INIT_ENV_ARG_LIMIT

extern void time_init(void);
/* Default late time init is NULL. archs can override this later. */
void (*__initdata late_time_init)(void);
extern void softirq_init(void);

/* Untouched command line saved by arch-specific code. */
char __initdata boot_command_line[COMMAND_LINE_SIZE];
/* Untouched saved command line (eg. for /proc) */
char *saved_command_line;
/* Command line for parameter parsing */
static char *static_command_line;

static char *execute_command;
static char *ramdisk_execute_command;

#ifdef CONFIG_SMP
/* Setup configured maximum number of CPUs to activate */
unsigned int __initdata setup_max_cpus = NR_CPUS;

/*
 * Setup routine for controlling SMP activation
 *
 * Command-line option of "nosmp" or "maxcpus=0" will disable SMP
 * activation entirely (the MPS table probe still happens, though).
 *
 * Command-line option of "maxcpus=<NUM>", where <NUM> is an integer
 * greater than 0, limits the maximum number of CPUs activated in
 * SMP mode to <NUM>.
 */

void __weak arch_disable_smp_support(void) { }

static int __init nosmp(char *str)
{
	setup_max_cpus = 0;
	arch_disable_smp_support();

	return 0;
}

early_param("nosmp", nosmp);

static int __init maxcpus(char *str)
{
	get_option(&str, &setup_max_cpus);
	if (setup_max_cpus == 0)
		arch_disable_smp_support();

	return 0;
}

early_param("maxcpus", maxcpus);
#else
const unsigned int setup_max_cpus = NR_CPUS;
#endif

/*
 * If set, this is an indication to the drivers that reset the underlying
 * device before going ahead with the initialization otherwise driver might
 * rely on the BIOS and skip the reset operation.
 *
 * This is useful if kernel is booting in an unreliable environment.
 * For ex. kdump situaiton where previous kernel has crashed, BIOS has been
 * skipped and devices will be in unknown state.
 */
unsigned int reset_devices;
EXPORT_SYMBOL(reset_devices);

static int __init set_reset_devices(char *str)
{
	reset_devices = 1;
	return 1;
}

__setup("reset_devices", set_reset_devices);

static char * argv_init[MAX_INIT_ARGS+2] = { "init", NULL, };
char * envp_init[MAX_INIT_ENVS+2] = { "HOME=/", "TERM=linux", NULL, };
static const char *panic_later, *panic_param;

extern struct obs_kernel_param __setup_start[], __setup_end[];

static int __init obsolete_checksetup(char *line)
{
	struct obs_kernel_param *p;
	int had_early_param = 0;

	p = __setup_start;
	do {
		int n = strlen(p->str);
		if (!strncmp(line, p->str, n)) {
			if (p->early) {
				/* Already done in parse_early_param?
				 * (Needs exact match on param part).
				 * Keep iterating, as we can have early
				 * params and __setups of same names 8( */
				if (line[n] == '\0' || line[n] == '=')
					had_early_param = 1;
			} else if (!p->setup_func) {
				printk(KERN_WARNING "Parameter %s is obsolete,"
				       " ignored\n", p->str);
				return 1;
			} else if (p->setup_func(line + n))
				return 1;
		}
		p++;
	} while (p < __setup_end);

	return had_early_param;
}

/*
 * This should be approx 2 Bo*oMips to start (note initial shift), and will
 * still work even if initially too large, it will just take slightly longer
 */
unsigned long loops_per_jiffy = (1<<12);

EXPORT_SYMBOL(loops_per_jiffy);

static int __init debug_kernel(char *str)
{
	console_loglevel = 10;
	return 0;
}

static int __init quiet_kernel(char *str)
{
	console_loglevel = 4;
	return 0;
}

early_param("debug", debug_kernel);
early_param("quiet", quiet_kernel);

static int __init loglevel(char *str)
{
	get_option(&str, &console_loglevel);
	return 0;
}

early_param("loglevel", loglevel);

/*
 * Unknown boot options get handed to init, unless they look like
 * failed parameters
 */
static int __init unknown_bootoption(char *param, char *val)
{
	/* Change NUL term back to "=", to make "param" the whole string. */
	if (val) {
		/* param=val or param="val"? */
		if (val == param+strlen(param)+1)
			val[-1] = '=';
		else if (val == param+strlen(param)+2) {
			val[-2] = '=';
			memmove(val-1, val, strlen(val)+1);
			val--;
		} else
			BUG();
	}

	/* Handle obsolete-style parameters */
	if (obsolete_checksetup(param))
		return 0;

	/*
	 * Preemptive maintenance for "why didn't my misspelled command
	 * line work?"
	 */
	if (strchr(param, '.') && (!val || strchr(param, '.') < val)) {
		printk(KERN_ERR "Unknown boot option `%s': ignoring\n", param);
		return 0;
	}

	if (panic_later)
		return 0;

	if (val) {
		/* Environment option */
		unsigned int i;
		for (i = 0; envp_init[i]; i++) {
			if (i == MAX_INIT_ENVS) {
				panic_later = "Too many boot env vars at `%s'";
				panic_param = param;
			}
			if (!strncmp(param, envp_init[i], val - param))
				break;
		}
		envp_init[i] = param;
	} else {
		/* Command line option */
		unsigned int i;
		for (i = 0; argv_init[i]; i++) {
			if (i == MAX_INIT_ARGS) {
				panic_later = "Too many boot init vars at `%s'";
				panic_param = param;
			}
		}
		argv_init[i] = param;
	}
	return 0;
}

#ifdef CONFIG_DEBUG_PAGEALLOC
int __read_mostly debug_pagealloc_enabled = 0;
#endif

static int __init init_setup(char *str)
{
	unsigned int i;

	execute_command = str;
	/*
	 * In case LILO is going to boot us with default command line,
	 * it prepends "auto" before the whole cmdline which makes
	 * the shell think it should execute a script with such name.
	 * So we ignore all arguments entered _before_ init=... [MJ]
	 */
	for (i = 1; i < MAX_INIT_ARGS; i++)
		argv_init[i] = NULL;
	return 1;
}
__setup("init=", init_setup);

static int __init rdinit_setup(char *str)
{
	unsigned int i;

	ramdisk_execute_command = str;
	/* See "auto" comment in init_setup */
	for (i = 1; i < MAX_INIT_ARGS; i++)
		argv_init[i] = NULL;
	return 1;
}
__setup("rdinit=", rdinit_setup);

#ifndef CONFIG_SMP

#ifdef CONFIG_X86_LOCAL_APIC
static void __init smp_init(void)
{
	APIC_init_uniprocessor();
}
#else
#define smp_init()	do { } while (0)
#endif

static inline void setup_per_cpu_areas(void) { }
static inline void setup_nr_cpu_ids(void) { }
static inline void smp_prepare_cpus(unsigned int maxcpus) { }

#else

#if NR_CPUS > BITS_PER_LONG
cpumask_t cpu_mask_all __read_mostly = CPU_MASK_ALL;
EXPORT_SYMBOL(cpu_mask_all);
#endif

/* Setup number of possible processor ids */
int nr_cpu_ids __read_mostly = NR_CPUS;
EXPORT_SYMBOL(nr_cpu_ids);

/* An arch may set nr_cpu_ids earlier if needed, so this would be redundant */
static void __init setup_nr_cpu_ids(void)
{
	nr_cpu_ids = find_last_bit(cpumask_bits(cpu_possible_mask),NR_CPUS) + 1;
}

#ifndef CONFIG_HAVE_SETUP_PER_CPU_AREA
unsigned long __per_cpu_offset[NR_CPUS] __read_mostly;

EXPORT_SYMBOL(__per_cpu_offset);

static void __init setup_per_cpu_areas(void)
{
	unsigned long size, i;
	char *ptr;
	unsigned long nr_possible_cpus = num_possible_cpus();

	/* Copy section for each CPU (we discard the original) */
	size = ALIGN(PERCPU_ENOUGH_ROOM, PAGE_SIZE);
	ptr = alloc_bootmem_pages(size * nr_possible_cpus);

	for_each_possible_cpu(i) {
		__per_cpu_offset[i] = ptr - __per_cpu_start;
		memcpy(ptr, __per_cpu_start, __per_cpu_end - __per_cpu_start);
		ptr += size;
	}
}
#endif /* CONFIG_HAVE_SETUP_PER_CPU_AREA */

#ifdef CONFIG_EJ_DELAY_BOOT_MAP
static cpumask_t __initdata delay_boot_map = (cpumask_t){{0x0}};

static int __init delayboot(char *str)
{
	unsigned long bitmap;

	get_option(&str, (int *)&bitmap);
	delay_boot_map = *to_cpumask(&bitmap);
	return 0;
}

early_param("delayboot", delayboot);

static unsigned long delay_boot_disable;

static int __init delayflag(char *str)
{
	unsigned long addr, data, flag;

	if (2 != get_option(&str, (int *)&addr)) {
		return 0;
	}
	if (1 != get_option(&str, (int *)&data)) {
		return 0;
	}

	flag = readb(addr);
	printk(KERN_INFO "delayflag=0x%02lx\n", flag);
	if (flag == data) {
		delay_boot_disable = 1;
	}
	return 0;
}

early_param("delayflag", delayflag);
#endif

/* Called by boot processor to activate the rest. */
static void __init smp_init(void)
{
	unsigned int cpu;

	/*
	 * Set up the current CPU as possible to migrate to.
	 * The other ones will be done by cpu_up/cpu_down()
	 */
	cpu = smp_processor_id();
	cpu_set(cpu, cpu_active_map);

	/* FIXME: This should be done in userspace --RR */
	for_each_present_cpu(cpu) {
		if (num_online_cpus() >= setup_max_cpus)
			break;
#ifdef CONFIG_EJ_DELAY_BOOT_MAP
		if (!delay_boot_disable) {
			if (cpu_isset(cpu, delay_boot_map))
				continue;
		}
#endif

		if (!cpu_online(cpu))
			cpu_up(cpu);
	}

	/* Any cleanup work */
	printk(KERN_INFO "Brought up %ld CPUs\n", (long)num_online_cpus());
	smp_cpus_done(setup_max_cpus);
}

#endif

/*
 * We need to store the untouched command line for future reference.
 * We also need to store the touched command line since the parameter
 * parsing is performed in place, and we should allow a component to
 * store reference of name/value for future reference.
 */
static void __init setup_command_line(char *command_line)
{
	saved_command_line = alloc_bootmem(strlen (boot_command_line)+1);
	static_command_line = alloc_bootmem(strlen (command_line)+1);
	strcpy (saved_command_line, boot_command_line);
	strcpy (static_command_line, command_line);
}

/*
 * We need to finalize in a non-__init function or else race conditions
 * between the root thread and the init thread may cause start_kernel to
 * be reaped by free_initmem before the root thread has proceeded to
 * cpu_idle.
 *
 * gcc-3.4 accidentally inlines this function, so use noinline.
 */

static noinline void __init_refok rest_init(void)
	__releases(kernel_lock)
{
	int pid;

	system_state = SYSTEM_BOOTING_SCHEDULER_OK;

	kernel_thread(kernel_init, NULL, CLONE_FS | CLONE_SIGHAND);
	numa_default_policy();
	pid = kernel_thread(kthreadd, NULL, CLONE_FS | CLONE_FILES);
	kthreadd_task = find_task_by_pid_ns(pid, &init_pid_ns);
	unlock_kernel();

	/*
	 * The boot idle thread must execute schedule()
	 * at least once to get things moving:
	 */
	init_idle_bootup_task(current);
	rcu_scheduler_starting();
	__preempt_enable_no_resched();
	schedule();
	preempt_disable();

	/* Call into cpu_idle with preempt disabled */
	cpu_idle();
}

/* Check for early params. */
static int __init do_early_param(char *param, char *val)
{
	struct obs_kernel_param *p;

	for (p = __setup_start; p < __setup_end; p++) {
		if ((p->early && strcmp(param, p->str) == 0) ||
		    (strcmp(param, "console") == 0 &&
		     strcmp(p->str, "earlycon") == 0)
		) {
			if (p->setup_func(val) != 0)
				printk(KERN_WARNING
				       "Malformed early option '%s'\n", param);
		}
	}
	/* We accept everything at this stage. */
	return 0;
}

/* Arch code calls this early on, or if not, just before other parsing. */
void __init parse_early_param(void)
{
	static __initdata int done = 0;
	static __initdata char tmp_cmdline[COMMAND_LINE_SIZE];

	if (done)
		return;

	/* All fall through to do_early_param. */
	strlcpy(tmp_cmdline, boot_command_line, COMMAND_LINE_SIZE);
	parse_args("early options", tmp_cmdline, NULL, 0, do_early_param);
	done = 1;
}

/*
 *	Activate the first processor.
 */

static void __init boot_cpu_init(void)
{
	int cpu = smp_processor_id();
	/* Mark the boot cpu "present", "online" etc for SMP and UP case */
	set_cpu_online(cpu, true);
	set_cpu_present(cpu, true);
	set_cpu_possible(cpu, true);
}

void __init __weak smp_setup_processor_id(void)
{
}

void __init __weak thread_info_cache_init(void)
{
}

asmlinkage void __init start_kernel(void)
{
	char * command_line;
	extern struct kernel_param __start___param[], __stop___param[];

	smp_setup_processor_id();

	/*
	 * Need to run as early as possible, to initialize the
	 * lockdep hash:
	 */
	lockdep_init();
	debug_objects_early_init();

	/*
	 * Set up the the initial canary ASAP:
	 */
	boot_init_stack_canary();

	cgroup_init_early();
	core_imv_update();

	local_irq_disable();
	early_boot_irqs_off();
	early_init_irq_lock_class();

/*
 * Interrupts are still disabled. Do necessary setups, then
 * enable them
 */
	lock_kernel();
	tick_init();
	boot_cpu_init();
	page_address_init();
	printk(KERN_NOTICE);
	printk(linux_banner);
#if defined(CONFIG_DEBUG_LL)
	printascii("setup_arch\n");
#endif
	setup_arch(&command_line);
	mm_init_owner(&init_mm, &init_task);
	setup_command_line(command_line);
	setup_per_cpu_areas();
	setup_nr_cpu_ids();
	smp_prepare_boot_cpu();	/* arch-specific boot-cpu hooks */

	/*
	 * Set up the scheduler prior starting any interrupts (such as the
	 * timer interrupt). Full topology setup happens at smp_init()
	 * time - but meanwhile we still have a functioning scheduler.
	 */
	sched_init();
	/*
	 * Disable preemption - early bootup scheduling is extremely
	 * fragile until we cpu_idle() for the first time.
	 */
	preempt_disable();

#if defined(CONFIG_DEBUG_LL)
	printascii("build_all_zonelists\n");
#endif
	build_all_zonelists();
#if defined(CONFIG_DEBUG_LL)
	printascii("page_alloc_init\n");
#endif
	page_alloc_init();
#ifdef CONFIG_SNSC_USE_NODE_ORDER
	nodeorder_init();
#endif
	early_init_hardirqs();
	printk(KERN_NOTICE "Kernel command line: %s\n", boot_command_line);
#if defined(CONFIG_DEBUG_LL)
	printascii("parse_early_param\n");
#endif
	parse_early_param();
	parse_args("Booting kernel", static_command_line, __start___param,
		   __stop___param - __start___param,
		   &unknown_bootoption);
	if (!irqs_disabled()) {
		printk(KERN_WARNING "start_kernel(): bug: interrupts were "
				"enabled *very* early, fixing it\n");
		local_irq_disable();
	}
	sort_main_extable();
	trap_init();
	rcu_init();
	/* init some links before init_ISA_irqs() */
	early_irq_init();
	init_IRQ();
	prio_tree_init();
	pidhash_init();
	init_timers();
	hrtimers_init();
	softirq_init();
	timekeeping_init();
	BOOT_TIME_ADD1("time_init()");
	time_init();
	BOOT_TIME_ADD1("profile_init()");
	sched_clock_init();
#if defined(CONFIG_DEBUG_LL)
	printascii("profile_init\n");
#endif
	profile_init();
	if (!irqs_disabled())
		printk(KERN_CRIT "start_kernel(): bug: interrupts were "
				 "enabled early\n");
	early_boot_irqs_on();
	local_irq_enable();

	/*
	 * HACK ALERT! This is early. We're enabling the console before
	 * we've done PCI setups etc, and console_init() must be aware of
	 * this. But we do want output early, in case something goes wrong.
	 */
#if defined(CONFIG_DEBUG_LL)
	printascii("console_init\n");
#endif
        BOOT_TIME_ADD1("console_init()");
	console_init();
#if defined(CONFIG_DEBUG_LL)
	printascii("console_init: done\n");
#endif
	if (panic_later)
		panic(panic_later, panic_param);

	lockdep_info();

	/*
	 * Need to run this when irqs are enabled, because it wants
	 * to self-test [hard/soft]-irqs on/off lock inversion bugs
	 * too:
	 */
#if defined(CONFIG_DEBUG_LL)
	printascii("locking_selftest\n");
#endif
	locking_selftest();

#ifdef CONFIG_BLK_DEV_INITRD
	if (initrd_start && !initrd_below_start_ok &&
	    page_to_pfn(virt_to_page((void *)initrd_start)) < min_low_pfn) {
		printk(KERN_CRIT "initrd overwritten (0x%08lx < 0x%08lx) - "
		    "disabling it.\n",
		    page_to_pfn(virt_to_page((void *)initrd_start)),
		    min_low_pfn);
		initrd_start = 0;
	}
#endif
	vmalloc_init();
#if defined(CONFIG_DEBUG_LL)
	printascii("vfs_caches_init_early\n");
#endif
	BOOT_TIME_ADD1("vfs_caches_init_early()");
	vfs_caches_init_early();
	cpuset_init_early();
	BOOT_TIME_ADD1("mem_init()");
	page_cgroup_init();
#if defined(CONFIG_DEBUG_LL)
	printascii("mem_init\n");
#endif
	mem_init();
#ifdef CONFIG_SNSC_PAGES_ACCOUNTING
	mem_init_state = 1;
#endif
	enable_debug_pagealloc();
	cpu_hotplug_init();
#if defined(CONFIG_DEBUG_LL)
	printascii("kmem_cache_init\n");
#endif
	kmem_cache_init();
	kmemleak_init();
 	kmemtrace_init();
	debug_objects_mem_init();
	idr_init_cache();
#if defined(CONFIG_DEBUG_LL)
	printascii("setup_per_cpu_pageset\n");
#endif
	setup_per_cpu_pageset();
#if defined(CONFIG_DEBUG_LL)
	printascii("numa_policy_init\n");
#endif
	numa_policy_init();
	if (late_time_init)
		late_time_init();
	calibrate_delay();
#if defined(CONFIG_DEBUG_LL)
	printascii("pidmap_init\n");
#endif
	pidmap_init();
#if defined(CONFIG_DEBUG_LL)
	printascii("pgtable_cache_init\n");
#endif
	pgtable_cache_init();
	anon_vma_init();
#ifdef CONFIG_X86
	if (efi_enabled)
		efi_enter_virtual_mode();
#endif
	thread_info_cache_init();
	cred_init();
#if defined(CONFIG_DEBUG_LL)
	printascii("fork_init\n");
#endif
	BOOT_TIME_ADD1("fork_init()");
	fork_init(num_physpages);
	proc_caches_init();
	buffer_init();
	key_init();
	security_init();
#if defined(CONFIG_DEBUG_LL)
	printascii("vfs_caches_init\n");
#endif
	vfs_caches_init(num_physpages);
	radix_tree_init();
	signals_init();
	/* rootfs populating might need page-writeback */
	page_writeback_init();
#ifdef CONFIG_PROC_FS
	proc_root_init();
#endif
	cgroup_init();
	cpuset_init();
	taskstats_init_early();
	delayacct_init();
	imv_init_complete();

#if defined(CONFIG_DEBUG_LL)
	printascii("check_bugs\n");
#endif
	check_bugs();

	acpi_early_init(); /* before LAPIC and SMP init */

	ftrace_init();

#ifdef CONFIG_PREEMPT_RT
	WARN_ON(irqs_disabled());
#endif
	/* Do the rest non-__init'ed, we're now alive */
#if defined(CONFIG_DEBUG_LL)
	printascii("rest_init\n");
#endif
        BOOT_TIME_ADD1("rest_init()");
	rest_init();
}

int initcall_debug;
core_param(initcall_debug, initcall_debug, bool, 0644);

static char msgbuf[64];
static struct boot_trace_call call;
static struct boot_trace_ret ret;

int do_one_initcall(initcall_t fn)
{
	int count = preempt_count();
	ktime_t calltime, delta, rettime;

	if (initcall_debug) {
		call.caller = task_pid_nr(current);
		printk("calling  %pF @ %i\n", fn, call.caller);
		calltime = ktime_get();
		trace_boot_call(&call, fn);
		enable_boot_trace();
	}

	ret.result = fn();

	if (initcall_debug) {
		disable_boot_trace();
		rettime = ktime_get();
		delta = ktime_sub(rettime, calltime);
		ret.duration = (unsigned long long) ktime_to_ns(delta) >> 10;
		trace_boot_ret(&ret, fn);
		printk("initcall %pF returned %d after %Ld usecs\n", fn,
			ret.result, ret.duration);
	}

	msgbuf[0] = 0;

	if (ret.result && ret.result != -ENODEV && initcall_debug)
		sprintf(msgbuf, "error code %d ", ret.result);

	if (preempt_count() != count) {
		strlcat(msgbuf, "preemption imbalance ", sizeof(msgbuf));
		preempt_count() = count;
	}
	if (irqs_disabled()) {
		strlcat(msgbuf, "disabled interrupts ", sizeof(msgbuf));
		local_irq_enable();
	}
	if (msgbuf[0]) {
		printk("initcall %pF returned with %s\n", fn, msgbuf);
	}

	return ret.result;
}


extern initcall_t __initcall_start[], __initcall_end[], __early_initcall_end[];

#ifdef CONFIG_SNSC_PARALLEL_INITCALLS
extern initcall_t __initcall0_start[], __initcall0_end[];
extern initcall_t __initcall0s_start[], __initcall0s_end[];
extern initcall_t __initcall1_start[], __initcall1_end[];
extern initcall_t __initcall1s_start[], __initcall1s_end[];
extern initcall_t __initcall2_start[], __initcall2_end[];
extern initcall_t __initcall2s_start[], __initcall2s_end[];
extern initcall_t __initcall3_start[], __initcall3_end[];
extern initcall_t __initcall3s_start[], __initcall3s_end[];
extern initcall_t __initcall4_start[], __initcall4_end[];
extern initcall_t __initcall4s_start[], __initcall4s_end[];
extern initcall_t __initcall5_start[], __initcall5_end[];
extern initcall_t __initcall5s_start[], __initcall5s_end[];
extern initcall_t __initcallrootfs_start[], __initcallrootfs_end[];
extern initcall_t __initcall6_start[], __initcall6_end[];
extern initcall_t __initcall6s_start[], __initcall6s_end[];
extern initcall_t __initcall7_start[], __initcall7_end[];
extern initcall_t __initcall7s_start[], __initcall7s_end[];

typedef enum {
	INITCALL_SERIAL   = 0,
	INITCALL_PARALLEL = 1
} initcall_mode_t;

#ifdef CONFIG_SNSC_PARALLEL_INITCALLS_ALL
#define INITCALL_NORMAL	INITCALL_PARALLEL
#else
#define INITCALL_NORMAL	INITCALL_SERIAL
#endif

typedef struct {
	initcall_t *start, *end;
	initcall_mode_t mode;
} initcall_data_t;

static initcall_data_t __initdata initcalls[] = {
	{ __initcall0_start,      __initcall0_end,      INITCALL_NORMAL   },
	{ __initcall0s_start,     __initcall0s_end,     INITCALL_SERIAL   },
	{ __initcall1_start,      __initcall1_end,      INITCALL_NORMAL   },
	{ __initcall1s_start,     __initcall1s_end,     INITCALL_SERIAL   },
	{ __initcall2_start,      __initcall2_end,      INITCALL_NORMAL   },
	{ __initcall2s_start,     __initcall2s_end,     INITCALL_SERIAL   },
	{ __initcall3_start,      __initcall3_end,      INITCALL_NORMAL   },
	{ __initcall3s_start,     __initcall3s_end,     INITCALL_SERIAL   },
	{ __initcall4_start,      __initcall4_end,      INITCALL_NORMAL   },
	{ __initcall4s_start,     __initcall4s_end,     INITCALL_SERIAL   },
	{ __initcall5_start,      __initcall5_end,      INITCALL_NORMAL   },
	{ __initcall5s_start,     __initcall5s_end,     INITCALL_SERIAL   },
	{ __initcallrootfs_start, __initcallrootfs_end, INITCALL_NORMAL   },
	{ __initcall6_start,      __initcall6_end,      INITCALL_PARALLEL },
	{ __initcall6s_start,     __initcall6s_end,     INITCALL_SERIAL   },
	{ __initcall7_start,      __initcall7_end,      INITCALL_NORMAL   },
	{ __initcall7s_start,     __initcall7s_end,     INITCALL_SERIAL   }
};

#define PARALLEL_MIN_INITCALL	2

#ifndef CONFIG_SNSC_PARALLEL_INITCALLS_USE_ASYNC_SCHEDULE
static DEFINE_SPINLOCK(initcalls_lock);
static initcall_t * __initdata initcall_pos;
static initcall_t * __initdata initcall_end;

static void __init set_initcalls(initcall_t *start, initcall_t *end)
{
	spin_lock(&initcalls_lock);
	initcall_pos = start;
	initcall_end = end;
	spin_unlock(&initcalls_lock);
}

static initcall_t * __init get_initcall(void)
{
	initcall_t *ret = NULL;

	spin_lock(&initcalls_lock);
	if (initcall_pos < initcall_end)
		ret = initcall_pos++;
	spin_unlock(&initcalls_lock);
	return ret;
}

static int __init __do_initcalls(void *arg)
{
	initcall_t *call;

	while ((call = get_initcall()) != NULL)
		do_one_initcall(*call);
	return 0;
}

#define PARALLEL_MAX_THREAD	(NR_CPUS + 1)
#else	/* CONFIG_SNSC_PARALLEL_INITCALLS_USE_ASYNC_SCHEDULE */
static void __init do_one_initcall_async(void *data, async_cookie_t cookie)
{
	initcall_t *call = (initcall_t *)data;

	do_one_initcall(*call);
}
#endif	/* !CONFIG_SNSC_PARALLEL_INITCALLS_USE_ASYNC_SCHEDULE */

static void __init do_initcall_parallel(unsigned int i)
{
#ifdef	CONFIG_SNSC_PARALLEL_INITCALLS_USE_ASYNC_SCHEDULE
	unsigned int n;
	struct list_head initcalls_async;
	initcall_t *call;
#else
	unsigned int n, n_thread, thread;
	int pid[PARALLEL_MAX_THREAD], ret;

	set_initcalls(initcalls[i].start, initcalls[i].end);
#endif

	n = initcalls[i].end - initcalls[i].start;
	if ((initcalls[i].mode != INITCALL_PARALLEL) ||
	    (n < PARALLEL_MIN_INITCALL)) {
#ifdef CONFIG_SNSC_PARALLEL_INITCALLS_USE_ASYNC_SCHEDULE
		for (call = initcalls[i].start; call < initcalls[i].end; call++)
			do_one_initcall(*call);
#else
		__do_initcalls(NULL);
#endif
		return;
	}

	BOOT_TIME_ADD1("parallel initcall start");

#ifdef	CONFIG_SNSC_PARALLEL_INITCALLS_USE_ASYNC_SCHEDULE
	INIT_LIST_HEAD(&initcalls_async);
	for (call = initcalls[i].start; call < initcalls[i].end; call++)
		async_schedule_domain(do_one_initcall_async,
				      call, &initcalls_async);
	async_synchronize_full_domain(&initcalls_async);
#else
	if (n < PARALLEL_MAX_THREAD)
		n_thread = n;
	else
		n_thread = PARALLEL_MAX_THREAD;

	for (thread = 0; thread < n_thread; thread++) {
		pid[thread] = kernel_thread(__do_initcalls, NULL,
		                            CLONE_FS | CLONE_FILES | SIGCHLD);
		BUG_ON(pid[thread] <= 0);
	}
	for (thread = 0; thread < n_thread; thread++) {
		ret = sys_wait4(pid[thread], NULL, 0, NULL);
		BUG_ON(ret != pid[thread]);
	}
#endif

	BOOT_TIME_ADD1("parallel initcall end");
}
#endif	/* CONFIG_SNSC_PARALLEL_INITCALLS */

static void __init do_initcalls(void)
{
#ifndef CONFIG_SNSC_PARALLEL_INITCALLS
	initcall_t *call;
#else
	unsigned int i;
#endif

	BOOT_TIME_ADD();
#ifndef CONFIG_SNSC_PARALLEL_INITCALLS
	for (call = __early_initcall_end; call < __initcall_end; call++)
		do_one_initcall(*call);
#else
	for (i = 0; i < sizeof(initcalls) / sizeof(initcalls[0]); i++)
		do_initcall_parallel(i);
#endif

	/* Make sure there is no pending stuff from the initcall sequence */
	flush_scheduled_work();
        BOOT_TIME_ADD1("do_initcalls() end\n");
}

/*
 * Ok, the machine is now initialized. None of the devices
 * have been touched yet, but the CPU subsystem is up and
 * running, and memory and process management works.
 *
 * Now we can finally start doing some real work..
 */
static void __init do_basic_setup(void)
{
	BOOT_TIME_ADD1("do_basic_setup()");
	rcu_init_sched(); /* needed by module_init stage. */
	init_workqueues();
	cpuset_init_smp();
	usermodehelper_init();
	driver_init();
	init_irq_proc();
	do_initcalls();
}

static void __init do_pre_smp_initcalls(void)
{
	initcall_t *call;
	extern int spawn_desched_task(void);

	/* kmemcheck must initialize before all early initcalls: */
	kmemcheck_init();

	for (call = __initcall_start; call < __early_initcall_end; call++)
		do_one_initcall(*call);
	spawn_desched_task();
}

static void run_init_process(char *init_filename)
{
#if defined(CONFIG_DEBUG_LL)
	printascii("run_init_process: ");
	printascii(init_filename);
	printascii("\n");
#endif
	argv_init[0] = init_filename;
	kernel_execve(init_filename, argv_init, envp_init);
}

#ifdef CONFIG_SNSC_FREE_MEM_MEASUREMENT
/* Code is taken from meminfo_read_proc() @ fs/proc/proc_misc.c. */
static void show_free_mem(void)
{
	struct sysinfo i;
	long cached;

#define K(x) ((x) << (PAGE_SHIFT - 10))
	si_meminfo(&i);
	si_swapinfo(&i);

	cached = global_page_state(NR_FILE_PAGES) -
		total_swapcache_pages - i.bufferram;
	if (cached < 0)
		cached = 0;
	printk("MemTotal:     %8lu kB\n"
	       "MemFree:      %8lu kB\n"
	       "Buffers:      %8lu kB\n"
	       "Cached:       %8lu kB\n"
	       "MemFree + Buffers/Cached: %8lu kB\n",
	       K(i.totalram),
	       K(i.freeram),
	       K(i.bufferram),
	       K(cached),
	       K(i.freeram) + K(i.bufferram) + K(cached)
		);
}
#endif

#ifdef CONFIG_SNSC_GROUP_SPARSEMEM_SECTIONS
extern void add_sec_grp(void);
#endif

/* This is a non __init function. Force it to be noinline otherwise gcc
 * makes it inline to init() and it becomes part of init.text section
 */
static noinline int init_post(void)
{
	/* need to finish all async __init code before freeing the memory */
#if defined(CONFIG_DEBUG_LL)
	printascii("init_post\n");
#endif
	async_synchronize_full();
	imv_unref_core_init();
	BOOT_TIME_ADD1("free_initmem()");
	free_initmem();
	unlock_kernel();
	mark_rodata_ro();
	system_state = SYSTEM_RUNNING;
#ifdef CONFIG_SNSC_GROUP_SPARSEMEM_SECTIONS
	add_sec_grp();
#endif

#ifdef CONFIG_SNSC_USE_NODE_ORDER
	nodeorder_make_zonelists();
#endif

#ifdef CONFIG_SNSC_FREE_MEM_MEASUREMENT
	show_free_mem();
#endif
	numa_default_policy();

#if defined(CONFIG_DEBUG_LL)
	printascii("sys_open /dev/console\n");
#endif
	if (sys_open((const char __user *) "/dev/console", O_RDWR, 0) < 0)
		printk(KERN_WARNING "Warning: unable to open an initial console.\n");

	(void) sys_dup(0);
	(void) sys_dup(0);

	current->signal->flags |= SIGNAL_UNKILLABLE;

	if (ramdisk_execute_command) {
		BOOT_TIME_ADD1("run_init_process()");
		run_init_process(ramdisk_execute_command);
		printk(KERN_WARNING "Failed to execute %s\n",
				ramdisk_execute_command);
	}
#ifdef CONFIG_PREEMPT_RT
	WARN_ON(irqs_disabled());
#endif

	/*
	 * We try each of these until one succeeds.
	 *
	 * The Bourne shell can be used instead of init if we are
	 * trying to recover a really broken machine.
	 */
	if (execute_command) {
		BOOT_TIME_ADD1("run_init_process()");
		run_init_process(execute_command);
		printk(KERN_WARNING "Failed to execute %s.  Attempting "
					"defaults...\n", execute_command);
	}
	BOOT_TIME_ADD1("run_init_process()");
	run_init_process("/sbin/init");
	run_init_process("/etc/init");
	run_init_process("/bin/init");
	run_init_process("/bin/sh");

	panic("No init found.  Try passing init= option to kernel.");
	return 0;
}

static int __init kernel_init(void * unused)
{
#if defined(CONFIG_DEBUG_LL)
	printascii("kernel_init\n");
#endif
        BOOT_TIME_ADD1("init()");
	lock_kernel();
	/*
	 * init can run on any cpu.
	 */
	set_cpus_allowed_ptr(current, CPU_MASK_ALL_PTR);
	/*
	 * Tell the world that we're going to be the grim
	 * reaper of innocent orphaned children.
	 *
	 * We don't want people to have to make incorrect
	 * assumptions about where in the task array this
	 * can be found.
	 */
	init_pid_ns.child_reaper = current;

	cad_pid = task_pid(current);

#if defined(CONFIG_DEBUG_LL)
	printascii("smp_prepare_cpus\n");
#endif
	smp_prepare_cpus(setup_max_cpus);

#if defined(CONFIG_DEBUG_LL)
	printascii("init_hardirqs\n");
#endif
	init_hardirqs();

#if defined(CONFIG_DEBUG_LL)
	printascii("do_pre_smp_initcalls\n");
#endif
	do_pre_smp_initcalls();
#if defined(CONFIG_DEBUG_LL)
	printascii("do_pre_smp_initcalls: done\n");
#endif
	start_boot_trace();

#if defined(CONFIG_DEBUG_LL)
	printascii("smp_init\n");
#endif
	smp_init();
#if defined(CONFIG_DEBUG_LL)
	printascii("sched_init_smp\n");
#endif
	sched_init_smp();

#if defined(CONFIG_DEBUG_LL)
	printascii("do_basic_setup\n");
#endif
	do_basic_setup();

	/*
	 * check if there is an early userspace init.  If yes, let it do all
	 * the work
	 */

	if (!ramdisk_execute_command)
		ramdisk_execute_command = "/init";

#if defined(CONFIG_DEBUG_LL)
	printascii("sys_access /init\n");
#endif
        BOOT_TIME_ADD1("sys_access(\"/init\")");
	if (sys_access((const char __user *) ramdisk_execute_command, 0) != 0) {
		ramdisk_execute_command = NULL;
		BOOT_TIME_ADD1("prepare_namespace()");
		prepare_namespace();
	}
#ifdef CONFIG_PREEMPT_RT
	WARN_ON(irqs_disabled());
#endif

#define DEBUG_COUNT (defined(CONFIG_DEBUG_RT_MUTEXES) + defined(CONFIG_IRQSOFF_TRACER) + defined(CONFIG_PREEMPT_TRACER) + defined(CONFIG_STACK_TRACER) + defined(CONFIG_INTERRUPT_OFF_HIST) + defined(CONFIG_PREEMPT_OFF_HIST) + defined(CONFIG_DEBUG_SLAB) + defined(CONFIG_DEBUG_PAGEALLOC) + defined(CONFIG_LOCKDEP) + (defined(CONFIG_FTRACE) - defined(CONFIG_FTRACE_MCOUNT_RECORD)))

#if DEBUG_COUNT > 0
	printk(KERN_ERR "*****************************************************************************\n");
	printk(KERN_ERR "*                                                                           *\n");
#if DEBUG_COUNT == 1
	printk(KERN_ERR "*  REMINDER, the following debugging option is turned on in your .config:   *\n");
#else
	printk(KERN_ERR "*  REMINDER, the following debugging options are turned on in your .config: *\n");
#endif
	printk(KERN_ERR "*                                                                           *\n");
#ifdef CONFIG_FTRACE
	printk(KERN_ERR "*        CONFIG_FTRACE                                                      *\n");
#endif
#ifdef CONFIG_DEBUG_RT_MUTEXES
	printk(KERN_ERR "*        CONFIG_DEBUG_RT_MUTEXES                                            *\n");
#endif
#ifdef CONFIG_IRQSOFF_TRACER
	printk(KERN_ERR "*        CONFIG_IRQSOFF_TRACER                                              *\n");
#endif
#ifdef CONFIG_PREEMPT_TRACER
	printk(KERN_ERR "*        CONFIG_PREEMPT_TRACER                                              *\n");
#endif
#ifdef CONFIG_INTERRUPT_OFF_HIST
	printk(KERN_ERR "*        CONFIG_INTERRUPT_OFF_HIST                                          *\n");
#endif
#ifdef CONFIG_PREEMPT_OFF_HIST
	printk(KERN_ERR "*        CONFIG_PREEMPT_OFF_HIST                                            *\n");
#endif
#ifdef CONFIG_DEBUG_SLAB
	printk(KERN_ERR "*        CONFIG_DEBUG_SLAB                                                  *\n");
#endif
#ifdef CONFIG_DEBUG_PAGEALLOC
	printk(KERN_ERR "*        CONFIG_DEBUG_PAGEALLOC                                             *\n");
#endif
#ifdef CONFIG_LOCKDEP
	printk(KERN_ERR "*        CONFIG_LOCKDEP                                                     *\n");
#endif
	printk(KERN_ERR "*                                                                           *\n");
#if DEBUG_COUNT == 1
	printk(KERN_ERR "*  it may increase runtime overhead and latencies.                          *\n");
#else
	printk(KERN_ERR "*  they may increase runtime overhead and latencies.                        *\n");
#endif
	printk(KERN_ERR "*                                                                           *\n");
	printk(KERN_ERR "*****************************************************************************\n");
#endif
	/*
	 * Ok, we have completed the initial bootup, and
	 * we're essentially up and running. Get rid of the
	 * initmem segments and start the user-mode stuff..
	 */

	init_post();
	WARN_ON(debug_direct_keyboard);

	return 0;
}
