/*--------------------------------------------------------------------------*
 * drivers/usb/scd/scd_trns.c
 * 
 * Gadget transaction functions
 * 
 * Copyright 2011 Sony Corporation
 *
 * This file is part of the HS-OTG Controller Driver SCD.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *---------------------------------------------------------------------------*/
 
#include <linux/kernel.h>
#include <linux/list.h>
#include <linux/timer.h>
#include <linux/spinlock.h>
#include <linux/errno.h>
#include <linux/interrupt.h>
#include <linux/init.h>
#include <linux/usb.h>
#include <linux/usb/ch9.h>
#include <linux/usb/gadget.h>
#include <linux/usb/scd/usb_otg_control.h>
#include <linux/dma-mapping.h>
#include <linux/dmapool.h>
#include <linux/wait.h>
#ifdef SCD_CACHE_INVALIDATE_AFTER_DMA
#include <linux/udif/cache.h>
#endif /* SCD_CACHE_INVALIDATE_AFTER_DMA */

#include "scd.h"
#include "scd_version.h"
#include "scd_register.h"
#include "scd_gadget.h"
#include "scd_debug.h"

/*============================================================================*/
void scd_g_ctrl_rx_int(struct scd_gadget *);
void scd_g_ctrl_tx_int(struct scd_gadget *);
void scd_g_rxtrans_int(struct scd_gadget *, u8);
void scd_g_txtrans_int(struct scd_gadget *, u8);
void scd_g_setup_int(struct scd_gadget *);
int scd_g_b2b_int(struct scd_gadget *, u8 *);
int scd_chg_ep0stage(struct scd_gadget *, int);
u8 scd_get_ep0stage(struct scd_gadget *);
int read_control_request(struct scd_gadget *, struct usb_ctrlrequest *);
int scd_g_ep0_data_out(struct scd_gadget *, gfp_t);
int scd_g_ep0_data_in(struct scd_gadget *, gfp_t);
int scd_g_ep0_status_in(struct scd_gadget *, gfp_t);
int scd_g_general_trns(struct scd_gadget *, u8, gfp_t);
int scd_g_general_trns_tx(struct scd_gadget *,struct usb_request *, u8);
int scd_g_general_trns_rx(struct scd_gadget *,struct usb_request *, u8);
u8 scd_get_substate(struct scd_gadget *, u8);
int scd_set_ctrl_substate(struct scd_gadget *, u8);
int scd_set_epstate(struct scd_gadget *, u8 , u8 );
void scd_g_complete_ep0_request(struct scd_gadget *, int);
int scd_g_ep0int_intknfemp_tx(struct scd_gadget *);
int scd_g_ep0int_xfercompl_rx(struct scd_gadget *);
int scd_g_ep0int_xfercompl_tx(struct scd_gadget *);
int scd_g_ep0int_tmout(struct scd_gadget *);
void scd_g_req_skip_after(struct scd_gadget *);
void scd_g_init_ep0buf(struct scd_dmaep0_buf *);
int scd_g_int_xfercompl_tx(struct scd_gadget *, u8 );
int scd_g_int_xfercompl_rx(struct scd_gadget *, u8);
int scd_g_int_nakeff(struct scd_gadget *, u8);
int scd_g_int_epdis(struct scd_gadget *, u8, u8);
int get_gwq_flag(struct scd_gadget_wq *);
int set_gwq_flag(struct scd_gadget_wq *, int);
static int scd_serach_trans_ep(struct scd_gadget *, u8);
int scd_gadget_set_halt_in(struct scd_gadget *, u8, int);
int scd_gadget_set_halt_out(struct scd_gadget *, u8, int);
int scd_gadget_set_halt_ep0(struct scd_gadget *, int );
int scd_g_setup_intr_int(struct scd_gadget *);
void scd_doneque_init(void);
int scd_gadget_complete_request(struct usb_request *, int);
int scd_gadget_complete_request_nolock(struct usb_request *, int);
void dma_unmap_req(struct scd_device *, struct usb_request *);
void scd_g_reset_setupcnt(struct scd_gadget *, u8);
void set_ep0_self_stall(struct scd_gadget *);
static int req_out_decode(struct scd_gadget *, struct usb_ctrlrequest *);
static int req_in_decode(struct scd_gadget *, struct usb_ctrlrequest *);
static int req_nodata_decode(struct scd_gadget *, struct usb_ctrlrequest *);
static int chk_ep0out_trnsz(struct scd_gadget *, struct usb_request *);
static int chk_ep0in_trnsz(struct scd_gadget *, struct usb_request *);
static u8 *get_dmasetup(struct scd_gadget *, struct scd_dmaep0_buf *, u8);
static int get_dmasetup_outdata(struct scd_gadget *, struct usb_request *
, u8 *);
static int chk_out_trnsz(struct scd_gadget *, struct usb_request *,u8);
static int chk_in_trnsz(struct scd_gadget *, struct usb_request *,u8 );
static int scd_g_setaddr_compl(struct scd_gadget *);
#ifdef SCD_POLL_ON
static int poll_wait_goutnakeff(struct scd_gadget *, u8);
static int poll_wait_outepdis(struct scd_gadget *, u8);
static int poll_wait_inepdis(struct scd_gadget *, u8);
static int poll_wait_innakeff(struct scd_gadget *, u8);
#endif
#ifdef CHK_SUPCNT
static void chk_setupcnt(struct scd_gadget *);
#endif
int scd_g_get_lastbuf(struct scd_gadget *, struct usb_request *, u8);
int scd_g_free_lastbuf(struct scd_gadget *, struct usb_request *, u8);
int scd_g_cp_buf(struct scd_gadget *, struct usb_request *, u8, int);
int scd_g_chk_lastbuf(struct usb_request *);

int scd_g_fifo_flush_tx(struct scd_gadget *, u8, u8);
int scd_g_fifo_flush_rx(struct scd_gadget *, u8, u8);
int scd_g_chk_translimit(struct scd_gadget *, struct usb_request *, u8);
int scd_g_retry_trans(struct scd_gadget *, struct usb_request *, u8);
int scd_g_xfercompl_rx_cont(struct scd_gadget *, struct usb_request *, u8);
int scd_g_xfercompl_rx_zero(struct scd_gadget *, struct usb_request *, u8);
extern int scd_chg_devstate(int, struct scd_gadget *);
extern u8 scd_g_get_devstate(struct scd_gadget *);
extern int chk_flush_comp(struct scd_gadget *, u32);
extern int scd_exec_test_mode(struct scd_gadget *);

/* a buffer for EP0 DMA */
struct scd_dmaep0_buf	scd_dma_ep0buf;

/* for scheduling completion-lists */
struct reqdone_queue	scd_ReqDoneQueue;

/*============================================================================*/

/*-------------------------------------------------------
	name	: scd_g_ctrl_rx_int
	argment	: struct scd_gadget *sgadget
	return	: none
	description:
			  Control rx transfer for EP0
		      BACK2BACKSETUP also is a type of write clear bit.
	date	: 2010/05/07
-------------------------------------------------------*/
void scd_g_ctrl_rx_int(struct scd_gadget *sgadget)
{
	u32 reg_val, msk_val, clr_val;
	struct scd_device *sdev=sgadget->g_device;
	
	reg_val = SCD__READ(sdev, DOEPINT(SCD_G_EP0));
	msk_val = SCD__READ(sdev, DOEPMSK);
	reg_val &= msk_val;
	
	DBG(2, "<== int:%x\n", reg_val);
	/*	
	 * װȽ
	*/
	if(reg_val&SCD_B_DXEPINTN_SETUP){
		/* 
		 * SETUP
		 */
		/* װꥢ*/
		clr_val = SCD_B_DXEPINTN_SETUP;
		SCD__WRITE(sdev,DOEPINT(SCD_G_EP0),clr_val);

		/* SETUP߽ */
		scd_g_setup_int(sgadget);
	}
	else if(reg_val&SCD_B_DXEPINTN_XFERCOMPL){
		/* 
		 * XferCompl
		 */
		/* װꥢ */
		clr_val = SCD_B_DXEPINTN_XFERCOMPL;
		SCD__WRITE(sdev,DOEPINT(SCD_G_EP0),clr_val);

		/* Controlž λ */
		scd_g_ep0int_xfercompl_rx(sgadget);
	}
	else if(reg_val&SCD_B_DXEPINTN_BBLEERRINTRPT){
		/* 
		 * Babble
		 */
		/* װꥢ */
		clr_val = SCD_B_DXEPINTN_BBLEERRINTRPT;
		SCD__WRITE(sdev,DOEPINT(SCD_G_EP0),clr_val);
		INFO("Babble occured!\n");
	}
	else if(reg_val&SCD_B_DXEPINTN_AHBERR){
		/* 
		 * AHBerr
		 */
		/* װꥢ*/
		clr_val = SCD_B_DXEPINTN_AHBERR;
		SCD__WRITE(sdev,DOEPINT(SCD_G_EP0),clr_val);
		INFO("non-support AHBerr\n");
	}
	else if(reg_val&SCD_B_DXEPINTN_EPDISBLD){
		/* 
		 * EPdis
		 */
		/* װꥢ
		 */
		clr_val = SCD_B_DXEPINTN_EPDISBLD;
		SCD__WRITE(sdev,DOEPINT(SCD_G_EP0),clr_val);

		/* NAKͭ */
		scd_g_int_epdis(sgadget, SCD_G_EP0, false);
	}
	else{
		ERR("non-support event occured! doepint:%x\n",reg_val);
	}
	DBG(2,"==>\n");
}
/*-------------------------------------------------------
	name	: scd_g_ctrl_tx_int
	argment	: struct scd_gadget *sgadget
	return	: none
	description:
			  Control tx transfer for EP0
	date	: 2010/05/07
-------------------------------------------------------*/
void scd_g_ctrl_tx_int(struct scd_gadget *sgadget)
{
	u32 reg_val, msk_val, clr_val;
	struct scd_device *sdev=sgadget->g_device;

	reg_val = SCD__READ(sdev, DIEPINT(SCD_G_EP0));
	msk_val = SCD__READ(sdev, DIEPMSK);
	reg_val &= msk_val;
	
	DBG(2, "<== epint:%x\n", reg_val);
	/*	
	 * װȽ
	*/
	if(reg_val&SCD_B_DXEPINTN_XFERCOMPL){
		/* 
		 * XferCompl
		 */
		/* װꥢ*/
		clr_val = SCD_B_DXEPINTN_XFERCOMPL;
		SCD__WRITE(sdev,DIEPINT(SCD_G_EP0),clr_val);

		/* Controlž λ */
		scd_g_ep0int_xfercompl_tx(sgadget);
	}
	else if(reg_val&SCD_B_DXEPINTN_BBLEERRINTRPT){
		/* 
		 * Babble
		 */
		/* װꥢ */
		clr_val = SCD_B_DXEPINTN_BBLEERRINTRPT;
		SCD__WRITE(sdev,DIEPINT(SCD_G_EP0),clr_val);

		DBG(2,"Babble occured!\n");
	}
	else if(reg_val&SCD_B_DXEPINTN_TXFEMP){
		/* 
		 * TxFemp
		 */
		/* װꥢ */
		clr_val = SCD_B_DXEPINTN_TXFEMP;
		SCD__WRITE(sdev,DIEPINT(SCD_G_EP0),clr_val);

		INFO("non-support TxFEmp \n");
	}
	else if(reg_val&SCD_B_DXEPINTN_INEPNAKEFF){
		/* 
		 * INEpNak
		 */
		/* װꥢ */
		clr_val = SCD_B_DXEPINTN_INEPNAKEFF;
		SCD__WRITE(sdev,DIEPINT(SCD_G_EP0),clr_val);

		/* NAKͭ */
		scd_g_int_nakeff(sgadget, SCD_G_EP0);
	}	
	else if(reg_val&SCD_B_DXEPINTN_INTKNTXFEMP){
		/* 
		 * INtknFemp 
		 */
		/* װꥢ */
		clr_val = SCD_B_DXEPINTN_INTKNTXFEMP;
		SCD__WRITE(sdev,DIEPINT(SCD_G_EP0),clr_val);
		
		/* INtknFemp߽ */
		scd_g_ep0int_intknfemp_tx(sgadget);
	}
	else if(reg_val&SCD_B_DXEPINTN_TIMEOUT){
		/* 
		 * Timeout 
		 */
		/* װꥢ */
		clr_val = SCD_B_DXEPINTN_TIMEOUT;
		SCD__WRITE(sdev,DIEPINT(SCD_G_EP0),clr_val);
		
		/* TimeOut߽ */
		scd_g_ep0int_tmout(sgadget);
	}
	else if(reg_val&SCD_B_DXEPINTN_AHBERR){
		/* 
		 * AHBerr 
		 */
		/* װꥢ */
		clr_val = SCD_B_DXEPINTN_AHBERR;
		SCD__WRITE(sdev,DIEPINT(SCD_G_EP0),clr_val);

		INFO("non-support AHBerr \n");
	}
	else if(reg_val&SCD_B_DXEPINTN_EPDISBLD){
		/* 
		 * EPdis
		 */
		/* װꥢ */
		clr_val = SCD_B_DXEPINTN_EPDISBLD;
		SCD__WRITE(sdev,DIEPINT(SCD_G_EP0),clr_val);

		/* NAKͭ */
		scd_g_int_epdis(sgadget, SCD_G_EP0, true);
	}
	else{
		ERR("non-support event occured! doepint:%x\n",reg_val);
	}
	DBG(2,"==>\n");
}
/*-------------------------------------------------------
	name	: scd_g_rxtrans_int
	argment	: struct scd_gadget *sgadget, u8 num
	return	: none
	description:
			  tx transfer except EP0
	date	: 2010/05/07
-------------------------------------------------------*/
void scd_g_rxtrans_int(struct scd_gadget *sgadget, u8 num)
{
	u32 reg_val, msk_val, clr_val;
	struct scd_device *sdev=sgadget->g_device;
	
	reg_val = SCD__READ(sdev, DOEPINT(num));
	msk_val = SCD__READ(sdev, DOEPMSK);
	reg_val &= msk_val;
	
	DBG(2, "<== epnum:%d \n", num);
	/*	
	 * װȽ
	*/
	if(reg_val&SCD_B_DXEPINTN_XFERCOMPL){
		/* 
		 * XferCompl
		 */
		/* װꥢ */
		clr_val = SCD_B_DXEPINTN_XFERCOMPL;
		SCD__WRITE(sdev,DOEPINT(num),clr_val);

		/* λ */
		scd_g_int_xfercompl_rx(sgadget, num);
	}
	else if(reg_val&SCD_B_DXEPINTN_NYETINTRPT){
		/* 
		 * NYET 
		 */
		/* װꥢ */
		clr_val = SCD_B_DXEPINTN_NYETINTRPT;
		SCD__WRITE(sdev,DOEPINT(num),clr_val);

		INFO("non-support NYET \n");
	}
	else if(reg_val&SCD_B_DXEPINTN_NAKINTRPT){
		/* 
		 * NAK
		 */
		/* װꥢ */
		clr_val = SCD_B_DXEPINTN_NAKINTRPT;
		SCD__WRITE(sdev,DOEPINT(num),clr_val);

		INFO("non-support NAK \n");
	}
	else if(reg_val&SCD_B_DXEPINTN_BBLEERRINTRPT){
		/* 
		 * Babble
		 */
		/* װꥢ */
		clr_val = SCD_B_DXEPINTN_BBLEERRINTRPT;
		SCD__WRITE(sdev,DOEPINT(num),clr_val);

		INFO("non-support Babble \n");
	}
	else if(reg_val&SCD_B_DXEPINTN_AHBERR){
		/* 
		 * AHBerr 
		 */
		/* װꥢ */
		clr_val = SCD_B_DXEPINTN_AHBERR;
		SCD__WRITE(sdev,DOEPINT(num),clr_val);

		INFO("non-support AHBerr \n");
	}
	else if(reg_val&SCD_B_DXEPINTN_EPDISBLD){
		/* 
		 * EPdis
		 */
		/* װꥢ */
		clr_val = SCD_B_DXEPINTN_EPDISBLD;
		SCD__WRITE(sdev,DOEPINT(num),clr_val);

		/* NAKͭ */
		scd_g_int_epdis(sgadget, num, false);
	}
	else{
		ERR("non-support event occured! doepint:%x\n",reg_val);
	}
	DBG(2,"==>\n");
}
/*-------------------------------------------------------
	name	: scd_g_txtrans_int
	argment	: struct scd_gadget *sgadget, u8 num
	return	: none
	description:
			  Control tx transfer except EP0
	date	: 2010/05/07
-------------------------------------------------------*/
void scd_g_txtrans_int(struct scd_gadget *sgadget, u8 num)
{
	u32 reg_val, msk_val, clr_val;
	struct scd_device *sdev=sgadget->g_device;
	
	reg_val = SCD__READ(sdev, DIEPINT(num));
	msk_val = SCD__READ(sdev, DIEPMSK);
	reg_val &= msk_val;
	
	DBG(2, "<== epnum:%d\n", num);
	/*	
	 * װȽ
	*/
	if(reg_val&SCD_B_DXEPINTN_XFERCOMPL){
		/* 
		 * XferCompl
		 */
		/* װꥢ */
		clr_val = SCD_B_DXEPINTN_XFERCOMPL;
		SCD__WRITE(sdev,DIEPINT(num),clr_val);
		
		/* λ */
		scd_g_int_xfercompl_tx(sgadget, num);
	}
	else if(reg_val&SCD_B_DXEPINTN_NAKINTRPT){
		/* 
		 * NAK
		 */
		/* װꥢ */
		clr_val = SCD_B_DXEPINTN_NAKINTRPT;
		SCD__WRITE(sdev,DIEPINT(num),clr_val);

		INFO("non-support NAK \n");
	}
	else if(reg_val&SCD_B_DXEPINTN_BBLEERRINTRPT){
		/* 
		 * Babble
		 */
		/* װꥢ */
		clr_val = SCD_B_DXEPINTN_BBLEERRINTRPT;
		SCD__WRITE(sdev,DIEPINT(num),clr_val);

		INFO("non-support Babble \n");
	}
	else if(reg_val&SCD_B_DXEPINTN_TXFEMP){
		/* 
		 * TxFemp
		 */
		/* װꥢ */
		clr_val = SCD_B_DXEPINTN_TXFEMP;
		SCD__WRITE(sdev,DIEPINT(num),clr_val);

		INFO("non-support TxFEmp \n");
	}
	else if(reg_val&SCD_B_DXEPINTN_INEPNAKEFF){
		/* 
		 * INEpNak
		 */
		/* װꥢ */
		clr_val = SCD_B_DXEPINTN_INEPNAKEFF;
		SCD__WRITE(sdev,DIEPINT(num),clr_val);

		/* NAKͭ */
		scd_g_int_nakeff(sgadget, num);
	}	
	else if(reg_val&SCD_B_DXEPINTN_INTKNTXFEMP){
		/* 
		 * INtknNakEff 
		 */
		/* װꥢ */
		clr_val = SCD_B_DXEPINTN_INTKNTXFEMP;
		SCD__WRITE(sdev,DIEPINT(num),clr_val);

		INFO("non-support INTknNAKEff \n");
	}
	else if(reg_val&SCD_B_DXEPINTN_AHBERR){
		/* 
		 * AHBerr 
		 */
		/* װꥢ */
		clr_val = SCD_B_DXEPINTN_AHBERR;
		SCD__WRITE(sdev,DIEPINT(num),clr_val);

		INFO("non-support AHBerr \n");
	}
	else if(reg_val&SCD_B_DXEPINTN_EPDISBLD){
		/* 
		 * EPdis
		 */
		/* װꥢ */
		clr_val = SCD_B_DXEPINTN_EPDISBLD;
		SCD__WRITE(sdev,DIEPINT(num),clr_val);

		/* NAKͭ */
		scd_g_int_epdis(sgadget, num, true);
	}
	else{
		ERR("non-support event occured! doepint:%x\n",reg_val);
	}
	DBG(2,"==>\n");
}
/*-------------------------------------------------------
	name	: scd_g_setup_int
	argment	: struct scd_gadget *sgadget
	return	: none
	description:
			  setup interrupt 
	date	: 2010/10/06
-------------------------------------------------------*/
void scd_g_setup_int(struct scd_gadget *sgadget)
{
	struct scd_device *sdev=sgadget->g_device;
	u8 cur_no	= 0;
	int rslt	= 0;
	u32	reg_val;
	u8 *req_cmd = NULL;
	u8 ep0stage;
	u8 index = 0;
	int intret = 0;
#ifdef DISPLAY_REQ
	int i;
#endif

	DBG(2,"<==\n");

	ep0stage = scd_get_ep0stage(sgadget);
	if(ep0stage!=SCD_EP0_STAGE_SETUP){
		
		DBG(1,"ep0stage:%x\n", ep0stage);
		/* stageǻsetup */
		intret = scd_g_setup_intr_int(sgadget);
		
		/* SETUPǤΥ  */
		rslt = scd_chg_ep0stage(sgadget, SCD_EP0_STAGE_SETUP);
		if(rslt){
			/* 顼Ϥꤨʤ */
			ERR("==> error ep0stage not changed to SETUP \n");
			return;
		}
		/* SubStateѹ */
		rslt = scd_set_ctrl_substate(sgadget, SCD_EP_IDLE);
		if(rslt) {
			/* 顼Ϥꤨʤ */
			ERR("==> abnormal error  not chaneged to IDLE \n");
			return;	/* ܥ顼 */
		}
	}
	/* SubStateѹ */
	rslt = scd_set_ctrl_substate(sgadget, SCD_EP_SETUP_DONE);
	if(rslt) {
		/* ꤨʤ */
		ERR("==> error not to SETUP_DONE \n");
		return;	/* ܥ顼 */
	}
	/* ߶ػ (for CPUmode)*/
	reg_val = SCD__READ(sdev, GINTMSK);
	reg_val	&= ~SCD_B_GINTMSK_RXFLVLMSK;
	SCD__WRITE(sdev, GINTMSK, reg_val);

	/* Setupȼ */
	reg_val = SCD__READ(sdev, DOEPTSIZ0);
	cur_no = (reg_val>>SCD_S_DOEPTSIZ0_SUPCNT)&SCD_M_DOEPTSIZ0_SUPCNT;

	DBG(1,"setcnt:%x\n",cur_no);

	/* ž⡼Ƚ */
	switch(sgadget->bTransMode){
	case SCD_GADGET_TRANS_DMA:
		if(!intret){	/* except DATA_OUT */
			/* DMAž */
			reg_val = SCD__READ(sdev, DOEPINT(SCD_G_EP0));
			if(reg_val&SCD_B_DXEPINTN_BACK2BACKSETUP){
				DBG(1,"Back2Back occured\n");
				/* Back2backװȯ */

				/* DMAɥ쥹ΰ */
				rslt = scd_g_b2b_int(sgadget, &index); 
				if(rslt){
					ERR("==> error b2b \n");
					return;
				}
			}else{
				/* ǥХåեФƾѤ˳Ǽɬפ롣 */
				/* scd_dma_ep0buf.buf */
				switch(SCD_SUPCNT_VAL-cur_no){
				case 1:
					index = SCD_GREQ_IND1;
					break;
				case 2:
					index = SCD_GREQ_IND2;
					break;
				case 3:
					index = SCD_GREQ_IND3;
					break;
				default:
					/* ꤨʤ顼 */
					ERR(">> error setup cnt abnormal error!\n");

					/* Хɥ쥹ᤷ */
					dma_unmap_single(sdev->dev, (dma_addr_t)scd_dma_ep0buf.dma_buf,
					sizeof(*(scd_dma_ep0buf.buf))*SCD_DMA_EP0_SZ, DMA_FROM_DEVICE);

#ifdef DISPLAY_REQ
					{
						for(i=0; i<SCD_DMA_EP0_SZ; i++){
							DBGS(1,"0x%x ",scd_dma_ep0buf.buf[i]);
						}
						DBGS(1,"\n");
					}
#endif 
					/* STALLԤ */
					ERR("self stall 2\n");
					set_ep0_self_stall(sgadget);

					/* setup */
					scd_g_reset_setupcnt(sgadget, SCD_G_SUPCNT_MODE1);
					ERR("==> error4 \n");
					return;
				}
			}
			/* Хɥ쥹ᤷ */
			dma_unmap_single(sdev->dev, (dma_addr_t)scd_dma_ep0buf.dma_buf,
				sizeof(*(scd_dma_ep0buf.buf))*SCD_DMA_EP0_SZ, DMA_FROM_DEVICE);

			/* ǸΥꥯȥǡ򻲾Ȥ */
			req_cmd = get_dmasetup(sgadget, &scd_dma_ep0buf, index);
			if(!req_cmd){
				/* ȥ顼 */
				ERR("==> error setup data abnormal error! \n");
				return;
			}
		}else{
			DBG(1,"setup interrupt at DATA_stage  \n");
			req_cmd = ((struct scd_ep0_buf *)(sgadget->pEp0buf))->req;
			break;
		}
		break;
	case SCD_GADGET_TRANS_CPU:
		/* CPUžϴFIFOФ */
		req_cmd = ((struct scd_ep0_buf *)(sgadget->pEp0buf))->req;
		break;
	}
	if(!req_cmd){
		/* 顼 FIFO˥ǡʤ*/
		ERR("==> error req_cmd NULL! \n");
		return;
	}
#ifdef DISPLAY_REQ
	{
		if((req_cmd[0]==0x00)&&(req_cmd[1]==0x00)){
			for(i=0; i<24; i++){
				printk("0x%x ",scd_dma_ep0buf.buf[i]);
			}
			printk("\n");
		}else{
			for(i=0; i<8; i++){
				DBGS(1,"0x%x ",req_cmd[i]);
			}
			DBGS(1,"\n");
		}
	}
#endif 
	/* setup */
	scd_g_reset_setupcnt(sgadget, SCD_G_SUPCNT_MODE1);

	sgadget->gep[SCD_G_EP0].bEpena    = true;
	/* ꥯȥɲ */
	rslt = read_control_request(sgadget, (struct usb_ctrlrequest *)req_cmd);
	DBG(3, "rslt:%x\n",rslt);
	switch(rslt){
	case SCD_GREQ_THRU:
		/*
		 * ̤Τ롣
		 */
		if (sgadget->pGadgetDriver) {
			if(sgadget->pGadgetDriver->setup){
				/* setup */
				DBG(2,">> setup notification \n");
				rslt = sgadget->pGadgetDriver->setup(sgadget->pGadget,
							   (struct usb_ctrlrequest *)req_cmd);
				if(rslt<0){
					/* STALLԤ */
					ERR("self stall\n");
					set_ep0_self_stall(sgadget);
				}
			}
		}
		break;
	case SCD_GREQ_SKIP:
		/*   
		 * SetAddressƥȥ⡼ɤΰ
		 */
		DBG(3,"SetAddress setup skip \n");

		ep0stage = scd_get_ep0stage(sgadget);

		switch(ep0stage){
		case SCD_EP0_STAGE_STATUS_IN:
			/* 
			 * STATUS_IN
			 * ep_queueȯԤʤΤǡǾ֤ʤƽ롣
			 */
			scd_g_req_skip_after(sgadget);
			break;
		default:
			ERR("non-support stage for SKIP \n");
		}
		break;
	default:
		ERR("==> error setup not notify stall\n");
		/* STALLԤ */
		set_ep0_self_stall(sgadget);
	}
	DBG(2,"==>\n");	
}

/*-------------------------------------------------------
	name	: scd_g_setup_intr_int
	argment	: struct scd_gadget *sgadget
	return	: int
	description:
			  setup interrupt operation in non setup stage
	date	: 2010/08/20
-------------------------------------------------------*/
int scd_g_setup_intr_int(struct scd_gadget *sgadget)
{
	struct scd_gep *gep = &sgadget->gep[SCD_G_EP0];
	u8 ep0stage;
	int status = 0;
	u8 *req_cmd = NULL;	/* for check */
	struct usb_request *req;
	unsigned long rwflags;
	int rslt = 0;

	DBG(1,"<== setup interrupt in non setup stage \n");
	
#ifdef RW_LOCK_ON
	SCD_G_READ_LOCK_IRQ(&gep->rwLock, rwflags);
#endif
	/* ߤΥꥯȼ */
	req = scd_get_currentreq(gep);
#ifdef RW_LOCK_ON
	SCD_G_READ_UNLOCK_IRQ(&gep->rwLock, rwflags);
#endif
	ep0stage = scd_get_ep0stage(sgadget);
	switch(ep0stage){
		case SCD_EP0_STAGE_DATA_IN:
			if(req!=0&&(req->status==-EINPROGRESS)){
				status = 0;
			}
			/* FIFOեå */
			scd_g_fifo_flush_tx(sgadget, SCD_G_EP0, SCD_G_FLUSH_STSKIP);
			break;
		case SCD_EP0_STAGE_DATA_OUT:
			if(req){
				/* ǸΥꥯȥǡ򻲾Ȥ */
				rslt = get_dmasetup_outdata(sgadget, req, req_cmd);
				if(rslt){
					/* ȥ顼 */
					ERR("==>ERROR setup data abnormal error! \n");
					return -ENOENT;
				}
			}else{
				ERR("no request \n");
				break;
			}
			if(req!=0&&(req->status==-EINPROGRESS)){
				status = -EPROTO;
			}
			rslt = SCD_G_DOSETUP;
			/* FIFOեå */
			scd_g_fifo_flush_rx(sgadget, SCD_G_EP0, SCD_G_FLUSH_STSKIP);
			break;
		case SCD_EP0_STAGE_STATUS_IN:
			/* FIFOեå */
			scd_g_fifo_flush_tx(sgadget, SCD_G_EP0, SCD_G_FLUSH_STSKIP);
			/* SCD_EP0_STAGE_STATUS_OUTƱԤ */
		case SCD_EP0_STAGE_STATUS_OUT:
			if(req!=0&&(req->status==-EINPROGRESS)){
				status = 0;	
			}
			break;
		default:
			/* setup stageϤˤϤʤ */
			ERR("abnormal error\n");
			return -EINVAL;
	}
	/* EP0ꥯȴλ */
	scd_g_complete_ep0_request(sgadget, status);

	DBG(1,"==>\n");
	return rslt;
}
/*-------------------------------------------------------
	name	: scd_g_b2b_int
	argment	: struct scd_gadget *sgadget
			  u8 *index
	return	: int
	description:
			  in back2back error
	date	: 2010/03/26
-------------------------------------------------------*/
int scd_g_b2b_int(struct scd_gadget *sgadget, u8 *index)
{
	struct scd_device *sdev = sgadget->g_device;
	u32 reg_val;
	u32 addr;
	u32 diff;

	reg_val = SCD_B_DXEPINTN_BACK2BACKSETUP;
	SCD__WRITE(sdev, DOEPINT(SCD_G_EP0), reg_val);

	reg_val = SCD__READ(sdev, DOEPDMA(SCD_G_EP0));
	addr = reg_val - SCD_GREQ_SZ;
	diff = (u32)addr - (u32)scd_dma_ep0buf.dma_buf;
	DBG(3,"diff:%x addr:%x dma_buf:%p \n",diff, addr, scd_dma_ep0buf.dma_buf);

	/* ̼ */
	if(diff >=0){
		*index = diff/SCD_GREQ_SZ;
		DBG(3,"*index:%x\n",*index);
	}else{
		ERR("==> back2back error \n");
		return -EPERM;
	}
	switch(*index){
	case 0:
		*index = SCD_GREQ_IND1;
		break;
	case 1:
		*index = SCD_GREQ_IND2;
		break;
	case 2:
		*index = SCD_GREQ_IND3;
		break;
	default:
		/* ꤨʤ顼 */
		ERR(">> error invalid index at back2back \n");
	}
	return 0;
}

/*-------------------------------------------------------
	name	: read_control_request
	argment	: struct scd_gadget *sgadget
			  struct usb_ctrlrequest *req
	return	: int
	description:
			  request decode
	date	: 2010/03/29
-------------------------------------------------------*/
int read_control_request(struct scd_gadget *sgadget,struct usb_ctrlrequest *req)
{
	int rslt = 0;
	int st_rslt = 0;
	u8 rdir;

	DBG(2, "<==\n");

	rdir = (req->bRequestType)&SCD_G_REQTYPE_DIR;
	DBG(2, "dir:%x\n",rdir);
	
	switch(rdir){
	case USB_DIR_OUT:
		if(req->wLength == 0){
			/* EP0stage STATUS_IN */
			scd_chg_ep0stage(sgadget, SCD_EP0_STAGE_STATUS_IN);
			/* ꥯȥǥ */
			rslt = req_nodata_decode(sgadget,req);
		}else{
			/* EP0stage DATA_OUT */
			scd_chg_ep0stage(sgadget, SCD_EP0_STAGE_DATA_OUT);
			/* ꥯȥǥ */
			rslt = req_out_decode(sgadget,req);
		}
		break;
	case USB_DIR_IN:
		/* EP0stage DATA_IN */
		scd_chg_ep0stage(sgadget, SCD_EP0_STAGE_DATA_IN);
		/* ꥯȥǥ */
		rslt = req_in_decode(sgadget,req);
		break;
	}
	/* SubStateѹ */
	st_rslt = scd_set_ctrl_substate(sgadget, SCD_EP_QUE_WAIT);
	if(st_rslt){
		/* SubStateѹ顼 */
		rslt = st_rslt;
	}
	DBG(2, "==>\n");
	return rslt;
}

/*-------------------------------------------------------
	name	: req_out_decode
	argment	: struct scd_gadget *sgadget
			  struct usb_ctrlrequest *req
	return	: int
	description:
			  out request decode
	date	: 2010/03/29
-------------------------------------------------------*/
static int req_out_decode(struct scd_gadget *sgadget, struct usb_ctrlrequest *req)
{
	DBG(2, "<==>\n");

	return  SCD_GREQ_THRU;
}

/*-------------------------------------------------------
	name	: req_in_decode
	argment	: struct scd_gadget *sgadget
			  struct usb_ctrlrequest *req
	return	: int
	description:
			  out request decode
	date	: 2010/03/29
-------------------------------------------------------*/
static int req_in_decode(struct scd_gadget *sgadget, struct usb_ctrlrequest *req)
{
	DBG(2, "<==>\n");

	return  SCD_GREQ_THRU;
}

/*-------------------------------------------------------
	name	: req_nodata_decode
	argment	: struct scd_gadget *sgadget
			  struct usb_ctrlrequest *req
	return	: int
	description:
			  2stage request decode
	date	: 2010/06/11
-------------------------------------------------------*/
static int req_nodata_decode(struct scd_gadget *sgadget, struct usb_ctrlrequest *req)
{
	int rslt;
	u8	addr;
	u32 reg_val;
	struct scd_device *sdev = sgadget->g_device;
	unsigned long flags;

	DBG(2, "<==\n");

	switch(req->bRequestType&USB_TYPE_MASK){
	/*	Standard type
	 */
	case USB_TYPE_STANDARD:
		switch(req->bRequest){
		/*	SetAddress  
		 */
		case USB_REQ_SET_ADDRESS:
			/*  */
			rslt = scd_chg_devstate(SCD_GST_ADDR, sgadget);
			if(rslt){
				ERR("error! can not change to ADDR state \n");
				DBG(2, "==>\n");
				return -EPERM;
			}
			if(req->wValue>=SCD_ADDR_LIMIT){
				/* ɥ쥹ϰϳ */
				ERR("error a device address is out of range!\n");
				DBG(2, "==>\n");
				return -EINVAL;
			}
			SCD_G_SPIN_LOCK_IRQ(&sgadget->gLock, flags);

			/* ɥ쥹 */
			addr = (u8)(req->wValue&SCD_M_DCFG_DEVADDR);
			reg_val = SCD__READ(sdev, DCFG);
			reg_val &= ~(SCD_M_DCFG_DEVADDR<<SCD_S_DCFG_DEVADDR);
			reg_val |= addr<<SCD_S_DCFG_DEVADDR;
			SCD__WRITE(sdev, DCFG, reg_val);
			
#ifdef DISPLAY_ADDR
			reg_val = SCD__READ(sdev, DCFG);
			DBG(1,"set addr :%d DCFG:%x \n", addr, reg_val);
#endif
			/* ѥ᡼Ǽ */
			sgadget->bDevAddr = addr;
			/* Τʤ 
			 * ơơѤΥåƥ󥰤ϿƴؿäƤ顣
			 */
			/* ե饰å */
			sgadget->bSetAddr = true;

			SCD_G_SPIN_UNLOCK_IRQ(&sgadget->gLock, flags);
			DBG(2, "==>\n");
			return SCD_GREQ_SKIP;

		case USB_REQ_SET_CONFIGURATION:
			/* ready for a FIFO mapping start */
			/*  */
			rslt = scd_chg_devstate(SCD_GST_CNFG, sgadget);
			if(rslt){
				ERR("error! can not change to CONFIGURATION state \n");
				return -EPERM;
			}	
			/* ե饰 */
			if(sgadget->bFifoCnfg){
				/* ˤϤǽޤäƤʤ */
				ERR("ERROR fifo the former config incompletion \n");

				DBG(2, "==>\n");
				return -EPERM;	
			}
			SCD_G_SPIN_LOCK_IRQ(&sgadget->gLock, flags);
			sgadget->bFifoCnfg	= true;	
			/* ᤷstatusơδλͽ */

			SCD_G_SPIN_UNLOCK_IRQ(&sgadget->gLock, flags);
			DBG(2, "==>\n");
			return SCD_GREQ_THRU;
		}
	}
	/* ʳϾ̤ */
	DBG(2, "==>\n");
	return  SCD_GREQ_THRU;
}

/*-------------------------------------------------------
	name	: scd_chg_ep0stage
	argment	: struct scd_gadget *sgadget
			  int
	return	: int
	description:
			  ep0stage state machine
	date	: 2010/06/11
-------------------------------------------------------*/
int scd_chg_ep0stage(struct scd_gadget *sgadget, int enter)
{
	int rslt = 0;
	unsigned long rwflags;

	DBG(2, "<==\n");
#ifdef RW_LOCK_ON
	SCD_G_WRITE_LOCK_IRQ(&sgadget->rwLock, rwflags);
#endif
	switch(enter){
	/*-- SETUP --*/
	case SCD_EP0_STAGE_SETUP:
		switch(sgadget->bEp0Stage){
		case SCD_EP0_STAGE_SETUP:
			DBG(2,"already state entered! SETUP in %x\n", sgadget->bEp0Stage);
			rslt = 0;	/* but normal */
			break;
		case SCD_EP0_STAGE_INIT:
		case SCD_EP0_STAGE_DATA_IN:
		case SCD_EP0_STAGE_DATA_OUT:
		case SCD_EP0_STAGE_STATUS_IN:
		case SCD_EP0_STAGE_STATUS_OUT:
			/* ѹ */
			sgadget->bEp0Stage = SCD_EP0_STAGE_SETUP;
			rslt = 0;
			break;
		default:
			ERR("error! ep0stage can not change to SETUP \n");
			rslt = -EINVAL;
		}
		break;	
	/*-- DATA_IN --*/
	case SCD_EP0_STAGE_DATA_IN:
		switch(sgadget->bEp0Stage){
		case SCD_EP0_STAGE_SETUP:
			/* ѹ */
			sgadget->bEp0Stage = SCD_EP0_STAGE_DATA_IN;
			rslt = 0;
			break;
		default:
			ERR("error! ep0stage can not change to DATA_IN \n");
			rslt = -EINVAL;
		}
		break;	
	/*-- DATA_OUT --*/
	case SCD_EP0_STAGE_DATA_OUT:
		switch(sgadget->bEp0Stage){
		case SCD_EP0_STAGE_SETUP:
			/* ѹ */
			sgadget->bEp0Stage = SCD_EP0_STAGE_DATA_OUT;
			rslt = 0;
			break;
		default:
			ERR("error! ep0stage can not change to DATA_OUT \n");
			rslt = -EINVAL;
		}
		break;	
	/*-- STATUS_IN --*/
	case SCD_EP0_STAGE_STATUS_IN:
		switch(sgadget->bEp0Stage){
		case SCD_EP0_STAGE_SETUP:
		case SCD_EP0_STAGE_DATA_OUT:
			/* ѹ */
			sgadget->bEp0Stage = SCD_EP0_STAGE_STATUS_IN;
			rslt = 0;
			break;
		default:
			ERR("error! ep0stage can not change to STATUS_IN \n");
			rslt = -EINVAL;
		}
		break;	
	/*-- STATUS_OUT --*/
	case SCD_EP0_STAGE_STATUS_OUT:
		switch(sgadget->bEp0Stage){
		case SCD_EP0_STAGE_DATA_IN:
			/* ѹ */
			sgadget->bEp0Stage = SCD_EP0_STAGE_STATUS_OUT;
			rslt = 0;
			break;
		default:
			ERR("error! ep0stage can not change to STATUS_OUT \n");
			rslt = -EINVAL;
		}
		break;	
	}
	DBG(2, "==> ep0stage:%x \n",sgadget->bEp0Stage);
#ifdef RW_LOCK_ON
	SCD_G_WRITE_UNLOCK_IRQ(&sgadget->rwLock, rwflags);
#endif
	return rslt;
}

/*-------------------------------------------------------
	name	: scd_get_ep0stage
	argment	: struct scd_gadget *sgadget
	return	: u8
	description:
			  ep0stage state machine
	date	: 2010/06/11
-------------------------------------------------------*/
u8 scd_get_ep0stage(struct scd_gadget *sgadget)
{
	u8 stage;
	unsigned long rwflags;

	DBG(2, "<==\n");
#ifdef RW_LOCK_ON
	SCD_G_READ_LOCK_IRQ(&sgadget->rwLock, rwflags);
#endif
	stage = sgadget->bEp0Stage;
#ifdef RW_LOCK_ON
	SCD_G_READ_UNLOCK_IRQ(&sgadget->rwLock, rwflags);
#endif
	DBG(2, "==>\n");
	return stage;
}

/*-------------------------------------------------------
	name	: scd_get_substate
	argment	: struct scd_gadget *sgadget
			  u8 epnum
	return	: u8
	description:
			  get a ep state
	date	: 2010/06/11
-------------------------------------------------------*/
u8 scd_get_substate(struct scd_gadget *sgadget, u8 epnum)
{
	struct scd_gep *gep=&(sgadget->gep[epnum]);
	unsigned long rwflags;
	u8 state;

	DBG(2, "<==\n");
	
#ifdef RW_LOCK_ON
	SCD_G_READ_LOCK_IRQ(&gep->rwLock, rwflags);
#endif
	/* ּ */
	state = gep->bState;
#ifdef RW_LOCK_ON
	SCD_G_READ_UNLOCK_IRQ(&gep->rwLock, rwflags);
#endif
	DBG(2, "==>\n");
	return state;
}

/*-------------------------------------------------------
	name	: scd_set_ctrl_substate
	argment	: struct scd_gadget *sgadget
			  u8 enter;
	return	: int
	description:
			  set the ep state
	date	: 2010/08/24
-------------------------------------------------------*/
int scd_set_ctrl_substate(struct scd_gadget *sgadget, u8 enter)
{
	int rslt=0;
	struct scd_gep *gep=&(sgadget->gep[SCD_G_EP0]);
	unsigned long rwflags;
	u8	mode = sgadget->bTransMode;
	u8	stage;
	u8	curstate;

	DBG(2, "<==\n");

#ifdef RW_LOCK_ON
	SCD_G_READ_LOCK_IRQ(&sgadget->rwLock, rwflags);
#endif
	stage = scd_get_ep0stage(sgadget);
#ifdef RW_LOCK_ON
	SCD_G_READ_UNLOCK_IRQ(&sgadget->rwLock, rwflags);
#endif
#ifdef RW_LOCK_ON
	SCD_G_READ_LOCK_IRQ(&gep->rwLock, rwflags);
#endif
	curstate = scd_get_substate(sgadget, SCD_G_EP0);
#ifdef RW_LOCK_ON
	SCD_G_READ_UNLOCK_IRQ(&gep->rwLock, rwflags);
#endif
	
	DBG(2,"change ctrl substate 0x%2x to 0x%2x \n", curstate, enter);

#ifdef RW_LOCK_ON
	SCD_G_WRITE_LOCK_IRQ(&gep->rwLock, rwflags);
#endif
	switch(stage){
/*---------------------------------------------------------*/
	case SCD_EP0_STAGE_SETUP:
		/* at SETUP stage */
		switch(enter){
		case SCD_EP_IDLE:
			switch(curstate){
			case SCD_EP_GET_SETUP:
				if(mode==SCD_GADGET_TRANS_DMA){
					/*  */
					goto out_mode;
				}
			case SCD_EP_IDLE:
			case SCD_EP_SETUP_DONE:
			case SCD_EP_INTR_SETUP:
			case SCD_EP_STALL:
			case SCD_EP_FLUSH:
			case SCD_EP_QUE_WAIT:		/* IDLEؤɲ */
			case SCD_EP_DATA_CONT:
				gep->bState = enter;
				break;
			case SCD_EP_OUT_READY:
			case SCD_EP_IN_READY:
			case SCD_EP_DATA_END:
			case SCD_EP_DATA_COMP:
			case SCD_EP_ZERO_END:
				DBG(1,"new applicable state transition \n");
				gep->bState = enter;
				break;
			default:
				/* ϰϳ */
				goto out_mode;
			}
			break;
		case SCD_EP_GET_SETUP:
			switch(curstate){
			case SCD_EP_IDLE:
				if(mode==SCD_GADGET_TRANS_DMA){
					/*  */
					goto out_mode;
				}				
				gep->bState = enter;
				break;
			default:
				/* ϰϳ */
				goto out_mode;
			}
			break;
		case SCD_EP_SETUP_DONE:
			switch(mode){
			case SCD_GADGET_TRANS_CPU:
				switch(curstate){
				case SCD_EP_GET_SETUP:
					gep->bState = enter;
					break;
				default:
					/* ϰϳ */
					goto out_mode;
				}
				break;
			case SCD_GADGET_TRANS_DMA:
				switch(curstate){
				case SCD_EP_IDLE:
					gep->bState = enter;
					break;
				default:
					/* ϰϳ */
					goto out_mode;
				}
				break;
			default:
				/* mode */
				goto ab_err;
			}
			break;
		case SCD_EP_FLUSH:
			switch(curstate){
			case SCD_EP_GET_SETUP:
				if(mode==SCD_GADGET_TRANS_DMA){
					/*  */
					goto out_mode;
				}
			case SCD_EP_IDLE:
			case SCD_EP_SETUP_DONE:
			case SCD_EP_INTR_SETUP:
			case SCD_EP_STALL:
			case SCD_EP_QUE_WAIT:		/* IDLEؤɲ */
				gep->bState = enter;
				break;
			default:
				/* ϰϳ */
				goto out_mode;
			}
			break;
		case SCD_EP_STALL:
			switch(curstate){
			case SCD_EP_SETUP_DONE:
				gep->bState = enter;
			}
			break;
		default:
			goto nochange;
		}
		break;
/*---------------------------------------------------------*/
	case SCD_EP0_STAGE_DATA_OUT:
		/* at data OUT stage */
		switch(enter){
		case SCD_EP_QUE_WAIT:
			/* SETUP stage -> data out */
			switch(curstate){
			case SCD_EP_SETUP_DONE:
				gep->bState = enter;
				break;
			default:
				/* ϰϳ */
				goto out_mode;
			}
			break;
		case SCD_EP_OUT_READY:
			switch(curstate){
			case SCD_EP_QUE_WAIT:
				gep->bState = enter;
				break;
			default:
				goto out_mode;
			}
			break;
		case SCD_EP_DATA_CONT:
			switch(curstate){
			case SCD_EP_OUT_READY:
			case SCD_EP_DATA_COMP:
				gep->bState = enter;
				break;
			default:
				goto out_mode;
			}
			break;
		case SCD_EP_DATA_END:
			switch(curstate){
			case SCD_EP_OUT_READY:
			case SCD_EP_DATA_COMP:
				gep->bState = enter;
				break;
			default:
				goto out_mode;
			}
			break;
		case SCD_EP_DATA_COMP:
			switch(curstate){
			case SCD_EP_OUT_READY:
				if(mode==SCD_GADGET_TRANS_CPU){
					/*  */
					goto out_mode;
				}
				gep->bState = enter;
				break;
			case SCD_EP_DATA_CONT:
			case SCD_EP_DATA_END:
				gep->bState = enter;
				break;
			case SCD_EP_STALL:
				gep->bState = enter;
				break;
			default:
				goto out_mode;
			}
			break;
		case SCD_EP_DATA_ERR:
			switch(curstate){
			case SCD_EP_OUT_READY:
				gep->bState = enter;
				break;
			case SCD_EP_DATA_CONT:
				if(mode==SCD_GADGET_TRANS_DMA){
					/*  */
					goto out_mode;
				}
				gep->bState = enter;
				break;
			default:
				goto out_mode;
			}
			break;
		case SCD_EP_INTR_SETUP:
			switch(curstate){
			case SCD_EP_QUE_WAIT:
			case SCD_EP_OUT_READY:
			case SCD_EP_DATA_CONT:
			case SCD_EP_DATA_END:
				gep->bState = enter;
				break;
			default:
				goto out_mode;
			}
			break;
		case SCD_EP_STALL:
			switch(curstate){	/* IDLEϤʤCOMPϥɥʳǤϤʤ */
			case SCD_EP_QUE_WAIT:
			case SCD_EP_OUT_READY:
			case SCD_EP_DATA_CONT:
			case SCD_EP_DATA_END:
			case SCD_EP_STALL:
				gep->bState = enter;
				break;
			default:
				goto out_mode;
			}
			break;
		case SCD_EP_FLUSH:
			switch(curstate){	/* IDLEϤʤCOMPϥɥʳǤϤʤ */
			case SCD_EP_QUE_WAIT:
			case SCD_EP_OUT_READY:
			case SCD_EP_DATA_CONT:
			case SCD_EP_DATA_END:
			case SCD_EP_STALL:
				gep->bState = enter;
				break;
			default:
				goto out_mode;
			}
			break;
		default:
			goto nochange;
		}
		break;
/*---------------------------------------------------------*/
	case SCD_EP0_STAGE_DATA_IN:
		/* at data IN stage */
		switch(enter){
		case SCD_EP_QUE_WAIT:
			switch(curstate){
			case SCD_EP_SETUP_DONE:
				gep->bState = enter;
				break;
			default:
				goto out_mode;
			}
			break;
		case SCD_EP_IN_READY:
			switch(curstate){
			case SCD_EP_QUE_WAIT:
				gep->bState = enter;
				break;
			default:
				goto out_mode;
			}
			break;
		case SCD_EP_DATA_CONT:
			switch(curstate){
			case SCD_EP_IN_READY:
			case SCD_EP_DATA_COMP:
				gep->bState = enter;
				break;
			default:
				goto out_mode;
			}
			break;
		case SCD_EP_DATA_END:
			switch(curstate){
			case SCD_EP_IN_READY:
			case SCD_EP_DATA_COMP:
				gep->bState = enter;
				break;
			default:
				goto out_mode;
			}
			break;
		case SCD_EP_DATA_COMP:
			switch(curstate){
			case SCD_EP_DATA_CONT:
			case SCD_EP_DATA_END:
			case SCD_EP_ZERO_END:
				gep->bState = enter;
				break;
			case SCD_EP_STALL:
				gep->bState = enter;
				break;
			default:
				goto out_mode;
			}
			break;
		case SCD_EP_ZERO_END:
			switch(curstate){
			case SCD_EP_DATA_COMP:
				gep->bState = enter;
				break;
			default:
				goto out_mode;
			}
			break;
		case SCD_EP_INTR_SETUP:
			switch(curstate){
			case SCD_EP_QUE_WAIT:
			case SCD_EP_DATA_CONT:
			case SCD_EP_DATA_END:
			case SCD_EP_ZERO_END:
				gep->bState = enter;
				break;
			case SCD_EP_IN_READY:
				if(mode==SCD_GADGET_TRANS_DMA){
					/*  */
					goto out_mode;
				}
				gep->bState = enter;
				break;
			default:
				goto out_mode;
			}
			break;
		case SCD_EP_STALL:
			switch(curstate){	/* IDLEϤʤCOMPϥɥʳǤϤʤ */
			case SCD_EP_QUE_WAIT:
			case SCD_EP_IN_READY:
			case SCD_EP_DATA_CONT:
			case SCD_EP_DATA_END:
			case SCD_EP_ZERO_END:
			case SCD_EP_STALL:
				gep->bState = enter;
				break;
			default:
				goto out_mode;
			}
			break;
		case SCD_EP_FLUSH:
			switch(curstate){	/* IDLEϤʤCOMPϥɥʳǤϤʤ */
			case SCD_EP_QUE_WAIT:
			case SCD_EP_IN_READY:
			case SCD_EP_DATA_CONT:
			case SCD_EP_DATA_END:
			case SCD_EP_ZERO_END:
			case SCD_EP_STALL:
				gep->bState = enter;
				break;
			default:
				goto out_mode;
			}
			break;
		default:
			goto nochange;
		}
		break;
/*---------------------------------------------------------*/
	case SCD_EP0_STAGE_STATUS_IN:
		/* at status IN stage */
		switch(enter){
		case SCD_EP_IDLE:
			switch(curstate){
			case SCD_EP_DATA_COMP:	/* from dataOUT */
				gep->bState = enter;
				break;
			case SCD_EP_QUE_WAIT:	/* from dataOUT(nodata) */
				gep->bState = enter;
				break;
			default:
				goto out_mode;
			}
			break;
		case SCD_EP_QUE_WAIT:
			switch(curstate){
			case SCD_EP_SETUP_DONE:	/* from SETUP */
				gep->bState = enter;
				break;
			default:
				goto out_mode;
			}
			break;
		case SCD_EP_DATA_END:
			switch(curstate){
			case SCD_EP_IDLE:
				if(mode==SCD_GADGET_TRANS_DMA){
					/*  */
					goto out_mode;
				}
				gep->bState = enter;
				break;
			default:
				goto out_mode;
			}
			break;
		case SCD_EP_DATA_COMP:
			switch(curstate){
			case SCD_EP_IDLE:
				if(mode==SCD_GADGET_TRANS_CPU){
					/*  */
					goto out_mode;
				}
				gep->bState = enter;
				break;
			case SCD_EP_QUE_WAIT:
				gep->bState = enter;
				break;
			default:
				goto out_mode;
			}
			break;
		case SCD_EP_DATA_ERR:
			switch(curstate){
			case SCD_EP_IDLE:
				gep->bState = enter;
				break;
			default:
				goto out_mode;
			}
			break;
		case SCD_EP_INTR_SETUP:
			switch(curstate){
			case SCD_EP_IDLE:
			case SCD_EP_DATA_END:
				gep->bState = enter;
				break;
			default:
				goto out_mode;
			}
			break;
		case SCD_EP_STALL:
			switch(curstate){	/* IDLEϤʤCOMPϥɥʳǤϤʤ */
			case SCD_EP_IDLE:
			case SCD_EP_QUE_WAIT:
			case SCD_EP_DATA_END:
			case SCD_EP_IN_READY:
			case SCD_EP_STALL:
				gep->bState = enter;
				break;
			default:
				goto out_mode;
			}
			break;
		case SCD_EP_FLUSH:
			switch(curstate){	/* COMPϥɥʳǤϤʤ */
			case SCD_EP_IDLE:
			case SCD_EP_QUE_WAIT:
			case SCD_EP_DATA_END:
			case SCD_EP_IN_READY:
			case SCD_EP_STALL:
				gep->bState = enter;
				break;
			default:
				goto out_mode;
			}
			break;
		default:
			goto nochange;
		}
		break;
/*---------------------------------------------------------*/
	case SCD_EP0_STAGE_STATUS_OUT:
		/* at status OUT stage */
		switch(enter){
		case SCD_EP_IDLE:
			switch(curstate){
			case SCD_EP_DATA_COMP:
			case SCD_EP_FLUSH:
				gep->bState = enter;
				break;
			default:
				goto out_mode;
			}
			break;
		case SCD_EP_DATA_END:
			switch(curstate){
			case SCD_EP_IDLE:
				if(mode==SCD_GADGET_TRANS_DMA){
					/*  */
					goto out_mode;
				}
				gep->bState = enter;
				break;
			default:
				goto out_mode;
			}
			break;
		case SCD_EP_DATA_COMP:
			switch(curstate){
			case SCD_EP_IDLE:
				if(mode==SCD_GADGET_TRANS_CPU){
					/*  */
					goto out_mode;
				}
				gep->bState = enter;
				break;
			case SCD_EP_DATA_END:
				if(mode==SCD_GADGET_TRANS_DMA){
					/*  */
					goto out_mode;
				}
				gep->bState = enter;
				break;
			default:
				goto out_mode;
			}
			break;
		case SCD_EP_DATA_ERR:
			switch(curstate){
			case SCD_EP_IDLE:
				gep->bState = enter;
				break;
			default:
				goto out_mode;
			}
			break;
		case SCD_EP_INTR_SETUP:
			switch(curstate){
			case SCD_EP_IDLE:
			case SCD_EP_DATA_END:
			case SCD_EP_DATA_COMP:
				gep->bState = enter;
				break;
			default:
				goto out_mode;
			}
			break;
		case SCD_EP_STALL:
			switch(curstate){	/* COMPϥϥɥʳǤϤʤ */
			case SCD_EP_IDLE:
			case SCD_EP_DATA_END:
			case SCD_EP_STALL:
				gep->bState = enter;
				break;
			default:
				goto out_mode;
			}
			break;
		case SCD_EP_FLUSH:
			switch(curstate){	
			case SCD_EP_IDLE:
			case SCD_EP_DATA_END:
			case SCD_EP_STALL:
				gep->bState = enter;
				break;
			default:
				goto out_mode;
			}
			break;
		default:
			goto nochange;
		}
		break;
	}
	DBG(2,"gep->state:%x\n",gep->bState);
#ifdef RW_LOCK_ON
	SCD_G_WRITE_UNLOCK_IRQ(&gep->rwLock, rwflags);
#endif
	return rslt;
nochange:
	/* cannot change  */
	ERR("cannot change to this state! 0x%2x to 0x%2x\n", curstate, enter);
#ifdef RW_LOCK_ON
	SCD_G_WRITE_UNLOCK_IRQ(&gep->rwLock, rwflags);
#endif
	return -EINVAL;
out_mode:
	/* out of mode */
	ERR("error not applicable mode:%x 0x%2x to 0x%2x\n",mode, curstate, enter);
#ifdef RW_LOCK_ON
	SCD_G_WRITE_UNLOCK_IRQ(&gep->rwLock, rwflags);
#endif
	return -EINVAL;
ab_err:
	/* at the unkown mode */
	ERR("error : unknown mode %x \n", mode);
#ifdef RW_LOCK_ON
	SCD_G_WRITE_UNLOCK_IRQ(&gep->rwLock, rwflags);
#endif
	return -EINVAL;
}

/*-------------------------------------------------------
	name	: scd_set_epstate
	argment	: struct scd_gadget *sgadget
			  u8 epnum
			  u8 enter
	return	: int
	description:
			  set the ep state except ctrl-ep
			  󥻥ĥɬפ(IDLEϤĤǤOKȤ)
	date	: 2010/04/13
-------------------------------------------------------*/
int scd_set_epstate(struct scd_gadget *sgadget, u8 epnum, u8 enter)
{
	struct scd_gep *gep=&(sgadget->gep[epnum]);
	u8	mode = sgadget->bTransMode;
	u8	curstate;
	unsigned long rwflags;

#ifdef RW_LOCK_ON
	SCD_G_READ_LOCK_IRQ(&gep->rwLock, rwflags);
#endif
	curstate = scd_get_substate(sgadget, epnum);
#ifdef RW_LOCK_ON
	SCD_G_READ_UNLOCK_IRQ(&gep->rwLock, rwflags);
#endif
	DBG(2,"<==change ep-state! 0x%2x to 0x%2x \n", curstate, enter);

#ifdef RW_LOCK_ON
	SCD_G_WRITE_LOCK_IRQ(&gep->rwLock, rwflags);
#endif
	if(gep->bIsTx){ /* Tx */

		switch(enter){
		case SCD_EP_IDLE:
		/*---------------------------------------------------------*/
			switch(curstate){
			case SCD_EP_IDLE:
			case SCD_EP_DATA_COMP:
			case SCD_EP_STALL:
			case SCD_EP_FLUSH:
			case SCD_EP_DATA_END:	/* 顼 */
			case SCD_EP_DATA_CONT:	/* 顼 */
			case SCD_EP_IN_READY:	/* 顼 */
			case SCD_EP_ZERO_END:	/* 顼 */
				gep->bState = enter;
				break;
			default:
				/* ϰϳ */
				goto out_mode;
			}
			break;
		case SCD_EP_IN_READY:
		/*---------------------------------------------------------*/
			switch(curstate){
			case SCD_EP_IDLE:
				gep->bState = enter;
				break;
			default:
				/* ϰϳ */
				goto out_mode;
			}
			break;
		case SCD_EP_DATA_CONT:
		/*---------------------------------------------------------*/
			switch(curstate){
			case SCD_EP_IN_READY:
			case SCD_EP_DATA_CONT:
				if(mode==SCD_GADGET_TRANS_DMA){
					/*  */
					goto out_mode;
				}
				gep->bState = enter;
				break;
			case SCD_EP_DATA_COMP:
				gep->bState = enter;
				break;
			default:
				/* ϰϳ */
				goto out_mode;
			}
			break;
		case SCD_EP_DATA_END:
		/*---------------------------------------------------------*/
			switch(curstate){
			case SCD_EP_IN_READY:
				if(mode==SCD_GADGET_TRANS_DMA){
					/*  */
					goto out_mode;
				}
				gep->bState = enter;
				break;
			case SCD_EP_DATA_CONT:
				if(mode==SCD_GADGET_TRANS_DMA){
					/*  */
					goto out_mode;
				}
				gep->bState = enter;
				break;
			default:
				/* ϰϳ */
				goto out_mode;
			}
			break;
		case SCD_EP_DATA_COMP:
		/*---------------------------------------------------------*/
			switch(curstate){
			case SCD_EP_IN_READY:
				if(mode==SCD_GADGET_TRANS_CPU){
					/*  */
					goto out_mode;
				}
				gep->bState = enter;
				break;
			case SCD_EP_DATA_CONT:
				gep->bState = enter;
				break;
			case SCD_EP_ZERO_END:
				gep->bState = enter;
				break;
			default:
				/* ϰϳ */
				goto out_mode;
			}
			break;
		case SCD_EP_ZERO_END:
		/*---------------------------------------------------------*/
			switch(curstate){
			case SCD_EP_DATA_COMP:
				gep->bState = enter;
				break;
			default:
				/* ϰϳ */
				goto out_mode;
			}
			break;
		case SCD_EP_DATA_ERR:
		/*---------------------------------------------------------*/
			switch(curstate){
			case SCD_EP_IDLE:
			case SCD_EP_IN_READY:
			case SCD_EP_DATA_CONT:
			case SCD_EP_DATA_END:
			case SCD_EP_DATA_COMP:
			case SCD_EP_ZERO_END:
				gep->bState = enter;
				break;
			default:
				/* ϰϳ */
				goto out_mode;
			}
			break;
		case SCD_EP_STALL:
		/*---------------------------------------------------------*/
			switch(curstate){
			case SCD_EP_IDLE:
			case SCD_EP_IN_READY:
			case SCD_EP_DATA_CONT:
			case SCD_EP_DATA_END:
			case SCD_EP_ZERO_END:
			case SCD_EP_DATA_ERR:
				gep->bState = enter;
				break;
			default:
				/* ϰϳ */
				goto out_mode;
			}
			break;
		case SCD_EP_FLUSH:
		/*---------------------------------------------------------*/
			switch(curstate){
			case SCD_EP_IDLE:
			case SCD_EP_IN_READY:
			case SCD_EP_DATA_CONT:
			case SCD_EP_DATA_END:
			case SCD_EP_ZERO_END:
			case SCD_EP_DATA_ERR:
			case SCD_EP_STALL:
				gep->bState = enter;
				break;
			default:
				/* ϰϳ */
				goto out_mode;
			}
			break;
		default:
			goto nochange;
		}
	/*---------------------------------------------------------*/
	}else{ /* Rx */
		switch(enter){
		case SCD_EP_IDLE:
			switch(curstate){
			case SCD_EP_IDLE:
			case SCD_EP_DATA_COMP:
			case SCD_EP_STALL:
			case SCD_EP_FLUSH:
			case SCD_EP_DATA_END:	/* 顼 */
			case SCD_EP_DATA_CONT:	/* 顼 */
			case SCD_EP_OUT_READY:	/* 顼 */
				gep->bState = enter;
				break;
			default:
				/* ϰϳ */
				goto out_mode;
			}
			break;
		/*---------------------------------------------------------*/
		case SCD_EP_OUT_READY:
			switch(curstate){
			case SCD_EP_IDLE:
				gep->bState = enter;
				break;
			default:
				/* ϰϳ */
				goto out_mode;
			}
			break;
		case SCD_EP_DATA_CONT:
		/*---------------------------------------------------------*/
			switch(curstate){
			case SCD_EP_OUT_READY:
			case SCD_EP_DATA_COMP:
				gep->bState = enter;
				break;
			case SCD_EP_DATA_CONT:
				if(mode==SCD_GADGET_TRANS_DMA){
					/*  */
					goto out_mode;
				}
				gep->bState = enter;
				break;
			default:
				/* ϰϳ */
				goto out_mode;
			}
			break;
		case SCD_EP_DATA_END:
		/*---------------------------------------------------------*/
			switch(curstate){
			case SCD_EP_OUT_READY:
			case SCD_EP_DATA_CONT:
			case SCD_EP_DATA_COMP:
				gep->bState = enter;
				break;
			default:
				/* ϰϳ */
				goto out_mode;
			}
			break;
		case SCD_EP_DATA_COMP:
		/*---------------------------------------------------------*/
			switch(curstate){
			case SCD_EP_OUT_READY:
				if(mode==SCD_GADGET_TRANS_CPU){
					/*  */
					goto out_mode;
				}
				gep->bState = enter;
				break;
			case SCD_EP_DATA_CONT:
			case SCD_EP_DATA_END:
				gep->bState = enter;
				break;
			case SCD_EP_ZERO_END:
				gep->bState = enter;
				break;
			default:
				/* ϰϳ */
				goto out_mode;
			}
			break;
		case SCD_EP_ZERO_END:
		/*---------------------------------------------------------*/
			switch(curstate){
			case SCD_EP_OUT_READY:
			case SCD_EP_DATA_COMP:
				gep->bState = enter;
				break;
			default:
				/* ϰϳ */
				goto out_mode;
			}
			break;
		case SCD_EP_DATA_ERR:
		/*---------------------------------------------------------*/
			switch(curstate){
			case SCD_EP_IDLE:
			case SCD_EP_OUT_READY:
			case SCD_EP_DATA_CONT:
			case SCD_EP_DATA_END:
			case SCD_EP_DATA_COMP:
			case SCD_EP_ZERO_END:
				gep->bState = enter;
				break;
			default:
				/* ϰϳ */
				goto out_mode;
			}
			break;
		case SCD_EP_STALL:
		/*---------------------------------------------------------*/
			switch(curstate){
			case SCD_EP_IDLE:
			case SCD_EP_OUT_READY:
			case SCD_EP_DATA_CONT:
			case SCD_EP_DATA_END:
			case SCD_EP_DATA_COMP:
			case SCD_EP_ZERO_END:
				gep->bState = enter;
				break;
			default:
				/* ϰϳ */
				goto out_mode;
			}
			break;
		case SCD_EP_FLUSH:
		/*---------------------------------------------------------*/
			switch(curstate){
			case SCD_EP_IDLE:
			case SCD_EP_OUT_READY:
			case SCD_EP_DATA_CONT:
			case SCD_EP_DATA_END:
			case SCD_EP_DATA_COMP:
			case SCD_EP_ZERO_END:
			case SCD_EP_DATA_ERR:
			case SCD_EP_STALL:
				gep->bState = enter;
				break;
			default:
				/* ϰϳ */
				goto out_mode;
			}
			break;
		default:
			goto nochange;
		}
	}
	DBG(2, "==>\n");
#ifdef RW_LOCK_ON
	SCD_G_WRITE_UNLOCK_IRQ(&gep->rwLock, rwflags);
#endif
	return 0;
out_mode:
	/* out of mode */
	ERR("==> error not applicable mode:%x :0x%2x to 0x%2x\n",mode, curstate, enter);
#ifdef RW_LOCK_ON
	SCD_G_WRITE_UNLOCK_IRQ(&gep->rwLock, rwflags);
#endif
	return -EINVAL;
nochange:
	/* cannot change  */
	ERR("==> error cannot change to this state! 0x%2x to 0x%2x \n", curstate, enter);
#ifdef RW_LOCK_ON
	SCD_G_WRITE_UNLOCK_IRQ(&gep->rwLock, rwflags);
#endif
	return -EINVAL;
}
/*-------------------------------------------------------
	name	: scd_g_ep0_data_out
	argment	: struct scd_gadget *sgadget
			  gfp_t gfp_flags
	return	: int
	description:
			  transfer request for a data out stage
	date	: 2010/03/31
-------------------------------------------------------*/
int scd_g_ep0_data_out(struct scd_gadget *sgadget, gfp_t gfp_flags)
{
	int rslt=0;
	u32 reg_val;
	struct scd_device *sdev = sgadget->g_device;
	struct scd_gep *gep = &(sgadget->gep[SCD_G_EP0]);
	struct usb_request *pRequest;
	unsigned req_len;
	u32	set_len;
	u16	maxp;
	u8	pktnum;
	unsigned rwflags;

	DBG(2, "<==\n");

#ifdef RW_LOCK_ON
	SCD_G_READ_LOCK_IRQ(&gep->rwLock, rwflags);
#endif
	/* ߤΥꥯȼ */
	pRequest = scd_get_currentreq(gep);
#ifdef RW_LOCK_ON
	SCD_G_READ_UNLOCK_IRQ(&gep->rwLock, rwflags);
#endif
	DBG(3,"pRequest:%p \n", pRequest);
	if(!pRequest){
		ERR("pRequest NULL \n");
		DBG(2, "==>\n");
		return -ENOENT;
	}
	/*
	 * setting parameters
	 */
	req_len		= pRequest->length;
	maxp		= gep->wMaxp;	/* usb_ep.maxpacket ˤ⤢ */
	pRequest->actual	=	0;	/* Initialize */

#ifdef DISPLAY_CTRL_DSTAGE
	DBG(1,"length:0x%x maxp:0x%x actual:0x%x \n",
		pRequest->length,gep->wMaxp,pRequest->actual);
#endif
	/* 쥸ꥵ */
	set_len	= maxp&SCD_M_DOEPTSIZ0_XFERSIZE;

	/* ѥåȥ */
	pktnum	= SCD_MAX_EP0_PACKET&SCD_M_DOEPTSIZ0_PKTCNT;	/* 1 */

#ifdef DISPLAY_CTRL_DSTAGE
	DBG(1,"set_len:%x pktnum:%x \n",gep->dwLen, pktnum);
#endif
	/* ž/ѥåȥ */
	reg_val = SCD__READ(sdev, DOEPTSIZ0);
	reg_val = (pktnum<<SCD_S_DOEPTSIZ0_PKTCNT)|set_len;
	SCD__WRITE(sdev, DOEPTSIZ0, reg_val);

	if(sgadget->bTransMode == SCD_GADGET_TRANS_DMA){
		if(req_len < maxp){
			rslt = scd_g_get_lastbuf(sgadget, pRequest, SCD_G_EP0);
			if(rslt){
				ERR("==> error can't ready a lastbuf \n");
				return -EPERM;
			}	
			reg_val = (u32)gep->lbuf.dma;
			SCD__WRITE(sdev, DOEPDMA(SCD_G_EP0), reg_val);

#ifdef DISPLAY_CTRL_DSTAGE
			reg_val = SCD__READ(sdev, DOEPDMA(SCD_G_EP0));
			DBG(3,"DOEPDMA0:%x \n",reg_val);
#endif
			/* ž */
			gep->dwLen = req_len;
		}else{
			/* DMAХåեݥ󥿽 */
			if(!pRequest->dma){
				/* DMAǤʤΤѴ롣
				 * ǤʤС顼ˤ롣
				 */
				/* Хɥ쥹 */
				pRequest->dma = dma_map_single(sdev->dev, pRequest->buf, 
					pRequest->length, DMA_FROM_DEVICE);
				DBG(3,"dma buf mapping buf:%p dmabuf:%p\n",(void *)pRequest->buf,(void *)pRequest->dma);
				if(!pRequest->dma){
					ERR("error pRequest->dma = NULL\n");
					DBG(2, "==>\n");
					return -EINVAL;
				}
				/* ° */
				((struct scd_grequest *)pRequest)->bAttribute |= SCD_REQ_RX_MAP;
			}else{
				/* åιθ */
				dma_sync_single_for_device(sdev->dev, pRequest->dma, 
					pRequest->length, DMA_FROM_DEVICE);	/* write */
				DBG(3,"dma buf sync buf:%p dmabuf:%p\n",(void *)pRequest->buf,(void *)pRequest->dma);
				/* ° */
				((struct scd_grequest *)pRequest)->bAttribute |= SCD_REQ_RX_SYNC;
			}
			reg_val = (u32)pRequest->dma;
			SCD__WRITE(sdev, DOEPDMA(SCD_G_EP0), reg_val);

#ifdef DISPLAY_CTRL_DSTAGE
			reg_val = SCD__READ(sdev, DOEPDMA(SCD_G_EP0));
			DBG(3,"DOEPDMA0:%x \n",reg_val);
#endif
			/* ž */
			gep->dwLen = maxp;
		}
	}
	/* SubStateѹ */
	rslt = scd_set_ctrl_substate(sgadget, SCD_EP_OUT_READY);
	if(rslt){
		/* 顼 */
		DBG(2, "==>\n");
		return rslt;
	}
	/* EP */
	reg_val = SCD__READ(sdev, DOEPCTL0);
	reg_val |= SCD_B_DOEPCTL0_EPENA;
	SCD__WRITE(sdev, DOEPCTL0, reg_val);
		

	/* NAK */
	reg_val = SCD__READ(sdev, DOEPCTL0);
	reg_val |= SCD_B_DOEPCTL0_CNAK;
	SCD__WRITE(sdev, DOEPCTL0, reg_val);
		
	/* wait for  the XferCompl interrupt. */

	/* OK */
	DBG(2, "==>\n");
	return 0;
}

/*-------------------------------------------------------
	name	: scd_g_ep0_data_in
	argment	: struct scd_gadget *sgadget
			  gfp_t gfp_flags
	return	: int
	description:
			  transfer request for a data in stage
	date	: 2010/03/31
-------------------------------------------------------*/
int scd_g_ep0_data_in(struct scd_gadget *sgadget, gfp_t gfp_flags)
{
	int rslt=0;
	u32 reg_val;
	struct scd_device *sdev = sgadget->g_device;
	struct scd_gep *gep = &(sgadget->gep[SCD_G_EP0]);
	struct usb_request *pRequest;
	unsigned req_len;
	u32	set_len;
	u16	maxp;
	u8	pktnum;
	unsigned long rwflags;

	DBG(2, "<==\n");

	/* SubStateѹ */
	rslt = scd_set_ctrl_substate(sgadget, SCD_EP_IN_READY);
	if(rslt){
		/* 顼 */
		DBG(2, "==>\n");
		return rslt;
	}

#ifdef RW_LOCK_ON
	SCD_G_READ_LOCK_IRQ(&gep->rwLock, rwflags);
#endif
	/* ߤΥꥯȼ */
	pRequest = scd_get_currentreq(gep);
#ifdef RW_LOCK_ON
	SCD_G_READ_UNLOCK_IRQ(&gep->rwLock, rwflags);
#endif
	DBG(3,"pRequest:%p \n", pRequest);
	if(!pRequest){
		ERR("pRequest NULL \n");
		DBG(2, "==>\n");
		return -ENOENT;
	}
	/*
	 * setting parameters
	 */
	req_len		= pRequest->length;
	maxp		= gep->wMaxp;	/* usb_ep.maxpacket ˤ⤢ */
	pRequest->actual	=	0;	/* Initialize */

#ifdef DISPLAY_CTRL_DSTAGE
	DBG(1,"length:0x%x maxp:0x%x actual:0x%x \n",
		pRequest->length,gep->wMaxp,pRequest->actual);
#endif
	if(req_len < maxp){
		/*  */
		set_len	= req_len&SCD_M_DIEPTSIZ0_XFERSIZE;
		
		/* SubStateѹ */
		rslt = scd_set_ctrl_substate(sgadget, SCD_EP_DATA_END);
		if(rslt){
			/* 顼 */
			DBG(2, "==>\n");
			return rslt;
		}
	}else{
		/* (maximum size up to maxp) */
		set_len	= maxp&SCD_M_DIEPTSIZ0_XFERSIZE;
		/* SubStateѹ */
		rslt = scd_set_ctrl_substate(sgadget, SCD_EP_DATA_CONT);
		if(rslt){
			/* 顼 */
			DBG(2, "==>\n");
			return rslt;
		}
	}
	/* ž */
	gep->dwLen = set_len;
	/* ѥåȥ */
	pktnum	= SCD_MAX_EP0_PACKET&SCD_M_DIEPTSIZ0_PKTCNT;	/* 1 */

#ifdef DISPLAY_CTRL_DSTAGE
	DBG(1,"set_len:%x pktnum:%x \n",gep->dwLen, pktnum);
#endif

	/* ž/ѥåȥ */
	reg_val = SCD__READ(sdev, DIEPTSIZ0);
	reg_val = (pktnum<<SCD_S_DIEPTSIZ0_PKTCNT)|set_len;
	SCD__WRITE(sdev, DIEPTSIZ0, reg_val);

	if(sgadget->bTransMode == SCD_GADGET_TRANS_DMA){
		/* DMAХåեݥ󥿽 */
		if(!pRequest->dma){
			/* DMAǤʤΤѴ롣
			 * ǤʤС顼ˤ롣
			 */
			/* Хɥ쥹 */
			pRequest->dma = dma_map_single(sdev->dev, pRequest->buf, 
				pRequest->length, DMA_TO_DEVICE);
			DBG(3,"dmabuf mapping buf:%p dmabuf:%p\n",(void *)pRequest->buf,(void *)pRequest->dma);
			/* °ե饰å */
			((struct scd_grequest *)pRequest)->bAttribute |= SCD_REQ_TX_MAP;
			if(!pRequest->dma){
				ERR("error pRequest->dma = NULL\n");
				DBG(2, "==>\n");
				return -EINVAL;
			}
		}else{
			/* åιθ */
			dma_sync_single_for_device(sdev->dev, pRequest->dma, 
				pRequest->length, DMA_TO_DEVICE);	/* write */
			DBG(3,"dmabuf sync buf:%p dmabuf:%p\n",(void *)pRequest->buf,(void *)pRequest->dma);
			/* ° */
			((struct scd_grequest *)pRequest)->bAttribute |= SCD_REQ_TX_SYNC;
		}
		reg_val = (u32)pRequest->dma;
		SCD__WRITE(sdev, DIEPDMA(SCD_G_EP0), reg_val);
	}
	/* EP */
	reg_val = SCD__READ(sdev, DIEPCTL0);
	reg_val |= SCD_B_DIEPCTL0_EPENA;
	/* NAK */
	reg_val |= SCD_B_DIEPCTL0_CNAK;
	SCD__WRITE(sdev, DIEPCTL0, reg_val);
		
	/* wait for  the XferCompl interrupt if in case of dma . */
	/* OK */
	DBG(2, "==>\n");
	return 0;
}

/*-------------------------------------------------------
	name	: scd_g_ep0_status_in
	argment	: struct scd_gadget *sgadget
			  gfp_t gfp_flags
	return	: int
	description:
			  transfer request for a status in stage
	date	: 2010/03/31
-------------------------------------------------------*/
int scd_g_ep0_status_in(struct scd_gadget *sgadget, gfp_t gfp_flags)
{
	int rslt=0;
	u32 reg_val;
	struct scd_device *sdev = sgadget->g_device;
	u8	substate;
	
	DBG(2, "<==\n");
	
	/* ־ּ */
	substate = scd_get_substate(sgadget, SCD_G_EP0);

	/* check whether this is a 2stage transfer */
	if(substate == SCD_EP_QUE_WAIT){

		/* ž/ѥåȥ */
		reg_val = SCD_G_PKT1<<SCD_S_DIEPTSIZ0_PKTCNT|SCD_G_SZ0;
		SCD__WRITE(sdev, DIEPTSIZ0, reg_val);

		/* setup */
		scd_g_reset_setupcnt(sgadget, SCD_G_SUPCNT_MODE0);

		/* EPEnaꤹINDMAEpEna
		   롣θNAKѤReadModifyWriteη֤ǺƥåȤ
		   ޤΤNAK椹롣*/
		/* NAK */
		reg_val = SCD__READ(sdev, DIEPCTL0);
		reg_val |= SCD_B_DIEPCTL0_CNAK;
		SCD__WRITE(sdev, DIEPCTL0, reg_val);

		/* EP for status */
		reg_val = SCD__READ(sdev, DIEPCTL0);
		reg_val |= SCD_B_DIEPCTL0_EPENA;
		SCD__WRITE(sdev, DIEPCTL0, reg_val);
		
		/* wait for  the XferCompl interrupt if in case of dma . */

		/*  */
		rslt = 0;
	}else{
		DBG(3,"the substate not applicable %x \n",substate);
		rslt = -EBUSY;	/* 顼 */
	}
	DBG(2, "==>\n");
	return rslt;
}

/*-------------------------------------------------------
	name	: scd_g_general_trns
	argment	: struct scd_gadget *sgadget
			  u8 epnum
			  gfp_t gfp_flags
	return	: int
	description:
			  transfer request for a bulk or interrupt ep
	date	: 2010/10/25
-------------------------------------------------------*/
int scd_g_general_trns(struct scd_gadget *sgadget, u8 epnum, gfp_t gfp_flags)
{
	int rslt=0;
	struct scd_gep *gep = &(sgadget->gep[epnum]);
	struct usb_request *pRequest;
	unsigned long rwflags;

	DBG(3, "<== epnum:%d \n", epnum);

#ifdef RW_LOCK_ON
	SCD_G_READ_LOCK_IRQ(&gep->rwLock, rwflags);
#endif
	/* ߤΥꥯȼ */
	pRequest = scd_get_currentreq(gep);
#ifdef RW_LOCK_ON
	SCD_G_READ_UNLOCK_IRQ(&gep->rwLock, rwflags);
#endif
	DBG(3,"pReq:%p \n",pRequest);
	if(!pRequest){
		ERR("pRequest NULL \n");
		DBG(3, "==> Request null \n");
		return -ENOENT;
	}
	
	if(sgadget->gep[epnum].bIsTx){
	/*- TX  -----------------------------------------------------*/
#ifdef DISPLAY_GTRANS_DIR
		DBG(1,"TX process\n");
#endif
		rslt = scd_g_general_trns_tx(sgadget, pRequest, epnum);
		if(rslt){
			return rslt;
		}
	}else{
	/*- RX  -----------------------------------------------------*/
#ifdef DISPLAY_GTRANS_DIR
		DBG(1,"RX process\n");
#endif
		rslt = scd_g_general_trns_rx(sgadget, pRequest, epnum);
		if(rslt){
			return rslt;
		}
	}
	DBG(3, "==>\n");
	return rslt;
}
/*-------------------------------------------------------
	name	: scd_g_general_trns_tx
	argment	: struct scd_gadget *sgadget
			  struct usb_request *pReq
			  u8 epnum
	return	: int
	description:
			  tx transfer for a bulk or interrupt ep request
	date	: 2010/10/26
-------------------------------------------------------*/
int scd_g_general_trns_tx(struct scd_gadget *sgadget, 
							struct usb_request *pReq, u8 epnum)
{
	int rslt=0;
	u32 reg_val;
	struct scd_device *sdev = sgadget->g_device;
	struct scd_gep *gep = &(sgadget->gep[epnum]);
	u32 req_len, set_len;
	unsigned long flags;
	u16	maxp;
	u16	pktnum;

	/*
	 * setting parameters
	 */
	req_len		= pReq->length;
	maxp		= gep->wMaxp;	/* usb_ep.maxpacket ˤ⤢ */
	pReq->actual	=	0;	/* Initialize */

	DBG(1,"req_len:0x%x maxp:0x%x actual:0x%x \n",req_len, maxp, pReq->actual);

	/* ž׵ĹĶȽ */
	rslt = scd_g_chk_translimit(sgadget, pReq, epnum);
	if(rslt == false){
		DBG(1,"old pass \n");
		/* ѥåȥ */
		if ((req_len%maxp != 0) || (req_len == 0)) {
			pktnum = (req_len -req_len%maxp)/maxp + 1;
		}else{
			pktnum = req_len/maxp;
		}
		if((pktnum&SCD_M_DXEPTSIZN_PKTCNT)==0){
			/* 顼 ꤨʤ */
			ERR("pktnum over %x \n",pktnum);
			DBG(3, "==>\n");
			return -EINVAL;
		}
		set_len = req_len;
	}else{
		/* ѥåʬ */
		pktnum = SCD_MAX_PKT_CNT;
		/* ž */
		set_len = pktnum * maxp;
	}
		
	if(sgadget->bTransMode == SCD_GADGET_TRANS_CPU){
		/* CPU */
		if(req_len<maxp){
			/* SubStateѹ */
			rslt = scd_set_epstate(sgadget, epnum,  SCD_EP_DATA_END);
			if(rslt){
				/* 顼 */
				DBG(2, "==> error epstate  \n");
				return rslt;
			}
			/*  */
			set_len	= req_len&SCD_M_DXEPTSIZN_XFERSIZE;
		}else{
			/* SubStateѹ */
			rslt = scd_set_epstate(sgadget, epnum, SCD_EP_DATA_CONT);
			if(rslt){
				/* 顼 */
				DBG(2, "==> error epstate\n");
				return rslt;
			}
			/* (maximum size up to maxp) */
			set_len	= maxp;
		}
	}

	/* ž */
	gep->dwLen = set_len;

	DBG(2,"after calc req_len:0x%x set_len:0x%x pktnum:0x%x \n",req_len, gep->dwLen, pktnum);

	SCD_G_SPIN_LOCK_IRQ(&gep->Lock, flags);

	/* ž/ѥåȥ */
	reg_val = ((pktnum&SCD_M_DXEPTSIZN_PKTCNT)<<SCD_S_DXEPTSIZN_PKTCNT)
					|set_len;
	SCD__WRITE(sdev, DIEPTSIZ(epnum), reg_val);

	SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);

	if(sgadget->bTransMode == SCD_GADGET_TRANS_DMA){
		/* DMAХåեݥ󥿽 */
		if(!pReq->dma){
			/* DMAǤʤΤѴ롣
			 * ǤʤС顼ˤ롣
			 */
			if(pReq->buf){
				/* Хɥ쥹 */
				pReq->dma = dma_map_single(sdev->dev, pReq->buf, 
					pReq->length, DMA_TO_DEVICE);
				/* ° */
				((struct scd_grequest *)pReq)->bAttribute |= SCD_REQ_TX_MAP;
			}else{
				ERR("error pReq->dma & buf = NULL\n");
				DBG(3, "==>\n");
				return -EINVAL;
			}
		}else{
			/* åιθ */
			dma_sync_single_for_device(sdev->dev, pReq->dma, 
				pReq->length, DMA_TO_DEVICE);	/* write */

			/* ° */
			((struct scd_grequest *)pReq)->bAttribute |= SCD_REQ_TX_SYNC;
		}
		DBG(3,"req_buf:0x%p req_dma:0x%p \n",pReq->buf, (void *)pReq->dma);

		SCD_G_SPIN_LOCK_IRQ(&gep->Lock, flags);

		reg_val = (u32)pReq->dma;
		SCD__WRITE(sdev, DIEPDMA(epnum), reg_val);

		SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);

	}
	/* EP-Stateѹ */
	rslt = scd_set_epstate(sgadget, epnum, SCD_EP_IN_READY);
	if(rslt){
		/* 顼 */
		/* unmap */
		dma_unmap_req(sdev, pReq);

		/* EPstateѹ */
		rslt = scd_set_epstate(sgadget, epnum, SCD_EP_IDLE);
		if(rslt){
			ERR("error epstate cannot change to IDLE \n");
			DBG(1, "==>\n");
			return -EPERM;
		}
		ERR("error epstate cannot change to IN_READY \n");
		ERR("==> error epstate \n");
		return rslt;
	}
	SCD_G_SPIN_LOCK_IRQ(&gep->Lock, flags);

	/* DIEPCTLå */
	reg_val = SCD__READ(sdev, DIEPCTL(epnum));
	/* EP */
	reg_val |= SCD_B_DXEPCTLN_EPENA;
	/* NAK */
	reg_val |= SCD_B_DXEPCTLN_CNAK;
	SCD__WRITE(sdev, DIEPCTL(epnum), reg_val);

	SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);

	/* wait for  the XferCompl interrupt if in case of dma . */
	return 0;
}
/*-------------------------------------------------------
	name	: scd_g_general_trns_rx
	argment	: struct scd_gadget *sgadget
			  struct usb_request *pReq
			  u8 epnum
	return	: int
	description:
			  rx transfer for a bulk or interrupt ep request
	date	: 2010/10/26
-------------------------------------------------------*/
int scd_g_general_trns_rx(struct scd_gadget *sgadget, 
							struct usb_request *pReq, u8 epnum)
{
	int rslt=0;
	u32 reg_val;
	struct scd_device *sdev = sgadget->g_device;
	struct scd_gep *gep = &(sgadget->gep[epnum]);
	u32 req_len, set_len;
	unsigned long flags;
	u16	maxp;
	u16	pktnum;

	/*
	* setting parameters
	*/

	req_len		= pReq->length;
	maxp		= gep->wMaxp;	/* usb_ep.maxpacket ˤ⤢ */
	pReq->actual	=	0;	/* Initialize */

	DBG(1,"req_len:0x%x maxp:0x%x actual:0x%x \n",req_len, maxp, pReq->actual);
	/* ž׵ĹĶȽ */
	rslt = scd_g_chk_translimit(sgadget, pReq, epnum);
	if(rslt == false){
		/* ѥ */
		DBG(1,"old pass \n");
		/* ѥåȥ */
		if ((req_len%maxp != 0) || (req_len == 0)) {
			if(req_len<maxp){
				/* 饹ȥХåեΰ */
				rslt = scd_g_get_lastbuf(sgadget, pReq, epnum);
				if(rslt){
					ERR("==> error can't ready a lastbuf \n");
					return -EPERM;
				}	
				pktnum = SCD_LAST_PACKET;	/* 1 */
			}else{
				pktnum = (req_len -req_len%maxp)/maxp;
			}
		}else{
			pktnum = req_len/maxp;
		}
		if((pktnum&SCD_M_DXEPTSIZN_PKTCNT)==0){
			/* 顼 ꤨʤ */
			ERR("pktnum over %x \n",pktnum);
			DBG(3, "==>\n");
			return -EINVAL;
		}
		/* ž */
		if(maxp%4!=0){
			req_len = pktnum * (maxp +4 -(maxp % 4));
			/* 󥿥ץ */
			((struct scd_grequest *)pReq)->bAttribute |= SCD_REQ_4ALIGIN;
		}else{
			req_len = pktnum * maxp;
		}
		if(sgadget->bTransMode == SCD_GADGET_TRANS_CPU){
			/* CPU */
			if(pReq->length<maxp){
				/*  */
				set_len	= req_len;
			}else{
				/* (maximum size up to maxp) */
				set_len	= maxp;
			}
			/* ž */
			gep->dwLen = set_len;
		}else{
			/* DMA */
			if(pReq->length<maxp){
				/* ž */
				gep->dwLen = pReq->length;
			}else{
				/* ž */
				gep->dwLen = req_len;
			}
			set_len = req_len;
		}
	}else{
		/* ѥåʬ */
		pktnum = SCD_MAX_PKT_CNT;
		/* ž */
		if(maxp%4!=0){
			req_len = pktnum * (maxp +4 -(maxp % 4));
			/* 󥿥ץ */
			((struct scd_grequest *)pReq)->bAttribute |= SCD_REQ_4ALIGIN;
		}else{
			req_len = pktnum * maxp;
		}			
		/* žĹ */
		gep->dwLen = req_len;
		/* ž */
		set_len = req_len;
	}

	DBG(1,"after calc req_len:0x%x set_len:0x%x pktnum:0x%x \n",req_len, gep->dwLen, pktnum);

	SCD_G_SPIN_LOCK_IRQ(&gep->Lock, flags);

	/* ž/ѥåȥ */
	reg_val = ((pktnum&SCD_M_DXEPTSIZN_PKTCNT)<<SCD_S_DXEPTSIZN_PKTCNT)|set_len;
	SCD__WRITE(sdev, DOEPTSIZ(epnum), reg_val);

	SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);

	/* üžɤǧ롣 */
	rslt = scd_g_chk_lastbuf(pReq);
	if(rslt == true){
		SCD_G_SPIN_LOCK_IRQ(&gep->Lock, flags);

		/* DMAХåե(lastbuf) */
		reg_val = (u32)gep->lbuf.dma;
		SCD__WRITE(sdev, DOEPDMA(epnum), reg_val);

		SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);

	}else{
		if(sgadget->bTransMode == SCD_GADGET_TRANS_DMA){
			if(!pReq->dma){
				/* DMAǤʤΤѴ롣
				 * ǤʤС顼ˤ롣
				 */
				if(pReq->buf){
					/* Хɥ쥹 */
					pReq->dma = dma_map_single(sdev->dev, pReq->buf, 							
						pReq->length, DMA_FROM_DEVICE);
					/* ° */
					((struct scd_grequest *)pReq)->bAttribute |= SCD_REQ_RX_MAP;
				}else{
					ERR("error pReq->dma = NULL\n");
					return -EINVAL;
				}
			}else{
				/* åιθ */
				dma_sync_single_for_device(sdev->dev, pReq->dma, 
					pReq->length, DMA_FROM_DEVICE);	/* read */

				/* ° */
				((struct scd_grequest *)pReq)->bAttribute 
					|= SCD_REQ_RX_SYNC;
			}
			DBG(2,"req_buf:0x%p req_dma:0x%p \n",pReq->buf, 
				(void *)pReq->dma);

			SCD_G_SPIN_LOCK_IRQ(&gep->Lock, flags);
				
			reg_val = (u32)pReq->dma;
			SCD__WRITE(sdev, DOEPDMA(epnum), reg_val);
			SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
		}
	}
	/* EP-Stateѹ */
	rslt = scd_set_epstate(sgadget, epnum, SCD_EP_OUT_READY);
	if(rslt){
		/* 顼 */
		rslt = scd_g_chk_lastbuf(pReq);
		if(rslt == true){
			scd_g_free_lastbuf(sgadget, pReq, epnum);
		}
		/* unmap */
		dma_unmap_req(sdev, pReq);

		/* EPstateѹ */
		rslt = scd_set_epstate(sgadget, epnum, SCD_EP_IDLE);
		if(rslt){
			ERR("error epstate cannot change to IDLE \n");
			DBG(1, "==>\n");
			return -EPERM;
		}
		ERR("==> error epstate\n");
		return rslt;
	}
	SCD_G_SPIN_LOCK_IRQ(&gep->Lock, flags);

	/* DOEPCTL */
	reg_val = SCD__READ(sdev, DOEPCTL(epnum));
	/* EP */
	reg_val |= SCD_B_DXEPCTLN_EPENA;
	/* NAK */
	reg_val |= SCD_B_DXEPCTLN_CNAK;
	SCD__WRITE(sdev, DOEPCTL(epnum), reg_val);

	SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);

	/* wait for  the XferCompl interrupt if in case of dma . */
	return 0;
}

/*-------------------------------------------------------
	name	: scd_g_ep0int_intknfemp_tx
	argment	: struct scd_gadget *sgadget
	return	: int
	description:
			  InTknFemp interrupt for Tx in ep0
	date	: 2010/04/05
-------------------------------------------------------*/
int scd_g_ep0int_intknfemp_tx(struct scd_gadget *sgadget)
{
	INFO("non-support \n");
	return 0;
}
/*-------------------------------------------------------
	name	: scd_g_ep0int_xfercompl_rx
	argment	: struct scd_gadget *sgadget
	return	: int
	description:
			  XferCompl interrupt for Rx
	date	: 2010/08/24
-------------------------------------------------------*/
int scd_g_ep0int_xfercompl_rx(struct scd_gadget *sgadget)
{
	struct scd_device *sdev = sgadget->g_device;
	struct scd_gep *gep = &(sgadget->gep[SCD_G_EP0]);
	struct usb_request *pRequest;
	u8 mode = sgadget->bTransMode;
	int rslt;
	u32 reg_val;
	u8	stage;
	u16	maxp;
	unsigned rest_len;
	u32 set_len;
	u8	pktnum;
	unsigned long rwflags;

	DBG(1, "<==\n");

	/* EP0stage */
	stage = scd_get_ep0stage(sgadget);

	switch(stage){
	case SCD_EP0_STAGE_DATA_OUT:
	case SCD_EP0_STAGE_STATUS_OUT:
		/* SubStateѹ */
		rslt = scd_set_ctrl_substate(sgadget, SCD_EP_DATA_COMP);
		if(rslt){
			DBG(2, "==>\n");
			return -EPERM;
		}
		break;
	default:
		ERR("==> not acceptable state:%x \n",stage);
		return -EPERM;
	}
#ifdef RW_LOCK_ON
	SCD_G_READ_LOCK_IRQ(&gep->rwLock, rwflags);
#endif
	/* ߤΥꥯȼ */
	pRequest = scd_get_currentreq(gep);
#ifdef RW_LOCK_ON
	SCD_G_READ_UNLOCK_IRQ(&gep->rwLock, rwflags);
#endif
	if(!pRequest){
		ERR("pRequest NULL \n");
		DBG(2, "==>\n");
		return -ENOENT;
	}
	switch(stage){
	case SCD_EP0_STAGE_DATA_OUT:
		/* check Transfer mode  */
		switch(mode){
		case SCD_GADGET_TRANS_DMA:

			/* setupϺ */
			/*
			 * Ƚ
			 */
			 rslt = chk_ep0out_trnsz(sgadget, pRequest);
			/* Ƚ */
			switch(rslt){
			case SCD_G_TRNS_ZERO:
				DBG(1,"TRANS_ZERO \n");
			case SCD_G_TRNS_CONT:
				/*
				 *	transfer the rest of an out data again.
				 */
				DBG(1,"TRANS_CONT \n");
				
				/*
				 * setting parameters
	 			*/
				rest_len	= pRequest->length - pRequest->actual;
				maxp		= gep->wMaxp;	/* usb_ep.maxpacket ˤ⤢ */

				DBG(1,"length:0x%x maxp:0x%x actual:0x%x \n",
					pRequest->length, maxp, pRequest->actual);

				/* žѥ᡼ */
				set_len	= maxp&SCD_M_DOEPTSIZ0_XFERSIZE;
				
				/* ѥåȥ */
				pktnum	= SCD_MAX_EP0_PACKET&SCD_M_DOEPTSIZ0_PKTCNT;	/* 1 */

				DBG(1,"pktnum:0x%x set_len:0x%x\n",pktnum, set_len);

				/* ž/ѥåȥ */
				reg_val = SCD__READ(sdev, DOEPTSIZ0);
				reg_val = (pktnum<<SCD_S_DOEPTSIZ0_PKTCNT)|set_len;
				DBG(3, "DOEPTSIZ0:%x\n",reg_val);
				SCD__WRITE(sdev, DOEPTSIZ0, reg_val);
				
				if(rest_len < maxp){
					/* 饹ȥХåեΰ */
					rslt = scd_g_get_lastbuf(sgadget, pRequest, SCD_G_EP0);
					if(rslt){
						ERR("==> error can't ready a lastbuf \n");
						goto err1;
					}
					/* ž */
					gep->dwLen = rest_len;
					/* dwLenѤlbuf.sizeѤƤ褤*/
				}else{
					/* ž */
					gep->dwLen = maxp;
				}
				/* üžɤǧ롣 */
				rslt = scd_g_chk_lastbuf(pRequest);
				if(rslt == true){
					/* DMAХåե(lastbuf) */
					reg_val = (u32)gep->lbuf.dma;
					SCD__WRITE(sdev, DOEPDMA(SCD_G_EP0), reg_val);
				}
				/* SubStateѹ */
				rslt = scd_set_ctrl_substate(sgadget, SCD_EP_DATA_CONT);
				if(rslt){
					/* 顼 */
					ERR("==> error data_cont\n");
					goto err2;
				}
				
				/* EP */
				reg_val = SCD__READ(sdev, DOEPCTL0);
				reg_val |= SCD_B_DOEPCTL0_EPENA;
				SCD__WRITE(sdev, DOEPCTL0, reg_val);

				/* NAK */
				reg_val = SCD__READ(sdev, DOEPCTL0);
				reg_val |= SCD_B_DOEPCTL0_CNAK;
				SCD__WRITE(sdev, DOEPCTL0, reg_val);
				
				/* wait for the XferCompl interrupt. */
				
				break;
			case SCD_G_TRNS_END:
			case SCD_G_TRNS_SHORT:
			case SCD_G_TRNS_OVER:
				/*
				 *	transition to a status stage
				 */
				DBG(1,"TRANS_END/SHORT/OVER \n");

				/*  */
				rslt = scd_chg_ep0stage(sgadget, SCD_EP0_STAGE_STATUS_IN);
				if(rslt){
					ERR("==> error status_in\n");
					return -EPERM;
				}
				/* SubStateѹ */
				rslt = scd_set_ctrl_substate(sgadget, SCD_EP_IDLE);
				if(rslt){
					ERR("==> error \n");
					return -EPERM;
				}
				/* setup */
				scd_g_reset_setupcnt(sgadget, SCD_G_SUPCNT_MODE1);
			
				/* ž/ѥåȥ */
				reg_val = SCD_G_PKT1<<SCD_S_DIEPTSIZ0_PKTCNT|SCD_G_SZ0;
				SCD__WRITE(sdev, DIEPTSIZ0, reg_val);
			
				reg_val = SCD__READ(sdev, DIEPCTL0);
				/* EP */
				reg_val |= SCD_B_DIEPCTL0_EPENA;
				/* NAK */
				reg_val |= SCD_B_DIEPCTL0_CNAK;
				SCD__WRITE(sdev, DIEPCTL0, reg_val);
				break;
			}
			break;
		default:
			ERR("non-support transfer mode %x \n",mode);
			ERR("==> error\n");
			return -EPERM;
		}
		break;

	case SCD_EP0_STAGE_STATUS_OUT:

		/* unmapscd_g_complete_ep0_requestˤƹԤ */
	
		/* 
		 * ̤إǡ 
		 */
		scd_g_complete_ep0_request(sgadget, SCD_G_REQ_OK);

		/*  */
		rslt = scd_chg_ep0stage(sgadget, SCD_EP0_STAGE_SETUP);
		if(rslt){
			/* 顼Ϥꤨʤ */
			ERR("==> error ep0stage not changed to SETUP \n");
			return -EPERM;
		}
		/* SubStateѹ */
		rslt = scd_set_ctrl_substate(sgadget, SCD_EP_IDLE);
		if(rslt){
			/* 顼Ϥꤨʤ */
			ERR("==> abnormal error  not chaneged to IDLE \n");
			return -EPERM;
		}

		/* SUPCNTϥåȤʤ */
		/* EPEna(SETUP)*/
		reg_val = SCD__READ(sdev, DOEPCTL0);
		reg_val |= SCD_B_DOEPCTL0_EPENA;
		SCD__WRITE(sdev, DOEPCTL0, reg_val);
		break;
	}
	DBG(1, "==>\n");
	return 0;
err2:
	/* üžɤǧ롣 */
	rslt = scd_g_chk_lastbuf(pRequest);
	if(rslt == true){
		scd_g_free_lastbuf(sgadget, pRequest, SCD_G_EP0);
	}
err1:
	/* unmap */
	dma_unmap_req(sdev, pRequest);
	
	/* ơ 顼 */
	pRequest->status = -EPERM;

	DBG(1, "==> error \n");	
	return -EPERM;
}
/*-------------------------------------------------------
	name	: scd_g_ep0int_xfercompl_tx
	argment	: struct scd_gadget *sgadget
	return	: int
	description:
			  XferCompl interrupt for TX
	date	: 2010/08/24
-------------------------------------------------------*/
int scd_g_ep0int_xfercompl_tx(struct scd_gadget *sgadget)
{
	struct scd_device *sdev = sgadget->g_device;
	struct usb_request *pRequest;
	u8 mode = sgadget->bTransMode;
	struct scd_gep *gep = &(sgadget->gep[SCD_G_EP0]);
	int rslt = 0;
	u32 reg_val;
	u8	stage;
	u32 rest_len, set_len = 0;
	u16  maxp;
	u8	pktnum;
	unsigned long rwflags;
	
	DBG(1,"<==\n");

	/*
	 *	ꥯ(SetAddress)
	 */
	if(sgadget->bSetAddr == true){
		/* SetAddressΤü찷 */

		/* EP0stage */
		stage = scd_get_ep0stage(sgadget);

		switch(stage){
		case SCD_EP0_STAGE_STATUS_IN:
			/* status stage */
			rslt = scd_g_setaddr_compl(sgadget);
			break;
		default:
			ERR("set address interrupted \n");
			rslt = -EPERM;
		}
		/* ե饰ꥢ */
		sgadget->bSetAddr = false;
		DBG(2,"==>\n");
		return rslt;
	}else if(sgadget->bTestMode == true){
		
		/* ƥȥ⡼ɼ¹ */
		/* bTestModeե饰ϴؿǥꥢ */
		scd_exec_test_mode(sgadget);

		/* setup */
#ifdef CHK_SUPCNT
		chk_setupcnt(sgadget);
#endif
		/*  */
		rslt = scd_chg_ep0stage(sgadget, SCD_EP0_STAGE_SETUP);
		if(rslt){
			/* 顼Ϥꤨʤ */
			ERR("==> error ep0stage not changed to SETUP \n");
			return -EPERM;
		}
		/* SubStateѹ */
		rslt = scd_set_ctrl_substate(sgadget, SCD_EP_IDLE);
		if(rslt){
			/* 顼Ϥꤨʤ */
			ERR("==> abnormal error  not chaneged to IDLE \n");
			return -EPERM;
		}
		/* setup */
		scd_g_reset_setupcnt(sgadget, SCD_G_SUPCNT_MODE1);
	}
	/*
	 *	̽ꥯ(SetAddressʳ)
	 */

#ifdef RW_LOCK_ON
		SCD_G_READ_LOCK_IRQ(&gep->rwLock, rwflags);
#endif	
		/* ߤΥꥯȼ */
		pRequest = scd_get_currentreq(gep);
#ifdef RW_LOCK_ON
		SCD_G_READ_UNLOCK_IRQ(&gep->rwLock, rwflags);
#endif	
	if(!pRequest){
		ERR("pRequest NULL \n");
		DBG(2, "==>\n");
		return -ENOENT;
	}
	/* EP0stage */
	stage = scd_get_ep0stage(sgadget);

	switch(stage){
	case SCD_EP0_STAGE_DATA_IN:
		/* SubStateѹ */
		rslt = scd_set_ctrl_substate(sgadget, SCD_EP_DATA_COMP);
		if(rslt){
			ERR("==> error data_comp\n");
			return -EPERM;
		}
		rslt = chk_ep0in_trnsz(sgadget, pRequest);
		DBG(2,"rslt :%x\n",rslt);
		switch(rslt){
		/*
		 * ǡž³
		 */
		case SCD_G_TRNS_CONT:
#ifdef DISPLAY_CTRL_DSTAGE
			DBG(2,"TRANS_CONT enter \n");
#endif
			rest_len	= pRequest->length - pRequest->actual;
			maxp		= gep->wMaxp;	/* usb_ep.maxpacket ˤ⤢ */
			if(rest_len < maxp){
				/*  */
				set_len	= rest_len&SCD_M_DIEPTSIZ0_XFERSIZE;
				/* SubStateѹ */
				rslt = scd_set_ctrl_substate(sgadget, SCD_EP_DATA_END);
				if(rslt){
					ERR("==> error data_end\n");
					return -EPERM;
				}
				DBG(1,"DATA_END \n");
			}else{
				/* (maximum size up to maxp) */
				set_len	= maxp&SCD_M_DIEPTSIZ0_XFERSIZE;
				/* SubStateѹ */
				rslt = scd_set_ctrl_substate(sgadget, SCD_EP_DATA_CONT);
				if(rslt){
					ERR("==> error data_cont\n");
					return -EPERM;
				}
				DBG(1,"DATA_CONT \n");
			}
			/* ž */
			gep->dwLen = set_len;
			/* ѥåȥ */
			pktnum	= SCD_MAX_EP0_PACKET&SCD_M_DIEPTSIZ0_PKTCNT;	/* 1 */
			/* ž/ѥåȥ */
			reg_val = SCD__READ(sdev, DIEPTSIZ0);
			reg_val = (pktnum<<SCD_S_DIEPTSIZ0_PKTCNT)|set_len;

			SCD__WRITE(sdev, DIEPTSIZ0, reg_val);

#ifdef DISPLAY_CTRL_DSTAGE
			DBG(1,"DIEPTSIZ0:%x length:%x actual:%x\n",reg_val, pRequest->length, pRequest->actual);			DBG(1,"set_len:%x pkt_num:%x\n",gep->dwLen, pktnum);
#endif
			/* DMAХåեݥ󥿤Ϻǽ˥åȤޤ */

			/* EP */
			reg_val = SCD__READ(sdev, DIEPCTL0);
			reg_val |= SCD_B_DIEPCTL0_EPENA;
			/* NAK */
			reg_val |= SCD_B_DIEPCTL0_CNAK;

			SCD__WRITE(sdev, DIEPCTL0, reg_val);

			/* wait for  the XferCompl interrupt. */
			break;
			
		/*
		 * Zeroǡž
		 */
		case SCD_G_TRNS_ZERO:
#ifdef DISPLAY_CTRL_DSTAGE
			DBG(1,"TRANS_ZERO enter \n");
#endif
			/* SubStateѹ */
			rslt = scd_set_ctrl_substate(sgadget, SCD_EP_ZERO_END);
			if(rslt){
				ERR("==> error zero_end\n");
				return -EPERM;
			}
			pRequest->zero = 0;	/* Initialize */
			set_len = 0;
			/* ž */
			gep->dwLen = set_len;
			/* ѥåȥ */
			pktnum	= SCD_MAX_EP0_PACKET&SCD_M_DIEPTSIZ0_PKTCNT;	/* 1 */
			/* ž/ѥåȥ */
			reg_val = SCD__READ(sdev, DIEPTSIZ0);
			reg_val = (pktnum<<SCD_S_DIEPTSIZ0_PKTCNT)|set_len;
			SCD__WRITE(sdev, DIEPTSIZ0, reg_val);
			
			/* DMAХåեݥ󥿤Ϻǽ˥åȤޤ */
				
			/* EP */
			reg_val = SCD__READ(sdev, DIEPCTL0);
			reg_val |= SCD_B_DIEPCTL0_EPENA;
			/* NAK */
			reg_val |= SCD_B_DIEPCTL0_CNAK;
			SCD__WRITE(sdev, DIEPCTL0, reg_val);
			
			/* wait for  the XferCompl interrupt. */
			break;

		/*
		 * ǽǡž
		 */
		case SCD_G_TRNS_END:
#ifdef DISPLAY_CTRL_DSTAGE
			DBG(1,"TRANS_END enter \n");
#endif
			/*  */
			rslt = scd_chg_ep0stage(sgadget, SCD_EP0_STAGE_STATUS_OUT);
			if(rslt){
				ERR("==> error \n");
				return -EPERM;
			}
			/* SubStateѹ */
			rslt = scd_set_ctrl_substate(sgadget, SCD_EP_IDLE);
			if(rslt){
				/* 顼 */
				ERR("==> error \n");
				return -EPERM;
			}
			/* ž/ѥåȥ */
			reg_val = SCD_G_PKT1<<SCD_S_DOEPTSIZ0_PKTCNT|SCD_G_SZ0;
			SCD__WRITE(sdev, DOEPTSIZ0, reg_val);

			/* setup *//* EPenableԤ */
			scd_g_reset_setupcnt(sgadget, SCD_G_SUPCNT_MODE0);

			/* NAK */
			reg_val = SCD__READ(sdev, DOEPCTL0);
			DBG(3,"DOEPCTL0 :%x \n",reg_val);
			reg_val |= SCD_B_DOEPCTL0_CNAK;
			SCD__WRITE(sdev, DOEPCTL0, reg_val);

			/* wait for  the XferCompl interrupt. */
		}
		break;
	case SCD_EP0_STAGE_STATUS_IN:
#ifndef ENABLE_AFTER_SET_CNFG	/* non-use  */
		if(sgadget->bFifoCnfg){
			/* SetConfigFIFOconfigѹ֤νλ */
			sgadget->bFifoCnfg = false;
		}
#endif
		/* check Transfer mode  */
		switch(mode){
		case SCD_GADGET_TRANS_DMA:
			/* SubStateѹ */
			rslt = scd_set_ctrl_substate(sgadget, SCD_EP_DATA_COMP);
			if(rslt){
				ERR("==> error \n");
				return -EPERM;
			}
			/* SUPCNTϥåȤʤ */

			/* 
			 * ̤إǡ 
			 */
			scd_g_complete_ep0_request(sgadget, SCD_G_REQ_OK);

			/*  */
			rslt = scd_chg_ep0stage(sgadget, SCD_EP0_STAGE_SETUP);
			if(rslt){
				/* 顼Ϥꤨʤ */
				ERR("==> error ep0stage not changed to SETUP \n");
				return -EPERM;
			}
			/* SubStateѹ*/
			rslt = scd_set_ctrl_substate(sgadget, SCD_EP_IDLE);
			if(rslt){
				/* 顼Ϥꤨʤ */
				ERR("==> abnormal error  not chaneged to IDLE \n");
				return -EPERM;
			}
			/* SUPCNTϥåȤʤ */
			/* EPEna(SETUP)*/
			reg_val = SCD__READ(sdev, DOEPCTL0);
			reg_val |= SCD_B_DOEPCTL0_EPENA;
			SCD__WRITE(sdev, DOEPCTL0, reg_val);
			break;
			
		default:
			ERR("non-support transfer mode %x \n",mode);
			DBG(1, "==>\n");
			return -EPERM;
		}
		break;
	}
	DBG(1, "==>\n");
	return 0;
}
/*-------------------------------------------------------
	name	: scd_g_int_xfercompl_tx
	argment	: struct scd_gadget *sgadget
			  u8 epnum
	return	: int
	description:
			  XferCompl interrupt for Tx
	date	: 2010/10/06
-------------------------------------------------------*/
int scd_g_int_xfercompl_tx(struct scd_gadget *sgadget, u8 epnum)
{
	struct scd_device *sdev = sgadget->g_device;
	struct usb_request *pRequest;
	struct scd_gep *gep = &(sgadget->gep[epnum]);
	int rslt;
	u32 reg_val;
	u32 set_len = 0;
	u16	pktnum;
	unsigned long rwflags;

	DBG(2, "<==\n");

#ifdef RW_LOCK_ON
	SCD_G_READ_LOCK_IRQ(&gep->rwLock, rwflags);
#endif
	/* ߤΥꥯȼ */
	pRequest = scd_get_currentreq(gep);
#ifdef RW_LOCK_ON
	SCD_G_READ_UNLOCK_IRQ(&gep->rwLock, rwflags);
#endif
	if(!pRequest){
		/* EP֤᤹ɲɬ */
		ERR("pRequest NULL \n");
		DBG(2, "==>\n");
		return -ENOENT;
	}
	DBG(2,"req:%p buf:%p dma:%p \n",pRequest, pRequest->buf, (void *)pRequest->dma);
	/* EP state */
	rslt = scd_set_epstate(sgadget, epnum, SCD_EP_DATA_COMP);
	if(rslt){
		ERR("==>error\n");
		return -EPERM;
	}

	rslt = chk_in_trnsz(sgadget, pRequest, epnum);
	switch(rslt){
	/*
	 * ǡž³
	 */
	case SCD_G_TRNS_CONT:
		if(sgadget->bTransMode==SCD_GADGET_TRANS_DMA){
			rslt = scd_g_retry_trans(sgadget, pRequest, epnum);
			if(rslt){
				ERR("==>error\n");
				return -EPERM;
			}
		}else{
			INFO("non-support CPU mode\n");
		}		
		break;
	/*
	 * Zeroǡž
	 */
	case SCD_G_TRNS_ZERO:
#ifdef DISPLAY_CLASS
		DBG(1,"TRNS_ZERO \n");
#endif
		DBG(1,"TRNS_ZERO \n");	
		/* EPstateѹ */
		rslt = scd_set_epstate(sgadget, epnum, SCD_EP_ZERO_END);
		if(rslt){
			ERR("==>error\n");
			return -EPERM;
		}
		set_len = 0;
		/* ž */
		gep->dwLen = set_len;
		/* ѥåȥ */
		pktnum	= 1;
		/* ž/ѥåȥ */
		reg_val = (pktnum<<SCD_S_DXEPTSIZN_PKTCNT)|set_len;
		SCD__WRITE(sdev, DIEPTSIZ(epnum), reg_val);
			
		/* DMAХåեݥ󥿤Ϻǽ˥åȤޤ */
		
		/* DIEPCTL */
		reg_val = SCD__READ(sdev, DIEPCTL(epnum));
		/* EP */
		reg_val |= SCD_B_DXEPCTLN_EPENA;
		/* NAK */
		reg_val |= SCD_B_DXEPCTLN_CNAK;
		SCD__WRITE(sdev, DIEPCTL(epnum), reg_val);
			
		/* wait for  the XferCompl interrupt. */
		break;
	/*
	 * ǽǡž
	 */
	case SCD_G_TRNS_END:
#ifdef DISPLAY_CLASS
		DBG(1,"TRNS_END \n");
#endif
		/* žλԤ */

		/* unmap */
		dma_unmap_req(sdev, pRequest);

		/* EPstateѹ */
		rslt = scd_set_epstate(sgadget, epnum, SCD_EP_IDLE);
		if(rslt){
			ERR("==> error\n");
			return -EPERM;
		}
		DBG(2,"complete_request\n");
		/* ̤žλ */
		scd_gadget_complete_request(pRequest, pRequest->status);
		break;
	}
	DBG(2, "==>\n");
	return 0;	
}
/*-------------------------------------------------------
	name	: scd_g_int_xfercompl_rx
	argment	: struct scd_gadget *sgadget
			  u8 epnum
	return	: int
	description:
			  XferCompl interrupt for Rx
	date	: 2010/06/10
-------------------------------------------------------*/
int scd_g_int_xfercompl_rx(struct scd_gadget *sgadget, u8 epnum)
{
	struct scd_device *sdev = sgadget->g_device;
	struct usb_request *pRequest;
	struct scd_gep *gep = &(sgadget->gep[epnum]);
	int rslt;
	unsigned long rwflags;
	
	DBG(2, "<==\n");

#ifdef RW_LOCK_ON
	SCD_G_READ_LOCK_IRQ(&gep->rwLock, rwflags);
#endif	
	/* ߤΥꥯȼ */
	pRequest = scd_get_currentreq(gep);
#ifdef RW_LOCK_ON
	SCD_G_READ_UNLOCK_IRQ(&gep->rwLock, rwflags);
#endif	
	if(!pRequest){
		ERR("==> pRequest NULL \n");
		return -ENOENT;
	}
	/* EP state */
	rslt = scd_set_epstate(sgadget, epnum, SCD_EP_DATA_COMP);
	if(rslt){
		ERR("==> error state can't change to DATA_COMP \n");
		return  -EPERM;
	}
	/* žȽ */
	rslt = chk_out_trnsz(sgadget, pRequest, epnum);
	switch(rslt){
	/*
	 * ǡž³ʳĥžüž
	 */
	case SCD_G_TRNS_CONT:
#ifdef DISPLAY_CLASS
		DBG(1,"TRNS_CONT \n");
#endif
		/* ǡž³ */
		rslt = scd_g_xfercompl_rx_cont(sgadget, pRequest, epnum);
		if(rslt<0){
			goto err;
		}
		break;
	/*
	 * ǡž³0Ĺ
	 */
	case SCD_G_TRNS_ZERO:
#ifdef DISPLAY_CLASS
		DBG(1,"TRNS_ZERO \n");
#endif
		/* ǡžzero */
		rslt = scd_g_xfercompl_rx_zero(sgadget, pRequest, epnum);
		if(rslt<0){
			goto err;
		}
		break;
	/*
	 * ǽǡž
	 */
	case SCD_G_TRNS_SHORT:
	case SCD_G_TRNS_END:
	case SCD_G_TRNS_OVER:
#ifdef DISPLAY_CLASS
		DBG(1,"TRNS_SHORT or END \n");
#endif
		/* žλԤ */

		/* unmap */
		dma_unmap_req(sdev, pRequest);

		/* EPstateѹ */
		rslt = scd_set_epstate(sgadget, epnum, SCD_EP_IDLE);
		if(rslt){
			DBG(1, "==>\n");
			return -EPERM;
		}
		DBG(2,"complete_request\n");
		/* ̤žλ */
		scd_gadget_complete_request(pRequest, pRequest->status);
		break;
	case SCD_G_TRNS_ERR:
		goto err;
		break;
	}
	DBG(2, "==>\n");
	return 0;	
err:
	/* üžɤǧ롣 */
	rslt = scd_g_chk_lastbuf(pRequest);
	if(rslt == true){
		scd_g_free_lastbuf(sgadget, pRequest, epnum);
	}
	/* unmap */
	dma_unmap_req(sdev, pRequest);
	
	/* EPstateѹ */
	scd_set_epstate(sgadget, epnum, SCD_EP_IDLE);
	if(rslt){
		ERR("==> error state can't change to IDLE \n");
		
	}
	/* ơ 顼 */
	pRequest->status = -EPERM;

	DBG(2,"complete_request\n");
	/* ̤žλ */
	scd_gadget_complete_request(pRequest, pRequest->status);

	return -EPERM;
}
/*-------------------------------------------------------
	name	: scd_g_complete_ep0_request
	argment	: struct scd_gadget *sgadget
			  int status
	return	: none
	description:
			  to indicate a callback in request completition
	date	: 2010/08/19
-------------------------------------------------------*/
void scd_g_complete_ep0_request(struct scd_gadget *sgadget, int status)
{
	struct usb_request *pRequest;
	struct scd_device *sdev = sgadget->g_device;
	struct scd_gep *gep = &sgadget->gep[SCD_G_EP0];
	unsigned long rwflags;
	int rslt = 0;

	DBG(2,"<==\n");

#ifdef RW_LOCK_ON
	SCD_G_READ_LOCK_IRQ(&gep->rwLock, rwflags);
#endif	
	/* ߤΥꥯȼ */
	pRequest = scd_get_currentreq(gep);
#ifdef RW_LOCK_ON
	SCD_G_READ_UNLOCK_IRQ(&gep->rwLock, rwflags);
#endif	
	/* EP0 ꥯȴλ */
	if (pRequest) {
		DBG(4, "completing request pRequest=%p\n", pRequest);
		/* lastbufȽ */
		rslt = scd_g_chk_lastbuf(pRequest);
		if(rslt == true){
			/* lastbuf unmap */
			scd_g_free_lastbuf(sgadget, pRequest, SCD_G_EP0);
		}	
		/* unmap */
		dma_unmap_req(sdev, pRequest);

		/* ơǼ */
		if(pRequest->status == -EINPROGRESS){
			DBG(1,"status change:%x to %x \n",pRequest->status, status);
			pRequest->status = status;
		}

#ifdef RW_LOCK_ON
		SCD_G_WRITE_LOCK_IRQ(&gep->rwLock, rwflags);
#endif
		/* ꥹȤꥯȺ */
		list_del(&pRequest->list);
#ifdef RW_LOCK_ON
		SCD_G_WRITE_UNLOCK_IRQ(&gep->rwLock, rwflags);
#endif
		if (pRequest->complete) {
			/* λ */
			pRequest->complete(&sgadget->gep[SCD_G_EP0].end_point, pRequest);
		}
	}
	DBG(2, "==>\n");
}

/*-------------------------------------------------------
	name	: scd_g_req_skip_after
	argment	: struct scd_gadget *sgadget
	return	: none
	description:
			  to process at the status-in stage 
			   without ep-queue from the upper.
	date	: 2010/04/02
-------------------------------------------------------*/
void scd_g_req_skip_after(struct scd_gadget *sgadget)
{
	DBG(2, "<==>\n");
	
	/* ep_queueν 
	 * the gfp_t flag is a dummy 
	 */
	scd_g_ep0_status_in(sgadget, GFP_KERNEL);
	
}

/*-------------------------------------------------------
	name	: chk_ep0out_trnsz
	argment	: struct scd_gadget *sgadget, 
			  struct usb_request *pRequest
	return	: int
	description:
			  to check transfer size at out.
	date	: 2010/05/06
-------------------------------------------------------*/
static int chk_ep0out_trnsz(struct scd_gadget *sgadget,
							struct usb_request *pRequest)
{
	struct scd_device *sdev = sgadget->g_device;
	struct scd_gep *gep = &(sgadget->gep[SCD_G_EP0]);
	int	rslt = SCD_G_TRNS_END;
	u32 xsize;
	int diff;
	u16  maxp = gep->wMaxp;

	DBG(2, "<==\n");

	/* μĥ */
	xsize = SCD__READ(sdev, DOEPTSIZ0);
	xsize &= SCD_M_DOEPTSIZ0_XFERSIZE;
	
	/* 1ñ̤Ǥμ */
	
	diff = maxp -xsize;

	/* üžɤǧ롣 */
	rslt = scd_g_chk_lastbuf(pRequest);
	if(rslt == true){

		/* unmap */	
		/* θ2Ť˸ƤФ륱⤢뤬
		   1ܤǤΤ߽뤫ʤ */
		dma_unmap_req(sdev, pRequest);
	
		if(diff <= gep->dwLen){
			/*	
			 *	JUST/SHORT size packet
			 */
			if(pRequest->short_not_ok){
				/* short transfer ng */
				ERR("rx short error short_not_ok=1\n");
				pRequest->status = -EREMOTEIO;
			}else{
				/* short transfer ok */
				pRequest->status = 0;
			}
			/* ̥Хåե˥ԡ */
			rslt = scd_g_cp_buf(sgadget, pRequest, SCD_G_EP0, diff);
			if(!rslt){
				/* ž */
				pRequest->actual += diff;
			}
			if(diff < gep->dwLen){
				rslt = SCD_G_TRNS_SHORT;
			}else{
				rslt = SCD_G_TRNS_END;
			}
#ifdef DISPLAY_CTRL_DSTAGE
			DBG(1,"[%d]0:JUST/1:SHORT xsize:%x actual:%x \n",((diff == gep->dwLen)?0:1),xsize, pRequest->actual);
#endif
		}else if(diff == gep->dwLen){
			/*	
			 *	JUST size packet
			 */
			pRequest->status = 0;
			/* ̥Хåե˥ԡ */
			rslt = scd_g_cp_buf(sgadget, pRequest, SCD_G_EP0, gep->dwLen);
			if(!rslt){
				/* ž */
				pRequest->actual += gep->dwLen;
			}
			rslt = SCD_G_TRNS_END;			
#ifdef DISPLAY_CTRL_DSTAGE
			DBG(1,"END xsize:%x actual:%x \n",xsize, pRequest->actual);
#endif
		}else{
			/*	
			 *	OVERDATA size packet
			 */
			ERR("error data over flow! \n");
			pRequest->status = -EOVERFLOW;
			/* ̥Хåե˥ԡ */
			rslt = scd_g_cp_buf(sgadget, pRequest, SCD_G_EP0, gep->dwLen);
			if(!rslt){
				/* ž */
				pRequest->actual += gep->dwLen;
			}
			rslt = SCD_G_TRNS_OVER;
		}
	}else{
		/* lengthMAXPܿξ硢ɤ */
		if(diff < gep->dwLen){
			/*	
			 *	SHORT size packet
			 */
			if(pRequest->short_not_ok){
				/* short transfer ng */
				ERR("rx short error short_not_ok=1\n");
				pRequest->status = -EREMOTEIO;
			}else{
				/* short transfer ok */
				pRequest->status = 0;
			}
			rslt = SCD_G_TRNS_SHORT;
			/* ž */
			pRequest->actual += diff;
#ifdef DISPLAY_CTRL_DSTAGE
			DBG(1,"SHORT xsize:%x actual:%x \n",xsize, pRequest->actual);
#endif
		}else if(diff == gep->dwLen){
			/*	
			 *	JUST size packet
			 */
			/* ž */
			pRequest->actual += gep->dwLen;
			/* ĥȽ */
			if((pRequest->length - pRequest->actual)>0){
				rslt = SCD_G_TRNS_CONT;
			}else if((pRequest->length - pRequest->actual)==0) {
				if(pRequest->zero){
					/* for zero packet*/
					rslt = SCD_G_TRNS_ZERO;
#ifdef DISPLAY_CTRL_DSTAGE
					DBG(1,"ZERO zero:%x xsize:%x actual:%x \n",pRequest->zero, xsize, pRequest->actual);
#endif
				}else{
					/* rx transfer ok */
					pRequest->status = 0;
					rslt = SCD_G_TRNS_END;
#ifdef DISPLAY_CTRL_DSTAGE
					DBG(1,"END xsize:%x actual:%x \n",xsize, pRequest->actual);
#endif
				}
			}
		}else{
			/*	
			 *	ä顢HWǤ롣
			 */
			ERR("error no support flow \n");
			pRequest->status = -EINVAL;
			rslt = SCD_G_TRNS_ERR;
		}
	}
	/* Initialize */
	gep->dwLen = 0;

	DBG(2, "<==\n");
	return rslt;
}

/*-------------------------------------------------------
	name	: chk_ep0in_trnsz
	argment	: struct scd_gadget *sgadget, 
			  struct usb_request *pRequest
	return	: int
	description:
			  to check transfer size at in.
	date	: 2010/04/05
-------------------------------------------------------*/
static int chk_ep0in_trnsz(struct scd_gadget *sgadget,
							struct usb_request *pRequest)
{
	struct scd_gep *gep = &(sgadget->gep[SCD_G_EP0]);
	int	rslt = SCD_G_TRNS_END;

	DBG(2, "<==\n");

	/* ž */
	pRequest->actual += gep->dwLen;
	/* ĥȽ */
	if((pRequest->length - pRequest->actual)>0){
		rslt = SCD_G_TRNS_CONT;
	}else{
		if(pRequest->zero){
			rslt = SCD_G_TRNS_ZERO;
			pRequest->zero = 0;
		}else{
			/* tx transfer ok */
			pRequest->status = 0;
			rslt = SCD_G_TRNS_END;
		}
	}
	/* Initialize */
	gep->dwLen = 0;

	DBG(2, "==>\n");
	return rslt;
}
/*-------------------------------------------------------
	name	: chk_out_trnsz
	argment	: struct scd_gadget *sgadget, 
			  struct usb_request *pRequest,
			  u8 epnum
	return	: int
	description:
			  to check transfer size at out.
	date	: 2010/10/25
-------------------------------------------------------*/
static int chk_out_trnsz(struct scd_gadget *sgadget,
						 struct usb_request *pRequest,
						 u8 epnum)
{
	struct scd_device *sdev = sgadget->g_device;
	struct scd_gep *gep = &(sgadget->gep[epnum]);
	int	rslt = SCD_G_TRNS_END;
	u32 xsize;
	u32 diff;

	u16	maxp = gep->wMaxp;

	DBG(2, "<== \n");

	/* μĥ */
	xsize = SCD__READ(sdev, DOEPTSIZ(epnum));
	xsize &= SCD_M_DXEPTSIZN_XFERSIZE;

	/* üž椫ǧ롣 */
	rslt = scd_g_chk_lastbuf(pRequest);
	if(rslt == true){	
	/*---------------------------------------------------------------*/
	/* üž */

		/* 1ñ̤Ǥμ */
		diff = maxp -xsize;

		/* 顼å */
		if(gep->dwLen >= maxp){
			DBG(1, "dwLen:%x maxp:%x\n", gep->dwLen, maxp);
			ERR("==> error dwLen is over out of the range \n");
			pRequest->status = -EINVAL;
			rslt = SCD_G_TRNS_ERR;
		}
		/* unmap */	
		/* θ2Ť˸ƤФ륱⤢뤬
		   1ܤǤΤ߽뤫ʤ */
		dma_unmap_req(sdev, pRequest);

		if(diff <= gep->dwLen){
			/*
			 *	JUST/SHORT size Packet
			 */
			if(pRequest->short_not_ok){
				/* short transfer ng */
				ERR("rx short error short_not_ok=1\n");
				pRequest->status = -EREMOTEIO;
			}else{
				/* short transfer ok */
				pRequest->status = 0;
#ifdef DISPLAY_CLASS
				DBG(1, "req status set OK \n");
#endif
			}
			/* ̥Хåե˥ԡ */
			rslt = scd_g_cp_buf(sgadget, pRequest, epnum, diff);
			if(!rslt){
				/* ž */
				pRequest->actual += diff;
			}
			if(diff < gep->dwLen){
				/*  */
				rslt = SCD_G_TRNS_SHORT;
#ifdef DISPLAY_CLASS
				DBG(1, "judge SHORT actual:%x += (%x - %x)\n",pRequest->actual,gep->dwLen, xsize);			
#endif
			}else{
				/*  */
				rslt = SCD_G_TRNS_END;
#ifdef DISPLAY_CLASS
				DBG(1, "judge END \n");	
#endif
			}
		}else if(diff == gep->dwLen){
			/*
			 *	JUST size Packet
			 */
			/* rx transfer ok */
			pRequest->status = 0;
			/* ̥Хåե˥ԡ */
			rslt = scd_g_cp_buf(sgadget, pRequest, epnum, gep->dwLen);
			if(!rslt){
				/* ž */
				pRequest->actual += gep->dwLen;
			}
			/*  */
			rslt = SCD_G_TRNS_END;
#ifdef DISPLAY_CLASS
			DBG(1, "judge END \n");	
#endif
		}else{
			/*
			 *	OVER DATA Packet
			 */
			ERR("error data over flow! \n");
			/* Сե */
			pRequest->status = -EOVERFLOW;
			/* ̥Хåե˥ԡ */
			rslt = scd_g_cp_buf(sgadget, pRequest, epnum, gep->dwLen);
			if(!rslt){
				/* ž */
				pRequest->actual += gep->dwLen;
			}
			/*  */
			rslt = SCD_G_TRNS_OVER;
		}
	}else{
	/*---------------------------------------------------------------*/
	/* MAXPܿž */

		/* ׵ĹȥåĹκ */
		diff = pRequest->length - gep->dwLen - pRequest->actual;

		if(diff==0){
			if(xsize >0){
				/*
				 *	SHORT size Packet
				 */
				if(pRequest->short_not_ok){
					/* short transfer ng */
					ERR("rx short error short_not_ok=1\n");
					pRequest->status = -EREMOTEIO;
				}else{
					/* short transfer ok */
					pRequest->status = 0;
#ifdef DISPLAY_CLASS
					DBG(1, "req status set OK \n");
#endif
				}
				rslt = SCD_G_TRNS_SHORT;
				/* ž */
				pRequest->actual += (gep->dwLen - xsize);
#ifdef DISPLAY_CLASS
				DBG(1, "judge SHORT actual:%x += (%x - %x)\n",pRequest->actual,gep->dwLen, xsize);			
#endif
			}else if(xsize==0){
				/*
				 *	JUST size Packet
			 	*/
				/* ž */
				pRequest->actual += gep->dwLen;
				if(pRequest->zero){
					rslt = SCD_G_TRNS_ZERO;
#ifdef DISPLAY_CLASS
					DBG(1, "judge ZERO \n");
#endif
				}else{
					/* rx transfer ok */
					pRequest->status = 0;
					rslt = SCD_G_TRNS_END;
#ifdef DISPLAY_CLASS
					DBG(1, "judge END \n");	
#endif
				}
			}else{
				ERR("error no support flow: xsize < 0\n");
				/* СǡȤƤޤ魯
				   äHWǤ롣 */
				pRequest->status = -EINVAL;
				rslt = SCD_G_TRNS_ERR;
			}
		}else if(diff >0){
			if(xsize>0){
				/*
				 *	SHORT Packet
			 	*/
				if(pRequest->short_not_ok){
					/* short transfer ng */
					ERR("rx short error short_not_ok=1\n");
					pRequest->status = -EREMOTEIO;
				}else{
					/* short transfer ok */
					pRequest->status = 0;
#ifdef DISPLAY_CLASS
					DBG(1, "req status set OK \n");
#endif
				}
				rslt = SCD_G_TRNS_SHORT;
				/* ž */
				pRequest->actual += (gep->dwLen - xsize);
#ifdef DISPLAY_CLASS
				DBG(1, "judge SHORT actual:%x += (%x - %x)\n",pRequest->actual,gep->dwLen, xsize);
#endif
			}else if(xsize==0){
				/*
				 *	Continue to the next transfer.
			 	*/
				/* ž */
				pRequest->actual += gep->dwLen;
				rslt = SCD_G_TRNS_CONT;
#ifdef DISPLAY_CLASS
				DBG(1, "judge CONT len:%x - actual:%x \n", pRequest->length, pRequest->actual);
#endif
			}else{
				ERR("error no support flow: xsize < 0\n");
				/* äHWǤ롣 */
				pRequest->status = -EINVAL;
				rslt = SCD_G_TRNS_ERR;
			}
		}else{
			ERR("error no support flow \n");
		}
	}
	/* Initialize */
	gep->dwLen = 0;

	DBG(2, "==>\n");
	return rslt;
}
/*-------------------------------------------------------
	name	: chk_in_trnsz
	argment	: struct scd_gadget *sgadget, 
			  struct usb_request *pRequest,
			  u8 epnum
	return	: int
	description:
			  to check transfer size at in.
	date	: 2010/04/05
-------------------------------------------------------*/
static int chk_in_trnsz(struct scd_gadget *sgadget,
						struct usb_request *pRequest,
						u8 epnum)
{
	struct scd_gep *gep = &(sgadget->gep[epnum]);
	int	rslt = SCD_G_TRNS_END;

	DBG(2, "<==\n");

	/* ž */
	pRequest->actual += gep->dwLen;

	/* ĥȽ */
	if((pRequest->length - pRequest->actual)>0){
		rslt = SCD_G_TRNS_CONT;
	}else{
		if(pRequest->zero){
			rslt = SCD_G_TRNS_ZERO;
			pRequest->zero = 0;
		}else{
			/* tx transfer ok */
			pRequest->status = 0;
			rslt = SCD_G_TRNS_END;
		}
	}
	/* Initialize */
	gep->dwLen = 0;

	DBG(2, "==>\n");
	return rslt;
}
/*-------------------------------------------------------
	name	: get_dmasetup
	argment	: struct scd_gadget *sgadget, 
			  struct scd_dmaep0_buf *setup_buf,
			  u8 no
	return	: u8 *
	description:
			  one:copy a valid request to buf for the upper
			  another:notify a head of status packet 
	date	: 2010/04/12
-------------------------------------------------------*/
static u8 *get_dmasetup(struct scd_gadget *sgadget,
						struct scd_dmaep0_buf *setup_buf, 
						u8 no)
{
	unsigned long flags;
	u8 *req = ((struct scd_ep0_buf *)(sgadget->pEp0buf))->req;

	DBG(2, "<==\n");

	/* index׻ */
	switch(no){
	case SCD_GREQ_IND1:
	case SCD_GREQ_IND2:
	case SCD_GREQ_IND3:
		SCD_G_SPIN_LOCK_IRQ(&setup_buf->lock, flags);
#ifdef SCD_CACHE_INVALIDATE_AFTER_DMA
		/* cache invalidate */
		udif_cache_ctrl_r((void *)&(setup_buf->buf[no]), 
						sizeof(*req)*SCD_GREQ_SZ,
                        UDIF_CACHE_INVALIDATE);
#endif /* SCD_CACHE_INVALIDATE_AFTER_DMA */
		/* ꥯȥԡ */
		memcpy((void *)req, (void *)&(setup_buf->buf[no]), 
			sizeof(*(setup_buf->buf))*SCD_GREQ_SZ);
		/* ȤindexֹǼ */
		scd_dma_ep0buf.dcnt = no/SCD_GREQ_SZ;

		SCD_G_SPIN_UNLOCK_IRQ(&setup_buf->lock, flags);
		break;
	default:
		ERR("parameter error! setup_cnt %d\n",no);
		DBG(2, "==>\n");
		return NULL;
	}
	DBG(2, "==>\n");
	return req;
}

/*-------------------------------------------------------
	name	: get_dmasetup_outdata
	argment	: struct scd_gadget *sgadget, 
			  struct usb_request *
			  u8 *reqcmd
	return	: int
	description:
			  to get a setup data at data_out 
	date	: 2010/08/18
-------------------------------------------------------*/
static int get_dmasetup_outdata(struct scd_gadget *sgadget,
						struct usb_request *pReq, u8 *reqcmd)
{
	struct scd_device *sdev = sgadget->g_device;
	struct scd_gep *gep = &(sgadget->gep[SCD_G_EP0]);
	u8 *req = ((struct scd_ep0_buf *)(sgadget->pEp0buf))->req;
	void *setup_buf;
	unsigned long rwflags;
	int rslt = 0;

	DBG(2, "<==\n");

	/* üž椫ǧ롣 */
	rslt = scd_g_chk_lastbuf(pReq);
	if(rslt == true){	/* üξϡΥХåե */
		DBG(1, "setup from lastbuf\n");
		/* üǡХåեunmap */
		rslt = scd_g_free_lastbuf(sgadget, pReq, SCD_G_EP0);
		if(rslt){
			ERR("==> error \n");
			return -EPERM;
		}
#ifdef SCD_CACHE_INVALIDATE_AFTER_DMA
		/* cache invalidate */
		udif_cache_ctrl_r((void *)(gep->lbuf.buf), 
				sizeof(*req)*SCD_GREQ_SZ, UDIF_CACHE_INVALIDATE);
#endif /* SCD_CACHE_INVALIDATE_AFTER_DMA */
#ifdef RW_LOCK_ON
		SCD_G_WRITE_LOCK_IRQ(&gep->rwLock, rwflags);
#endif
		/* ǡԡ */
		memcpy((void *)req, (void *)(gep->lbuf.buf), 
			sizeof(*req)*SCD_GREQ_SZ);

#ifdef RW_LOCK_ON
		SCD_G_WRITE_UNLOCK_IRQ(&gep->rwLock, rwflags);
#endif
	}else{	/* maxpܿξϡ̤ΥХåեо */
		DBG(1, "setup from dataout buf\n");

		/* Хɥ쥹ᤷ */
		dma_unmap_req(sdev, pReq);

		/* Хåե */
		setup_buf = (void *)&((char *)(pReq->buf))[pReq->actual];
#ifdef SCD_CACHE_INVALIDATE_AFTER_DMA
		/* cache invalidate */
		udif_cache_ctrl_r((void *)setup_buf,
			sizeof(*req)*SCD_GREQ_SZ, UDIF_CACHE_INVALIDATE);
#endif /* SCD_CACHE_INVALIDATE_AFTER_DMA */
#ifdef RW_LOCK_ON
		SCD_G_WRITE_LOCK_IRQ(&gep->rwLock, rwflags);
#endif
		/* ꥯȥԡ */
		memcpy((void *)req, (void *)setup_buf, 
			sizeof(*req)*SCD_GREQ_SZ);

#ifdef RW_LOCK_ON
		SCD_G_WRITE_UNLOCK_IRQ(&gep->rwLock, rwflags);
#endif
		reqcmd = req;
	}
	DBG(2, "==>\n");
	return 0;
}
/*-------------------------------------------------------
	name	: scd_g_init_ep0buf
	argment	: struct scd_dmaep0_buf *ep0buf,
	return	: none
	description:
			  init scd_dmaep0_buf 
	date	: 2010/10/06
-------------------------------------------------------*/
void scd_g_init_ep0buf(struct scd_dmaep0_buf *setup_buf)
{
	unsigned long flags;

	DBG(2, "<==\n");

	SCD_G_SPIN_LOCK_IRQ(&setup_buf->lock, flags);
	
	memset(setup_buf->buf, 0x00, sizeof(*(scd_dma_ep0buf.buf))*SCD_DMA_EP0_SZ);
	setup_buf->dcnt		= 0;
	setup_buf->dma_buf	= NULL;
	SCD_G_SPIN_UNLOCK_IRQ(&setup_buf->lock, flags);
	DBG(2, "==>\n");

}

/*----------------------------------------------------------------------------
	Tasklet for interrupt
----------------------------------------------------------------------------*/
/*-------------------------------------------------------
	name	: scd_gadget_requests_notify
	argment	: unsigned long data
	return	: void
	description:
			  tasklet for requests completion
	date	: 2011/02/02
-------------------------------------------------------*/
static void scd_gadget_requests_notify(unsigned long data)
{
	struct list_head done_list;
	struct scd_gadget *sgadget = scd_gadget_info;
	struct scd_grequest *req = NULL, *tmp = NULL;
	struct scd_gep *gep;
	struct usb_request *pRequest;
	struct reqdone_queue *Donequeue = (struct reqdone_queue *)data;
	u8	   epnum;
	unsigned long flags;
	unsigned long dflags;
	unsigned long rwflags;
	int rslt;

	DBG(2, "<==\n");

	/* ꥹȥإåɽ */
	INIT_LIST_HEAD(&done_list);

	SCD_G_SPIN_LOCK_IRQ(&Donequeue->lock, dflags);
	
	list_for_each_entry_safe(req, tmp, &Donequeue->req_list, compl_list){
		/* 
		 *	taskletǤΥ륭塼ˤĤʤʤ
		 *	Υ塼Ϻ 
		 */
		list_move_tail(&req->compl_list, &done_list);
#ifdef RW_LOCK_ON
		/* EPֹ */
		epnum = req->bEp;
		/* gep */
		gep = &sgadget->gep[epnum];

		SCD_G_WRITE_LOCK(&gep->rwLock);
#endif		
		list_del(&((struct usb_request *)req)->list);

#ifdef RW_LOCK_ON
		SCD_G_WRITE_UNLOCK(&gep->rwLock);
#endif		
	}
	/* 륭塼θ */
	list_for_each_entry_safe(req, tmp, &done_list, compl_list){

		pRequest = (struct usb_request *)req;

		/* λꥹȺ */
		list_del(&req->compl_list);

		/* EPֹ */
		epnum = req->bEp;
		/* gep */
		gep = &sgadget->gep[epnum];

		if(pRequest) {
			if (pRequest->complete) {
				DBG(2,"target ep:%d pRequest:%p buf:%p \n",epnum, pRequest, pRequest->buf);
				/* λ */
				pRequest->complete(&gep->end_point, pRequest);
				DBG(2,"complete to the upper \n");
			}
		}
		/* 塼Ѥޤ줿ΤΤ롣 */
		SCD_G_SPIN_LOCK_IRQ(&gep->Lock, flags);

#ifdef RW_LOCK_ON
		SCD_G_READ_LOCK_IRQ(&gep->rwLock, rwflags);
#endif	
		pRequest = scd_get_currentreq(gep);
#ifdef RW_LOCK_ON
		SCD_G_READ_UNLOCK_IRQ(&gep->rwLock, rwflags);
#endif	
		if(pRequest){
			if(pRequest->status == -EALREADY){
				DBG(0,"Req ops by tasklet schedule\n");
				pRequest->status = -EINPROGRESS;	/* λ/Ϥstatus */
				SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);

				rslt = scd_g_general_trns(sgadget , epnum, 0);/* gfpե饰̤ */
				if(rslt<0){
					ERR("no request or error:%x in ep %d\n",rslt, epnum);
					/* ̤žλ */
					scd_gadget_complete_request_nolock(pRequest, rslt);
				}
				SCD_G_SPIN_LOCK_IRQ(&gep->Lock, flags);

			}
		}else{
			DBG(2,"pRequest NULL\n");
		}
		SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
	}
	SCD_G_SPIN_UNLOCK_IRQ(&Donequeue->lock, dflags);

	DBG(2, "==>\n");
}
/*----------------------------------------------------------------------------
	Tadklet Declaration
----------------------------------------------------------------------------*/
DECLARE_TASKLET(gadget_requests_notify, scd_gadget_requests_notify, (unsigned long)&scd_ReqDoneQueue);

/*-------------------------------------------------------
	name	: scd_gadget_complete_request
	argment	: struct usb_request *pRequest,
			  int status 
	return	: int
	description:
			  class transfer completion
	date	: 2011/02/02
-------------------------------------------------------*/
int scd_gadget_complete_request(struct usb_request *pRequest, int status)
{
	struct scd_gadget *sgadget = scd_gadget_info;
	struct scd_grequest *req = (struct scd_grequest *)pRequest;
	struct usb_request *pReq;
	unsigned long dflags;
	unsigned long rwflags;
	struct scd_gep *gep = &(sgadget->gep[req->bEp]);

	DBG(2, "<==\n");

	/* spin_lock_irqsave */
	SCD_G_SPIN_LOCK_IRQ(&scd_ReqDoneQueue.lock, dflags);

#ifdef RW_LOCK_ON
	SCD_G_READ_LOCK_IRQ(&gep->rwLock, rwflags);
#endif
	/* ߤΥꥯȼ */
	pReq = scd_get_currentreq(gep);
#ifdef RW_LOCK_ON
	SCD_G_READ_UNLOCK_IRQ(&gep->rwLock, rwflags);
#endif
	if(!req||(pReq != pRequest)) {
		ERR("error requestl\n");
		/* spin_unlock_irqrestore */
		SCD_G_SPIN_UNLOCK_IRQ(&scd_ReqDoneQueue.lock, dflags);
		DBG(2, "==>\n");
		return -EINVAL;
	}
	DBG(2, "scheduling completition of %p on ep=0x%x\n", req, req->bEp);
	
	if((pRequest->status==-EINPROGRESS)||(pRequest->status == -EALREADY)){
		INFO("not set status yet case or reset case\n");
		pRequest->status = status;
	}
	/* ꥯȴλꥹȤ򥿥åȥ塼إ */
	list_add_tail(&(req->compl_list), &(scd_ReqDoneQueue.req_list));

	/* spin_unlock_irqrestore */
	SCD_G_SPIN_UNLOCK_IRQ(&scd_ReqDoneQueue.lock, dflags);

	/* åȽ˰Ѥ */
	tasklet_schedule(&gadget_requests_notify);

	DBG(2, "==>\n");
	return 0;
}
/*-------------------------------------------------------
	name	: scd_gadget_complete_request_nolock
	argment	: struct usb_request *pRequest,
			  int status 
	return	: int
	description:
			  class transfer completion without lock
	date	: 2011/02/02
-------------------------------------------------------*/
int scd_gadget_complete_request_nolock(
	struct usb_request *pRequest, int status)
{
	struct scd_grequest *req = (struct scd_grequest *)pRequest;

	DBG(2, "<==\n");

	if (!req) {
		ERR("error request is Null\n");
		DBG(1, "==>\n");
		return -EINVAL;
	}
	DBG(2, "scheduling completition of %p on ep=0x%x\n", req, req->bEp);
	
	if((pRequest->status==-EINPROGRESS)||(pRequest->status == -EALREADY)){
		INFO("not set status yet case or reset case\n");
		pRequest->status = status;
	}
	/* ꥯȴλꥹȤ򥿥åȥ塼إ */
	list_add_tail(&(req->compl_list), &(scd_ReqDoneQueue.req_list));

	/* åȽ˰Ѥ */
	tasklet_schedule(&gadget_requests_notify);

	DBG(2, "==>\n");
	return 0;
}
/*-------------------------------------------------------
	name	: scd_doneque_init
	argment	: none
	return	: none
	description:
			  init request doneque parameter
	date	: 2011/01/29
-------------------------------------------------------*/
void scd_doneque_init(void)
{
	DBG(2, "<==\n");

	/* ԥå */
	spin_lock_init(&scd_ReqDoneQueue.lock);
	
	/* ꥹƬ */
	INIT_LIST_HEAD(&scd_ReqDoneQueue.req_list);
	
	scd_ReqDoneQueue.cnt	=	0;	/* for debug */

	DBG(2, "==>\n");
}

/*-------------------------------------------------------
	name	: dma_unmap_req
	argment	: struct scd_device *sdev,
			  struct usb_request *req
	return	: none
	description:
			  unmap request for dma
	date	: 2010/04/18
-------------------------------------------------------*/
void dma_unmap_req(struct scd_device *sdev, struct usb_request *req)
{
	struct scd_grequest *greq = (struct scd_grequest *)req;

	DBG(2, "<==\n");
	
	switch(greq->bAttribute&SCD_REQ_MAPMSK){
	case SCD_REQ_TX_SYNC:
		dma_sync_single_for_cpu(sdev->dev, req->dma, req->length, DMA_TO_DEVICE);
		DBG(2, "tx sync_single_for_cpu\n");
		break;
	case SCD_REQ_TX_MAP:
		dma_unmap_single(sdev->dev, req->dma, req->length, DMA_TO_DEVICE);
#ifdef UCT
		req->dma = 0;	/* ᤹ */
#endif
		DBG(2, "tx unmap_single\n");
		break;
	case SCD_REQ_RX_SYNC:
		dma_sync_single_for_cpu(sdev->dev, req->dma, req->length, DMA_FROM_DEVICE);
		DBG(2, "rx sync_single_for_cpu\n");
		break;
	case SCD_REQ_RX_MAP:
		dma_unmap_single(sdev->dev, req->dma, req->length, DMA_FROM_DEVICE);
#ifdef UCT
		req->dma = 0;	/* ᤹ */
#endif
		DBG(2, "rx unmap_single\n");
		break;
	default:
		DBG(2,"nothing unmapping\n");
	}
	greq->bAttribute &= ~SCD_REQ_MAPMSK;	/*  */
	DBG(2, "==>\n");
}

/*-------------------------------------------------------
	name	: scd_g_setaddr_compl
	argment	: struct scd_gadget *sgadget
	return	: int
	description:
			  status OUT process for internal control transfer
			  now only for set address
			  the caller may call spin_lock_**
	date	: 2010/08/24
-------------------------------------------------------*/
static int scd_g_setaddr_compl(struct scd_gadget *sgadget)
{
	u8 mode = sgadget->bTransMode;
	struct scd_device *sdev = sgadget->g_device;
	int rslt = 0;
	u32 reg_val;

	DBG(2, "<==\n");

	switch(mode){
	case SCD_GADGET_TRANS_DMA:

		/* setup */
#ifdef CHK_SUPCNT
		chk_setupcnt(sgadget);
#endif
		/*  */
		rslt = scd_chg_ep0stage(sgadget, SCD_EP0_STAGE_SETUP);
		if(rslt){
			/* 顼Ϥꤨʤ */
			ERR("==> error ep0stage not changed to SETUP \n");
			return -EPERM;
		}
		/* SubStateѹ */
		rslt = scd_set_ctrl_substate(sgadget, SCD_EP_IDLE);
		if(rslt){
			/* 顼Ϥꤨʤ */
			ERR("==> abnormal error  not chaneged to IDLE \n");
			return -EPERM;
		}
		/* SUPCNTϥåȤʤ */
		/* EPEna(SETUP)*/
		reg_val = SCD__READ(sdev, DOEPCTL0);
		reg_val |= SCD_B_DOEPCTL0_EPENA;
		SCD__WRITE(sdev, DOEPCTL0, reg_val);
		break;
	default:
		ERR("non-support transfer mode %x \n",mode);
		DBG(2, "==>\n");
		return -EPERM;
	}
	DBG(2, "==>\n");
	return 0;
}
/*-------------------------------------------------------
	name	: scd_g_reset_setupcnt
	argment	: struct scd_gadget *sgadget
			  u8 type 0:readmodifywrite 1: clean set
	return	: none
	description:
			  reset SUPCNT and DMA buffer
	date	: 2010/05/07
-------------------------------------------------------*/
void scd_g_reset_setupcnt(struct scd_gadget *sgadget, u8 type)
{
	struct scd_device *sdev = sgadget->g_device;
	u32 reg_val;
	u8 mode = sgadget->bTransMode;

	DBG(2, "<==\n");
#ifdef CHK_SUPCNT
	chk_setupcnt(sgadget);
#endif
	/* setup */
	reg_val = SCD__READ(sdev, DOEPTSIZ0);
	if(type){	/* clean set */
		reg_val = (SCD_SUPCNT_VAL<<SCD_S_DOEPTSIZ0_SUPCNT);
	}else{	/* read&modify-write */
		reg_val &= ~(SCD_M_DOEPTSIZ0_SUPCNT<<SCD_S_DOEPTSIZ0_SUPCNT);
		reg_val |= (SCD_SUPCNT_VAL<<SCD_S_DOEPTSIZ0_SUPCNT);
	}
	SCD__WRITE(sdev, DOEPTSIZ0, reg_val);

	switch(mode){
	case SCD_GADGET_TRANS_DMA:
		/* EP0Хåե */
		scd_g_init_ep0buf(&scd_dma_ep0buf);
		/* Хɥ쥹 */
		scd_dma_ep0buf.dma_buf 
			= (void *)dma_map_single(sdev->dev, 
				scd_dma_ep0buf.buf, 
					sizeof(*(scd_dma_ep0buf.buf))*SCD_DMA_EP0_SZ, 
						DMA_BIDIRECTIONAL);
		/* DMAХåե */
		SCD__WRITE(sdev, DOEPDMA(SCD_G_EP0), scd_dma_ep0buf.dma_buf);
		
#ifdef EPENA_CARE
		/* EPEnaǧ */
		reg_val = SCD__READ(sdev, DOEPCTL0);
		if(reg_val&SCD_B_DOEPCTL0_EPENA){
			INFO("EPEna already set DOEPCTL0:%x \n",reg_val);
		}else{
			/* EPEna(SETUP)*/
			reg_val = SCD__READ(sdev, DOEPCTL0);
			reg_val &= ~(SCD_B_DOEPCTL0_EPENA|SCD_B_DOEPCTL0_STALL);
			reg_val |= SCD_B_DOEPCTL0_EPENA;
			SCD__WRITE(sdev, DOEPCTL0, reg_val);
		}
#else
		/* EPEna(SETUP)*/
		reg_val = SCD__READ(sdev, DOEPCTL0);
		reg_val |= SCD_B_DOEPCTL0_EPENA;
		SCD__WRITE(sdev, DOEPCTL0, reg_val);
#endif
		
		break;
	}
	DBG(2, "==>\n");
}
/*-------------------------------------------------------
	name	: scd_g_irq_gintsts_goutnakeff
	argment	: struct scd_gadget *sgadget
	return	: int
	description:
			  to do after Global OUT Nak is Effective.
			nonuse
	date	: 2010/05/17
-------------------------------------------------------*/
void scd_g_irq_gintsts_goutnakeff(struct scd_gadget *sgadget)
{
	struct scd_device *sdev = sgadget->g_device;
	struct scd_gep *gep;
	struct scd_gadget_wq *gevent;
	unsigned long flags;
	u32 reg_val;
	int i, rslt;
	
	DBG(2,"<== \n");

	SCD_G_SPIN_LOCK_IRQ(&sgadget->gLock, flags);

	/* GlobalNAKEFF߶ػ */
	reg_val = SCD__READ(sdev, GINTMSK);
	reg_val &= ~SCD_B_GINTMSK_GOUTNAKEFFMSK;
	SCD__WRITE(sdev, GINTMSK, reg_val);

	for(i=0; i<sgadget->bEps+SCD_NUM_EP0; i++){
		gep = &sgadget->gep[i];
		gevent = &(gep->wq);

		if(get_gwq_flag(gevent) == SCD_GQ_WAIT_NAKEFF){
			/*  */
			rslt = set_gwq_flag(gevent, SCD_GQ_NAKEFF);
			if(rslt){
				ERR("error can't change to nakeff\n");
				SCD_G_SPIN_UNLOCK_IRQ(&sgadget->gLock, flags);
				DBG(2,"==> \n");
				return;
			}
		}
	}
	SCD_G_SPIN_UNLOCK_IRQ(&sgadget->gLock, flags);
	DBG(2,"==> \n");
	return;
}

/*-------------------------------------------------------
	name	: scd_g_int_nakeff
	argment	: struct scd_gadget *sgadget
			  u8 epnum
	return	: int
	description:
			  to do after Nak is Effective.
	date	: 2010/05/17
-------------------------------------------------------*/
int scd_g_int_nakeff(struct scd_gadget *sgadget, u8 epnum)
{
	struct scd_device *sdev = sgadget->g_device;
	struct scd_gep *gep = &sgadget->gep[epnum];
	struct scd_gadget_wq *gevent = &(gep->wq);
	unsigned long flags;
	int rslt;
	u32 reg_val;

	DBG(2,"<== \n");

	SCD_G_SPIN_LOCK_IRQ(&gep->Lock, flags);

	if(get_gwq_flag(gevent) == SCD_GQ_WAIT_NAKEFF){
		/* NAKEFF߶ػ */
		reg_val = SCD__READ(sdev, DIEPMSK);
		reg_val &= ~SCD_B_DIEPMSK_INEPNAKEFFMSK;
		SCD__WRITE(sdev, DIEPMSK, reg_val);

		/*  */
		rslt = set_gwq_flag(gevent, SCD_GQ_NAKEFF);
		if(rslt){
			ERR("error can't change to nakeff\n");
			SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
			DBG(2,"==> \n");
			return rslt;
		}
		SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
	}else{
		SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
		ERR("error Nakeff not match the condition \n");
	}
	DBG(2,"==> \n");
	return 0;
}

/*-------------------------------------------------------
	name	: scd_g_int_epdis
	argment	: struct scd_gadget *sgadget
			  u8 epnum
			  u8 istx
	return	: int
	description:
			  to do after ep disabled.
	date	: 2010/05/17
-------------------------------------------------------*/
int scd_g_int_epdis(struct scd_gadget *sgadget, u8 epnum, u8 istx)
{
	struct scd_gep *gep = &sgadget->gep[epnum];
	struct scd_gadget_wq *gevent = &(gep->wq);
	unsigned long flags;
	int rslt;
	
	DBG(2,"<== \n");
	
	SCD_G_SPIN_LOCK_IRQ(&gep->Lock, flags);
	
	if(istx){
		/* TX */
		if(get_gwq_flag(gevent) == SCD_GQ_WAIT_EPDIS){
			/*  */
			rslt = set_gwq_flag(gevent, SCD_GQ_EPDIS);
			if(rslt){
				ERR("error int epdis\n");
				SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
				DBG(2,"==> \n");
				return rslt;
			}
			SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
		}else{
			SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
			ERR("ERROR EPDIS not match the condition \n");
		}
	}else{
		/* RX */
		SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
	}
	DBG(2,"==> \n");
	return 0;
}
/*-------------------------------------------------------
	name	: get_gwq_flag
	argment	: struct scd_gadget_wq *gevent
	return	: int
	description:
			  get a wq_flag state
	date	: 2010/05/10
-------------------------------------------------------*/
int get_gwq_flag(struct scd_gadget_wq *gevent)
{
	DBG(3,"<==> \n");
	return gevent->wq_flag;
}
/*-------------------------------------------------------
	name	: set_gwq_flag
	argment	: struct scd_gadget_wq *gevent
			  int state
	return	: int
	description:
			  set a wq_flag state
	date	: 2010/05/10
-------------------------------------------------------*/
int set_gwq_flag(struct scd_gadget_wq *gevent, int state)
{
	int cur_flag;

	DBG(2,"<== state:%x \n",state);
	/* ߤͼ */
	cur_flag = get_gwq_flag(gevent);

	switch(state){
	case SCD_GQ_IDLE:
		switch(cur_flag){
		case SCD_GQ_IDLE:
		case SCD_GQ_WAIT_NAKEFF:
		case SCD_GQ_NAKEFF:
		case SCD_GQ_WAIT_EPDIS:
		case SCD_GQ_EPDIS:
		case SCD_GQ_CONNRESET:
		case SCD_GQ_SHUTDOWN:
			gevent->wq_flag = state;
			DBG(2,"changed to IDLE from %x at wq_flag \n",cur_flag);
			break;
		default:
			goto err;
		}
		break;
	case SCD_GQ_WAIT_NAKEFF:
		switch(cur_flag){
		case SCD_GQ_CONNRESET:	/* ǻνб */
		case SCD_GQ_SHUTDOWN:	/* ǻνб */
		case SCD_GQ_IDLE:
			gevent->wq_flag = state;
			DBG(2,"changed to WAIT_NAKEFF from %x at wq_flag \n",cur_flag);
			break;
		default:
			goto err;
		}
		break;
	case SCD_GQ_NAKEFF:
		switch(cur_flag){
		case SCD_GQ_IDLE:
		case SCD_GQ_WAIT_NAKEFF:
			gevent->wq_flag = state;
			DBG(2,"changed to NAKEFF from %x at wq_flag \n",cur_flag);
			break;
		default:
			goto err;
		}
		break;
	case SCD_GQ_WAIT_EPDIS:
		switch(cur_flag){
		case SCD_GQ_NAKEFF:	/* for periodic */
		case SCD_GQ_IDLE:	/* for non-periodic */
			gevent->wq_flag = state;
			DBG(2,"changed to WAIT_EPDIS from %x at wq_flag \n",cur_flag);
			break;
		default:
			goto err;
		}			
		break;
	case SCD_GQ_EPDIS:
		switch(cur_flag){
		case SCD_GQ_WAIT_EPDIS:
			gevent->wq_flag = state;
			DBG(2,"changed to EPDIS from %x at wq_flag \n",cur_flag);
			break;
		default:
			goto err;
		}
		break;
	case SCD_GQ_CONNRESET:
		switch(cur_flag){
		case SCD_GQ_IDLE:
		case SCD_GQ_WAIT_NAKEFF:
		case SCD_GQ_NAKEFF:
		case SCD_GQ_WAIT_EPDIS:
		case SCD_GQ_EPDIS:		
			gevent->wq_flag = state;
			DBG(2,"changed to CONNRESET from %x at wq_flag \n",cur_flag);
			break;
		default:
			goto err;
		}
		break;
	case SCD_GQ_SHUTDOWN:
		switch(cur_flag){
		case SCD_GQ_IDLE:
		case SCD_GQ_WAIT_NAKEFF:
		case SCD_GQ_NAKEFF:
		case SCD_GQ_WAIT_EPDIS:
		case SCD_GQ_EPDIS:	
		case SCD_GQ_CONNRESET:
		case SCD_GQ_SHUTDOWN:
			gevent->wq_flag = state;
			DBG(2,"changed to SHUTDOWN from %x at wq_flag \n",cur_flag);
			break;
		default:
			goto err;
		}
		break;
	}
	DBG(2,"==> %x to %x \n",cur_flag, state);
	return 0;
err:
	ERR("not changed to %x from %x at wq_flag \n", state ,cur_flag);
	DBG(2,"==> \n");
	return -EPERM;
}

/*-------------------------------------------------------
	name	: scd_g_fifo_flush_tx
	argment	: struct scd_gadget *sgadget
			  u8 epnum
			  u8 flag
	return	: int
	description:
			  tx fifo flush
	date	: 2010/08/26
-------------------------------------------------------*/
int scd_g_fifo_flush_tx(struct scd_gadget *sgadget, u8 epnum, u8 flag)
{
	struct scd_device *sdev = sgadget->g_device;
	struct scd_gep *gep = &sgadget->gep[epnum];
	struct scd_gadget_wq *gevent = &(gep->wq);
	unsigned long flags;
	u8 state;
	int rslt;
	u32 reg_val;
	u8	err_flag = 0;

	DBG(1,"<==\n");

	SCD_G_SPIN_LOCK_IRQ(&gep->Lock, flags);

	state = scd_get_substate(sgadget, epnum);
	
	if(epnum==SCD_G_EP0){ /* EP0 */
		switch(state){
		case SCD_EP_IDLE:
		case SCD_EP_IN_READY:
		case SCD_EP_QUE_WAIT:	/* Ǥʤɤȯ롣 */
		case SCD_EP_DATA_END:
		case SCD_EP_DATA_CONT:	/* ENDƱͤ˰ */
			goto act;
			break;
		case SCD_EP_DATA_COMP:
			SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
			ERR("error impossible case:%x \n",state);
			DBG(2,"==>\n");
			return -EPERM;
			break;
		case SCD_EP_FLUSH:	
		case SCD_EP_CANCEL:	/* ǧ cancelflushϤΤޤ޽λ */
		case SCD_EP_STALL:
			SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
			INFO("inprogress cancel state:%x \n",state);
			DBG(2,"==>\n");
			return 0;
			break;
		default:
			SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
			ERR("error non-support state:%x\n",state);
			DBG(2,"==>\n");
			return -EPERM;
		}
		
	}else{	/* EP0ʳ */
		/* ߤEP0Ʊĥ䤹ʬ
		 */
		switch(state){
		case SCD_EP_IDLE:
		case SCD_EP_IN_READY:
		case SCD_EP_QUE_WAIT:	/* Ǥʤɤȯ롣 */
		case SCD_EP_DATA_END:
		case SCD_EP_DATA_CONT:
			goto act;
			break;
		case SCD_EP_DATA_COMP:
			SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
			ERR("error impossible case:%x \n",state);
			DBG(2,"==>\n");
			return -EPERM;
			break;
		case SCD_EP_FLUSH:	
		case SCD_EP_CANCEL:	/* ǧ cancelflushϤΤޤ޽λ */
		case SCD_EP_STALL:
			SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
			INFO("inprogress cancel state:%x \n",state);
			DBG(2,"==>\n");
			return 0;
			break;
		default:
			SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
			ERR("error non-support state:%x\n",state);
			DBG(2,"==>\n");
			return -EPERM;
		}		
	}
act:
	/* EP */
	if(epnum==SCD_G_EP0){
		rslt = scd_set_ctrl_substate(sgadget, SCD_EP_FLUSH);
		if(rslt){
			ERR("error not change to FLUSH state \n");
			SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
			DBG(2,"==>\n");
			return -EPERM;
		}
	}else{
		rslt = scd_set_epstate(sgadget, epnum, SCD_EP_FLUSH);
		if(rslt){
			ERR("error not change to FLUSH state \n");
			SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
			DBG(2,"==>\n");
			return -EPERM;
		}
	}
	if(epnum==SCD_G_EP0){	/* EP0 */
			reg_val = SCD__READ(sdev, DIEPCTL0);
			reg_val |= SCD_B_DIEPCTL0_SNAK;	/* SNAKå */
			SCD__WRITE(sdev, DIEPCTL0, reg_val);
			
			reg_val = SCD__READ(sdev, DIEPCTL0);
			if(reg_val&SCD_B_DIEPCTL0_EPENA){
				DBG(1,"Epena alredy set\n");
				/* NAKEFFԤ */
				/*  */
				rslt = set_gwq_flag(gevent, SCD_GQ_WAIT_NAKEFF);
				if(rslt){
					ERR("error can't change to nakeff\n");
					SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
					DBG(2,"==>\n");
					return rslt;
				}
#ifdef SCD_POLL_ON
				SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);

				rslt = poll_wait_innakeff(sgadget, epnum);
				if(rslt){
					err_flag = true;
					goto err;
				}
				SCD_G_SPIN_LOCK_IRQ(&gep->Lock, flags);
#endif
			}
	}else{	/* except EP0 */
			reg_val = SCD__READ(sdev, DIEPCTL(epnum));
			reg_val |= SCD_B_DXEPCTLN_SNAK;	/* SNAKå */
			SCD__WRITE(sdev, DIEPCTL(epnum), reg_val);

			reg_val = SCD__READ(sdev, DIEPCTL(epnum));
			if(reg_val&SCD_B_DXEPCTLN_EPENA){
				DBG(1,"Epena already set\n");
				/* NAKEFFԤ */
				/*  */
				rslt = set_gwq_flag(gevent, SCD_GQ_WAIT_NAKEFF);
				if(rslt){
					ERR("error int epdis\n");
					SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
					DBG(2,"==>\n");
					return rslt;
				}
				SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
#ifdef SCD_POLL_ON
				rslt = poll_wait_innakeff(sgadget, epnum);
				if(rslt){
					err_flag = true;
					goto err;
				}
#endif
				SCD_G_SPIN_LOCK_IRQ(&gep->Lock, flags);
			}
	}
	if(epnum==SCD_G_EP0){
		reg_val = SCD__READ(sdev, DIEPCTL0);
		if(reg_val&SCD_B_DIEPCTL0_EPENA){
			/* EPEnaͭΤ᥻å */
			reg_val |= SCD_B_DIEPCTL0_EPDIS;
			SCD__WRITE(sdev, DIEPCTL0, reg_val);
			DBG(1,"Epdis set at ep:%x\n",epnum);
		}else{
			goto epdisskip;
		}
	}else{
		reg_val = SCD__READ(sdev, DIEPCTL(epnum));
		if(reg_val&SCD_B_DXEPCTLN_EPENA){
			/* EPEnaͭΤ᥻å */
			reg_val |= SCD_B_DXEPCTLN_EPDIS;
			SCD__WRITE(sdev, DIEPCTL(epnum), reg_val);
		}else{
			goto epdisskip;
		}
	}
	SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
#ifdef 	SCD_POLL_ON
	/* װȯԤ */
	rslt = poll_wait_inepdis(sgadget, epnum);
	if(rslt){
		err_flag = true;
		goto err;
	}
#endif
	SCD_G_SPIN_LOCK_IRQ(&gep->Lock, flags);

epdisskip:

	/* FIFO flush */
	reg_val = SCD__READ(sdev, GRSTCTL);
	reg_val &= ~(SCD_M_GRSTCTL_TXFNUM<<SCD_S_GRSTCTL_TXFNUM|
		SCD_B_GRSTCTL_TXFFLSH|SCD_B_GRSTCTL_RXFFLSH);
	reg_val |= epnum<<SCD_S_GRSTCTL_TXFNUM|SCD_B_GRSTCTL_TXFFLSH;
	SCD__WRITE(sdev, GRSTCTL, reg_val);

	SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
			
	/* FLUSHλԤ */
	rslt = chk_flush_comp(sgadget, SCD_B_GRSTCTL_TXFFLSH);
	if(rslt){
		ERR("==>error flush comp\n");
		/* 顼ʸߤϥ롼Ǥ */
		return rslt;
	}

err:	/* ʲ롣 */
	if(!flag){	/* ̾ */

		SCD_G_SPIN_LOCK_IRQ(&gep->Lock, flags);

		/*  */
		rslt = set_gwq_flag(gevent, SCD_GQ_IDLE);
		if(rslt){
				ERR("error int epdis\n");
				SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
				return rslt;
		}
		/* EP */
		if(epnum==SCD_G_EP0){	/* EP0 */
				rslt = scd_chg_ep0stage(sgadget, SCD_EP0_STAGE_SETUP);
				if(rslt){
					/* 顼Ϥꤨʤ */
					ERR("==> error ep0stage not changed to SETUP \n");
					SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
					return -EPERM;
				}
				rslt = scd_set_ctrl_substate(sgadget, SCD_EP_IDLE);
				if(rslt){
					/* 顼Ϥꤨʤ */
					ERR("==> abnormal error  not chaneged to IDLE \n");
					SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
					return -EPERM;
				}
				/* reset setup count */
				scd_g_reset_setupcnt(sgadget, SCD_G_SUPCNT_MODE1);
		}else{
				rslt = scd_set_epstate(sgadget, epnum, SCD_EP_IDLE);
				if(rslt){
					SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
					ERR("error not change to IDLE state \n");
					return -EPERM;
				}
		}
		SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
	}else{
		/*  */
		rslt = set_gwq_flag(gevent, SCD_GQ_IDLE);
		if(rslt){
				ERR("error int epdis\n");
				SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
				return rslt;
		}
		SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
	}
	if(err_flag){
		ERR("==> error or interrupt end\n");
		return -EPERM;
	}else{
		DBG(1,"==>\n");
		return 0;
	}
}
/*-------------------------------------------------------
	name	: scd_g_fifo_flush_rx
	argment	: struct scd_gadget *sgadget
			  u8 epnum,
			  u8 flag
	return	: int
	description:
			  rx fifo flush
	date	: 2010/08/24
-------------------------------------------------------*/
int scd_g_fifo_flush_rx(struct scd_gadget *sgadget, u8 epnum, u8 flag)
{
	struct scd_device *sdev = sgadget->g_device;
	struct scd_gep *gep = &sgadget->gep[epnum];
	struct scd_gadget_wq *gevent = &(gep->wq);
	unsigned long flags;
	u8 state;
	int rslt;
	u32 reg_val;
	u8	err_flag = 0;

	DBG(1,"<==\n");

	SCD_G_SPIN_LOCK_IRQ(&gep->Lock, flags);

	state = scd_get_substate(sgadget, epnum);
	
	/* ϰϥåϾ̤ǹԤ */
	if(epnum==SCD_G_EP0){ /* EP0 */
		switch(state){
		case SCD_EP_IDLE:
		case SCD_EP_OUT_READY:
		case SCD_EP_QUE_WAIT:	/* Ǥʤɤȯ롣 */
		case SCD_EP_DATA_END:
		case SCD_EP_DATA_CONT:
			goto act;
			break;
		case SCD_EP_DATA_COMP:
			SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
			ERR("error impossible case:%x \n",state);
			DBG(2,"==>\n");
			return -EPERM;
			break;
		case SCD_EP_FLUSH:	
		case SCD_EP_CANCEL:	/* ǧ cancelflushϤΤޤ޽λ */
		case SCD_EP_STALL:
			SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
			INFO("inprogress cancel state:%x \n",state);
			DBG(2,"==>\n");
			return 0;
			break;
		default:
			SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
			ERR("error non-support state:%x\n",state);
			DBG(2,"==>\n");
			return -EPERM;
		}
		
	}else{	/* EP0ʳ */
		/* ߤEP0Ʊĥ䤹ʬ
		 */
		switch(state){
		case SCD_EP_IDLE:
		case SCD_EP_OUT_READY:
		case SCD_EP_QUE_WAIT:	/* Ǥʤɤȯ롣 */
		case SCD_EP_DATA_END:
		case SCD_EP_DATA_CONT:
			goto act;
			break;
		case SCD_EP_DATA_COMP:
			SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
			ERR("error impossible case:%x \n",state);
			DBG(2,"==>\n");
			return -EPERM;
			break;
		case SCD_EP_FLUSH:	
		case SCD_EP_CANCEL:	/* ǧ cancelflushϤΤޤ޽λ */
		case SCD_EP_STALL:
			SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
			INFO("inprogress cancel state:%x \n",state);
			DBG(2,"==>\n");
			return 0;
			break;
		default:
			SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
			ERR("error non-support state:%x\n",state);
			DBG(2,"==>\n");
			return -EPERM;
		}
	}
act:
	/* EP */
	if(epnum==SCD_G_EP0){
		rslt = scd_set_ctrl_substate(sgadget, SCD_EP_FLUSH);
		if(rslt){
			ERR("error not change to FLUSH state \n");
			SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
			DBG(2,"==>\n");
			return -EPERM;
		}
	}else{
		rslt = scd_set_epstate(sgadget, epnum, SCD_EP_FLUSH);
		if(rslt){
			ERR("error not change to FLUSH state \n");
			SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
			DBG(2,"==>\n");
			return -EPERM;
		}
	}
	/* NAKEFFԤ */
	/*  */
	rslt = set_gwq_flag(gevent, SCD_GQ_WAIT_NAKEFF);
	if(rslt){
		SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
		ERR("error can't changed to wait_nakeff \n");
		DBG(2,"==>\n");
		return rslt;
	}
	/* DCTL */
	reg_val = SCD__READ(sdev, DCTL);
	reg_val |= SCD_B_DCTL_SGOUTNAK;	/* GlobalNAKå */
	SCD__WRITE(sdev, DCTL, reg_val);		

	SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
	
	/* GOUT-NAKEFFװȯԤ */
#ifdef 	SCD_POLL_ON
	rslt = poll_wait_goutnakeff(sgadget, epnum);
	if(rslt){
		DBG(1,"error wait goutnakeff\n");
		err_flag = true;
		SCD_G_SPIN_LOCK_IRQ(&gep->Lock, flags);

		/* DCTL */
		reg_val = SCD__READ(sdev, DCTL);
		reg_val |= SCD_B_DCTL_CGOUTNAK;	/* GlobalNAKꥢ */
		SCD__WRITE(sdev, DCTL, reg_val);

		goto err;
	}
#endif
	SCD_G_SPIN_LOCK_IRQ(&gep->Lock, flags);

	/*  */
	rslt = set_gwq_flag(gevent, SCD_GQ_NAKEFF);
	if(rslt){
		ERR("error int epdis\n");
		SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
		DBG(2,"==>\n");
		return rslt;
	}
	/*  */
	rslt = set_gwq_flag(gevent, SCD_GQ_WAIT_EPDIS);
	if(rslt){
		ERR("error int epdis\n");
		SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
		DBG(2,"==>\n");
		return rslt;
	}
	/* DOEPCTL */
	if(epnum!=SCD_G_EP0){	/* EP0noOUTEPdisǤʤ */
		reg_val = SCD__READ(sdev, DOEPCTL(epnum));
		reg_val &= ~SCD_B_DXEPCTLN_EPENA;
		reg_val |=  SCD_B_DXEPCTLN_SNAK;
		SCD__WRITE(sdev, DOEPCTL(epnum), reg_val);
		reg_val = SCD__READ(sdev, DOEPCTL(epnum));
		if(reg_val&SCD_B_DXEPCTLN_EPENA){
			/* EPEnaͭ椨å */
			reg_val |= SCD_B_DXEPCTLN_EPDIS;
			SCD__WRITE(sdev, DOEPCTL(epnum), reg_val);
			DBG(1,"OUT Epdis set at ep:%x\n",epnum);

			SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
#ifdef 	SCD_POLL_ON
			/* װȯԤ */
			rslt = poll_wait_outepdis(sgadget, epnum);
			if(rslt){
				DBG(1,"error wait outepdis \n");
				err_flag = true;
				SCD_G_SPIN_LOCK_IRQ(&gep->Lock, flags);

				/* DCTL */
				reg_val = SCD__READ(sdev, DCTL);
				reg_val |= SCD_B_DCTL_CGOUTNAK;	/* GlobalNAKꥢ */
				SCD__WRITE(sdev, DCTL, reg_val);

				goto err;
			}
#endif
			SCD_G_SPIN_LOCK_IRQ(&gep->Lock, flags);
		}
	}
	else{

		reg_val = SCD__READ(sdev, DOEPCTL(epnum));
		reg_val &= ~SCD_B_DOEPCTL0_EPENA;
		reg_val |=  SCD_B_DOEPCTL0_SNAK;
		SCD__WRITE(sdev, DOEPCTL(epnum), reg_val);
	}
	/* DCTL */
	reg_val = SCD__READ(sdev, DCTL);
	reg_val |= SCD_B_DCTL_CGOUTNAK;	/* GlobalNAKꥢ */
	SCD__WRITE(sdev, DCTL, reg_val);

	SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);

	/* žEP */
	rslt = scd_serach_trans_ep(sgadget, epnum);
	if(rslt){	/* 0:thru */
		SCD_G_SPIN_LOCK_IRQ(&gep->Lock, flags);

		/* FIFO flush */
		reg_val = SCD__READ(sdev, GRSTCTL);
		reg_val &= ~(SCD_M_GRSTCTL_TXFNUM<<SCD_S_GRSTCTL_TXFNUM|
			SCD_B_GRSTCTL_TXFFLSH|SCD_B_GRSTCTL_RXFFLSH);
		reg_val |= SCD_B_GRSTCTL_RXFFLSH;
		SCD__WRITE(sdev, GRSTCTL, reg_val);

		SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
			
		/* FLUSHλԤ */
		rslt = chk_flush_comp(sgadget, SCD_B_GRSTCTL_RXFFLSH);
		if(rslt){
			/* 顼ʸߤϥ롼Ǥ */
			ERR("==>error\n");
			return rslt;
		}
	}
	SCD_G_SPIN_LOCK_IRQ(&gep->Lock, flags);

	/*  */
	rslt = set_gwq_flag(gevent, SCD_GQ_IDLE);
	if(rslt){
		ERR("error int epdis\n");
		SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
		DBG(2,"==>\n");
		return rslt;
	}
err:	/* ʲ롣 */
	if(!flag){	/* normal */
		/*  */
		rslt = set_gwq_flag(gevent, SCD_GQ_IDLE);
		if(rslt){
				ERR("error int epdis\n");
				SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
				DBG(2,"==>\n");
				return rslt;
		}
		/* EP */
		if(epnum==SCD_G_EP0){	/* EP0 */
				rslt = scd_chg_ep0stage(sgadget, SCD_EP0_STAGE_SETUP);
				if(rslt){
					/* 顼Ϥꤨʤ */
					ERR("==> error not change ep0stage to SETUP\n");
					SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
					return -EPERM;
				}
				rslt = scd_set_ctrl_substate(sgadget, SCD_EP_IDLE);
				if(rslt){
					/* 顼Ϥꤨʤ */
					ERR("==>error not change to IDLE state \n");
					SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
					return -EPERM;
				}
				/* reset setup count */
				scd_g_reset_setupcnt(sgadget, SCD_G_SUPCNT_MODE1);
		}else{
				rslt = scd_set_epstate(sgadget, epnum, SCD_EP_IDLE);
				if(rslt){
					SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
					ERR("error not change to IDLE state \n");
					return -EPERM;
				}
		}
		SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
	}else{
		/*  */
		rslt = set_gwq_flag(gevent, SCD_GQ_IDLE);
		if(rslt){
				ERR("error int epdis\n");
				SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
				return rslt;
		}
		SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
	}
	if(err_flag){
		ERR("==> error or interrupt end\n");
		return -EPERM;
	}else{
		DBG(1,"==>\n");
		return 0;
	}
}
/*-------------------------------------------------------
	name	: scd_serach_trans_ep
	argment	: struct scd_gadget *sgadget
			  u8 epnum
	return	: int 1: flushǽ0flushػ
	description:
			  search eps for flush
			  RxFIFOhaGlobalFIFO椨¾ǻѤƤ
			  Flushػߤ롣
	date	: 2010/10/15
-------------------------------------------------------*/
static int scd_serach_trans_ep(struct scd_gadget *sgadget, u8 epnum)
{
	u8 state;
	int i;
	u8 flush = true;

	state = scd_get_substate(sgadget, epnum);
	DBG(3,"state:%x \n",state);

	if(epnum==SCD_G_EP0){
		switch(state){
		case SCD_EP_IDLE:		/* žIDLE */
		case SCD_EP_OUT_READY:	/* Ѥ */
		case SCD_EP_QUE_WAIT:	/* 塼Ԥ */
		case SCD_EP_DATA_CONT:	/* ǡ³ */
		case SCD_EP_DATA_END:	/* ǡ饹 */
		case SCD_EP_DATA_COMP:	/* ǡλ */
		case SCD_EP_ZERO_END:	/* ZeroĹ */
			for(i=1; i<sgadget->bEps+SCD_NUM_EP0; i++){
				/* EPּ GlobalFIFO椨*/
				state = scd_get_substate(sgadget, epnum);
				if(state!=SCD_EP_IDLE){
					/* 椨flushԲ */
					flush = false;
					break;
				}
			}
			break;
		case SCD_EP_FLUSH:		/* FLUSH */
		case SCD_EP_CANCEL:		/* 󥻥 */
		case SCD_EP_STALL:		/* STALL */
		case SCD_EP_DATA_ERR:	/* ǡ顼 */
			flush = true;
		default:
			flush = false;
			break;
		}
	}else{
		switch(state){
		case SCD_EP_IDLE:		/* žIDLE */
		case SCD_EP_OUT_READY:	/* Ѥ */
		case SCD_EP_QUE_WAIT:	/* 塼Ԥ */
		case SCD_EP_DATA_CONT:  /* ǡ³ */
		case SCD_EP_DATA_END:   /* ǡ饹 */
		case SCD_EP_DATA_COMP:  /* ǡλ */
		case SCD_EP_ZERO_END:   /* ZeroĹ */
			for(i=0; i<sgadget->bEps+SCD_NUM_EP0; i++){
				/* EPּ GlobalFIFO椨*/
				if(i!=epnum){
					state = scd_get_substate(sgadget, epnum);
					if(state!=SCD_EP_IDLE){
						/* 椨flushԲ */
						flush = false;
						break;
					}
				}
			}
			break;
		case SCD_EP_FLUSH:		/* FLUSH */
		case SCD_EP_CANCEL:		/* 󥻥 */
		case SCD_EP_STALL:		/* STALL */
		case SCD_EP_DATA_ERR:	/* ǡ顼 */
			flush = true;
		default:
			flush = false;
			break;
		}
	}
	DBG(2,"==> flush ok:%x \n", flush);
	return flush;

}

/*-------------------------------------------------------
	name	: scd_gadget_set_halt_ep0
	argment	: struct scd_gadget *sgadget
			  int value
	return	: int
	description:
			  stall operation for EP0
	date	: 2010/05/12
-------------------------------------------------------*/
int scd_gadget_set_halt_ep0(struct scd_gadget *sgadget, int value)
{
	u8 stage;

	DBG(1,"<== val:%d \n", value);
	
	stage = scd_get_ep0stage(sgadget);
	switch(stage){
	case SCD_EP0_STAGE_DATA_IN:
	case SCD_EP0_STAGE_STATUS_IN:
	case SCD_EP0_STAGE_DATA_OUT:
	case SCD_EP0_STAGE_STATUS_OUT:
	case SCD_EP0_STAGE_SETUP:
		scd_gadget_set_halt_out(sgadget, SCD_G_EP0, value);
		scd_gadget_set_halt_in(sgadget, SCD_G_EP0, value);
		break;
	default:
		DBG(1,"no stall because of stage:%d\n",stage);
	}
	DBG(1,"==> \n");
	return 0;
}

/*-------------------------------------------------------
	name	: scd_gadget_set_halt_in
	argment	: struct scd_gadget *sgadget
			  u8 epnum
			  int value
	return	: int
	description:
			  stall operations for tx-ep except EP0
	date	: 2010/06/03
-------------------------------------------------------*/
int scd_gadget_set_halt_in(struct scd_gadget *sgadget, u8 epnum, int value)
{
	struct scd_device *sdev = sgadget->g_device;
	struct scd_gep *gep = &sgadget->gep[epnum];
	struct scd_gadget_wq *gevent = &(gep->wq);
	unsigned long flags;
	int rslt;
	u32 reg_val;

	DBG(2,"<== epnum:%x \n", epnum);

	SCD_G_SPIN_LOCK_IRQ(&gep->Lock, flags);
	/* flushȰ㤤֤Ƕ̤ʤ */

	/*-----------------------------------------------------------------*/
	if(value){	/* set STALL */
#ifdef DISPLAY_STALL_VAL
		DBG(1," set STALL ep:%x \n",epnum);
#endif
		/* EP */
		if(epnum==SCD_G_EP0){	/* EP0 */
			rslt = scd_set_ctrl_substate(sgadget, SCD_EP_STALL);
		}else{
			rslt = scd_set_epstate(sgadget, epnum, SCD_EP_STALL);
		}
		if(rslt){
			SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
			ERR("==>error not change to STALL state \n");
			return -EPERM;
		}
		/* DIEPCTL */
		if(epnum==SCD_G_EP0){	/* EP0 */
			reg_val = SCD__READ(sdev, DIEPCTL0);
			/* SNAK/STALLå */
			/* NAKEFFߤEPDIS˰¸ƤߤǤ롦 */
			reg_val |= SCD_B_DIEPCTL0_SNAK|SCD_B_DIEPCTL0_STALL;
			SCD__WRITE(sdev, DIEPCTL0, reg_val);
			
			reg_val = SCD__READ(sdev, DIEPCTL0);
			if(reg_val&SCD_B_DIEPCTL0_EPENA){
				/* NAKEFFԤ */
				/*  */
				rslt = set_gwq_flag(gevent, SCD_GQ_WAIT_NAKEFF);
				if(rslt){
					ERR("error int wait_nakeff\n");
					SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
					goto err;
				}
				SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);

#ifdef SCD_POLL_ON
				rslt = poll_wait_innakeff(sgadget, epnum);
				if(rslt) goto err;
#endif
				SCD_G_SPIN_LOCK_IRQ(&gep->Lock, flags);
			}
		}else{	/* except EP0 */
			reg_val = SCD__READ(sdev, DIEPCTL(epnum));
			/* SNAK/STALLå */
			reg_val |= SCD_B_DXEPCTLN_SNAK|SCD_B_DXEPCTLN_STALL;
			SCD__WRITE(sdev, DIEPCTL(epnum), reg_val);

			reg_val = SCD__READ(sdev, DIEPCTL(epnum));
			if(reg_val&SCD_B_DXEPCTLN_EPENA){
				DBG(1,"Epena already set\n");
				/* NAKEFFԤ */
				/*  */
				rslt = set_gwq_flag(gevent, SCD_GQ_WAIT_NAKEFF);
				if(rslt){
					SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
					ERR("error can't changed to wait_nakeff \n");
					goto err;
				}
				SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);

#ifdef SCD_POLL_ON
				rslt = poll_wait_innakeff(sgadget, epnum);
				if(rslt) goto err;
#endif
				SCD_G_SPIN_LOCK_IRQ(&gep->Lock, flags);
			}
		}
		/*  */
		rslt = set_gwq_flag(gevent, SCD_GQ_NAKEFF);
		if(rslt){
			ERR("error int nakeff\n");
			SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
			goto err;
		}
		/*  */
		rslt = set_gwq_flag(gevent, SCD_GQ_WAIT_EPDIS);
		if(rslt){
			ERR("error int epdis\n");
			SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
			goto err;
		}
		/* DIEPCTL */
		if(epnum==SCD_G_EP0){
			reg_val = SCD__READ(sdev, DIEPCTL0);
			if(reg_val&SCD_B_DIEPCTL0_EPENA){
				/* EPEnaͭΤ᥻å */
				reg_val |= SCD_B_DIEPCTL0_EPDIS;

				SCD__WRITE(sdev, DIEPCTL0, reg_val);
				DBG(1,"Epdis set at ep:%x\n",epnum);

				SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
#ifdef 	SCD_POLL_ON
				/* װȯԤ */
				rslt = poll_wait_inepdis(sgadget, SCD_G_EP0);
				if(rslt) goto err;
#endif
				SCD_G_SPIN_LOCK_IRQ(&gep->Lock, flags);
			}
		}else{
			reg_val = SCD__READ(sdev, DIEPCTL(epnum));
			if(reg_val&SCD_B_DXEPCTLN_EPENA){
				/* EPEnaͭΤ᥻å */
				reg_val |= SCD_B_DXEPCTLN_EPDIS;
				SCD__WRITE(sdev, DIEPCTL(epnum), reg_val);
				DBG(1,"Epdis set at ep:%x\n",epnum);

				SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
#ifdef 	SCD_POLL_ON
				/* װȯԤ */
				rslt = poll_wait_inepdis(sgadget, epnum);
				if(rslt) goto err;
#endif
				SCD_G_SPIN_LOCK_IRQ(&gep->Lock, flags);
			}
		}
		/* FIFO flush */
		reg_val = SCD__READ(sdev, GRSTCTL);
		reg_val &= ~(SCD_M_GRSTCTL_TXFNUM<<SCD_S_GRSTCTL_TXFNUM|
			SCD_B_GRSTCTL_TXFFLSH|SCD_B_GRSTCTL_RXFFLSH);
		reg_val |= epnum<<SCD_S_GRSTCTL_TXFNUM|SCD_B_GRSTCTL_TXFFLSH;
		SCD__WRITE(sdev, GRSTCTL, reg_val);

		SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
			
		/* FLUSHλԤ */
		rslt = chk_flush_comp(sgadget, SCD_B_GRSTCTL_TXFFLSH);
		if(rslt){
			ERR("==> error txfflush\n");
			goto err;
		}
		SCD_G_SPIN_LOCK_IRQ(&gep->Lock, flags);
		/*  */
		rslt = set_gwq_flag(gevent, SCD_GQ_IDLE);
		if(rslt){
			ERR("error int epdis\n");
			SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
			goto err;
		}
	/*-----------------------------------------------------------------*/
	}else{	/* clear STALL */
#ifdef DISPLAY_STALL_VAL
		DBG(1," cleat STALL ep:%x\n",epnum);
#endif
		/* EP */
		if(epnum==SCD_G_EP0){	/* EP0 */
			/* DIEPCTL */
			reg_val = SCD__READ(sdev, DIEPCTL0);
			if(reg_val&SCD_B_DIEPCTL0_EPDIS){
				/* ä顢HWλ;SWǥեʤȤʤ */
				ERR("warn epdis remains at ep0 in \n");
				reg_val |= SCD_B_DIEPCTL0_EPENA; /* EnaӥåΩƤȾä*/
												 /* װ⤿ġâ */
			}
			SCD__WRITE(sdev, DIEPCTL0, reg_val);

			/*  */
			rslt = scd_chg_ep0stage(sgadget, SCD_EP0_STAGE_SETUP);
			if(rslt){
				/* 顼Ϥꤨʤ */
				ERR("==> error not change ep0stage to SETUP\n");
				SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
				return -EPERM;
			}
			rslt = scd_set_ctrl_substate(sgadget, SCD_EP_IDLE);
			if(rslt){
				/* 顼Ϥꤨʤ */
				ERR("==> abnormal error  not chaneged to IDLE \n");
				SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
				return -EPERM;
			}
			/* reset setup count */
			scd_g_reset_setupcnt(sgadget, SCD_G_SUPCNT_MODE1);

		}else{	/* EP0ʳ */
			/* DIEPCTL */
			reg_val = SCD__READ(sdev, DIEPCTL(epnum));
			if(reg_val&SCD_B_DXEPCTLN_EPDIS){
				/* ä顢HWλ;SWǥեʤȤʤ */
				ERR("WARN epdis remains at ep%d in \n",epnum);
				reg_val |= SCD_B_DXEPCTLN_EPENA; /* EnaӥåΩƤȾä*/
												 /* װ⤿ġâ */
			}
			/* STALLꥢ */
			reg_val &= ~SCD_B_DXEPCTLN_STALL;
			/* ȥ륯ꥢ */
			reg_val |= SCD_B_DXEPCTLN_SETD0PID;
			SCD__WRITE(sdev, DIEPCTL(epnum), reg_val);

			rslt = scd_set_epstate(sgadget, epnum, SCD_EP_IDLE);
			if(rslt){
				SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
				ERR("==>error not change to STALL state \n");
				return -EPERM;
			}
		}
	}
	SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
	DBG(2,"==>\n");
	return 0;	
err:
	SCD_G_SPIN_LOCK_IRQ(&gep->Lock, flags);

	/*  */
	rslt = set_gwq_flag(gevent, SCD_GQ_IDLE);
	if(rslt){
			ERR("error int epdis\n");
			SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
			DBG(2,"==>\n");
			return rslt;
	}
	/* EP */
	if(epnum==SCD_G_EP0){	/* EP0 */
			rslt = scd_chg_ep0stage(sgadget, SCD_EP0_STAGE_SETUP);
			if(rslt){
				/* 顼Ϥꤨʤ */
				ERR("==> error ep0stage not changed to SETUP \n");
				SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
				return -EPERM;
			}
			rslt = scd_set_ctrl_substate(sgadget, SCD_EP_IDLE);
			if(rslt){
				/* 顼Ϥꤨʤ */
				ERR("==> abnormal error  not chaneged to IDLE \n");
				SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
				return -EPERM;
			}
			/* reset setup count */
			scd_g_reset_setupcnt(sgadget, SCD_G_SUPCNT_MODE1);
	}else{
			rslt = scd_set_epstate(sgadget, epnum, SCD_EP_IDLE);
			if(rslt){
				SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
				ERR("==>error not change to IDLE state \n");
				return -EPERM;
			}
	}
	SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
	ERR("==> error or interrupt end\n");
	return -EPERM;
}

/*-------------------------------------------------------
	name	: scd_gadget_set_halt_out
	argment	: struct scd_gadget *sgadget
			  u8 epnum
			  int value
	return	: int
	description:
			  stall operation for rx-ep except EP0
	date	: 2010/08/24
-------------------------------------------------------*/
int scd_gadget_set_halt_out(struct scd_gadget *sgadget, u8 epnum, int value)
{
	struct scd_device *sdev = sgadget->g_device;
	struct scd_gep *gep = &sgadget->gep[epnum];
	struct scd_gadget_wq *gevent = &(gep->wq);
	unsigned long flags;
	int rslt;
	u32 reg_val;

	DBG(2,"<==\n");

	SCD_G_SPIN_LOCK_IRQ(&gep->Lock, flags);

	/* flushȰ㤤֤Ƕ̤ʤ */
	/*-----------------------------------------------------------------*/
	if(value){	/* set STALL */
#ifdef DISPLAY_STALL_VAL
		DBG(1," set STALL ep:%x \n",epnum);
#endif
		/* EP */
		if(epnum==SCD_G_EP0){	/* EP0 */
			rslt = scd_set_ctrl_substate(sgadget, SCD_EP_STALL);
		}else{
			rslt = scd_set_epstate(sgadget, epnum, SCD_EP_STALL);
		}
		if(rslt){
			ERR("error not change to STALL state \n");
			SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
			return -EPERM;
		}
		/*  */
		rslt = set_gwq_flag(gevent, SCD_GQ_WAIT_NAKEFF);
		if(rslt){
			ERR("error can't changed to nakeff \n");
			SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
			goto err;
		}
		/* DCTL */
		reg_val = SCD__READ(sdev, DCTL);
		reg_val |= SCD_B_DCTL_SGOUTNAK;	/* GlobalNAKå */
		SCD__WRITE(sdev, DCTL, reg_val);

		SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);

		/* GOUT-NAKEFFװȯԤ */
#ifdef 	SCD_POLL_ON
		rslt = poll_wait_goutnakeff(sgadget, epnum);
		if(rslt) goto err;

#endif
		SCD_G_SPIN_LOCK_IRQ(&gep->Lock, flags);

		/*  */
		rslt = set_gwq_flag(gevent, SCD_GQ_NAKEFF);
		if(rslt){
			ERR("error can't changed to nakeff \n");
			SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
			goto err;
		}
		/*  */
		rslt = set_gwq_flag(gevent, SCD_GQ_WAIT_EPDIS);
		if(rslt){
			ERR("error int epdis\n");
			SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
			goto err;
		}
		/* DOEPCTL */
		if(epnum==SCD_G_EP0){	/* EP0 */
			reg_val = SCD__READ(sdev, DOEPCTL0);
			/* STALLå */
			/* 쥸ͽOUTEPDisʤ */
			reg_val |= SCD_B_DOEPCTL0_STALL;
			SCD__WRITE(sdev, DOEPCTL0, reg_val);
		}else{	/* except EP0 */
			reg_val = SCD__READ(sdev, DOEPCTL(epnum));
			if(reg_val&SCD_B_DXEPCTLN_EPENA){
				/* EPDIS/STALLå */
				reg_val |= SCD_B_DXEPCTLN_EPDIS|SCD_B_DXEPCTLN_STALL;
				SCD__WRITE(sdev, DOEPCTL(epnum), reg_val);

				SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
#ifdef 	SCD_POLL_ON
				/* װȯԤ */
				rslt = poll_wait_outepdis(sgadget, epnum);
				if(rslt) goto err;
#endif
				SCD_G_SPIN_LOCK_IRQ(&gep->Lock, flags);
			}else{
				/* STALLΤߥå */
				reg_val |= SCD_B_DXEPCTLN_STALL;
				SCD__WRITE(sdev, DOEPCTL(epnum), reg_val);
			}
		}
		/* DCTL */
		reg_val = SCD__READ(sdev, DCTL);
		reg_val |= SCD_B_DCTL_CGOUTNAK;	/* GlobalNAKꥢ */
		SCD__WRITE(sdev, DCTL, reg_val);

		SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);

		/* žEP */
		rslt = scd_serach_trans_ep(sgadget, epnum);
		if(rslt){	/* 0:thru */
	
			SCD_G_SPIN_LOCK_IRQ(&gep->Lock, flags);

			/* FIFO flush */
			reg_val = SCD__READ(sdev, GRSTCTL);
			reg_val &= ~(SCD_M_GRSTCTL_TXFNUM<<SCD_S_GRSTCTL_TXFNUM|
				SCD_B_GRSTCTL_TXFFLSH|SCD_B_GRSTCTL_RXFFLSH);
			reg_val |= SCD_B_GRSTCTL_RXFFLSH;
			SCD__WRITE(sdev, GRSTCTL, reg_val);

			SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
			/* FLUSHλԤ */
			rslt = chk_flush_comp(sgadget, SCD_B_GRSTCTL_TXFFLSH);
			if(rslt){
				/* 顼ʸߤϥ롼Ǥ */
				ERR("==>error txfflush\n");
				goto err;
			}
		}
		SCD_G_SPIN_LOCK_IRQ(&gep->Lock, flags);

		/*  */
		rslt = set_gwq_flag(gevent, SCD_GQ_IDLE);
		if(rslt){
			ERR("error int epdis\n");
			SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
			goto err;
		}
	/*-----------------------------------------------------------------*/
	}else{	/* clear STALL */
#ifdef DISPLAY_STALL_VAL
		DBG(1," clear STALL ep:%x \n",epnum);
#endif
		/* EP */
		if(epnum==SCD_G_EP0){	/* EP0 */
			/*  */
			rslt = scd_chg_ep0stage(sgadget, SCD_EP0_STAGE_SETUP);
			if(rslt){
			/* 顼Ϥꤨʤ */
				ERR("==> error ep0stage not changed to SETUP \n");
				SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
				DBG(1, "==>\n");
				return -EPERM;
			}
			rslt = scd_set_ctrl_substate(sgadget, SCD_EP_IDLE);
			if(rslt){
				/* 顼Ϥꤨʤ */
				ERR("==> abnormal error  not chaneged to IDLE \n");
				SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
				return -EPERM;
			}
			
			/* reset setup count */
			scd_g_reset_setupcnt(sgadget, SCD_G_SUPCNT_MODE1);

		}else{	/* EP0ʳ */
			rslt = scd_set_epstate(sgadget, epnum, SCD_EP_IDLE);
			if(rslt){
				ERR("==>error not change to STALL state \n");
				SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
				return -EPERM;
			}
		}
		/* DOEPCTL */
		if(epnum==SCD_G_EP0){	/* EP0 */
			reg_val = SCD__READ(sdev, DOEPCTL0);
			/* STALLꥢ */
			reg_val &= ~SCD_B_DOEPCTL0_STALL;
			SCD__WRITE(sdev, DOEPCTL0, reg_val);
		}else{	/* except EP0 */
			reg_val = SCD__READ(sdev, DOEPCTL(epnum));
			if(reg_val&SCD_B_DXEPCTLN_EPDIS){
				/* ä顢HWλ;SWǥեʤȤʤ */
				ERR("WARN epdis remains at ep%d out \n",epnum);
				reg_val |= SCD_B_DXEPCTLN_EPENA; /* EnaӥåΩƤȾä*/
												 /* װ⤿ġâ */
			}
			/* STALLꥢ */
			reg_val &= ~SCD_B_DXEPCTLN_STALL;
			/* ȥ륯ꥢ */
			reg_val |= SCD_B_DXEPCTLN_SETD0PID;
			SCD__WRITE(sdev, DOEPCTL(epnum), reg_val);
		}
	}
	SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
	
	DBG(2,"==> \n");
	return 0;
err:
	SCD_G_SPIN_LOCK_IRQ(&gep->Lock, flags);

	/* DCTL */
	reg_val = SCD__READ(sdev, DCTL);
	reg_val |= SCD_B_DCTL_CGOUTNAK;	/* GlobalNAKꥢ */
	SCD__WRITE(sdev, DCTL, reg_val);

	/*  */
	rslt = set_gwq_flag(gevent, SCD_GQ_IDLE);
	if(rslt){
		/* 顼Ϥꤨʤ */
		ERR("error int epdis\n");
		DBG(1,"==>\n");
		return rslt;
	}
	/* EP */
	if(epnum==SCD_G_EP0){	/* EP0 */
			rslt = scd_chg_ep0stage(sgadget, SCD_EP0_STAGE_SETUP);
			if(rslt){
				/* 顼Ϥꤨʤ */
				ERR("==> error ep0stage not changed to SETUP \n");
				SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
				return -EPERM;
			}
			rslt = scd_set_ctrl_substate(sgadget, SCD_EP_IDLE);
			if(rslt){
				/* 顼Ϥꤨʤ */
				ERR("==> abnormal error  not chaneged to IDLE \n");
				SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
				return -EPERM;
			}
			/* reset setup count */
			scd_g_reset_setupcnt(sgadget, SCD_G_SUPCNT_MODE1);
	}else{
			rslt = scd_set_epstate(sgadget, epnum, SCD_EP_IDLE);
			if(rslt){
				SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
				ERR("==>error not change to IDLE state \n");
				return -EPERM;
			}
	}
	SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
	ERR("==> error or interrupt end\n");
	return -EPERM;
}

#ifdef CHK_SUPCNT
/*-------------------------------------------------------
	name	: chk_setupcnt
	argment	: struct scd_gadget *sgadget
	return	: none
	description:
			  forr debug to check setup count
	date	: 2010/06/17
-------------------------------------------------------*/
static void chk_setupcnt(struct scd_gadget *sgadget)

{
	struct scd_device *sdev=sgadget->g_device;
	u32 reg_val;
	u32 val;

	/* setup */
	reg_val = SCD__READ(sdev, DOEPTSIZ0);

	val = (reg_val&(SCD_M_DOEPTSIZ0_SUPCNT<<SCD_S_DOEPTSIZ0_SUPCNT))
			>>SCD_S_DOEPTSIZ0_SUPCNT;
	if(val!=SCD_SUPCNT_VAL)
		DBG(1,"set supcnt:%x\n",val);

}
#endif

#ifdef SCD_POLL_ON
/*-------------------------------------------------------
	name	: poll_wait_goutnakeff
	argment	: struct scd_gadget *sgadget
	return	: int
	description:
			  wait until gout-nakeff is set
	date	: 2010/06/17
-------------------------------------------------------*/
static int poll_wait_goutnakeff(struct scd_gadget *sgadget, u8 epnum)
{
	struct scd_device *sdev = sgadget->g_device;
	struct scd_gep *gep = &sgadget->gep[epnum];
	struct scd_gadget_wq *gevent = &(gep->wq);
	u32 reg_val;
	unsigned long flags;
	scd_usec start, end;
	int cnt=0;
	int rslt = 0;
	int wq_flag = 0;

	DBG(2,"<==\n");
	SCD_G_SPIN_LOCK_IRQ(&gep->Lock, flags);

	/* λǧ */
	start = scd_device_get_time(sdev);
	while(1){
		reg_val = SCD__READ(sdev, GINTSTS);
		if(reg_val&SCD_B_GINTSTS_GOUTNAKEFF){
			/* loop exit */
			DBG(2,"Detect GOUTNakeff ep:%x\n",epnum);
			break;
		}
		wq_flag = get_gwq_flag(gevent);
		switch(wq_flag){
		case SCD_GQ_IDLE:		/* idle */
		case SCD_GQ_CONNRESET:	/* reset */
		case SCD_GQ_SHUTDOWN:	/* disconnect */
			DBG(1,"exit loop by state:%x\n",wq_flag);
			rslt = SCD_POLL_SKIP;
			goto skip;
		}
		udelay(SCD_POLL_UDELAY);
		cnt++;
		if(cnt>=SCD_POLL_CNT){
			end = scd_device_get_time(sdev);
			/* Ϥ絬֤timeout */
			if (end - start > SCD_POLL_WAIT_GOUTNAKEFF) {
				ERR("GOUTNAK wait failed timeout:%ld \n",(end-start));
				rslt = -ETIME;
				break;
			}
		}
	}
skip:
	SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
	DBG(2,"==>\n");
	return rslt;
}
/*-------------------------------------------------------
	name	: poll_wait_innakeff
	argment	: struct scd_gadget *sgadget
	return	: int
	description:
			  wait until gout-nakeff is set
	date	: 2010/05/14
-------------------------------------------------------*/
static int poll_wait_innakeff(struct scd_gadget *sgadget, u8 epnum)
{
	struct scd_device *sdev = sgadget->g_device;
	struct scd_gep *gep = &sgadget->gep[epnum];
	struct scd_gadget_wq *gevent = &(gep->wq);
	u32 reg_val;
	unsigned long flags;
	scd_usec start, end;
	int cnt=0;
	int rslt = 0;
	int wq_flag = 0;

	DBG(1,"<==\n");
	SCD_G_SPIN_LOCK_IRQ(&gep->Lock, flags);

	/* λǧ */
	start = scd_device_get_time(sdev);
	while(1){
		reg_val = SCD__READ(sdev, DIEPINT(epnum));
		if(reg_val&SCD_B_DXEPINTN_INEPNAKEFF){
			/* loop exit */
			DBG(2,"Detect IN NakEff ep:%x\n",epnum);
			break;
		}
		wq_flag = get_gwq_flag(gevent);
		switch(wq_flag){
		case SCD_GQ_IDLE:		/* idle */
		case SCD_GQ_CONNRESET:	/* reset */
		case SCD_GQ_SHUTDOWN:	/* disconnect */
			DBG(1,"exit loop by state:%x\n",wq_flag);
			rslt = SCD_POLL_SKIP;
			goto skip;
		}
		udelay(SCD_POLL_UDELAY);
		cnt++;
		if(cnt>=SCD_POLL_CNT){
			end = scd_device_get_time(sdev);
			/* Ϥ絬֤timeout */
			if (end - start > SCD_POLL_WAIT_INNAKEFF) {
				ERR("GOUTNAK wait failed timeout:%ld \n",(end-start));
				rslt =  -ETIME;
				break;
			}
		}
	}
skip:
	SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
	DBG(1,"==>\n");
	return rslt;
}
/*-------------------------------------------------------
	name	: poll_wait_outepdis
	argment	: struct scd_gadget *sgadget
			  u8 epnum
	return	: int
	description:
			  wait until epdis is set
	date	: 2010/05/14
-------------------------------------------------------*/
static int poll_wait_outepdis(struct scd_gadget *sgadget, u8 epnum)
{
	struct scd_device *sdev = sgadget->g_device;
	struct scd_gep *gep = &sgadget->gep[epnum];
	struct scd_gadget_wq *gevent = &(gep->wq);
	u32 reg_val;
	unsigned long flags;
	scd_usec start, end;
	int cnt=0;
	int rslt = 0;
	int wq_flag = 0;

	DBG(2,"<==\n");

	SCD_G_SPIN_LOCK_IRQ(&gep->Lock, flags);

	/* λǧ */
	start = scd_device_get_time(sdev);
	while(1){
		reg_val = SCD__READ(sdev, DOEPINT(epnum));
		if(reg_val&SCD_B_DXEPINTN_EPDISBLD){
			/* loop exit */
			DBG(2,"Detect EPDis ep:%x\n",epnum);
			break;
		}
		wq_flag = get_gwq_flag(gevent);
		switch(wq_flag){
		case SCD_GQ_IDLE:		/* idle */
		case SCD_GQ_CONNRESET:	/* reset */
		case SCD_GQ_SHUTDOWN:	/* disconnect */
			DBG(2,"exit loop by state:%x\n",wq_flag);
			rslt = SCD_POLL_SKIP;
			goto skip;
		}
		udelay(SCD_POLL_UDELAY);
		cnt++;
		if(cnt>=SCD_POLL_CNT){
			end = scd_device_get_time(sdev);
			/* Ϥ絬֤timeout */
			if (end - start > SCD_POLL_WAIT_EPDIS) {
				ERR("epdis wait failed timeout:%ld \n",(end-start));
				rslt = -ETIME;
				break;
			}
		}
	}
skip:
	SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
	DBG(2,"==>\n");
	return rslt;
}
/*-------------------------------------------------------
	name	: poll_wait_inepdis
	argment	: struct scd_gadget *sgadget
			  u8 epnum
	return	: int
	description:
			  wait until epdis is set
	date	: 2010/05/14
-------------------------------------------------------*/
static int poll_wait_inepdis(struct scd_gadget *sgadget, u8 epnum)
{
	struct scd_device *sdev = sgadget->g_device;
	struct scd_gep *gep = &sgadget->gep[epnum];
	struct scd_gadget_wq *gevent = &(gep->wq);
	u32 reg_val;
	unsigned long flags;
	scd_usec start, end;
	int cnt=0;
	int rslt = 0;
	int wq_flag = 0;

	DBG(2,"<==\n");

	SCD_G_SPIN_LOCK_IRQ(&gep->Lock, flags);

	/* λǧ */
	start = scd_device_get_time(sdev);
	while(1){
		reg_val = SCD__READ(sdev, DIEPINT(epnum));
		if(reg_val&SCD_B_DXEPINTN_EPDISBLD){
			/* loop exit */
			DBG(2,"Detect EPDis ep:%x\n",epnum);
			break;
		}
		wq_flag = get_gwq_flag(gevent);
		switch(wq_flag){
		case SCD_GQ_IDLE:		/* idle */
		case SCD_GQ_CONNRESET:	/* reset */
		case SCD_GQ_SHUTDOWN:	/* disconnect */
			DBG(2,"exit loop by state:%x\n",wq_flag);
			rslt = SCD_POLL_SKIP;
			goto skip;
		}
		udelay(SCD_POLL_UDELAY);
		cnt++;
		if(cnt>=SCD_POLL_CNT){
			end = scd_device_get_time(sdev);
			/* Ϥ絬֤timeout */
			if (end - start > SCD_POLL_WAIT_EPDIS) {
				ERR("epdis wait failed timeout:%ld \n",(end-start));
				rslt = -ETIME;
				break;
			}
		}
	}
skip:
	SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);
	DBG(2,"==>\n");
	return rslt;
}
#endif

/*-------------------------------------------------------
	name	: set_ep0_self_stall
	argment	: struct scd_gadget *sgadget
	return	: void
	description:
			  set stall for ep0 by oneself
	date	: 2010/05/19
-------------------------------------------------------*/
void set_ep0_self_stall(struct scd_gadget *sgadget){

	/* EP0 stall */
	scd_gadget_set_halt_ep0(sgadget, SCD_G_SET_STALL);
}

/*-------------------------------------------------------
	name	: scd_g_get_lastbuf
	argment	: struct scd_gadget *sgadget
			  struct usb_request *pRequest, 
			  u8 epnum,
	return	: int
	description:
			  to get the buffer for last short rx data 
	date	: 2010/10/06
-------------------------------------------------------*/
int scd_g_get_lastbuf(struct scd_gadget *sgadget, 
	struct usb_request *pRequest, u8 epnum)
{
	struct scd_device *sdev = sgadget->g_device;
	struct scd_gep *gep = &sgadget->gep[epnum];

	DBG(3,"<==\n");

	if(gep->lbuf.buf){
		/* Хɥ쥹 */
		gep->lbuf.dma = 
		 (u8 *)dma_map_single(sdev->dev, gep->lbuf.buf, gep->wMaxp, DMA_FROM_DEVICE);
		if(!(gep->lbuf.dma)){
			ERR("==> error no mapping memory lastdma \n");
			return -EPERM;
		}
	}else{
		/* 顼ɽ */
		ERR("error no memory lastbuf \n");
		DBG(1,"==>\n");
		return -ENOMEM;
	}
	if(!pRequest){
		ERR("==> error no pRequest \n");
		return -ENOENT;
	}
	/* ° */
	((struct scd_grequest *)pRequest)->bAttribute |= SCD_REQ_ADDBUF;

	DBG(3,"==>\n");
	return 0;
}

/*-------------------------------------------------------
	name	: scd_g_free_lastbuf
	argment	: struct scd_gadget *sgadget
			  struct usb_request *pRequest, 
			  u8 epnum,
	return	: int
	description:
			  to free the buffer for last short rx data 
	date	: 2010/06/09
-------------------------------------------------------*/
int scd_g_free_lastbuf(struct scd_gadget *sgadget, 
	struct usb_request *pRequest, u8 epnum)
{
	struct scd_device *sdev = sgadget->g_device;
	struct scd_gep *gep = &sgadget->gep[epnum];

	DBG(3,"<==\n");

	if(gep->lbuf.dma){
		/* Хɥ쥹 */
		dma_unmap_single(sdev->dev, (dma_addr_t)(gep->lbuf.dma), gep->wMaxp, DMA_FROM_DEVICE);
	}else{
		/* 顼ɽ */
		ERR("==> error no memory lastdma \n");
		return -ENOMEM;
	}
	if(!pRequest){
		ERR("==> error no pRequest \n");
		return -ENOENT;
	}
	/* ° */
	((struct scd_grequest *)pRequest)->bAttribute &= ~SCD_REQ_ADDBUF;

	DBG(3,"==>\n");
	return 0;
}

/*-------------------------------------------------------
	name	: scd_g_cp_buf
	argment	: struct scd_gadget *sgadget
			  struct usb_request *pRequest, 
			  u8 epnum,
			  int size
	return	: int
	description:
			  to copy the data of last short rx data 
	date	: 2010/06/09
-------------------------------------------------------*/
int scd_g_cp_buf(struct scd_gadget *sgadget, 
	struct usb_request *pRequest, u8 epnum, int size){

	struct scd_gep *gep = &sgadget->gep[epnum];
	int rslt = 0;
	
	DBG(3,"<==\n");

	/* üǡХåեunmap */
	rslt = scd_g_free_lastbuf(sgadget, pRequest, epnum);
	
	if(rslt){
		DBG(1,"==> error \n");
		return -EPERM;
	}
	if(!pRequest){
		DBG(1,"==> error no pRequest \n");
		return -ENOENT;
	}
#ifdef SCD_CACHE_INVALIDATE_AFTER_DMA
	/* cache invalidate */
	udif_cache_ctrl_r((void *)(gep->lbuf.buf), size,
                         UDIF_CACHE_INVALIDATE);
#endif /* SCD_CACHE_INVALIDATE_AFTER_DMA */
	/* ǡԡ */
	memcpy((void *)&((char *)(pRequest->buf))[pRequest->actual],
 		(void *)(gep->lbuf.buf), size);

	DBG(3,"==>\n");
	return 0;
}

/*-------------------------------------------------------
	name	: scd_g_chk_lastbuf
	argment	: struct usb_request *pRequest
	return	: int
	description:
			  to check  whether the lastbuf attribute has been set.
	date	: 2010/06/09
-------------------------------------------------------*/
int scd_g_chk_lastbuf(struct usb_request *pRequest)
{
	if((((struct scd_grequest *)pRequest)->bAttribute&SCD_REQ_ADDBUFMSK)
 	  == SCD_REQ_ADDBUF){
		return true;
	}else{
		return false;
	}
}

/*-------------------------------------------------------
	name	: scd_g_ep0int_tmout
	argment	: struct scd_gadget *sgadget
	return	: int
	description:
			  to do for ep0 after detecting a timeout.
	date	: 2010/08/09
-------------------------------------------------------*/
int scd_g_ep0int_tmout(struct scd_gadget *sgadget){

	u8 ep0stage;
	int rslt = 0;
	u32 reg_val;
	struct scd_device *sdev = sgadget->g_device;
	struct usb_request *pRequest;
	struct scd_gep *gep = &(sgadget->gep[SCD_G_EP0]);

	DBG(1,"<== \n");
	/* EP0stage */
	ep0stage = scd_get_ep0stage(sgadget);

	switch(ep0stage){
	case SCD_EP0_STAGE_DATA_IN:
		/* the ready for Status IN */

		/* ߤΥꥯȼ */
		pRequest = scd_get_currentreq(gep);

		/* ǽѥåȤACKʤΤ */
		if(pRequest->length == pRequest->actual + gep->dwLen){
			DBG(1,"change status becase of last packet \n");

			/* FIFOեå */
			scd_g_fifo_flush_tx(sgadget, SCD_G_EP0, SCD_G_FLUSH_STSKIP);

			/*  */
			rslt = scd_chg_ep0stage(sgadget, SCD_EP0_STAGE_STATUS_OUT);
			if(rslt){
				ERR("==> error \n");
				return -EPERM;
			}
			/* SubStateѹ */
			rslt = scd_set_ctrl_substate(sgadget, SCD_EP_IDLE);
			if(rslt){
				/* 顼 */
				ERR("==> error \n");
				return -EPERM;
			}
			/* ž/ѥåȥ */
			reg_val = SCD_G_PKT1<<SCD_S_DOEPTSIZ0_PKTCNT|SCD_G_SZ0;
			SCD__WRITE(sdev, DOEPTSIZ0, reg_val);

			/* setup *//* EPenableԤ */
			scd_g_reset_setupcnt(sgadget, SCD_G_SUPCNT_MODE0);
		
			/* NAK */
			reg_val = SCD__READ(sdev, DOEPCTL0);
			DBG(3,"DOEPCTL0 :%x \n",reg_val);
			reg_val |= SCD_B_DOEPCTL0_CNAK;
			SCD__WRITE(sdev, DOEPCTL0, reg_val);

			/* wait for  the XferCompl interrupt. */
		}else{
			DBG(1,"no care becase of not last packet \n");
		}
		rslt = 0;
		break;
	default:
		ERR("ep0stage is not applicable:%x \n",ep0stage);
		rslt = -EINVAL;
	}
	DBG(1,"==> \n");
	return rslt;
}
/*-------------------------------------------------------
	name	: scd_g_chk_translimit
	argment	: struct scd_gadget *sgadget
			  struct usb_request *pReq
			  u8 epnum
	return	: int
	description:
			  to check whether the request length is over
			  the limit length as the HW specified.
	date	: 2010/10/22
-------------------------------------------------------*/
int scd_g_chk_translimit(struct scd_gadget *sgadget, struct usb_request *pReq, u8 epnum)
{
	struct scd_gep *gep = &(sgadget->gep[epnum]);
	unsigned long rwflags;
	u16	maxp = gep->wMaxp;
	u32 limitsz, restsz;
	u8 rslt = false;

#ifdef RW_LOCK_ON
	SCD_G_READ_LOCK_IRQ(&gep->rwLock, rwflags);
#endif	

	/* 1DMAǤκžǽ */
	limitsz = maxp * SCD_MAX_PKT_CNT;	/* žĹ */
	restsz	= pReq->length - pReq->actual;

	DBG(1,"limitsz:%d restsz:%d \n",limitsz, restsz);
	
	if(restsz > limitsz){
		/* over */
		DBG(1,"Request->length is over the limit\n");
		
		rslt = true;
	}else{
		rslt = false;
	}
#ifdef RW_LOCK_ON
	SCD_G_READ_UNLOCK_IRQ(&gep->rwLock, rwflags);
#endif
	return rslt;
}

/*-------------------------------------------------------
	name	: scd_g_retry_trans
	argment	: struct scd_gadget *sgadget
			  struct usb_request *pReq
			  u8 epnum
	return	: u8
	description:
			  to restart DMA for a general tx transfer
	date	: 2010/10/25
-------------------------------------------------------*/
int scd_g_retry_trans(struct scd_gadget *sgadget, struct usb_request *pReq, u8 epnum)
{
	int rslt=0;
	u32 reg_val;
	struct scd_device *sdev = sgadget->g_device;
	struct scd_gep *gep = &(sgadget->gep[epnum]);
	u32 req_len, set_len;
	unsigned long flags;
	u16	maxp;
	u16	pktnum;

	DBG(3,"<==\n");
#ifdef RW_LOCK_ON
	DBG(1,"gep r+1 dep:%d\n",gep->rwLock.read_depth);
	SCD_G_READ_LOCK(&gep->rwLock);
#endif	

	req_len		= pReq->length - pReq->actual;
	maxp		= gep->wMaxp;	/* usb_ep.maxpacket ˤ⤢ */

	DBG(1,"req_len:0x%x maxp:0x%x actual:0x%x \n",req_len, maxp, pReq->actual);

	/* ž׵ĹĶȽ */
	rslt = scd_g_chk_translimit(sgadget, pReq, epnum);
	if(rslt == false){
		/* ǽѥ(shortǽ륱ɬ */
		DBG(1,"last pass \n");
		/* ѥåȥ */
		if(req_len%maxp!=0){
			pktnum = (req_len -req_len%maxp)/maxp + 1;
		}else{
			pktnum = req_len/maxp;
		}
		set_len = req_len;
	}else{
		/* ѥåʬ */
		pktnum = SCD_MAX_PKT_CNT;
		/* ž */
		set_len = pktnum * maxp;
	}
	/* ž */
	gep->dwLen = set_len;

	DBG(1,"after calc req_len:0x%x set_len:0x%x pktnum:0x%x \n",req_len, gep->dwLen, pktnum);
	SCD_G_SPIN_LOCK_IRQ(&gep->Lock, flags);

	/* ž/ѥåȥ */
	reg_val = ((pktnum&SCD_M_DXEPTSIZN_PKTCNT)<<SCD_S_DXEPTSIZN_PKTCNT)|set_len;
	SCD__WRITE(sdev, DIEPTSIZ(epnum), reg_val);

	SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);

#ifdef RW_LOCK_ON
	DBG(1,"gep r-1 dep:%d\n",gep->rwLock.read_depth);
	SCD_G_READ_UNLOCK(&gep->rwLock);
#endif	

	/* DMAХåեϤΤޤ */
	
	/* EP-Stateѹ */
	rslt = scd_set_epstate(sgadget, epnum, SCD_EP_DATA_CONT);
	if(rslt){
		/* 顼 */
		/* unmap */
		dma_unmap_req(sdev, pReq);

		/* EPstateѹ */
		rslt = scd_set_epstate(sgadget, epnum, SCD_EP_IDLE);
		if(rslt){
			ERR("error epstate cannot change to IDLE \n");
			DBG(1, "==>\n");
			return -EPERM;
		}
		ERR("error epstate cannot change to IN_READY \n");
		ERR("==> error epstate \n");

#ifdef RW_LOCK_ON
		DBG(1,"gep w+1 dep:%d\n",gep->rwLock.read_depth);
		SCD_G_WRITE_LOCK(&gep->rwLock);
#endif	

		pReq->status = -EPERM;

#ifdef RW_LOCK_ON
		DBG(1,"gep w-1 dep:%d\n",gep->rwLock.read_depth);
		SCD_G_WRITE_UNLOCK(&gep->rwLock);
#endif	

		DBG(2,"complete_request\n");
		/* ̤žλ */
		scd_gadget_complete_request(pReq, pReq->status);

		return rslt;
	}
	SCD_G_SPIN_LOCK_IRQ(&gep->Lock, flags);

	/* DIEPCTLå */
	reg_val = SCD__READ(sdev, DIEPCTL(epnum));
	reg_val &= ~(SCD_B_DXEPCTLN_EPDIS|SCD_B_DXEPCTLN_STALL);

	/* EP */
	reg_val |= SCD_B_DXEPCTLN_EPENA;
	/* NAK */
	reg_val |= SCD_B_DXEPCTLN_CNAK;
	
	SCD__WRITE(sdev, DIEPCTL(epnum), reg_val);

	SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);

	return 0;
}

/*-------------------------------------------------------
	name	: scd_g_xfercompl_rx_cont
	argment	: struct scd_gadget *sgadget
			  struct usb_request *pReq
			  u8 epnum
	return	: u8
	description:
			  to restart DMA for a general rx transfer
	date	: 2010/10/25
-------------------------------------------------------*/
int scd_g_xfercompl_rx_cont(struct scd_gadget *sgadget, struct usb_request *pReq, u8 epnum)
{
	struct scd_device *sdev = sgadget->g_device;
	struct scd_gep *gep = &(sgadget->gep[epnum]);
	unsigned long flags;
	long rest_len;
	int rslt;
	u32 set_len = 0;
	u32 req_len;
	u16 maxp;
	u16	pktnum;
	u32 reg_val;

	DBG(1,"<==\n");

	/* ĥ */
	rest_len = pReq->length - pReq->actual;
	/* MAXP */
	maxp	 = gep->wMaxp;

	/* ž׵ĹĶȽ */
	rslt = scd_g_chk_translimit(sgadget, pReq, epnum);
	if(rslt == false){
		if(rest_len%maxp!=0){
			/* MAXPʲ */
			if(rest_len<maxp){
				/* 饹ȥХåեΰ */
				rslt = scd_g_get_lastbuf(sgadget, pReq, epnum);
				if(rslt){
					ERR("==> error can't ready a lastbuf \n");
					return -EPERM;
				}	
				pktnum = SCD_LAST_PACKET;	/* 1 */
				/* EP֥å */
				rslt = scd_set_epstate(sgadget, epnum, SCD_EP_DATA_END);
				if(rslt){
					return -EPERM;
				}
			}else{
				pktnum = (rest_len -rest_len%maxp)/maxp;
				/* EP֥å */
				rslt = scd_set_epstate(sgadget, epnum, SCD_EP_DATA_CONT);
				if(rslt){
					return -EPERM;
				}
			}
		}else{
			pktnum = rest_len/maxp;
			/* EP֥å */
			rslt = scd_set_epstate(sgadget, epnum, SCD_EP_DATA_CONT);
			if(rslt){
				return -EPERM;
			}
		}
		/* ž */
		if(maxp%4!=0){
			req_len = pktnum * (maxp +4 -(maxp % 4));
			/* 󥿥ץ */
			((struct scd_grequest *)pReq)->bAttribute |= SCD_REQ_4ALIGIN;
		}else{
			req_len = pktnum * maxp;
		}
		if(sgadget->bTransMode == SCD_GADGET_TRANS_CPU){
			/* CPU */
			if(rest_len<maxp){
				/*  */
				set_len	= req_len;
			}else{
				/* (maximum size up to maxp) */
				set_len	= maxp;
			}
			/* ž */
			gep->dwLen = set_len;
		}else{
			/* DMA */
			if(rest_len<maxp){
				/* ž */
				gep->dwLen = rest_len;
			}else{
				/* ž */
				gep->dwLen = req_len;
			}
			set_len = req_len;
		}
		SCD_G_SPIN_LOCK_IRQ(&gep->Lock, flags);	

		/* ž/ѥåȥ */
		reg_val = ((pktnum&SCD_M_DXEPTSIZN_PKTCNT)<<SCD_S_DXEPTSIZN_PKTCNT)|set_len;
		SCD__WRITE(sdev, DOEPTSIZ(epnum), reg_val);

		/* üžɤǧ롣 */
		rslt = scd_g_chk_lastbuf(pReq);
		if(rslt == true){
			/* DMAХåե(lastbuf) */
			reg_val = (u32)gep->lbuf.dma;
			SCD__WRITE(sdev, DOEPDMA(epnum), reg_val);
		}
		/* DOEPCTL */
		reg_val = SCD__READ(sdev, DOEPCTL(epnum));
		/* EP */
		reg_val |= SCD_B_DXEPCTLN_EPENA;
		/* NAK */
		reg_val |= SCD_B_DXEPCTLN_CNAK;
		SCD__WRITE(sdev, DOEPCTL(epnum), reg_val);

		SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);			
			
	}else{
		/* ž */
		/* ѥåʬ */
		pktnum = SCD_MAX_PKT_CNT;
		/* ž */
		if(maxp%4!=0){
			req_len = pktnum * (maxp +4 -(maxp % 4));
			/* 󥿥ץ */
			((struct scd_grequest *)pReq)->bAttribute |= SCD_REQ_4ALIGIN;
		}else{
			req_len = pktnum * maxp;
		}			
		/* žĹ */
		gep->dwLen = req_len;

		SCD_G_SPIN_LOCK_IRQ(&gep->Lock, flags);			

		/* ž */
		set_len = req_len;
			
		/* ž/ѥåȥ */
		reg_val = ((pktnum&SCD_M_DXEPTSIZN_PKTCNT)<<SCD_S_DXEPTSIZN_PKTCNT)|set_len;
		SCD__WRITE(sdev, DOEPTSIZ(epnum), reg_val);

		SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);			

		/* EP֥å */
		rslt = scd_set_epstate(sgadget, epnum, SCD_EP_DATA_CONT);
		if(rslt){
			return -EPERM;
		}
		SCD_G_SPIN_LOCK_IRQ(&gep->Lock, flags);			

		/* DOEPCTL */
		reg_val = SCD__READ(sdev, DOEPCTL(epnum));
		/* EP */
		reg_val |= SCD_B_DXEPCTLN_EPENA;
		/* NAK */
		reg_val |= SCD_B_DXEPCTLN_CNAK;
		SCD__WRITE(sdev, DOEPCTL(epnum), reg_val);

		SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);			
	}
	DBG(1,"==>\n");
	return 0;
}
/*-------------------------------------------------------
	name	: scd_g_xfercompl_rx_zero
	argment	: struct scd_gadget *sgadget
			  struct usb_request *pReq
			  u8 epnum
	return	: u8
	description:
			  to restart DMA for a general rx zero transfer
	date	: 2010/10/25
-------------------------------------------------------*/
int scd_g_xfercompl_rx_zero(struct scd_gadget *sgadget, struct usb_request *pReq, u8 epnum)
{
	struct scd_device *sdev = sgadget->g_device;
	struct scd_gep *gep = &(sgadget->gep[epnum]);
	unsigned long flags;
	long rest_len;
	int rslt;
	u32 set_len = 0;
	u16 maxp;
	u16	pktnum;
	u32 reg_val;

	/* ĥ */
	rest_len = pReq->length - pReq->actual;
	/* MAXP */
	maxp	 = gep->wMaxp;

	/* 쥸ꥵ */
	set_len = maxp&SCD_M_DXEPTSIZN_XFERSIZE;
	/* »ĥ */
	gep->dwLen = rest_len;
	DBG(3,"lastbuf action dwLen:%x\n",gep->dwLen);
	if(rest_len >= maxp){
		/* ϰϳλĥǤС顼  */
		ERR("==>error no support rest size \n");
		return -EPERM;
	}
	/* ѥåȥ */
	pktnum = 1;

	SCD_G_SPIN_LOCK_IRQ(&gep->Lock, flags);			
		
	/* ž/ѥåȥ */
	reg_val = ((pktnum&SCD_M_DXEPTSIZN_PKTCNT)<<SCD_S_DXEPTSIZN_PKTCNT)|set_len;
	SCD__WRITE(sdev, DOEPTSIZ(epnum), reg_val);

	SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);			

	/* DMAž⡼Ƚ */
	if(sgadget->bTransMode==SCD_GADGET_TRANS_DMA){
		/* 饹ȥХåեΰ */
		rslt = scd_g_get_lastbuf(sgadget, pReq, epnum);
		if(rslt){
			ERR("==> error can't ready a lastbuf \n");
			return -EPERM;
		}
			
		/* EP֥å */
		rslt = scd_set_epstate(sgadget, epnum, SCD_EP_DATA_END);
		if(rslt){
			return -EPERM;
		}
		/* üžɤǧ롣 */
		rslt = scd_g_chk_lastbuf(pReq);
		if(rslt == true){
			SCD_G_SPIN_LOCK_IRQ(&gep->Lock, flags);			

			/* DMAХåե(lastbuf) */
			reg_val = (u32)gep->lbuf.dma;
			SCD__WRITE(sdev, DOEPDMA(epnum), reg_val);
			SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);			
		}
	}else{
		ERR("error no support CPU mode \n");
	}
	SCD_G_SPIN_LOCK_IRQ(&gep->Lock, flags);			

	/* DOEPCTL */
	reg_val = SCD__READ(sdev, DOEPCTL(epnum));
	/* EP */
	reg_val |= SCD_B_DXEPCTLN_EPENA;
	/* NAK */
	reg_val |= SCD_B_DXEPCTLN_CNAK;
	SCD__WRITE(sdev, DOEPCTL(epnum), reg_val);

	SCD_G_SPIN_UNLOCK_IRQ(&gep->Lock, flags);			

	return 0;
}

